From b6ac87209b89e2c66a1b4844caac9ce12338046c Mon Sep 17 00:00:00 2001
From: Stefan Metzmacher <metze@samba.org>
Date: Thu, 10 Jul 2014 08:21:22 +0200
Subject: [PATCH 01/94] s4:libcli/smb2: remove unused 'primary' argument of
 smb2_session_init()

This also gets rid of a talloc_reference() usage.

Signed-off-by: Stefan Metzmacher <metze@samba.org>
Reviewed-by: Volker Lendecke <vl@samba.org>
---
 source4/libcli/smb2/connect.c | 2 +-
 source4/libcli/smb2/session.c | 8 ++------
 source4/torture/smb2/util.c   | 2 +-
 3 files changed, 4 insertions(+), 8 deletions(-)

diff --git a/source4/libcli/smb2/connect.c b/source4/libcli/smb2/connect.c
index 18977d8..7a69b3e 100644
--- a/source4/libcli/smb2/connect.c
+++ b/source4/libcli/smb2/connect.c
@@ -172,7 +172,7 @@ static void smb2_connect_negprot_done(struct tevent_req *subreq)
 	/* This is a hack... */
 	smb2cli_conn_set_max_credits(transport->conn, 30);
 
-	state->session = smb2_session_init(transport, state->gensec_settings, state, true);
+	state->session = smb2_session_init(transport, state->gensec_settings, state);
 	if (tevent_req_nomem(state->session, req)) {
 		return;
 	}
diff --git a/source4/libcli/smb2/session.c b/source4/libcli/smb2/session.c
index dfc607e..4d1402a 100644
--- a/source4/libcli/smb2/session.c
+++ b/source4/libcli/smb2/session.c
@@ -36,7 +36,7 @@
  */
 struct smb2_session *smb2_session_init(struct smb2_transport *transport,
 				       struct gensec_settings *settings,
-				       TALLOC_CTX *parent_ctx, bool primary)
+				       TALLOC_CTX *parent_ctx)
 {
 	struct smb2_session *session;
 	NTSTATUS status;
@@ -45,11 +45,7 @@ struct smb2_session *smb2_session_init(struct smb2_transport *transport,
 	if (!session) {
 		return NULL;
 	}
-	if (primary) {
-		session->transport = talloc_steal(session, transport);
-	} else {
-		session->transport = talloc_reference(session, transport);
-	}
+	session->transport = talloc_steal(session, transport);
 
 	session->smbXcli = smbXcli_session_create(session, transport->conn);
 	if (session->smbXcli == NULL) {
diff --git a/source4/torture/smb2/util.c b/source4/torture/smb2/util.c
index 97761d7..4297cee 100644
--- a/source4/torture/smb2/util.c
+++ b/source4/torture/smb2/util.c
@@ -323,7 +323,7 @@ bool torture_smb2_session_setup(struct torture_context *tctx,
 
 	session = smb2_session_init(transport,
 				    lpcfg_gensec_settings(tctx, tctx->lp_ctx),
-				    mem_ctx, true);
+				    mem_ctx);
 
 	if (session == NULL) {
 		return false;
-- 
1.8.1.2


From 22a29b87b6aa10204df1750182518eacc3a291e7 Mon Sep 17 00:00:00 2001
From: Stefan Metzmacher <metze@samba.org>
Date: Tue, 24 Jun 2014 00:22:30 +0200
Subject: [PATCH 02/94] s4:libcli/smb2: add smb2_session_channel()

Signed-off-by: Stefan Metzmacher <metze@samba.org>
Reviewed-by: Volker Lendecke <vl@samba.org>
---
 source4/libcli/smb2/session.c | 83 +++++++++++++++++++++++++++++++++++++------
 source4/libcli/smb2/smb2.h    |  1 +
 2 files changed, 74 insertions(+), 10 deletions(-)

diff --git a/source4/libcli/smb2/session.c b/source4/libcli/smb2/session.c
index 4d1402a..d33ecaf 100644
--- a/source4/libcli/smb2/session.c
+++ b/source4/libcli/smb2/session.c
@@ -66,11 +66,54 @@ struct smb2_session *smb2_session_init(struct smb2_transport *transport,
 	return session;
 }
 
+/*
+ * Note: that the caller needs to keep 'transport' arround as
+ *       long as the returned session is active!
+ */
+struct smb2_session *smb2_session_channel(struct smb2_transport *transport,
+					  struct gensec_settings *settings,
+					  TALLOC_CTX *parent_ctx,
+					  struct smb2_session *base_session)
+{
+	struct smb2_session *session;
+	NTSTATUS status;
+
+	session = talloc_zero(parent_ctx, struct smb2_session);
+	if (!session) {
+		return NULL;
+	}
+	session->transport = transport;
+
+	status = smb2cli_session_create_channel(session,
+						base_session->smbXcli,
+						transport->conn,
+						&session->smbXcli);
+	if (!NT_STATUS_IS_OK(status)) {
+		talloc_free(session);
+		return NULL;
+	}
+
+	session->needs_bind = true;
+
+	/* prepare a gensec context for later use */
+	status = gensec_client_start(session, &session->gensec,
+				     settings);
+	if (!NT_STATUS_IS_OK(status)) {
+		talloc_free(session);
+		return NULL;
+	}
+
+	gensec_want_feature(session->gensec, GENSEC_FEATURE_SESSION_KEY);
+
+	return session;
+}
+
 struct smb2_session_setup_spnego_state {
 	struct tevent_context *ev;
 	struct smb2_session *session;
 	struct cli_credentials *credentials;
 	uint64_t previous_session_id;
+	bool session_bind;
 	bool reauth;
 	NTSTATUS gensec_status;
 	DATA_BLOB in_secblob;
@@ -98,6 +141,7 @@ struct tevent_req *smb2_session_setup_spnego_send(
 	const DATA_BLOB *server_gss_blob;
 	DATA_BLOB negprot_secblob = data_blob_null;
 	uint32_t timeout_msec;
+	uint8_t in_flags = 0;
 
 	timeout_msec = session->transport->options.request_timeout * 1000;
 
@@ -112,10 +156,11 @@ struct tevent_req *smb2_session_setup_spnego_send(
 	state->previous_session_id = previous_session_id;
 
 	current_session_id = smb2cli_session_current_id(state->session->smbXcli);
-	if (current_session_id != 0) {
+	if (state->session->needs_bind) {
+		state->session_bind = true;
+	} else if (current_session_id != 0) {
 		state->reauth = true;
 	}
-
 	server_gss_blob = smbXcli_conn_server_gss_blob(session->transport->conn);
 	if (server_gss_blob) {
 		negprot_secblob = *server_gss_blob;
@@ -158,11 +203,15 @@ struct tevent_req *smb2_session_setup_spnego_send(
 	}
 	state->gensec_status = status;
 
+	if (state->session_bind) {
+		in_flags |= SMB2_SESSION_FLAG_BINDING;
+	}
+
 	subreq = smb2cli_session_setup_send(state, state->ev,
 					    session->transport->conn,
 					    timeout_msec,
 					    session->smbXcli,
-					    0, /* in_flags */
+					    in_flags,
 					    0, /* in_capabilities */
 					    0, /* in_channel */
 					    state->previous_session_id,
@@ -191,6 +240,7 @@ static void smb2_session_setup_spnego_done(struct tevent_req *subreq)
 	NTSTATUS status;
 	struct iovec *recv_iov;
 	uint32_t timeout_msec;
+	uint8_t in_flags = 0;
 
 	timeout_msec = session->transport->options.request_timeout * 1000;
 
@@ -246,22 +296,35 @@ static void smb2_session_setup_spnego_done(struct tevent_req *subreq)
 			return;
 		}
 
-		status = smb2cli_session_set_session_key(session->smbXcli,
-							 session_key,
-							 recv_iov);
-		if (tevent_req_nterror(req, status)) {
-			return;
+		if (state->session_bind) {
+			status = smb2cli_session_set_channel_key(session->smbXcli,
+								 session_key,
+								 recv_iov);
+			if (tevent_req_nterror(req, status)) {
+				return;
+			}
+			session->needs_bind = false;
+		} else {
+			status = smb2cli_session_set_session_key(session->smbXcli,
+								 session_key,
+								 recv_iov);
+			if (tevent_req_nterror(req, status)) {
+				return;
+			}
 		}
-
 		tevent_req_done(req);
 		return;
 	}
 
+	if (state->session_bind) {
+		in_flags |= SMB2_SESSION_FLAG_BINDING;
+	}
+
 	subreq = smb2cli_session_setup_send(state, state->ev,
 					    session->transport->conn,
 					    timeout_msec,
 					    session->smbXcli,
-					    0, /* in_flags */
+					    in_flags,
 					    0, /* in_capabilities */
 					    0, /* in_channel */
 					    state->previous_session_id,
diff --git a/source4/libcli/smb2/smb2.h b/source4/libcli/smb2/smb2.h
index 101ef73..73e117a 100644
--- a/source4/libcli/smb2/smb2.h
+++ b/source4/libcli/smb2/smb2.h
@@ -116,6 +116,7 @@ struct smb2_session {
 	struct smb2_transport *transport;
 	struct gensec_security *gensec;
 	struct smbXcli_session *smbXcli;
+	bool needs_bind;
 };
 
 
-- 
1.8.1.2


From 054b723ee71e40bc3a249cdef4121a2c014042ba Mon Sep 17 00:00:00 2001
From: Volker Lendecke <vl@samba.org>
Date: Wed, 23 Jul 2014 11:25:00 +0000
Subject: [PATCH 03/94] s4:libcli/smb2: Fix a typo

Signed-off-by: Volker Lendecke <vl@samba.org>
---
 source4/libcli/smb2/session.c | 2 +-
 1 file changed, 1 insertion(+), 1 deletion(-)

diff --git a/source4/libcli/smb2/session.c b/source4/libcli/smb2/session.c
index d33ecaf..9ab96a3 100644
--- a/source4/libcli/smb2/session.c
+++ b/source4/libcli/smb2/session.c
@@ -67,7 +67,7 @@ struct smb2_session *smb2_session_init(struct smb2_transport *transport,
 }
 
 /*
- * Note: that the caller needs to keep 'transport' arround as
+ * Note: that the caller needs to keep 'transport' around as
  *       long as the returned session is active!
  */
 struct smb2_session *smb2_session_channel(struct smb2_transport *transport,
-- 
1.8.1.2


From 1a0cffe18e7eee098c8ee8ed79b08b0a96d01d83 Mon Sep 17 00:00:00 2001
From: Stefan Metzmacher <metze@samba.org>
Date: Tue, 24 Jun 2014 06:40:29 +0200
Subject: [PATCH 04/94] s4:torture/smb2: add smb2.session.bind1

This is a very simple multi-channel test

Signed-off-by: Stefan Metzmacher <metze@samba.org>
Reviewed-by: Volker Lendecke <vl@samba.org>
---
 source4/torture/smb2/session.c | 132 +++++++++++++++++++++++++++++++++++++++++
 1 file changed, 132 insertions(+)

diff --git a/source4/torture/smb2/session.c b/source4/torture/smb2/session.c
index 2b90c3f..3a4b3bd 100644
--- a/source4/torture/smb2/session.c
+++ b/source4/torture/smb2/session.c
@@ -1071,6 +1071,137 @@ done:
 	return ret;
 }
 
+bool test_session_bind1(struct torture_context *tctx, struct smb2_tree *tree1)
+{
+	const char *host = torture_setting_string(tctx, "host", NULL);
+	const char *share = torture_setting_string(tctx, "share", NULL);
+	struct cli_credentials *credentials = cmdline_credentials;
+	NTSTATUS status;
+	TALLOC_CTX *mem_ctx = talloc_new(tctx);
+	char fname[256];
+	struct smb2_handle _h1;
+	struct smb2_handle *h1 = NULL;
+	struct smb2_create io1;
+	union smb_fileinfo qfinfo;
+	bool ret = false;
+	struct smb2_tree *tree2 = NULL;
+	struct smb2_transport *transport1 = tree1->session->transport;
+	struct smb2_transport *transport2 = NULL;
+	struct smb2_session *session1_1 = tree1->session;
+	struct smb2_session *session1_2 = NULL;
+	struct smb2_session *session2_1 = NULL;
+	struct smb2_session *session2_2 = NULL;
+	uint32_t caps;
+
+	caps = smb2cli_conn_server_capabilities(transport1->conn);
+	if (!(caps & SMB2_CAP_MULTI_CHANNEL)) {
+		torture_skip(tctx, "server doesn't support SMB2_CAP_MULTI_CHANNEL\n");
+	}
+
+	/* Add some random component to the file name. */
+	snprintf(fname, 256, "session_bind1_%s.dat",
+		 generate_random_str(tctx, 8));
+
+	smb2_util_unlink(tree1, fname);
+
+	smb2_oplock_create_share(&io1, fname,
+				 smb2_util_share_access(""),
+				 smb2_util_oplock_level("b"));
+
+	status = smb2_create(tree1, mem_ctx, &io1);
+	CHECK_STATUS(status, NT_STATUS_OK);
+	_h1 = io1.out.file.handle;
+	h1 = &_h1;
+	CHECK_CREATED(&io1, CREATED, FILE_ATTRIBUTE_ARCHIVE);
+	CHECK_VAL(io1.out.oplock_level, smb2_util_oplock_level("b"));
+
+	status = smb2_connect(tctx,
+			      host,
+			      lpcfg_smb_ports(tctx->lp_ctx),
+			      share,
+			      lpcfg_resolve_context(tctx->lp_ctx),
+			      credentials,
+			      &tree2,
+			      tctx->ev,
+			      &transport1->options,
+			      lpcfg_socket_options(tctx->lp_ctx),
+			      lpcfg_gensec_settings(tctx, tctx->lp_ctx)
+			      );
+	torture_assert_ntstatus_ok_goto(tctx, status, ret, done,
+					"smb2_connect failed");
+	session2_2 = tree2->session;
+	transport2 = tree2->session->transport;
+
+	/*
+	 * Now bind the 2nd transport connection to the 1st session
+	 */
+	session1_2 = smb2_session_channel(transport2,
+					  lpcfg_gensec_settings(tctx, tctx->lp_ctx),
+					  tree2,
+					  session1_1);
+	torture_assert(tctx, session1_2 != NULL, "smb2_session_channel failed");
+
+	status = smb2_session_setup_spnego(session1_2,
+					   cmdline_credentials,
+					   0 /* previous_session_id */);
+	CHECK_STATUS(status, NT_STATUS_OK);
+
+	/* use the 1st connection, 1st session */
+	ZERO_STRUCT(qfinfo);
+	qfinfo.generic.level = RAW_FILEINFO_POSITION_INFORMATION;
+	qfinfo.generic.in.file.handle = _h1;
+	tree1->session = session1_1;
+	status = smb2_getinfo_file(tree1, mem_ctx, &qfinfo);
+	CHECK_STATUS(status, NT_STATUS_OK);
+
+	/* use the 2nd connection, 1st session */
+	ZERO_STRUCT(qfinfo);
+	qfinfo.generic.level = RAW_FILEINFO_POSITION_INFORMATION;
+	qfinfo.generic.in.file.handle = _h1;
+	tree1->session = session1_2;
+	status = smb2_getinfo_file(tree1, mem_ctx, &qfinfo);
+	CHECK_STATUS(status, NT_STATUS_OK);
+
+	tree1->session = session1_1;
+	smb2_util_close(tree1, *h1);
+	h1 = NULL;
+
+	/*
+	 * Now bind the 1st transport connection to the 2nd session
+	 */
+	session2_1 = smb2_session_channel(transport1,
+					  lpcfg_gensec_settings(tctx, tctx->lp_ctx),
+					  tree1,
+					  session2_2);
+	torture_assert(tctx, session2_1 != NULL, "smb2_session_channel failed");
+
+	status = smb2_session_setup_spnego(session2_1,
+					   cmdline_credentials,
+					   0 /* previous_session_id */);
+	CHECK_STATUS(status, NT_STATUS_OK);
+
+	tree2->session = session2_1;
+	status = smb2_util_unlink(tree2, fname);
+	CHECK_STATUS(status, NT_STATUS_OK);
+
+	ret = true;
+done:
+	talloc_free(tree2);
+	tree1->session = session1_1;
+
+	if (h1 != NULL) {
+		smb2_util_close(tree1, *h1);
+	}
+
+	smb2_util_unlink(tree1, fname);
+
+	talloc_free(tree1);
+
+	talloc_free(mem_ctx);
+
+	return ret;
+}
+
 struct torture_suite *torture_smb2_session_init(void)
 {
 	struct torture_suite *suite =
@@ -1085,6 +1216,7 @@ struct torture_suite *torture_smb2_session_init(void)
 	torture_suite_add_1smb2_test(suite, "reauth5", test_session_reauth5);
 	torture_suite_add_1smb2_test(suite, "reauth6", test_session_reauth6);
 	torture_suite_add_simple_test(suite, "expire1", test_session_expire1);
+	torture_suite_add_1smb2_test(suite, "bind1", test_session_bind1);
 
 	suite->description = talloc_strdup(suite, "SMB2-SESSION tests");
 
-- 
1.8.1.2


From 5a8c5d537c6f4d5f5261bdba1f5e27f961e3140c Mon Sep 17 00:00:00 2001
From: Volker Lendecke <vl@samba.org>
Date: Wed, 23 Jul 2014 11:37:28 +0000
Subject: [PATCH 05/94] torture4: Use sizeof(char-array) instead of constant

Signed-off-by: Volker Lendecke <vl@samba.org>
---
 source4/torture/smb2/session.c | 24 ++++++++++++------------
 1 file changed, 12 insertions(+), 12 deletions(-)

diff --git a/source4/torture/smb2/session.c b/source4/torture/smb2/session.c
index 3a4b3bd..ee51d55 100644
--- a/source4/torture/smb2/session.c
+++ b/source4/torture/smb2/session.c
@@ -75,7 +75,7 @@ bool test_session_reconnect1(struct torture_context *tctx, struct smb2_tree *tre
 	union smb_fileinfo qfinfo;
 
 	/* Add some random component to the file name. */
-	snprintf(fname, 256, "session_reconnect_%s.dat",
+	snprintf(fname, sizeof(fname), "session_reconnect_%s.dat",
 		 generate_random_str(tctx, 8));
 
 	smb2_util_unlink(tree, fname);
@@ -158,7 +158,7 @@ bool test_session_reconnect2(struct torture_context *tctx, struct smb2_tree *tre
 	union smb_fileinfo qfinfo;
 
 	/* Add some random component to the file name. */
-	snprintf(fname, 256, "session_reconnect_%s.dat",
+	snprintf(fname, sizeof(fname), "session_reconnect_%s.dat",
 		 generate_random_str(tctx, 8));
 
 	smb2_util_unlink(tree, fname);
@@ -216,7 +216,7 @@ bool test_session_reauth1(struct torture_context *tctx, struct smb2_tree *tree)
 	union smb_fileinfo qfinfo;
 
 	/* Add some random component to the file name. */
-	snprintf(fname, 256, "session_reauth1_%s.dat",
+	snprintf(fname, sizeof(fname), "session_reauth1_%s.dat",
 		 generate_random_str(tctx, 8));
 
 	smb2_util_unlink(tree, fname);
@@ -285,7 +285,7 @@ bool test_session_reauth2(struct torture_context *tctx, struct smb2_tree *tree)
 	struct cli_credentials *anon_creds = NULL;
 
 	/* Add some random component to the file name. */
-	snprintf(fname, 256, "session_reauth2_%s.dat",
+	snprintf(fname, sizeof(fname), "session_reauth2_%s.dat",
 		 generate_random_str(tctx, 8));
 
 	smb2_util_unlink(tree, fname);
@@ -369,7 +369,7 @@ bool test_session_reauth3(struct torture_context *tctx, struct smb2_tree *tree)
 				| SECINFO_UNPROTECTED_DACL;
 
 	/* Add some random component to the file name. */
-	snprintf(fname, 256, "session_reauth3_%s.dat",
+	snprintf(fname, sizeof(fname), "session_reauth3_%s.dat",
 		 generate_random_str(tctx, 8));
 
 	smb2_util_unlink(tree, fname);
@@ -473,7 +473,7 @@ bool test_session_reauth4(struct torture_context *tctx, struct smb2_tree *tree)
 	struct dom_sid *extra_sid;
 
 	/* Add some random component to the file name. */
-	snprintf(fname, 256, "session_reauth4_%s.dat",
+	snprintf(fname, sizeof(fname), "session_reauth4_%s.dat",
 		 generate_random_str(tctx, 8));
 
 	smb2_util_unlink(tree, fname);
@@ -600,9 +600,9 @@ bool test_session_reauth5(struct torture_context *tctx, struct smb2_tree *tree)
 	struct dom_sid *extra_sid;
 
 	/* Add some random component to the file name. */
-	snprintf(dname, 256, "session_reauth5_%s.d",
+	snprintf(dname, sizeof(dname), "session_reauth5_%s.d",
 		 generate_random_str(tctx, 8));
-	snprintf(fname, 256, "%s\\file.dat", dname);
+	snprintf(fname, sizeof(fname), "%s\\file.dat", dname);
 
 	ok = smb2_util_setup_dir(tctx, tree, dname);
 	CHECK_VAL(ok, true);
@@ -647,7 +647,7 @@ bool test_session_reauth5(struct torture_context *tctx, struct smb2_tree *tree)
 
 	/* try to rename the file: fails */
 
-	snprintf(fname2, 256, "%s\\file2.dat", dname);
+	snprintf(fname2, sizeof(fname2), "%s\\file2.dat", dname);
 
 	smb2_util_unlink(tree, fname2);
 
@@ -886,7 +886,7 @@ bool test_session_reauth6(struct torture_context *tctx, struct smb2_tree *tree)
 	encrypted = smb2cli_tcon_is_encryption_on(tree->smbXcli);
 
 	/* Add some random component to the file name. */
-	snprintf(fname, 256, "session_reauth1_%s.dat",
+	snprintf(fname, sizeof(fname), "session_reauth1_%s.dat",
 		 generate_random_str(tctx, 8));
 
 	smb2_util_unlink(tree, fname);
@@ -1004,7 +1004,7 @@ static bool test_session_expire1(struct torture_context *tctx)
 					"smb2_connect failed");
 
 	/* Add some random component to the file name. */
-	snprintf(fname, 256, "session_expire1_%s.dat",
+	snprintf(fname, sizeof(fname), "session_expire1_%s.dat",
 		 generate_random_str(tctx, 8));
 
 	smb2_util_unlink(tree, fname);
@@ -1099,7 +1099,7 @@ bool test_session_bind1(struct torture_context *tctx, struct smb2_tree *tree1)
 	}
 
 	/* Add some random component to the file name. */
-	snprintf(fname, 256, "session_bind1_%s.dat",
+	snprintf(fname, sizeof(fname), "session_bind1_%s.dat",
 		 generate_random_str(tctx, 8));
 
 	smb2_util_unlink(tree1, fname);
-- 
1.8.1.2


From c0185dc0c68669460e28ef7cbe9e187234d8ea9d Mon Sep 17 00:00:00 2001
From: Anubhav Rakshit <anubhav.rakshit@gmail.com>
Date: Mon, 16 Jun 2014 15:30:18 +0530
Subject: [PATCH 06/94] librpc: Add NETWORK_INTERFACE_INFO IDL data structure

This is in preparation for adding NETWORK_INTERFACE_INFO Ioctl smbtorture test
case. This data structure is described in MS-SMB2 Sec. 2.2.32.5

Pair-Programmed-With: Stefan Metzmacher <metze@samba.org>
Signed-off-by: Anubhav Rakshit <anubhav.rakshit@gmail.com>
Signed-off-by: Stefan Metzmacher <metze@samba.org>
Reviewed-by: Volker Lendecke <vl@samba.org>
---
 librpc/idl/ioctl.idl   | 45 +++++++++++++++++++++++++++++++++++++++++++++
 librpc/ndr/ndr_ioctl.c | 40 ++++++++++++++++++++++++++++++++++++++++
 librpc/wscript_build   |  2 +-
 3 files changed, 86 insertions(+), 1 deletion(-)
 create mode 100644 librpc/ndr/ndr_ioctl.c

diff --git a/librpc/idl/ioctl.idl b/librpc/idl/ioctl.idl
index b4485e0..a0a9950 100644
--- a/librpc/idl/ioctl.idl
+++ b/librpc/idl/ioctl.idl
@@ -99,3 +99,48 @@ interface compression
 		uint16 format;
 	} compression_state;
 }
+
+interface netinterface
+{
+	typedef [bitmap32bit] bitmap {
+		FSCTL_NET_IFACE_RSS_CAPABLE = 0x00000001,
+		FSCTL_NET_IFACE_RDMA_CAPABLE = 0x00000002
+	} fsctl_net_iface_capability;
+
+	typedef [enum16bit] enum {
+		FSCTL_NET_IFACE_AF_INET = 0x0002,
+		FSCTL_NET_IFACE_AF_INET6 = 0x0017
+	} fsctl_sockaddr_af;
+
+	typedef [flag(NDR_NOALIGN)] struct {
+		[value(0)] uint16 port;
+		[flag(NDR_BIG_ENDIAN)] ipv4address ipv4;
+		[value(0)] hyper reserved;
+	} fsctl_sockaddr_in;
+
+	typedef [flag(NDR_NOALIGN)] struct {
+		[value(0)] uint16 port;
+		[value(0)] uint32 flowinfo;
+		[flag(NDR_BIG_ENDIAN)] ipv6address ipv6;
+		[value(0)] uint32 scopeid;
+	} fsctl_sockaddr_in6;
+
+	typedef [nodiscriminant,flag(NDR_NOALIGN)] union {
+		[case (FSCTL_NET_IFACE_AF_INET)] fsctl_sockaddr_in saddr_in;
+		[case (FSCTL_NET_IFACE_AF_INET6)] fsctl_sockaddr_in6 saddr_in6;
+	} fsctl_sockaddr_union;
+
+	typedef [flag(NDR_NOALIGN)] struct {
+		fsctl_sockaddr_af family;
+		[subcontext(0),subcontext_size(126),switch_is(family)] fsctl_sockaddr_union saddr;
+	} fsctl_sockaddr_storage;
+
+	typedef [public,relative_base,noprint] struct {
+		[relative] fsctl_net_iface_info *next;
+		uint32 ifindex;
+		fsctl_net_iface_capability capability;
+		[value(0)] uint32 reserved;
+		hyper linkspeed;
+		fsctl_sockaddr_storage sockaddr;
+	} fsctl_net_iface_info;
+}
diff --git a/librpc/ndr/ndr_ioctl.c b/librpc/ndr/ndr_ioctl.c
new file mode 100644
index 0000000..7e76abc
--- /dev/null
+++ b/librpc/ndr/ndr_ioctl.c
@@ -0,0 +1,40 @@
+/*
+   Unix SMB/CIFS implementation.
+
+   Manually parsed structures for IOCTL/FSCTL
+
+   Copyright (C) Stefan Metzmacher 2014
+
+   This program is free software; you can redistribute it and/or modify
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation; either version 3 of the License, or
+   (at your option) any later version.
+
+   This program is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License for more details.
+
+   You should have received a copy of the GNU General Public License
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.
+*/
+
+#include "includes.h"
+#include "librpc/gen_ndr/ndr_ioctl.h"
+
+_PUBLIC_ void ndr_print_fsctl_net_iface_info(struct ndr_print *ndr, const char *name, const struct fsctl_net_iface_info *r)
+{
+	ndr_print_struct(ndr, name, "fsctl_net_iface_info");
+	if (r == NULL) { ndr_print_null(ndr); return; }
+	ndr->depth++;
+	ndr_print_ptr(ndr, "next", r->next);
+	ndr_print_uint32(ndr, "ifindex", r->ifindex);
+	ndr_print_fsctl_net_iface_capability(ndr, "capability", r->capability);
+	ndr_print_uint32(ndr, "reserved", (ndr->flags & LIBNDR_PRINT_SET_VALUES)?0:r->reserved);
+	ndr_print_hyper(ndr, "linkspeed", r->linkspeed);
+	ndr_print_fsctl_sockaddr_storage(ndr, "sockaddr", &r->sockaddr);
+	ndr->depth--;
+	if (r->next) {
+		ndr_print_fsctl_net_iface_info(ndr, "next", r->next);
+	}
+}
diff --git a/librpc/wscript_build b/librpc/wscript_build
index b21619e..0a60b62 100644
--- a/librpc/wscript_build
+++ b/librpc/wscript_build
@@ -616,7 +616,7 @@ bld.SAMBA_SUBSYSTEM('RPC_NDR_DNSSERVER',
     )
 
 bld.SAMBA_SUBSYSTEM('NDR_IOCTL',
-    source='gen_ndr/ndr_ioctl.c',
+    source='gen_ndr/ndr_ioctl.c ndr/ndr_ioctl.c',
     public_deps='ndr'
     )
 
-- 
1.8.1.2


From 6fbff835af7f75dfd062305106421ac579827ae9 Mon Sep 17 00:00:00 2001
From: Anubhav Rakshit <anubhav.rakshit@gmail.com>
Date: Wed, 18 Jun 2014 22:04:14 +0530
Subject: [PATCH 07/94] torture: Add FSCTL_QUERY_NETWORK_INTERFACE_INFO torture
 test case to source4/torture/smb2/ioctl.c

Pair-Programmed-With: Stefan Metzmacher <metze@samba.org>
Signed-off-by: Anubhav Rakshit <anubhav.rakshit@gmail.com>
Signed-off-by: Stefan Metzmacher <metze@samba.org>
Reviewed-by: Volker Lendecke <vl@samba.org>
---
 source4/torture/smb2/ioctl.c | 47 ++++++++++++++++++++++++++++++++++++++++++++
 1 file changed, 47 insertions(+)

diff --git a/source4/torture/smb2/ioctl.c b/source4/torture/smb2/ioctl.c
index 3ccd1d0..053a7c8 100644
--- a/source4/torture/smb2/ioctl.c
+++ b/source4/torture/smb2/ioctl.c
@@ -25,6 +25,7 @@
 #include "libcli/smb2/smb2_calls.h"
 #include "torture/torture.h"
 #include "torture/smb2/proto.h"
+#include "../libcli/smb/smbXcli_base.h"
 #include "librpc/gen_ndr/ndr_ioctl.h"
 
 #define FNAME	"testfsctl.dat"
@@ -2413,6 +2414,49 @@ static bool test_ioctl_compress_perms(struct torture_context *torture,
 }
 
 /*
+   basic testing of the SMB2 FSCTL_QUERY_NETWORK_INTERFACE_INFO ioctl
+*/
+static bool test_ioctl_network_interface_info(struct torture_context *torture,
+				      struct smb2_tree *tree)
+{
+	union smb_ioctl ioctl;
+	struct smb2_handle fh;
+	NTSTATUS status;
+	TALLOC_CTX *tmp_ctx = talloc_new(tree);
+	struct fsctl_net_iface_info net_iface;
+	enum ndr_err_code ndr_ret;
+	uint32_t caps;
+
+	caps = smb2cli_conn_server_capabilities(tree->session->transport->conn);
+	if (!(caps & SMB2_CAP_MULTI_CHANNEL)) {
+		torture_skip(torture, "server doesn't support SMB2_CAP_MULTI_CHANNEL\n");
+	}
+
+	ZERO_STRUCT(ioctl);
+	ioctl.smb2.level = RAW_IOCTL_SMB2;
+	fh.data[0] = UINT64_MAX;
+	fh.data[1] = UINT64_MAX;
+	ioctl.smb2.in.file.handle = fh;
+	ioctl.smb2.in.function = FSCTL_QUERY_NETWORK_INTERFACE_INFO;
+	ioctl.smb2.in.max_response_size = 0x10000; /* Windows client sets this to 64KiB */
+	ioctl.smb2.in.flags = SMB2_IOCTL_FLAG_IS_FSCTL;
+
+	status = smb2_ioctl(tree, tmp_ctx, &ioctl.smb2);
+	torture_assert_ntstatus_ok(torture, status, "FSCTL_QUERY_NETWORK_INTERFACE_INFO");
+
+	ndr_ret = ndr_pull_struct_blob(&ioctl.smb2.out.out, tmp_ctx, &net_iface,
+			(ndr_pull_flags_fn_t)ndr_pull_fsctl_net_iface_info);
+	torture_assert_ndr_success(torture, ndr_ret,
+				   "ndr_pull_fsctl_net_iface_info");
+
+	ndr_print_debug((ndr_print_fn_t)ndr_print_fsctl_net_iface_info
+			, "Network Interface Info", &net_iface);
+
+	talloc_free(tmp_ctx);
+	return true;
+}
+
+/*
    basic testing of SMB2 ioctls
 */
 struct torture_suite *torture_smb2_ioctl_init(void)
@@ -2477,6 +2521,9 @@ struct torture_suite *torture_smb2_ioctl_init(void)
 				     test_ioctl_compress_set_file_attr);
 	torture_suite_add_1smb2_test(suite, "compress_perms",
 				     test_ioctl_compress_perms);
+	torture_suite_add_1smb2_test(suite, "network_interface_info",
+				     test_ioctl_network_interface_info);
+
 
 	suite->description = talloc_strdup(suite, "SMB2-IOCTL tests");
 
-- 
1.8.1.2


From 8a89df2e20dcf7c0e16fe6201198fffde62d0596 Mon Sep 17 00:00:00 2001
From: Stefan Metzmacher <metze@samba.org>
Date: Fri, 23 May 2014 12:37:14 +0200
Subject: [PATCH 08/94] s3:smbd: avoid using client_get_tcp_info()

We already have the required information as stack variable
in the current function. There's no need to call get[peer|sock]name()
twice.

Signed-off-by: Stefan Metzmacher <metze@samba.org>
Reviewed-by: Volker Lendecke <vl@samba.org>
---
 source3/smbd/process.c | 42 +++++++++++++++++-------------------------
 1 file changed, 17 insertions(+), 25 deletions(-)

diff --git a/source3/smbd/process.c b/source3/smbd/process.c
index dd931e0..5bc57da 100644
--- a/source3/smbd/process.c
+++ b/source3/smbd/process.c
@@ -3512,8 +3512,12 @@ void smbd_process(struct tevent_context *ev_ctx,
 	TALLOC_CTX *frame = talloc_stackframe();
 	struct smbXsrv_connection *conn;
 	struct smbd_server_connection *sconn;
-	struct sockaddr_storage ss;
-	struct sockaddr *sa = NULL;
+	struct sockaddr_storage ss_srv;
+	void *sp_srv = (void *)&ss_srv;
+	struct sockaddr *sa_srv = (struct sockaddr *)sp_srv;
+	struct sockaddr_storage ss_clnt;
+	void *sp_clnt = (void *)&ss_clnt;
+	struct sockaddr *sa_clnt = (struct sockaddr *)sp_clnt;
 	socklen_t sa_socklen;
 	struct tsocket_address *local_address = NULL;
 	struct tsocket_address *remote_address = NULL;
@@ -3580,16 +3584,15 @@ void smbd_process(struct tevent_context *ev_ctx,
 	set_socket_options(sconn->sock, "SO_KEEPALIVE");
 	set_socket_options(sconn->sock, lp_socket_options());
 
-	sa = (struct sockaddr *)(void *)&ss;
-	sa_socklen = sizeof(ss);
-	ret = getpeername(sconn->sock, sa, &sa_socklen);
+	sa_socklen = sizeof(ss_clnt);
+	ret = getpeername(sock_fd, sa_clnt, &sa_socklen);
 	if (ret != 0) {
 		int level = (errno == ENOTCONN)?2:0;
 		DEBUG(level,("getpeername() failed - %s\n", strerror(errno)));
 		exit_server_cleanly("getpeername() failed.\n");
 	}
 	ret = tsocket_address_bsd_from_sockaddr(sconn,
-						sa, sa_socklen,
+						sa_clnt, sa_socklen,
 						&remote_address);
 	if (ret != 0) {
 		DEBUG(0,("%s: tsocket_address_bsd_from_sockaddr remote failed - %s\n",
@@ -3597,16 +3600,15 @@ void smbd_process(struct tevent_context *ev_ctx,
 		exit_server_cleanly("tsocket_address_bsd_from_sockaddr remote failed.\n");
 	}
 
-	sa = (struct sockaddr *)(void *)&ss;
-	sa_socklen = sizeof(ss);
-	ret = getsockname(sconn->sock, sa, &sa_socklen);
+	sa_socklen = sizeof(ss_srv);
+	ret = getsockname(sock_fd, sa_srv, &sa_socklen);
 	if (ret != 0) {
 		int level = (errno == ENOTCONN)?2:0;
 		DEBUG(level,("getsockname() failed - %s\n", strerror(errno)));
 		exit_server_cleanly("getsockname() failed.\n");
 	}
 	ret = tsocket_address_bsd_from_sockaddr(sconn,
-						sa, sa_socklen,
+						sa_srv, sa_socklen,
 						&local_address);
 	if (ret != 0) {
 		DEBUG(0,("%s: tsocket_address_bsd_from_sockaddr remote failed - %s\n",
@@ -3792,22 +3794,12 @@ void smbd_process(struct tevent_context *ev_ctx,
 		 * tickle acks, triggering a reconnection by the
 		 * client.
 		 */
+		NTSTATUS status;
 
-		struct sockaddr_storage srv, clnt;
-
-		if (client_get_tcp_info(sconn->sock, &srv, &clnt) == 0) {
-			NTSTATUS status;
-			status = smbd_register_ips(sconn, &srv, &clnt);
-			if (!NT_STATUS_IS_OK(status)) {
-				DEBUG(0, ("ctdbd_register_ips failed: %s\n",
-					  nt_errstr(status)));
-			}
-		} else {
-			int level = (errno == ENOTCONN)?2:0;
-			DEBUG(level,("Unable to get tcp info for "
-				     "smbd_register_ips: %s\n",
-				     strerror(errno)));
-			exit_server_cleanly("client_get_tcp_info() failed.\n");
+		status = smbd_register_ips(sconn, &ss_srv, &ss_clnt);
+		if (!NT_STATUS_IS_OK(status)) {
+			DEBUG(0, ("ctdbd_register_ips failed: %s\n",
+				  nt_errstr(status)));
 		}
 	}
 
-- 
1.8.1.2


From 47d2d45e31fc7e17ebac5f6842a96dd958ca024e Mon Sep 17 00:00:00 2001
From: Stefan Metzmacher <metze@samba.org>
Date: Fri, 23 May 2014 12:40:31 +0200
Subject: [PATCH 09/94] s3:smbd: remove unused client_get_tcp_info()

Signed-off-by: Stefan Metzmacher <metze@samba.org>
Reviewed-by: Volker Lendecke <vl@samba.org>
---
 source3/smbd/process.c | 15 ---------------
 1 file changed, 15 deletions(-)

diff --git a/source3/smbd/process.c b/source3/smbd/process.c
index 5bc57da..127bbb3 100644
--- a/source3/smbd/process.c
+++ b/source3/smbd/process.c
@@ -2670,21 +2670,6 @@ static NTSTATUS smbd_register_ips(struct smbd_server_connection *sconn,
 	return ctdbd_register_ips(cconn, srv, clnt, release_ip, state);
 }
 
-static int client_get_tcp_info(int sock, struct sockaddr_storage *server,
-			       struct sockaddr_storage *client)
-{
-	socklen_t length;
-	length = sizeof(*server);
-	if (getsockname(sock, (struct sockaddr *)server, &length) != 0) {
-		return -1;
-	}
-	length = sizeof(*client);
-	if (getpeername(sock, (struct sockaddr *)client, &length) != 0) {
-		return -1;
-	}
-	return 0;
-}
-
 static void msg_kill_client_ip(struct messaging_context *msg_ctx,
 				  void *private_data, uint32_t msg_type,
 				  struct server_id server_id, DATA_BLOB *data)
-- 
1.8.1.2


From 5ba214b2e381e6b52e29dbca6ef1deabac936836 Mon Sep 17 00:00:00 2001
From: Stefan Metzmacher <metze@samba.org>
Date: Mon, 21 Jul 2014 18:58:52 +0200
Subject: [PATCH 10/94] s3:smbd: add smbXsrv_connection_dbg()

This is similar to fsp_fnum_dbg() and fsp_str_dbg().

Signed-off-by: Stefan Metzmacher <metze@samba.org>
Reviewed-by: Volker Lendecke <vl@samba.org>
---
 source3/smbd/globals.h |  2 ++
 source3/smbd/process.c | 23 +++++++++++++++++++++++
 2 files changed, 25 insertions(+)

diff --git a/source3/smbd/globals.h b/source3/smbd/globals.h
index 70148fb..3b9882d 100644
--- a/source3/smbd/globals.h
+++ b/source3/smbd/globals.h
@@ -389,6 +389,8 @@ struct smbXsrv_connection {
 	struct smbXsrv_open_table *open_table;
 };
 
+const char *smbXsrv_connection_dbg(const struct smbXsrv_connection *xconn);
+
 NTSTATUS smbXsrv_version_global_init(const struct server_id *server_id);
 uint32_t smbXsrv_version_global_current(void);
 
diff --git a/source3/smbd/process.c b/source3/smbd/process.c
index 127bbb3..d1074e2 100644
--- a/source3/smbd/process.c
+++ b/source3/smbd/process.c
@@ -3485,6 +3485,29 @@ static void smbd_tevent_trace_callback(enum tevent_trace_point point,
 	}
 }
 
+/**
+ * Create a debug string for the connection
+ *
+ * This is allocated to talloc_tos() or a string constant
+ * in certain corner cases. The returned string should
+ * hence not be free'd directly but only via the talloc stack.
+ */
+const char *smbXsrv_connection_dbg(const struct smbXsrv_connection *xconn)
+{
+	const char *ret;
+
+	/*
+	 * TODO: this can be improved later
+	 * maybe including the client guid or more
+	 */
+	ret = tsocket_address_string(xconn->remote_address, talloc_tos());
+	if (ret == NULL) {
+		return "<tsocket_address_string() failed>";
+	}
+
+	return ret;
+}
+
 /****************************************************************************
  Process commands from the client
 ****************************************************************************/
-- 
1.8.1.2


From 386f2b5c884724c5c8b69b9ae22a6addbf20b4dc Mon Sep 17 00:00:00 2001
From: Stefan Metzmacher <metze@samba.org>
Date: Fri, 23 May 2014 12:59:43 +0200
Subject: [PATCH 11/94] TODO/REVIEW s3:smbd: avoid get_peer_addr() and use
 smbXsrv_connection_dbg()

There's no need to do syscalls, if we already have the information
in userspace.

Signed-off-by: Stefan Metzmacher <metze@samba.org>

On the one hand, there is this hunk:

-               if (write_data(fsp->conn->sconn->sock, buf, cur_read)
-                   != cur_read) {
-                       char addr[INET6_ADDRSTRLEN];
+               ret = write_data(xconn->sconn->sock, buf, cur_read);
+               if (ret != cur_read) {
+                       int saved_errno = errno;

but also

                        to_write = MIN(SHORT_SEND_BUFSIZE, smb_maxcnt - nread);
-                       if (write_data(fsp->conn->sconn->sock, buf, to_write)
+                       if (write_data(xconn->sconn->sock, buf, to_write)
                            != to_write) {

So in some of the hunks we have the "never do function calls in
expressions", convention pushed through, but not in all of them. Is
there any reason for this?

Also, in some places I don't see errno is saved around the DEBUG message
where it was not before. DEBUG itself can change the errno.
---
 source3/smbd/process.c   | 17 +++++----
 source3/smbd/reply.c     | 93 +++++++++++++++++++++++++++---------------------
 source3/smbd/smb2_read.c | 49 ++++++++++++-------------
 3 files changed, 85 insertions(+), 74 deletions(-)

diff --git a/source3/smbd/process.c b/source3/smbd/process.c
index d1074e2..1b80ba5 100644
--- a/source3/smbd/process.c
+++ b/source3/smbd/process.c
@@ -215,6 +215,7 @@ bool srv_send_smb(struct smbd_server_connection *sconn, char *buffer,
 		  bool do_encrypt,
 		  struct smb_perfcount_data *pcd)
 {
+	struct smbXsrv_connection *xconn = sconn->conn;
 	size_t len = 0;
 	ssize_t ret;
 	char *buf_out = buffer;
@@ -248,16 +249,16 @@ bool srv_send_smb(struct smbd_server_connection *sconn, char *buffer,
 
 	ret = write_data(sconn->sock, buf_out, len);
 	if (ret <= 0) {
-
-		char addr[INET6_ADDRSTRLEN];
+		int saved_errno = errno;
 		/*
 		 * Try and give an error message saying what
 		 * client failed.
 		 */
 		DEBUG(1,("pid[%d] Error writing %d bytes to client %s. %d. (%s)\n",
 			 (int)getpid(), (int)len,
-			 get_peer_addr(sconn->sock, addr, sizeof(addr)),
-			 (int)ret, strerror(errno) ));
+			 smbXsrv_connection_dbg(xconn),
+			 (int)ret, strerror(saved_errno)));
+		errno = saved_errno;
 
 		srv_free_enc_buffer(sconn, buf_out);
 		goto out;
@@ -2703,6 +2704,7 @@ static bool keepalive_fn(const struct timeval *now, void *private_data)
 {
 	struct smbd_server_connection *sconn = talloc_get_type_abort(
 		private_data, struct smbd_server_connection);
+	struct smbXsrv_connection *xconn = sconn->conn;
 	bool ret;
 
 	if (sconn->using_smb2) {
@@ -2715,15 +2717,16 @@ static bool keepalive_fn(const struct timeval *now, void *private_data)
 	smbd_unlock_socket(sconn);
 
 	if (!ret) {
-		char addr[INET6_ADDRSTRLEN];
+		int saved_errno = errno;
 		/*
 		 * Try and give an error message saying what
 		 * client failed.
 		 */
 		DEBUG(0, ("send_keepalive failed for client %s. "
 			  "Error %s - exiting\n",
-			  get_peer_addr(sconn->sock, addr, sizeof(addr)),
-			  strerror(errno)));
+			  smbXsrv_connection_dbg(xconn),
+			  strerror(saved_errno)));
+		errno = saved_errno;
 		return False;
 	}
 	return True;
diff --git a/source3/smbd/reply.c b/source3/smbd/reply.c
index d54326a..1cf6a03 100644
--- a/source3/smbd/reply.c
+++ b/source3/smbd/reply.c
@@ -3023,6 +3023,7 @@ static void fail_readraw(void)
 
 ssize_t fake_sendfile(files_struct *fsp, off_t startpos, size_t nread)
 {
+	struct smbXsrv_connection *xconn = fsp->conn->sconn->conn;
 	size_t bufsize;
 	size_t tosend = nread;
 	char *buf;
@@ -3057,19 +3058,19 @@ ssize_t fake_sendfile(files_struct *fsp, off_t startpos, size_t nread)
 			memset(buf + ret, '\0', cur_read - ret);
 		}
 
-		if (write_data(fsp->conn->sconn->sock, buf, cur_read)
-		    != cur_read) {
-			char addr[INET6_ADDRSTRLEN];
+		ret = write_data(xconn->sconn->sock, buf, cur_read);
+		if (ret != cur_read) {
+			int saved_errno = errno;
 			/*
 			 * Try and give an error message saying what
 			 * client failed.
 			 */
 			DEBUG(0, ("write_data failed for client %s. "
 				  "Error %s\n",
-				  get_peer_addr(fsp->conn->sconn->sock, addr,
-						sizeof(addr)),
-				  strerror(errno)));
+				  smbXsrv_connection_dbg(xconn),
+				  strerror(saved_errno)));
 			SAFE_FREE(buf);
+			errno = saved_errno;
 			return -1;
 		}
 		tosend -= cur_read;
@@ -3090,6 +3091,8 @@ void sendfile_short_send(files_struct *fsp,
 				size_t headersize,
 				size_t smb_maxcnt)
 {
+	struct smbXsrv_connection *xconn = fsp->conn->sconn->conn;
+
 #define SHORT_SEND_BUFSIZE 1024
 	if (nread < headersize) {
 		DEBUG(0,("sendfile_short_send: sendfile failed to send "
@@ -3128,19 +3131,18 @@ void sendfile_short_send(files_struct *fsp,
 			size_t to_write;
 
 			to_write = MIN(SHORT_SEND_BUFSIZE, smb_maxcnt - nread);
-			if (write_data(fsp->conn->sconn->sock, buf, to_write)
+			if (write_data(xconn->sconn->sock, buf, to_write)
 			    != to_write) {
-				char addr[INET6_ADDRSTRLEN];
+				int saved_errno = errno;
 				/*
 				 * Try and give an error message saying what
 				 * client failed.
 				 */
 				DEBUG(0, ("write_data failed for client %s. "
 					  "Error %s\n",
-					  get_peer_addr(
-						  fsp->conn->sconn->sock, addr,
-						  sizeof(addr)),
-					  strerror(errno)));
+					  smbXsrv_connection_dbg(xconn),
+					  strerror(saved_errno)));
+				errno = saved_errno;
 				exit_server_cleanly("sendfile_short_send: "
 						    "write_data failed");
 			}
@@ -3156,21 +3158,23 @@ void sendfile_short_send(files_struct *fsp,
 
 static void reply_readbraw_error(struct smbd_server_connection *sconn)
 {
+	struct smbXsrv_connection *xconn = sconn->conn;
 	char header[4];
 
 	SIVAL(header,0,0);
 
 	smbd_lock_socket(sconn);
 	if (write_data(sconn->sock,header,4) != 4) {
-		char addr[INET6_ADDRSTRLEN];
+		int saved_errno = errno;
 		/*
 		 * Try and give an error message saying what
 		 * client failed.
 		 */
 		DEBUG(0, ("write_data failed for client %s. "
 			  "Error %s\n",
-			  get_peer_addr(sconn->sock, addr, sizeof(addr)),
-			  strerror(errno)));
+			  smbXsrv_connection_dbg(xconn),
+			  strerror(saved_errno)));
+		errno = saved_errno;
 
 		fail_readraw();
 	}
@@ -3189,6 +3193,7 @@ static void send_file_readbraw(connection_struct *conn,
 			       ssize_t mincount)
 {
 	struct smbd_server_connection *sconn = req->sconn;
+	struct smbXsrv_connection *xconn = sconn->conn;
 	char *outbuf = NULL;
 	ssize_t ret=0;
 
@@ -3288,16 +3293,15 @@ normal_readbraw:
 
 	_smb_setlen(outbuf,ret);
 	if (write_data(sconn->sock, outbuf, 4+ret) != 4+ret) {
-		char addr[INET6_ADDRSTRLEN];
+		int saved_errno = errno;
 		/*
 		 * Try and give an error message saying what
 		 * client failed.
 		 */
-		DEBUG(0, ("write_data failed for client %s. "
-			  "Error %s\n",
-			  get_peer_addr(fsp->conn->sconn->sock, addr,
-					sizeof(addr)),
-			  strerror(errno)));
+		DEBUG(0, ("write_data failed for client %s. Error %s\n",
+			  smbXsrv_connection_dbg(xconn),
+			  strerror(saved_errno)));
+		errno = saved_errno;
 
 		fail_readraw();
 	}
@@ -3690,6 +3694,7 @@ static void send_file_readX(connection_struct *conn, struct smb_request *req,
 			    files_struct *fsp, off_t startpos,
 			    size_t smb_maxcnt)
 {
+	struct smbXsrv_connection *xconn = req->sconn->conn;
 	ssize_t nread = -1;
 	struct lock_struct lock;
 	int saved_errno = 0;
@@ -3746,6 +3751,8 @@ static void send_file_readX(connection_struct *conn, struct smb_request *req,
 		nread = SMB_VFS_SENDFILE(req->sconn->sock, fsp, &header,
 					 startpos, smb_maxcnt);
 		if (nread == -1) {
+			saved_errno = errno;
+
 			/* Returning ENOSYS means no data at all was sent.
 			   Do this as a normal read. */
 			if (errno == ENOSYS) {
@@ -3767,9 +3774,12 @@ static void send_file_readX(connection_struct *conn, struct smb_request *req,
 				if (nread == -1) {
 					DEBUG(0,("send_file_readX: "
 						 "fake_sendfile failed for "
-						 "file %s (%s).\n",
+						 "file %s (%s) for client %s. "
+						 "Terminating\n",
 						 fsp_str_dbg(fsp),
-						 strerror(errno)));
+						 smbXsrv_connection_dbg(xconn),
+						 strerror(saved_errno)));
+					errno = saved_errno;
 					exit_server_cleanly("send_file_readX: fake_sendfile failed");
 				}
 				DEBUG(3, ("send_file_readX: fake_sendfile %s max=%d nread=%d\n",
@@ -3813,35 +3823,37 @@ normal_read:
 
 	if ((smb_maxcnt & 0xFF0000) > 0x10000) {
 		uint8 headerbuf[smb_size + 2*12];
+		ssize_t ret;
 
 		construct_reply_common_req(req, (char *)headerbuf);
 		setup_readX_header(req, (char *)headerbuf, smb_maxcnt);
 
 		/* Send out the header. */
-		if (write_data(req->sconn->sock, (char *)headerbuf,
-			       sizeof(headerbuf)) != sizeof(headerbuf)) {
-
-			char addr[INET6_ADDRSTRLEN];
+		ret = write_data(req->sconn->sock, (char *)headerbuf,
+				 sizeof(headerbuf));
+		if (ret != sizeof(headerbuf)) {
+			saved_errno = errno;
 			/*
 			 * Try and give an error message saying what
 			 * client failed.
 			 */
-			DEBUG(0, ("write_data failed for client %s. "
-				  "Error %s\n",
-				  get_peer_addr(req->sconn->sock, addr,
-						sizeof(addr)),
-				  strerror(errno)));
-
 			DEBUG(0,("send_file_readX: write_data failed for file "
-				 "%s (%s). Terminating\n", fsp_str_dbg(fsp),
-				 strerror(errno)));
+				 "%s (%s) for client %s. Terminating\n",
+				 fsp_str_dbg(fsp),
+				 smbXsrv_connection_dbg(xconn),
+				 strerror(saved_errno)));
+			errno = saved_errno;
 			exit_server_cleanly("send_file_readX sendfile failed");
 		}
 		nread = fake_sendfile(fsp, startpos, smb_maxcnt);
 		if (nread == -1) {
-			DEBUG(0,("send_file_readX: fake_sendfile failed for "
-				 "file %s (%s).\n", fsp_str_dbg(fsp),
-				 strerror(errno)));
+			saved_errno = errno;
+			DEBUG(0,("send_file_readX: fake_sendfile failed for file "
+				 "%s (%s) for client %s. Terminating\n",
+				 fsp_str_dbg(fsp),
+				 smbXsrv_connection_dbg(xconn),
+				 strerror(saved_errno)));
+			errno = saved_errno;
 			exit_server_cleanly("send_file_readX: fake_sendfile failed");
 		}
 		goto strict_unlock;
@@ -4103,6 +4115,7 @@ static NTSTATUS read_smb_length(int fd, char *inbuf, unsigned int timeout,
 void reply_writebraw(struct smb_request *req)
 {
 	connection_struct *conn = req->conn;
+	struct smbXsrv_connection *xconn = req->sconn->conn;
 	char *buf = NULL;
 	ssize_t nwritten=0;
 	ssize_t total_written=0;
@@ -4266,14 +4279,12 @@ void reply_writebraw(struct smb_request *req)
 		status = read_data(req->sconn->sock, buf+4, numtowrite);
 
 		if (!NT_STATUS_IS_OK(status)) {
-			char addr[INET6_ADDRSTRLEN];
 			/* Try and give an error message
 			 * saying what client failed. */
 			DEBUG(0, ("reply_writebraw: Oversize secondary write "
 				  "raw read failed (%s) for client %s. "
 				  "Terminating\n", nt_errstr(status),
-				  get_peer_addr(req->sconn->sock, addr,
-						sizeof(addr))));
+				  smbXsrv_connection_dbg(xconn)));
 			exit_server_cleanly("secondary writebraw failed");
 		}
 
diff --git a/source3/smbd/smb2_read.c b/source3/smbd/smb2_read.c
index 603c4a0..f27f1c0 100644
--- a/source3/smbd/smb2_read.c
+++ b/source3/smbd/smb2_read.c
@@ -185,8 +185,10 @@ static int smb2_sendfile_send_data(struct smbd_smb2_read_state *state)
 	uint64_t in_offset = state->in_offset;
 	files_struct *fsp = state->fsp;
 	const DATA_BLOB *hdr = state->smb2req->queue_entry.sendfile_header;
+	struct smbXsrv_connection *xconn = state->smb2req->sconn->conn;
 	ssize_t nread;
 	ssize_t ret;
+	int saved_errno;
 
 	nread = SMB_VFS_SENDFILE(fsp->conn->sconn->sock,
 				 fsp,
@@ -198,6 +200,8 @@ static int smb2_sendfile_send_data(struct smbd_smb2_read_state *state)
 		fsp_str_dbg(fsp) ));
 
 	if (nread == -1) {
+		saved_errno = errno;
+
 		/*
 		 * Returning ENOSYS means no data at all was sent.
 		   Do this as a normal read. */
@@ -214,11 +218,12 @@ static int smb2_sendfile_send_data(struct smbd_smb2_read_state *state)
 			set_use_sendfile(SNUM(fsp->conn), false);
 			nread = fake_sendfile(fsp, in_offset, in_length);
 			if (nread == -1) {
-				DEBUG(0,("smb2_sendfile_send_data: "
-					"fake_sendfile failed for "
-					"file %s (%s).\n",
-					fsp_str_dbg(fsp),
-					strerror(errno)));
+				saved_errno = errno;
+				DEBUG(0,("smb2_sendfile_send_data: fake_sendfile "
+					 "failed for file %s (%s) for client %s. "
+					 "Terminating\n",
+					 fsp_str_dbg(fsp), strerror(saved_errno),
+					 smbXsrv_connection_dbg(xconn)));
 				exit_server_cleanly("smb2_sendfile_send_data: "
 					"fake_sendfile failed");
 			}
@@ -226,9 +231,9 @@ static int smb2_sendfile_send_data(struct smbd_smb2_read_state *state)
 		}
 
 		DEBUG(0,("smb2_sendfile_send_data: sendfile failed for file "
-			"%s (%s). Terminating\n",
-			fsp_str_dbg(fsp),
-			strerror(errno)));
+			 "%s (%s) for client %s. Terminating\n",
+			 fsp_str_dbg(fsp), strerror(saved_errno),
+			 smbXsrv_connection_dbg(xconn)));
 		exit_server_cleanly("smb2_sendfile_send_data: sendfile failed");
 	} else if (nread == 0) {
 		/*
@@ -254,29 +259,21 @@ normal_read:
 	ret = write_data(fsp->conn->sconn->sock,
 			 (const char *)hdr->data, hdr->length);
 	if (ret != hdr->length) {
-		char addr[INET6_ADDRSTRLEN];
-		/*
-		 * Try and give an error message saying what
-		 * client failed.
-		 */
-		DEBUG(0, ("smb2_sendfile_send_data: write_data failed "
-			  "for client %s. Error %s\n",
-			  get_peer_addr(fsp->conn->sconn->sock, addr,
-					sizeof(addr)),
-			  strerror(errno)));
-
+		saved_errno = errno;
 		DEBUG(0,("smb2_sendfile_send_data: write_data failed for file "
-			 "%s (%s). Terminating\n", fsp_str_dbg(fsp),
-			 strerror(errno)));
+			 "%s (%s) for client %s. Terminating\n",
+			 fsp_str_dbg(fsp), strerror(saved_errno),
+			 smbXsrv_connection_dbg(xconn)));
 		exit_server_cleanly("smb2_sendfile_send_data: write_data failed");
 	}
 	nread = fake_sendfile(fsp, in_offset, in_length);
 	if (nread == -1) {
-		DEBUG(0,("smb2_sendfile_send_data: "
-			"fake_sendfile failed for file "
-			"%s (%s). Terminating\n",
-			fsp_str_dbg(fsp),
-			strerror(errno)));
+		saved_errno = errno;
+		DEBUG(0,("smb2_sendfile_send_data: fake_sendfile "
+			 "failed for file %s (%s) for client %s. "
+			 "Terminating\n",
+			 fsp_str_dbg(fsp), strerror(saved_errno),
+			 smbXsrv_connection_dbg(xconn)));
 		exit_server_cleanly("smb2_sendfile_send_data: "
 			"fake_sendfile failed");
 	}
-- 
1.8.1.2


From 9fd68975b36759687588b91e0d646a3267aa9c1e Mon Sep 17 00:00:00 2001
From: Stefan Metzmacher <metze@samba.org>
Date: Tue, 20 May 2014 19:59:12 +0200
Subject: [PATCH 12/94] s3:smbd: move sconn->nbt.* to xconn->transport.nbt.*

This prepares the structures for multi-channel support.

Signed-off-by: Stefan Metzmacher <metze@samba.org>
Reviewed-by: Volker Lendecke <vl@samba.org>
---
 source3/smbd/globals.h | 9 ++++++---
 source3/smbd/process.c | 2 --
 source3/smbd/reply.c   | 5 +++--
 3 files changed, 9 insertions(+), 7 deletions(-)

diff --git a/source3/smbd/globals.h b/source3/smbd/globals.h
index 3b9882d..8f4a021 100644
--- a/source3/smbd/globals.h
+++ b/source3/smbd/globals.h
@@ -345,6 +345,12 @@ struct smbXsrv_connection {
 
 	struct {
 		struct {
+			bool got_session;
+		} nbt;
+	} transport;
+
+	struct {
+		struct {
 			uint32_t capabilities;
 			struct GUID guid;
 			uint16_t security_mode;
@@ -658,9 +664,6 @@ struct smbd_server_connection {
 	struct messaging_context *msg_ctx;
 	struct sys_notify_context *sys_notify_ctx;
 	struct notify_context *notify_ctx;
-	struct {
-		bool got_session;
-	} nbt;
 	bool using_smb2;
 	int trans_num;
 
diff --git a/source3/smbd/process.c b/source3/smbd/process.c
index 1b80ba5..0e468a2 100644
--- a/source3/smbd/process.c
+++ b/source3/smbd/process.c
@@ -3814,8 +3814,6 @@ void smbd_process(struct tevent_context *ev_ctx,
 		}
 	}
 
-	sconn->nbt.got_session = false;
-
 	tmp = lp_max_xmit();
 	tmp = MAX(tmp, SMB_BUFFER_SIZE_MIN);
 	tmp = MIN(tmp, SMB_BUFFER_SIZE_MAX);
diff --git a/source3/smbd/reply.c b/source3/smbd/reply.c
index 1cf6a03..be65cee 100644
--- a/source3/smbd/reply.c
+++ b/source3/smbd/reply.c
@@ -533,6 +533,7 @@ static void reply_called_name_not_present(char *outbuf)
 
 void reply_special(struct smbd_server_connection *sconn, char *inbuf, size_t inbuf_size)
 {
+	struct smbXsrv_connection *xconn = sconn->conn;
 	int msg_type = CVAL(inbuf,0);
 	int msg_flags = CVAL(inbuf,1);
 	/*
@@ -556,7 +557,7 @@ void reply_special(struct smbd_server_connection *sconn, char *inbuf, size_t inb
 
 		*name1 = *name2 = 0;
 
-		if (sconn->nbt.got_session) {
+		if (xconn->transport.nbt.got_session) {
 			exit_server_cleanly("multiple session request not permitted");
 		}
 
@@ -642,7 +643,7 @@ void reply_special(struct smbd_server_connection *sconn, char *inbuf, size_t inb
 		reload_services(sconn, conn_snum_used, true);
 		reopen_logs();
 
-		sconn->nbt.got_session = true;
+		xconn->transport.nbt.got_session = true;
 		break;
 	}
 
-- 
1.8.1.2


From 7d06ad0a921e0e641b434076e27ba56ffb375122 Mon Sep 17 00:00:00 2001
From: Stefan Metzmacher <metze@samba.org>
Date: Wed, 21 May 2014 23:23:34 +0200
Subject: [PATCH 13/94] s3:smbd: move sconn->sock to xconn->transport.sock

This prepares the structures for multi-channel support.

Signed-off-by: Stefan Metzmacher <metze@samba.org>
Reviewed-by: Volker Lendecke <vl@samba.org>
---
 source3/smbd/globals.h       |  3 ++-
 source3/smbd/msdfs.c         |  1 -
 source3/smbd/process.c       | 41 ++++++++++++++++++++++++-----------------
 source3/smbd/reply.c         | 29 ++++++++++++++++-------------
 source3/smbd/server_reload.c | 11 ++++++++---
 source3/smbd/smb2_read.c     |  4 ++--
 source3/smbd/smb2_server.c   | 15 ++++++++++-----
 source3/smbd/vfs.c           |  4 ++--
 8 files changed, 64 insertions(+), 44 deletions(-)

diff --git a/source3/smbd/globals.h b/source3/smbd/globals.h
index 8f4a021..82b09f0 100644
--- a/source3/smbd/globals.h
+++ b/source3/smbd/globals.h
@@ -344,6 +344,8 @@ struct smbXsrv_connection {
 	enum protocol_types protocol;
 
 	struct {
+		int sock;
+
 		struct {
 			bool got_session;
 		} nbt;
@@ -656,7 +658,6 @@ struct user_struct {
 
 struct smbd_server_connection {
 	NTSTATUS status;
-	int sock;
 	const struct tsocket_address *local_address;
 	const struct tsocket_address *remote_address;
 	const char *remote_hostname;
diff --git a/source3/smbd/msdfs.c b/source3/smbd/msdfs.c
index 1239ade..681ebc7 100644
--- a/source3/smbd/msdfs.c
+++ b/source3/smbd/msdfs.c
@@ -246,7 +246,6 @@ static NTSTATUS create_conn_struct_as_root(TALLOC_CTX *ctx,
 
 	sconn->ev_ctx = ev;
 	sconn->msg_ctx = msg;
-	sconn->sock = -1;
 	smbd_echo_init(sconn);
 
 	conn = conn_new(sconn);
diff --git a/source3/smbd/process.c b/source3/smbd/process.c
index 0e468a2..41547bb 100644
--- a/source3/smbd/process.c
+++ b/source3/smbd/process.c
@@ -247,7 +247,7 @@ bool srv_send_smb(struct smbd_server_connection *sconn, char *buffer,
 
 	len = smb_len_large(buf_out) + 4;
 
-	ret = write_data(sconn->sock, buf_out, len);
+	ret = write_data(xconn->transport.sock, buf_out, len);
 	if (ret <= 0) {
 		int saved_errno = errno;
 		/*
@@ -2442,6 +2442,7 @@ static void smbd_server_connection_write_handler(
 static void smbd_server_connection_read_handler(
 	struct smbd_server_connection *sconn, int fd)
 {
+	struct smbXsrv_connection *xconn = sconn->conn;
 	uint8_t *inbuf = NULL;
 	size_t inbuf_len = 0;
 	size_t unread_bytes = 0;
@@ -2464,7 +2465,7 @@ static void smbd_server_connection_read_handler(
 		}
 	}
 
-	from_client = (sconn->sock == fd);
+	from_client = (xconn->transport.sock == fd);
 
 	if (async_echo && from_client) {
 		smbd_lock_socket(sconn);
@@ -2511,6 +2512,7 @@ static void smbd_server_connection_handler(struct tevent_context *ev,
 {
 	struct smbd_server_connection *conn = talloc_get_type(private_data,
 					      struct smbd_server_connection);
+	struct smbXsrv_connection *xconn = conn->conn;
 
 	if (!NT_STATUS_IS_OK(conn->status)) {
 		/*
@@ -2526,7 +2528,7 @@ static void smbd_server_connection_handler(struct tevent_context *ev,
 		return;
 	}
 	if (flags & TEVENT_FD_READ) {
-		smbd_server_connection_read_handler(conn, conn->sock);
+		smbd_server_connection_read_handler(conn, xconn->transport.sock);
 		return;
 	}
 }
@@ -2713,7 +2715,7 @@ static bool keepalive_fn(const struct timeval *now, void *private_data)
 	}
 
 	smbd_lock_socket(sconn);
-	ret = send_keepalive(sconn->sock);
+	ret = send_keepalive(xconn->transport.sock);
 	smbd_unlock_socket(sconn);
 
 	if (!ret) {
@@ -2802,6 +2804,7 @@ static struct tevent_req *smbd_echo_read_send(
 {
 	struct tevent_req *req, *subreq;
 	struct smbd_echo_read_state *state;
+	struct smbXsrv_connection *xconn = sconn->conn;
 
 	req = tevent_req_create(mem_ctx, &state,
 				struct smbd_echo_read_state);
@@ -2811,7 +2814,7 @@ static struct tevent_req *smbd_echo_read_send(
 	state->ev = ev;
 	state->sconn = sconn;
 
-	subreq = wait_for_read_send(state, ev, sconn->sock);
+	subreq = wait_for_read_send(state, ev, xconn->transport.sock);
 	if (tevent_req_nomem(subreq, req)) {
 		return tevent_req_post(req, ev);
 	}
@@ -2854,6 +2857,7 @@ static void smbd_echo_read_waited(struct tevent_req *subreq)
 	struct smbd_echo_read_state *state = tevent_req_data(
 		req, struct smbd_echo_read_state);
 	struct smbd_server_connection *sconn = state->sconn;
+	struct smbXsrv_connection *xconn = sconn->conn;
 	bool ok;
 	NTSTATUS status;
 	size_t unread = 0;
@@ -2873,7 +2877,7 @@ static void smbd_echo_read_waited(struct tevent_req *subreq)
 		return;
 	}
 
-	if (!fd_is_readable(sconn->sock)) {
+	if (!fd_is_readable(xconn->transport.sock)) {
 		DEBUG(10,("echo_handler[%d] the parent smbd was faster\n",
 			  (int)getpid()));
 
@@ -2885,7 +2889,8 @@ static void smbd_echo_read_waited(struct tevent_req *subreq)
 			return;
 		}
 
-		subreq = wait_for_read_send(state, state->ev, sconn->sock);
+		subreq = wait_for_read_send(state, state->ev,
+					    xconn->transport.sock);
 		if (tevent_req_nomem(subreq, req)) {
 			return;
 		}
@@ -2893,7 +2898,9 @@ static void smbd_echo_read_waited(struct tevent_req *subreq)
 		return;
 	}
 
-	status = receive_smb_talloc(state, sconn, sconn->sock, &state->buf,
+	status = receive_smb_talloc(state, sconn,
+				    xconn->transport.sock,
+				    &state->buf,
 				    0 /* timeout */,
 				    &unread,
 				    &encrypted,
@@ -3546,6 +3553,7 @@ void smbd_process(struct tevent_context *ev_ctx,
 
 	conn->ev_ctx = ev_ctx;
 	conn->msg_ctx = msg_ctx;
+	conn->transport.sock = sock_fd;
 
 	sconn = talloc_zero(conn, struct smbd_server_connection);
 	if (!sconn) {
@@ -3562,7 +3570,6 @@ void smbd_process(struct tevent_context *ev_ctx,
 
 	sconn->ev_ctx = ev_ctx;
 	sconn->msg_ctx = msg_ctx;
-	sconn->sock = sock_fd;
 	smbd_echo_init(sconn);
 
 	if (!interactive) {
@@ -3590,10 +3597,10 @@ void smbd_process(struct tevent_context *ev_ctx,
 	}
 
 	/* Ensure child is set to blocking mode */
-	set_blocking(sconn->sock,True);
+	set_blocking(sock_fd,True);
 
-	set_socket_options(sconn->sock, "SO_KEEPALIVE");
-	set_socket_options(sconn->sock, lp_socket_options());
+	set_socket_options(sock_fd, "SO_KEEPALIVE");
+	set_socket_options(sock_fd, lp_socket_options());
 
 	sa_socklen = sizeof(ss_clnt);
 	ret = getpeername(sock_fd, sa_clnt, &sa_socklen);
@@ -3828,11 +3835,11 @@ void smbd_process(struct tevent_context *ev_ctx,
 	}
 
 	sconn->smb1.fde = tevent_add_fd(ev_ctx,
-						  sconn,
-						  sconn->sock,
-						  TEVENT_FD_READ,
-						  smbd_server_connection_handler,
-						  sconn);
+					sconn,
+					sock_fd,
+					TEVENT_FD_READ,
+					smbd_server_connection_handler,
+					sconn);
 	if (!sconn->smb1.fde) {
 		exit_server("failed to create smbd_server_connection fde");
 	}
diff --git a/source3/smbd/reply.c b/source3/smbd/reply.c
index be65cee..fb695a2 100644
--- a/source3/smbd/reply.c
+++ b/source3/smbd/reply.c
@@ -431,6 +431,7 @@ bool check_fsp_ntquota_handle(connection_struct *conn, struct smb_request *req,
 static bool netbios_session_retarget(struct smbd_server_connection *sconn,
 				     const char *name, int name_type)
 {
+	struct smbXsrv_connection *xconn = sconn->conn;
 	char *trim_name;
 	char *trim_name_type;
 	const char *retarget_parm;
@@ -443,7 +444,7 @@ static bool netbios_session_retarget(struct smbd_server_connection *sconn,
 	bool ret = false;
 	uint8_t outbuf[10];
 
-	if (get_socket_port(sconn->sock) != NBT_SMB_PORT) {
+	if (get_socket_port(xconn->transport.sock) != NBT_SMB_PORT) {
 		return false;
 	}
 
@@ -3059,7 +3060,7 @@ ssize_t fake_sendfile(files_struct *fsp, off_t startpos, size_t nread)
 			memset(buf + ret, '\0', cur_read - ret);
 		}
 
-		ret = write_data(xconn->sconn->sock, buf, cur_read);
+		ret = write_data(xconn->transport.sock, buf, cur_read);
 		if (ret != cur_read) {
 			int saved_errno = errno;
 			/*
@@ -3130,10 +3131,11 @@ void sendfile_short_send(files_struct *fsp,
 			 * about efficiency here :-)
 			 */
 			size_t to_write;
+			ssize_t ret;
 
 			to_write = MIN(SHORT_SEND_BUFSIZE, smb_maxcnt - nread);
-			if (write_data(xconn->sconn->sock, buf, to_write)
-			    != to_write) {
+			ret = write_data(xconn->transport.sock, buf, to_write);
+			if (ret != to_write) {
 				int saved_errno = errno;
 				/*
 				 * Try and give an error message saying what
@@ -3165,7 +3167,7 @@ static void reply_readbraw_error(struct smbd_server_connection *sconn)
 	SIVAL(header,0,0);
 
 	smbd_lock_socket(sconn);
-	if (write_data(sconn->sock,header,4) != 4) {
+	if (write_data(xconn->transport.sock,header,4) != 4) {
 		int saved_errno = errno;
 		/*
 		 * Try and give an error message saying what
@@ -3215,7 +3217,7 @@ static void send_file_readbraw(connection_struct *conn,
 		_smb_setlen(header,nread);
 		header_blob = data_blob_const(header, 4);
 
-		sendfile_read = SMB_VFS_SENDFILE(sconn->sock, fsp,
+		sendfile_read = SMB_VFS_SENDFILE(xconn->transport.sock, fsp,
 						 &header_blob, startpos,
 						 nread);
 		if (sendfile_read == -1) {
@@ -3293,7 +3295,7 @@ normal_readbraw:
 	}
 
 	_smb_setlen(outbuf,ret);
-	if (write_data(sconn->sock, outbuf, 4+ret) != 4+ret) {
+	if (write_data(xconn->transport.sock, outbuf, 4+ret) != 4+ret) {
 		int saved_errno = errno;
 		/*
 		 * Try and give an error message saying what
@@ -3749,7 +3751,7 @@ static void send_file_readX(connection_struct *conn, struct smb_request *req,
 		construct_reply_common_req(req, (char *)headerbuf);
 		setup_readX_header(req, (char *)headerbuf, smb_maxcnt);
 
-		nread = SMB_VFS_SENDFILE(req->sconn->sock, fsp, &header,
+		nread = SMB_VFS_SENDFILE(xconn->transport.sock, fsp, &header,
 					 startpos, smb_maxcnt);
 		if (nread == -1) {
 			saved_errno = errno;
@@ -3830,7 +3832,7 @@ normal_read:
 		setup_readX_header(req, (char *)headerbuf, smb_maxcnt);
 
 		/* Send out the header. */
-		ret = write_data(req->sconn->sock, (char *)headerbuf,
+		ret = write_data(xconn->transport.sock, (char *)headerbuf,
 				 sizeof(headerbuf));
 		if (ret != sizeof(headerbuf)) {
 			saved_errno = errno;
@@ -4253,7 +4255,7 @@ void reply_writebraw(struct smb_request *req)
 	}
 
 	/* Now read the raw data into the buffer and write it */
-	status = read_smb_length(req->sconn->sock, buf, SMB_SECONDARY_WAIT,
+	status = read_smb_length(xconn->transport.sock, buf, SMB_SECONDARY_WAIT,
 				 &numtowrite);
 	if (!NT_STATUS_IS_OK(status)) {
 		exit_server_cleanly("secondary writebraw failed");
@@ -4277,7 +4279,7 @@ void reply_writebraw(struct smb_request *req)
 				(int)tcount,(int)nwritten,(int)numtowrite));
 		}
 
-		status = read_data(req->sconn->sock, buf+4, numtowrite);
+		status = read_data(xconn->transport.sock, buf+4, numtowrite);
 
 		if (!NT_STATUS_IS_OK(status)) {
 			/* Try and give an error message
@@ -4340,7 +4342,7 @@ void reply_writebraw(struct smb_request *req)
 		 * sending a NBSSkeepalive. Thanks to DaveCB at Sun for this.
 		 * JRA.
 		 */
-		if (!send_keepalive(req->sconn->sock)) {
+		if (!send_keepalive(xconn->transport.sock)) {
 			exit_server_cleanly("reply_writebraw: send of "
 				"keepalive failed");
 		}
@@ -4708,6 +4710,7 @@ bool is_valid_writeX_buffer(struct smbd_server_connection *sconn,
 void reply_write_and_X(struct smb_request *req)
 {
 	connection_struct *conn = req->conn;
+	struct smbXsrv_connection *xconn = req->sconn->conn;
 	files_struct *fsp;
 	struct lock_struct lock;
 	off_t startpos;
@@ -4864,7 +4867,7 @@ void reply_write_and_X(struct smb_request *req)
 out:
 	if (req->unread_bytes) {
 		/* writeX failed. drain socket. */
-		if (drain_socket(req->sconn->sock, req->unread_bytes) !=
+		if (drain_socket(xconn->transport.sock, req->unread_bytes) !=
 				req->unread_bytes) {
 			smb_panic("failed to drain pending bytes");
 		}
diff --git a/source3/smbd/server_reload.c b/source3/smbd/server_reload.c
index e1e3f9a..627ad8b 100644
--- a/source3/smbd/server_reload.c
+++ b/source3/smbd/server_reload.c
@@ -139,8 +139,13 @@ bool reload_services(struct smbd_server_connection *sconn,
 		     bool (*snumused) (struct smbd_server_connection *, int),
 		     bool test)
 {
+	struct smbXsrv_connection *xconn = NULL;
 	bool ret;
 
+	if (sconn != NULL) {
+		xconn = sconn->conn;
+	}
+
 	if (lp_loaded()) {
 		char *fname = lp_next_configfile(talloc_tos());
 		if (file_exist(fname) &&
@@ -173,9 +178,9 @@ bool reload_services(struct smbd_server_connection *sconn,
 
 	load_interfaces();
 
-	if (sconn != NULL) {
-		set_socket_options(sconn->sock, "SO_KEEPALIVE");
-		set_socket_options(sconn->sock, lp_socket_options());
+	if (xconn != NULL) {
+		set_socket_options(xconn->transport.sock, "SO_KEEPALIVE");
+		set_socket_options(xconn->transport.sock, lp_socket_options());
 	}
 
 	mangle_reset_cache();
diff --git a/source3/smbd/smb2_read.c b/source3/smbd/smb2_read.c
index f27f1c0..b68a1b6 100644
--- a/source3/smbd/smb2_read.c
+++ b/source3/smbd/smb2_read.c
@@ -190,7 +190,7 @@ static int smb2_sendfile_send_data(struct smbd_smb2_read_state *state)
 	ssize_t ret;
 	int saved_errno;
 
-	nread = SMB_VFS_SENDFILE(fsp->conn->sconn->sock,
+	nread = SMB_VFS_SENDFILE(xconn->transport.sock,
 				 fsp,
 				 hdr,
 				 in_offset,
@@ -256,7 +256,7 @@ static int smb2_sendfile_send_data(struct smbd_smb2_read_state *state)
 
 normal_read:
 	/* Send out the header. */
-	ret = write_data(fsp->conn->sconn->sock,
+	ret = write_data(xconn->transport.sock,
 			 (const char *)hdr->data, hdr->length);
 	if (ret != hdr->length) {
 		saved_errno = errno;
diff --git a/source3/smbd/smb2_server.c b/source3/smbd/smb2_server.c
index d61929e..59af738 100644
--- a/source3/smbd/smb2_server.c
+++ b/source3/smbd/smb2_server.c
@@ -200,6 +200,8 @@ bool smbd_is_smb2_header(const uint8_t *inbuf, size_t size)
 
 static NTSTATUS smbd_initialize_smb2(struct smbd_server_connection *sconn)
 {
+	struct smbXsrv_connection *xconn = sconn->conn;
+
 	TALLOC_FREE(sconn->smb1.fde);
 
 	sconn->smb2.send_queue = NULL;
@@ -216,7 +218,7 @@ static NTSTATUS smbd_initialize_smb2(struct smbd_server_connection *sconn)
 
 	sconn->smb2.fde = tevent_add_fd(sconn->ev_ctx,
 					sconn,
-					sconn->sock,
+					xconn->transport.sock,
 					TEVENT_FD_READ,
 					smbd_smb2_connection_handler,
 					sconn);
@@ -225,7 +227,7 @@ static NTSTATUS smbd_initialize_smb2(struct smbd_server_connection *sconn)
 	}
 
 	/* Ensure child is set to non-blocking mode */
-	set_blocking(sconn->sock, false);
+	set_blocking(xconn->transport.sock, false);
 	return NT_STATUS_OK;
 }
 
@@ -2661,6 +2663,7 @@ NTSTATUS smbd_smb2_request_error_ex(struct smbd_smb2_request *req,
 				    DATA_BLOB *info,
 				    const char *location)
 {
+	struct smbXsrv_connection *xconn = req->sconn->conn;
 	DATA_BLOB body;
 	DATA_BLOB _dyn;
 	uint8_t *outhdr = SMBD_SMB2_OUT_HDR_PTR(req);
@@ -2675,7 +2678,7 @@ NTSTATUS smbd_smb2_request_error_ex(struct smbd_smb2_request *req,
 		size_t ret;
 
 		errno = 0;
-		ret = drain_socket(req->sconn->sock, unread_bytes);
+		ret = drain_socket(xconn->transport.sock, unread_bytes);
 		if (ret != unread_bytes) {
 			NTSTATUS error;
 
@@ -3096,6 +3099,7 @@ static int socket_error_from_errno(int ret,
 
 static NTSTATUS smbd_smb2_flush_send_queue(struct smbd_server_connection *sconn)
 {
+	struct smbXsrv_connection *xconn = sconn->conn;
 	int ret;
 	int err;
 	bool retry;
@@ -3148,7 +3152,7 @@ static NTSTATUS smbd_smb2_flush_send_queue(struct smbd_server_connection *sconn)
 			continue;
 		}
 
-		ret = writev(sconn->sock, e->vector, e->count);
+		ret = writev(xconn->transport.sock, e->vector, e->count);
 		if (ret == 0) {
 			/* propagate end of file */
 			return NT_STATUS_INTERNAL_ERROR;
@@ -3206,6 +3210,7 @@ static NTSTATUS smbd_smb2_flush_send_queue(struct smbd_server_connection *sconn)
 static NTSTATUS smbd_smb2_io_handler(struct smbd_server_connection *sconn,
 				     uint16_t fde_flags)
 {
+	struct smbXsrv_connection *xconn = sconn->conn;
 	struct smbd_smb2_request_read_state *state = &sconn->smb2.request_read_state;
 	struct smbd_smb2_request *req = NULL;
 	size_t min_recvfile_size = UINT32_MAX;
@@ -3248,7 +3253,7 @@ again:
 		state->vector.iov_len = NBT_HDR_SIZE;
 	}
 
-	ret = readv(sconn->sock, &state->vector, 1);
+	ret = readv(xconn->transport.sock, &state->vector, 1);
 	if (ret == 0) {
 		/* propagate end of file */
 		return NT_STATUS_END_OF_FILE;
diff --git a/source3/smbd/vfs.c b/source3/smbd/vfs.c
index 3574049..82e0cdf 100644
--- a/source3/smbd/vfs.c
+++ b/source3/smbd/vfs.c
@@ -406,7 +406,7 @@ ssize_t vfs_write_data(struct smb_request *req,
 	ssize_t ret;
 
 	if (req && req->unread_bytes) {
-		int sockfd = req->sconn->sock;
+		int sockfd = req->sconn->conn->transport.sock;
 		int old_flags;
 		SMB_ASSERT(req->unread_bytes == N);
 		/* VFS_RECVFILE must drain the socket
@@ -450,7 +450,7 @@ ssize_t vfs_pwrite_data(struct smb_request *req,
 	ssize_t ret;
 
 	if (req && req->unread_bytes) {
-		int sockfd = req->sconn->sock;
+		int sockfd = req->sconn->conn->transport.sock;
 		SMB_ASSERT(req->unread_bytes == N);
 		/* VFS_RECVFILE must drain the socket
 		 * before returning. */
-- 
1.8.1.2


From ea67e823bfbe9b25d08910905f8c3180b9409860 Mon Sep 17 00:00:00 2001
From: Stefan Metzmacher <metze@samba.org>
Date: Fri, 23 May 2014 08:45:01 +0200
Subject: [PATCH 14/94] s3:smbd: move sconn->smb1.fde to xconn->transport.fde

This prepares the structures for multi-channel support.

Signed-off-by: Stefan Metzmacher <metze@samba.org>
Reviewed-by: Volker Lendecke <vl@samba.org>
---
 source3/smbd/globals.h     |  3 +--
 source3/smbd/process.c     | 18 +++++++++---------
 source3/smbd/smb2_server.c |  2 +-
 3 files changed, 11 insertions(+), 12 deletions(-)

diff --git a/source3/smbd/globals.h b/source3/smbd/globals.h
index 82b09f0..a928123 100644
--- a/source3/smbd/globals.h
+++ b/source3/smbd/globals.h
@@ -345,6 +345,7 @@ struct smbXsrv_connection {
 
 	struct {
 		int sock;
+		struct tevent_fd *fde;
 
 		struct {
 			bool got_session;
@@ -700,8 +701,6 @@ struct smbd_server_connection {
 	} oplocks;
 
 	struct {
-		struct tevent_fd *fde;
-
 		struct {
 
 			/*
diff --git a/source3/smbd/process.c b/source3/smbd/process.c
index 41547bb..b720f9b 100644
--- a/source3/smbd/process.c
+++ b/source3/smbd/process.c
@@ -2518,8 +2518,8 @@ static void smbd_server_connection_handler(struct tevent_context *ev,
 		/*
 		 * we're not supposed to do any io
 		 */
-		TEVENT_FD_NOT_READABLE(conn->smb1.fde);
-		TEVENT_FD_NOT_WRITEABLE(conn->smb1.fde);
+		TEVENT_FD_NOT_READABLE(xconn->transport.fde);
+		TEVENT_FD_NOT_WRITEABLE(xconn->transport.fde);
 		return;
 	}
 
@@ -3834,13 +3834,13 @@ void smbd_process(struct tevent_context *ev_ctx,
 		exit_server("init_dptrs() failed");
 	}
 
-	sconn->smb1.fde = tevent_add_fd(ev_ctx,
-					sconn,
-					sock_fd,
-					TEVENT_FD_READ,
-					smbd_server_connection_handler,
-					sconn);
-	if (!sconn->smb1.fde) {
+	conn->transport.fde = tevent_add_fd(ev_ctx,
+					    sconn,
+					    sock_fd,
+					    TEVENT_FD_READ,
+					    smbd_server_connection_handler,
+					    sconn);
+	if (!conn->transport.fde) {
 		exit_server("failed to create smbd_server_connection fde");
 	}
 
diff --git a/source3/smbd/smb2_server.c b/source3/smbd/smb2_server.c
index 59af738..20fc92b 100644
--- a/source3/smbd/smb2_server.c
+++ b/source3/smbd/smb2_server.c
@@ -202,7 +202,7 @@ static NTSTATUS smbd_initialize_smb2(struct smbd_server_connection *sconn)
 {
 	struct smbXsrv_connection *xconn = sconn->conn;
 
-	TALLOC_FREE(sconn->smb1.fde);
+	TALLOC_FREE(xconn->transport.fde);
 
 	sconn->smb2.send_queue = NULL;
 
-- 
1.8.1.2


From c2d05b4c26aadace47e9ad85412fcc1e80bf57ec Mon Sep 17 00:00:00 2001
From: Stefan Metzmacher <metze@samba.org>
Date: Fri, 23 May 2014 09:07:47 +0200
Subject: [PATCH 15/94] s3:smbd: move sconn->status to xconn->transport.status

This prepares the structures for multi-channel support.

Signed-off-by: Stefan Metzmacher <metze@samba.org>
Reviewed-by: Volker Lendecke <vl@samba.org>
---
 source3/smbd/globals.h     |  2 +-
 source3/smbd/process.c     | 15 +++++++++------
 source3/smbd/server_exit.c |  6 +++---
 source3/smbd/smb2_server.c |  5 +++--
 4 files changed, 16 insertions(+), 12 deletions(-)

diff --git a/source3/smbd/globals.h b/source3/smbd/globals.h
index a928123..4d18308 100644
--- a/source3/smbd/globals.h
+++ b/source3/smbd/globals.h
@@ -344,6 +344,7 @@ struct smbXsrv_connection {
 	enum protocol_types protocol;
 
 	struct {
+		NTSTATUS status;
 		int sock;
 		struct tevent_fd *fde;
 
@@ -658,7 +659,6 @@ struct user_struct {
 };
 
 struct smbd_server_connection {
-	NTSTATUS status;
 	const struct tsocket_address *local_address;
 	const struct tsocket_address *remote_address;
 	const char *remote_hostname;
diff --git a/source3/smbd/process.c b/source3/smbd/process.c
index b720f9b..3fc92a0 100644
--- a/source3/smbd/process.c
+++ b/source3/smbd/process.c
@@ -220,7 +220,7 @@ bool srv_send_smb(struct smbd_server_connection *sconn, char *buffer,
 	ssize_t ret;
 	char *buf_out = buffer;
 
-	if (!NT_STATUS_IS_OK(sconn->status)) {
+	if (!NT_STATUS_IS_OK(xconn->transport.status)) {
 		/*
 		 * we're not supposed to do any io
 		 */
@@ -2514,7 +2514,7 @@ static void smbd_server_connection_handler(struct tevent_context *ev,
 					      struct smbd_server_connection);
 	struct smbXsrv_connection *xconn = conn->conn;
 
-	if (!NT_STATUS_IS_OK(conn->status)) {
+	if (!NT_STATUS_IS_OK(xconn->transport.status)) {
 		/*
 		 * we're not supposed to do any io
 		 */
@@ -2540,8 +2540,9 @@ static void smbd_server_echo_handler(struct tevent_context *ev,
 {
 	struct smbd_server_connection *conn = talloc_get_type(private_data,
 					      struct smbd_server_connection);
+	struct smbXsrv_connection *xconn = conn->conn;
 
-	if (!NT_STATUS_IS_OK(conn->status)) {
+	if (!NT_STATUS_IS_OK(xconn->transport.status)) {
 		/*
 		 * we're not supposed to do any io
 		 */
@@ -2574,8 +2575,9 @@ static void smbd_release_ip_immediate(struct tevent_context *ctx,
 	struct smbd_release_ip_state *state =
 		talloc_get_type_abort(private_data,
 		struct smbd_release_ip_state);
+	struct smbXsrv_connection *xconn = state->sconn->conn;
 
-	if (!NT_STATUS_EQUAL(state->sconn->status, NT_STATUS_ADDRESS_CLOSED)) {
+	if (!NT_STATUS_EQUAL(xconn->transport.status, NT_STATUS_ADDRESS_CLOSED)) {
 		/*
 		 * smbd_server_connection_terminate() already triggered ?
 		 */
@@ -2593,10 +2595,11 @@ static bool release_ip(const char *ip, void *priv)
 	struct smbd_release_ip_state *state =
 		talloc_get_type_abort(priv,
 		struct smbd_release_ip_state);
+	struct smbXsrv_connection *xconn = state->sconn->conn;
 	const char *addr = state->addr;
 	const char *p = addr;
 
-	if (!NT_STATUS_IS_OK(state->sconn->status)) {
+	if (!NT_STATUS_IS_OK(xconn->transport.status)) {
 		/* avoid recursion */
 		return false;
 	}
@@ -2638,7 +2641,7 @@ static bool release_ip(const char *ip, void *priv)
 		/*
 		 * Make sure we don't get any io on the connection.
 		 */
-		state->sconn->status = NT_STATUS_ADDRESS_CLOSED;
+		xconn->transport.status = NT_STATUS_ADDRESS_CLOSED;
 		return true;
 	}
 
diff --git a/source3/smbd/server_exit.c b/source3/smbd/server_exit.c
index 357c69a..4c91bfb 100644
--- a/source3/smbd/server_exit.c
+++ b/source3/smbd/server_exit.c
@@ -102,13 +102,13 @@ static void exit_server_common(enum server_exit_reason how,
 	if (sconn) {
 		NTSTATUS status;
 
-		if (NT_STATUS_IS_OK(sconn->status)) {
+		if (NT_STATUS_IS_OK(conn->transport.status)) {
 			switch (how) {
 			case SERVER_EXIT_ABNORMAL:
-				sconn->status = NT_STATUS_INTERNAL_ERROR;
+				conn->transport.status = NT_STATUS_INTERNAL_ERROR;
 				break;
 			case SERVER_EXIT_NORMAL:
-				sconn->status = NT_STATUS_LOCAL_DISCONNECT;
+				conn->transport.status = NT_STATUS_LOCAL_DISCONNECT;
 				break;
 			}
 		}
diff --git a/source3/smbd/smb2_server.c b/source3/smbd/smb2_server.c
index 20fc92b..bce6034 100644
--- a/source3/smbd/smb2_server.c
+++ b/source3/smbd/smb2_server.c
@@ -2954,10 +2954,11 @@ static bool is_smb2_recvfile_write(struct smbd_smb2_request_read_state *state)
 static NTSTATUS smbd_smb2_request_next_incoming(struct smbd_server_connection *sconn)
 {
 	struct smbd_smb2_request_read_state *state = &sconn->smb2.request_read_state;
+	struct smbXsrv_connection *xconn = sconn->conn;
 	size_t max_send_queue_len;
 	size_t cur_send_queue_len;
 
-	if (!NT_STATUS_IS_OK(sconn->status)) {
+	if (!NT_STATUS_IS_OK(xconn->transport.status)) {
 		/*
 		 * we're not supposed to do any io
 		 */
@@ -3220,7 +3221,7 @@ static NTSTATUS smbd_smb2_io_handler(struct smbd_server_connection *sconn,
 	NTSTATUS status;
 	NTTIME now;
 
-	if (!NT_STATUS_IS_OK(sconn->status)) {
+	if (!NT_STATUS_IS_OK(xconn->transport.status)) {
 		/*
 		 * we're not supposed to do any io
 		 */
-- 
1.8.1.2


From 150a65e292649a1bad2bd606117fa11d984e69f8 Mon Sep 17 00:00:00 2001
From: Stefan Metzmacher <metze@samba.org>
Date: Wed, 21 May 2014 10:27:50 +0200
Subject: [PATCH 16/94] s3:smbd: move sconn->smb1.negprot.* to
 xconn->smb1.negprot.*

This prepares the structures for multi-channel support.

Signed-off-by: Stefan Metzmacher <metze@samba.org>
Reviewed-by: Volker Lendecke <vl@samba.org>
---
 source3/smbd/globals.h     | 25 +++++++++++-----------
 source3/smbd/negprot.c     | 52 ++++++++++++++++++++++++++--------------------
 source3/smbd/process.c     |  2 +-
 source3/smbd/reply.c       |  6 ++++--
 source3/smbd/server_exit.c |  2 +-
 source3/smbd/sesssetup.c   | 11 +++++-----
 6 files changed, 53 insertions(+), 45 deletions(-)

diff --git a/source3/smbd/globals.h b/source3/smbd/globals.h
index 4d18308..625bff6 100644
--- a/source3/smbd/globals.h
+++ b/source3/smbd/globals.h
@@ -355,6 +355,19 @@ struct smbXsrv_connection {
 
 	struct {
 		struct {
+			bool encrypted_passwords;
+			bool spnego;
+			struct auth4_context *auth_context;
+			bool done;
+			/*
+			 * Size of the data we can receive. Set by us.
+			 * Can be modified by the max xmit parameter.
+			 */
+			int max_recv;
+		} negprot;
+	} smb1;
+	struct {
+		struct {
 			uint32_t capabilities;
 			struct GUID guid;
 			uint16_t security_mode;
@@ -735,18 +748,6 @@ struct smbd_server_connection {
 		} echo_handler;
 
 		struct {
-			bool encrypted_passwords;
-			bool spnego;
-			struct auth4_context *auth_context;
-			bool done;
-			/*
-			 * Size of the data we can receive. Set by us.
-			 * Can be modified by the max xmit parameter.
-			 */
-			int max_recv;
-		} negprot;
-
-		struct {
 			uint16_t client_major;
 			uint16_t client_minor;
 			uint32_t client_cap_low;
diff --git a/source3/smbd/negprot.c b/source3/smbd/negprot.c
index 4cd12d8..8c657e1 100644
--- a/source3/smbd/negprot.c
+++ b/source3/smbd/negprot.c
@@ -33,27 +33,28 @@ extern fstring remote_proto;
 
 static void get_challenge(struct smbd_server_connection *sconn, uint8 buff[8])
 {
+	struct smbXsrv_connection *xconn = sconn->conn;
 	NTSTATUS nt_status;
 
 	/* We might be called more than once, multiple negprots are
 	 * permitted */
-	if (sconn->smb1.negprot.auth_context) {
+	if (xconn->smb1.negprot.auth_context) {
 		DEBUG(3, ("get challenge: is this a secondary negprot? "
 			  "sconn->negprot.auth_context is non-NULL!\n"));
-			TALLOC_FREE(sconn->smb1.negprot.auth_context);
+		TALLOC_FREE(xconn->smb1.negprot.auth_context);
 	}
 
 	DEBUG(10, ("get challenge: creating negprot_global_auth_context\n"));
 	nt_status = make_auth4_context(
-		sconn, &sconn->smb1.negprot.auth_context);
+		xconn, &xconn->smb1.negprot.auth_context);
 	if (!NT_STATUS_IS_OK(nt_status)) {
 		DEBUG(0, ("make_auth_context_subsystem returned %s",
 			  nt_errstr(nt_status)));
 		smb_panic("cannot make_negprot_global_auth_context!");
 	}
 	DEBUG(10, ("get challenge: getting challenge\n"));
-	sconn->smb1.negprot.auth_context->get_ntlm_challenge(
-		sconn->smb1.negprot.auth_context, buff);
+	xconn->smb1.negprot.auth_context->get_ntlm_challenge(
+		xconn->smb1.negprot.auth_context, buff);
 }
 
 /****************************************************************************
@@ -65,6 +66,7 @@ static void reply_lanman1(struct smb_request *req, uint16 choice)
 	int secword=0;
 	time_t t = time(NULL);
 	struct smbd_server_connection *sconn = req->sconn;
+	struct smbXsrv_connection *xconn = sconn->conn;
 	uint16_t raw;
 	if (lp_async_smb_echo_handler()) {
 		raw = 0;
@@ -72,19 +74,19 @@ static void reply_lanman1(struct smb_request *req, uint16 choice)
 		raw = (lp_read_raw()?1:0) | (lp_write_raw()?2:0);
 	}
 
-	sconn->smb1.negprot.encrypted_passwords = lp_encrypt_passwords();
+	xconn->smb1.negprot.encrypted_passwords = lp_encrypt_passwords();
 
 	secword |= NEGOTIATE_SECURITY_USER_LEVEL;
-	if (sconn->smb1.negprot.encrypted_passwords) {
+	if (xconn->smb1.negprot.encrypted_passwords) {
 		secword |= NEGOTIATE_SECURITY_CHALLENGE_RESPONSE;
 	}
 
-	reply_outbuf(req, 13, sconn->smb1.negprot.encrypted_passwords?8:0);
+	reply_outbuf(req, 13, xconn->smb1.negprot.encrypted_passwords?8:0);
 
 	SSVAL(req->outbuf,smb_vwv0,choice);
 	SSVAL(req->outbuf,smb_vwv1,secword);
 	/* Create a token value and add it to the outgoing packet. */
-	if (sconn->smb1.negprot.encrypted_passwords) {
+	if (xconn->smb1.negprot.encrypted_passwords) {
 		get_challenge(sconn, (uint8 *)smb_buf(req->outbuf));
 		SSVAL(req->outbuf,smb_vwv11, 8);
 	}
@@ -93,7 +95,7 @@ static void reply_lanman1(struct smb_request *req, uint16 choice)
 
 	/* Reply, SMBlockread, SMBwritelock supported. */
 	SCVAL(req->outbuf,smb_flg, FLAG_REPLY|FLAG_SUPPORT_LOCKREAD);
-	SSVAL(req->outbuf,smb_vwv2, sconn->smb1.negprot.max_recv);
+	SSVAL(req->outbuf,smb_vwv2, xconn->smb1.negprot.max_recv);
 	SSVAL(req->outbuf,smb_vwv3, lp_max_mux()); /* maxmux */
 	SSVAL(req->outbuf,smb_vwv4, 1);
 	SSVAL(req->outbuf,smb_vwv5, raw); /* tell redirector we support
@@ -115,6 +117,7 @@ static void reply_lanman2(struct smb_request *req, uint16 choice)
 	int secword=0;
 	time_t t = time(NULL);
 	struct smbd_server_connection *sconn = req->sconn;
+	struct smbXsrv_connection *xconn = sconn->conn;
 	uint16_t raw;
 	if (lp_async_smb_echo_handler()) {
 		raw = 0;
@@ -122,21 +125,21 @@ static void reply_lanman2(struct smb_request *req, uint16 choice)
 		raw = (lp_read_raw()?1:0) | (lp_write_raw()?2:0);
 	}
 
-	sconn->smb1.negprot.encrypted_passwords = lp_encrypt_passwords();
+	xconn->smb1.negprot.encrypted_passwords = lp_encrypt_passwords();
 
 	secword |= NEGOTIATE_SECURITY_USER_LEVEL;
-	if (sconn->smb1.negprot.encrypted_passwords) {
+	if (xconn->smb1.negprot.encrypted_passwords) {
 		secword |= NEGOTIATE_SECURITY_CHALLENGE_RESPONSE;
 	}
 
-	reply_outbuf(req, 13, sconn->smb1.negprot.encrypted_passwords?8:0);
+	reply_outbuf(req, 13, xconn->smb1.negprot.encrypted_passwords?8:0);
 
 	SSVAL(req->outbuf,smb_vwv0, choice);
 	SSVAL(req->outbuf,smb_vwv1, secword);
 	SIVAL(req->outbuf,smb_vwv6, getpid());
 
 	/* Create a token value and add it to the outgoing packet. */
-	if (sconn->smb1.negprot.encrypted_passwords) {
+	if (xconn->smb1.negprot.encrypted_passwords) {
 		get_challenge(sconn, (uint8 *)smb_buf(req->outbuf));
 		SSVAL(req->outbuf,smb_vwv11, 8);
 	}
@@ -145,7 +148,7 @@ static void reply_lanman2(struct smb_request *req, uint16 choice)
 
 	/* Reply, SMBlockread, SMBwritelock supported. */
 	SCVAL(req->outbuf,smb_flg,FLAG_REPLY|FLAG_SUPPORT_LOCKREAD);
-	SSVAL(req->outbuf,smb_vwv2,sconn->smb1.negprot.max_recv);
+	SSVAL(req->outbuf,smb_vwv2,xconn->smb1.negprot.max_recv);
 	SSVAL(req->outbuf,smb_vwv3,lp_max_mux());
 	SSVAL(req->outbuf,smb_vwv4,1);
 	SSVAL(req->outbuf,smb_vwv5,raw); /* readbraw and/or writebraw */
@@ -159,6 +162,7 @@ static void reply_lanman2(struct smb_request *req, uint16 choice)
 
 DATA_BLOB negprot_spnego(TALLOC_CTX *ctx, struct smbd_server_connection *sconn)
 {
+	struct smbXsrv_connection *xconn = sconn->conn;
 	DATA_BLOB blob = data_blob_null;
 	DATA_BLOB blob_out = data_blob_null;
 	nstring dos_name;
@@ -188,7 +192,7 @@ DATA_BLOB negprot_spnego(TALLOC_CTX *ctx, struct smbd_server_connection *sconn)
 		TALLOC_FREE(gensec_security);
 	}
 
-	sconn->smb1.negprot.spnego = true;
+	xconn->smb1.negprot.spnego = true;
 
 	/* strangely enough, NT does not sent the single OID NTLMSSP when
 	   not a ADS member, it sends no OIDs at all
@@ -250,10 +254,11 @@ static void reply_nt1(struct smb_request *req, uint16 choice)
 	struct timespec ts;
 	ssize_t ret;
 	struct smbd_server_connection *sconn = req->sconn;
+	struct smbXsrv_connection *xconn = sconn->conn;
 	bool signing_desired = false;
 	bool signing_required = false;
 
-	sconn->smb1.negprot.encrypted_passwords = lp_encrypt_passwords();
+	xconn->smb1.negprot.encrypted_passwords = lp_encrypt_passwords();
 
 	/* Check the flags field to see if this is Vista.
 	   WinXP sets it and Vista does not. But we have to 
@@ -272,7 +277,7 @@ static void reply_nt1(struct smb_request *req, uint16 choice)
 	/* do spnego in user level security if the client
 	   supports it and we can do encrypted passwords */
 
-	if (sconn->smb1.negprot.encrypted_passwords &&
+	if (xconn->smb1.negprot.encrypted_passwords &&
 	    lp_use_spnego() &&
 	    (req->flags2 & FLAGS2_EXTENDED_SECURITY)) {
 		negotiate_spnego = True;
@@ -309,7 +314,7 @@ static void reply_nt1(struct smb_request *req, uint16 choice)
 		capabilities |= CAP_DFS;
 
 	secword |= NEGOTIATE_SECURITY_USER_LEVEL;
-	if (sconn->smb1.negprot.encrypted_passwords) {
+	if (xconn->smb1.negprot.encrypted_passwords) {
 		secword |= NEGOTIATE_SECURITY_CHALLENGE_RESPONSE;
 	}
 
@@ -333,7 +338,7 @@ static void reply_nt1(struct smb_request *req, uint16 choice)
 	SSVAL(req->outbuf,smb_vwv1+1, lp_max_mux()); /* maxmpx */
 	SSVAL(req->outbuf,smb_vwv2+1, 1); /* num vcs */
 	SIVAL(req->outbuf,smb_vwv3+1,
-	      sconn->smb1.negprot.max_recv); /* max buffer. LOTS! */
+	      xconn->smb1.negprot.max_recv); /* max buffer. LOTS! */
 	SIVAL(req->outbuf,smb_vwv5+1, 0x10000); /* raw size. full 64k */
 	SIVAL(req->outbuf,smb_vwv7+1, getpid()); /* session key */
 	SIVAL(req->outbuf,smb_vwv9+1, capabilities); /* capabilities */
@@ -343,7 +348,7 @@ static void reply_nt1(struct smb_request *req, uint16 choice)
 
 	if (!negotiate_spnego) {
 		/* Create a token value and add it to the outgoing packet. */
-		if (sconn->smb1.negprot.encrypted_passwords) {
+		if (xconn->smb1.negprot.encrypted_passwords) {
 			uint8 chal[8];
 			/* note that we do not send a challenge at all if
 			   we are using plaintext */
@@ -516,14 +521,15 @@ void reply_negprot(struct smb_request *req)
 	int i;
 	size_t converted_size;
 	struct smbd_server_connection *sconn = req->sconn;
+	struct smbXsrv_connection *xconn = sconn->conn;
 
 	START_PROFILE(SMBnegprot);
 
-	if (sconn->smb1.negprot.done) {
+	if (xconn->smb1.negprot.done) {
 		END_PROFILE(SMBnegprot);
 		exit_server_cleanly("multiple negprot's are not permitted");
 	}
-	sconn->smb1.negprot.done = true;
+	xconn->smb1.negprot.done = true;
 
 	if (req->buflen == 0) {
 		DEBUG(0, ("negprot got no protocols\n"));
diff --git a/source3/smbd/process.c b/source3/smbd/process.c
index 3fc92a0..3c5d025 100644
--- a/source3/smbd/process.c
+++ b/source3/smbd/process.c
@@ -3828,7 +3828,7 @@ void smbd_process(struct tevent_context *ev_ctx,
 	tmp = MAX(tmp, SMB_BUFFER_SIZE_MIN);
 	tmp = MIN(tmp, SMB_BUFFER_SIZE_MAX);
 
-	sconn->smb1.negprot.max_recv = tmp;
+	conn->smb1.negprot.max_recv = tmp;
 
 	sconn->smb1.sessions.done_sesssetup = false;
 	sconn->smb1.sessions.max_send = SMB_BUFFER_SIZE_MAX;
diff --git a/source3/smbd/reply.c b/source3/smbd/reply.c
index fb695a2..2336893 100644
--- a/source3/smbd/reply.c
+++ b/source3/smbd/reply.c
@@ -694,6 +694,7 @@ void reply_tcon(struct smb_request *req)
 	const char *p2;
 	TALLOC_CTX *ctx = talloc_tos();
 	struct smbd_server_connection *sconn = req->sconn;
+	struct smbXsrv_connection *xconn = sconn->conn;
 	NTTIME now = timeval_to_nttime(&req->request_time);
 
 	START_PROFILE(SMBtcon);
@@ -735,7 +736,7 @@ void reply_tcon(struct smb_request *req)
 	}
 
 	reply_outbuf(req, 2, 0);
-	SSVAL(req->outbuf,smb_vwv0,sconn->smb1.negprot.max_recv);
+	SSVAL(req->outbuf,smb_vwv0,xconn->smb1.negprot.max_recv);
 	SSVAL(req->outbuf,smb_vwv1,conn->cnum);
 	SSVAL(req->outbuf,smb_tid,conn->cnum);
 
@@ -771,6 +772,7 @@ void reply_tcon_and_X(struct smb_request *req)
 	bool session_key_updated = false;
 	uint16_t optional_support = 0;
 	struct smbd_server_connection *sconn = req->sconn;
+	struct smbXsrv_connection *xconn = sconn->conn;
 
 	START_PROFILE(SMBtconX);
 
@@ -818,7 +820,7 @@ void reply_tcon_and_X(struct smb_request *req)
 		return;
 	}
 
-	if (sconn->smb1.negprot.encrypted_passwords) {
+	if (xconn->smb1.negprot.encrypted_passwords) {
 		p = req->buf + passlen;
 	} else {
 		p = req->buf + passlen + 1;
diff --git a/source3/smbd/server_exit.c b/source3/smbd/server_exit.c
index 4c91bfb..077b81a 100644
--- a/source3/smbd/server_exit.c
+++ b/source3/smbd/server_exit.c
@@ -113,7 +113,7 @@ static void exit_server_common(enum server_exit_reason how,
 			}
 		}
 
-		TALLOC_FREE(sconn->smb1.negprot.auth_context);
+		TALLOC_FREE(conn->smb1.negprot.auth_context);
 
 		if (lp_log_writeable_files_on_exit()) {
 			bool found = false;
diff --git a/source3/smbd/sesssetup.c b/source3/smbd/sesssetup.c
index 1a0cb5e..a908f69 100644
--- a/source3/smbd/sesssetup.c
+++ b/source3/smbd/sesssetup.c
@@ -590,11 +590,10 @@ void reply_sesssetup_and_X(struct smb_request *req)
 	uint16_t action = 0;
 	NTTIME now = timeval_to_nttime(&req->request_time);
 	struct smbXsrv_session *session = NULL;
-
 	NTSTATUS nt_status;
 	struct smbd_server_connection *sconn = req->sconn;
-
-	bool doencrypt = sconn->smb1.negprot.encrypted_passwords;
+	struct smbXsrv_connection *xconn = sconn->conn;
+	bool doencrypt = xconn->smb1.negprot.encrypted_passwords;
 	bool signing_allowed = false;
 	bool signing_mandatory = false;
 
@@ -627,7 +626,7 @@ void reply_sesssetup_and_X(struct smb_request *req)
 	if (req->wct == 12 &&
 	    (req->flags2 & FLAGS2_EXTENDED_SECURITY)) {
 
-		if (!sconn->smb1.negprot.spnego) {
+		if (!xconn->smb1.negprot.spnego) {
 			DEBUG(0,("reply_sesssetup_and_X:  Rejecting attempt "
 				 "at SPNEGO session setup when it was not "
 				 "negotiated.\n"));
@@ -837,7 +836,7 @@ void reply_sesssetup_and_X(struct smb_request *req)
 				domain, user, get_remote_machine_name()));
 
 	if (*user) {
-		if (sconn->smb1.negprot.spnego) {
+		if (xconn->smb1.negprot.spnego) {
 
 			/* This has to be here, because this is a perfectly
 			 * valid behaviour for guest logons :-( */
@@ -865,7 +864,7 @@ void reply_sesssetup_and_X(struct smb_request *req)
 
 	} else if (doencrypt) {
 		struct auth4_context *negprot_auth_context = NULL;
-		negprot_auth_context = sconn->smb1.negprot.auth_context;
+		negprot_auth_context = xconn->smb1.negprot.auth_context;
 		if (!negprot_auth_context) {
 			DEBUG(0, ("reply_sesssetup_and_X:  Attempted encrypted "
 				"session setup without negprot denied!\n"));
-- 
1.8.1.2


From 48963f512f349bc07a67f61a45d7bb04cd2dfd6b Mon Sep 17 00:00:00 2001
From: Stefan Metzmacher <metze@samba.org>
Date: Wed, 21 May 2014 10:56:26 +0200
Subject: [PATCH 17/94] s3:smbd: move sconn->smb1.sessions.* to
 xconn->smb1.sessions.*

This prepares the structures for multi-channel support.

Signed-off-by: Stefan Metzmacher <metze@samba.org>
Reviewed-by: Volker Lendecke <vl@samba.org>
---
 source3/smbd/globals.h   | 19 ++++++++++---------
 source3/smbd/ipc.c       |  3 ++-
 source3/smbd/nttrans.c   |  3 ++-
 source3/smbd/process.c   |  4 ++--
 source3/smbd/reply.c     | 23 ++++++++++++++---------
 source3/smbd/sesssetup.c | 17 +++++++++--------
 source3/smbd/trans2.c    |  3 ++-
 7 files changed, 41 insertions(+), 31 deletions(-)

diff --git a/source3/smbd/globals.h b/source3/smbd/globals.h
index 625bff6..e077133 100644
--- a/source3/smbd/globals.h
+++ b/source3/smbd/globals.h
@@ -365,6 +365,16 @@ struct smbXsrv_connection {
 			 */
 			int max_recv;
 		} negprot;
+
+		struct {
+			bool done_sesssetup;
+			/*
+			 * Size of data we can send to client. Set
+			 *  by the client for all protocols above CORE.
+			 *  Set by us for CORE protocol.
+			 */
+			int max_send;
+		} sessions;
 	} smb1;
 	struct {
 		struct {
@@ -754,15 +764,6 @@ struct smbd_server_connection {
 			uint32_t client_cap_high;
 		} unix_info;
 
-		struct {
-			bool done_sesssetup;
-			/*
-			 * Size of data we can send to client. Set
-			 *  by the client for all protocols above CORE.
-			 *  Set by us for CORE protocol.
-			 */
-			int max_send;
-		} sessions;
 		struct smb_signing_state *signing_state;
 
 		struct notify_mid_map *notify_mid_maps;
diff --git a/source3/smbd/ipc.c b/source3/smbd/ipc.c
index dbb259c..1c8033d 100644
--- a/source3/smbd/ipc.c
+++ b/source3/smbd/ipc.c
@@ -108,7 +108,8 @@ void send_trans_reply(connection_struct *conn,
 	int ldata  = rdata  ? rdata_len : 0;
 	int lparam = rparam ? rparam_len : 0;
 	struct smbd_server_connection *sconn = req->sconn;
-	int max_send = sconn->smb1.sessions.max_send;
+	struct smbXsrv_connection *xconn = sconn->conn;
+	int max_send = xconn->smb1.sessions.max_send;
 	/* HACK: make sure we send at least 128 byte in one go */
 	int hdr_overhead = SMB_BUFFER_SIZE_MIN - 128;
 
diff --git a/source3/smbd/nttrans.c b/source3/smbd/nttrans.c
index bca4cd8..018a89d 100644
--- a/source3/smbd/nttrans.c
+++ b/source3/smbd/nttrans.c
@@ -68,7 +68,8 @@ static void send_nt_replies(connection_struct *conn,
 	int alignment_offset = 1;
 	int data_alignment_offset = 0;
 	struct smbd_server_connection *sconn = req->sconn;
-	int max_send = sconn->smb1.sessions.max_send;
+	struct smbXsrv_connection *xconn = sconn->conn;
+	int max_send = xconn->smb1.sessions.max_send;
 
 	/*
 	 * If there genuinely are no parameters or data to send just send
diff --git a/source3/smbd/process.c b/source3/smbd/process.c
index 3c5d025..c789157 100644
--- a/source3/smbd/process.c
+++ b/source3/smbd/process.c
@@ -3830,8 +3830,8 @@ void smbd_process(struct tevent_context *ev_ctx,
 
 	conn->smb1.negprot.max_recv = tmp;
 
-	sconn->smb1.sessions.done_sesssetup = false;
-	sconn->smb1.sessions.max_send = SMB_BUFFER_SIZE_MAX;
+	conn->smb1.sessions.done_sesssetup = false;
+	conn->smb1.sessions.max_send = SMB_BUFFER_SIZE_MAX;
 
 	if (!init_dptrs(sconn)) {
 		exit_server("init_dptrs() failed");
diff --git a/source3/smbd/reply.c b/source3/smbd/reply.c
index 2336893..4d5c9ea 100644
--- a/source3/smbd/reply.c
+++ b/source3/smbd/reply.c
@@ -1556,6 +1556,7 @@ void reply_search(struct smb_request *req)
 	bool ask_sharemode = lp_parm_bool(SNUM(conn), "smbd", "search ask sharemode", true);
 	struct dptr_struct *dirptr = NULL;
 	struct smbd_server_connection *sconn = req->sconn;
+	struct smbXsrv_connection *xconn = sconn->conn;
 
 	START_PROFILE(SMBsearch);
 
@@ -1705,7 +1706,7 @@ void reply_search(struct smb_request *req)
 	} else {
 		unsigned int i;
 		size_t hdr_size = ((uint8_t *)smb_buf(req->outbuf) + 3 - req->outbuf);
-		size_t available_space = sconn->smb1.sessions.max_send - hdr_size;
+		size_t available_space = xconn->smb1.sessions.max_send - hdr_size;
 
 		maxentries = MIN(maxentries, available_space/DIR_STRUCT_SIZE);
 
@@ -3477,6 +3478,7 @@ void reply_lockread(struct smb_request *req)
 	struct byte_range_lock *br_lck = NULL;
 	char *p = NULL;
 	struct smbd_server_connection *sconn = req->sconn;
+	struct smbXsrv_connection *xconn = sconn->conn;
 
 	START_PROFILE(SMBlockread);
 
@@ -3531,13 +3533,13 @@ void reply_lockread(struct smb_request *req)
 	/*
 	 * However the requested READ size IS affected by max_send. Insanity.... JRA.
 	 */
-	maxtoread = sconn->smb1.sessions.max_send - (smb_size + 5*2 + 3);
+	maxtoread = xconn->smb1.sessions.max_send - (smb_size + 5*2 + 3);
 
 	if (numtoread > maxtoread) {
 		DEBUG(0,("reply_lockread: requested read size (%u) is greater than maximum allowed (%u/%u). \
 Returning short read of maximum allowed for compatibility with Windows 2000.\n",
 			(unsigned int)numtoread, (unsigned int)maxtoread,
-			(unsigned int)sconn->smb1.sessions.max_send));
+			(unsigned int)xconn->smb1.sessions.max_send));
 		numtoread = maxtoread;
 	}
 
@@ -3586,6 +3588,7 @@ void reply_read(struct smb_request *req)
 	files_struct *fsp;
 	struct lock_struct lock;
 	struct smbd_server_connection *sconn = req->sconn;
+	struct smbXsrv_connection *xconn = sconn->conn;
 
 	START_PROFILE(SMBread);
 
@@ -3614,13 +3617,13 @@ void reply_read(struct smb_request *req)
 	/*
 	 * The requested read size cannot be greater than max_send. JRA.
 	 */
-	maxtoread = sconn->smb1.sessions.max_send - (smb_size + 5*2 + 3);
+	maxtoread = xconn->smb1.sessions.max_send - (smb_size + 5*2 + 3);
 
 	if (numtoread > maxtoread) {
 		DEBUG(0,("reply_read: requested read size (%u) is greater than maximum allowed (%u/%u). \
 Returning short read of maximum allowed for compatibility with Windows 2000.\n",
 			(unsigned int)numtoread, (unsigned int)maxtoread,
-			(unsigned int)sconn->smb1.sessions.max_send));
+			(unsigned int)xconn->smb1.sessions.max_send));
 		numtoread = maxtoread;
 	}
 
@@ -3898,16 +3901,18 @@ nosendfile_read:
 
 static size_t calc_max_read_pdu(const struct smb_request *req)
 {
+	struct smbXsrv_connection *xconn = req->sconn->conn;
+
 	if (req->sconn->conn->protocol < PROTOCOL_NT1) {
-		return req->sconn->smb1.sessions.max_send;
+		return xconn->smb1.sessions.max_send;
 	}
 
 	if (!lp_large_readwrite()) {
-		return req->sconn->smb1.sessions.max_send;
+		return xconn->smb1.sessions.max_send;
 	}
 
 	if (req_is_in_chain(req)) {
-		return req->sconn->smb1.sessions.max_send;
+		return xconn->smb1.sessions.max_send;
 	}
 
 	if (req->encrypted) {
@@ -3916,7 +3921,7 @@ static size_t calc_max_read_pdu(const struct smb_request *req)
 		 * limit. There are padding considerations
 		 * that make that tricky.
 		 */
-		return req->sconn->smb1.sessions.max_send;
+		return xconn->smb1.sessions.max_send;
 	}
 
 	if (srv_is_signing_active(req->sconn)) {
diff --git a/source3/smbd/sesssetup.c b/source3/smbd/sesssetup.c
index a908f69..75da143 100644
--- a/source3/smbd/sesssetup.c
+++ b/source3/smbd/sesssetup.c
@@ -132,6 +132,7 @@ static void reply_sesssetup_and_X_spnego(struct smb_request *req)
 	uint64_t vuid = req->vuid;
 	NTSTATUS status = NT_STATUS_OK;
 	struct smbd_server_connection *sconn = req->sconn;
+	struct smbXsrv_connection *xconn = sconn->conn;
 	uint16_t action = 0;
 	NTTIME now = timeval_to_nttime(&req->request_time);
 	struct smbXsrv_session *session = NULL;
@@ -140,7 +141,7 @@ static void reply_sesssetup_and_X_spnego(struct smb_request *req)
 
 	DEBUG(3,("Doing spnego session setup\n"));
 
-	if (!sconn->smb1.sessions.done_sesssetup) {
+	if (!xconn->smb1.sessions.done_sesssetup) {
 		global_client_caps = client_caps;
 
 		if (!(global_client_caps & CAP_STATUS32)) {
@@ -381,13 +382,13 @@ static void reply_sesssetup_and_X_spnego(struct smb_request *req)
 			return;
 		}
 
-		if (!sconn->smb1.sessions.done_sesssetup) {
+		if (!xconn->smb1.sessions.done_sesssetup) {
 			if (smb_bufsize < SMB_BUFFER_SIZE_MIN) {
 				reply_force_doserror(req, ERRSRV, ERRerror);
 				return;
 			}
-			sconn->smb1.sessions.max_send = smb_bufsize;
-			sconn->smb1.sessions.done_sesssetup = true;
+			xconn->smb1.sessions.max_send = smb_bufsize;
+			xconn->smb1.sessions.done_sesssetup = true;
 		}
 
 		/* current_user_info is changed on new vuid */
@@ -683,7 +684,7 @@ void reply_sesssetup_and_X(struct smb_request *req)
 		const uint8_t *save_p = req->buf;
 		uint16 byte_count;
 
-		if (!sconn->smb1.sessions.done_sesssetup) {
+		if (!xconn->smb1.sessions.done_sesssetup) {
 			global_client_caps = IVAL(req->vwv+11, 0);
 
 			if (!(global_client_caps & CAP_STATUS32)) {
@@ -1090,14 +1091,14 @@ void reply_sesssetup_and_X(struct smb_request *req)
 	SSVAL(discard_const_p(char, req->inbuf),smb_uid,sess_vuid);
 	req->vuid = sess_vuid;
 
-	if (!sconn->smb1.sessions.done_sesssetup) {
+	if (!xconn->smb1.sessions.done_sesssetup) {
 		if (smb_bufsize < SMB_BUFFER_SIZE_MIN) {
 			reply_force_doserror(req, ERRSRV, ERRerror);
 			END_PROFILE(SMBsesssetupX);
 			return;
 		}
-		sconn->smb1.sessions.max_send = smb_bufsize;
-		sconn->smb1.sessions.done_sesssetup = true;
+		xconn->smb1.sessions.max_send = smb_bufsize;
+		xconn->smb1.sessions.done_sesssetup = true;
 	}
 
 	END_PROFILE(SMBsesssetupX);
diff --git a/source3/smbd/trans2.c b/source3/smbd/trans2.c
index 5304052..76e410f 100644
--- a/source3/smbd/trans2.c
+++ b/source3/smbd/trans2.c
@@ -860,7 +860,8 @@ void send_trans2_replies(connection_struct *conn,
 	int data_alignment_offset = 0;
 	bool overflow = False;
 	struct smbd_server_connection *sconn = req->sconn;
-	int max_send = sconn->smb1.sessions.max_send;
+	struct smbXsrv_connection *xconn = sconn->conn;
+	int max_send = xconn->smb1.sessions.max_send;
 
 	/* Modify the data_to_send and datasize and set the error if
 	   we're trying to send more than max_data_bytes. We still send
-- 
1.8.1.2


From 59a4346dd17458bd4cf4092f6590a7f57f506eeb Mon Sep 17 00:00:00 2001
From: Stefan Metzmacher <metze@samba.org>
Date: Wed, 21 May 2014 14:44:01 +0200
Subject: [PATCH 18/94] s3:smbd: move sconn->smb1.signing_state to
 xconn->smb1.signing_state

This prepares the structures for multi-channel support.

Signed-off-by: Stefan Metzmacher <metze@samba.org>
Reviewed-by: Volker Lendecke <vl@samba.org>
---
 source3/smbd/globals.h   |  3 +--
 source3/smbd/negprot.c   |  4 ++--
 source3/smbd/nttrans.c   |  4 +++-
 source3/smbd/process.c   | 10 ++++++----
 source3/smbd/proto.h     | 18 ++++++++++--------
 source3/smbd/reply.c     | 11 ++++++-----
 source3/smbd/service.c   |  3 ++-
 source3/smbd/sesssetup.c | 10 +++++-----
 source3/smbd/signing.c   | 16 ++++++++--------
 source3/smbd/trans2.c    |  5 +++--
 10 files changed, 46 insertions(+), 38 deletions(-)

diff --git a/source3/smbd/globals.h b/source3/smbd/globals.h
index e077133..63bd731 100644
--- a/source3/smbd/globals.h
+++ b/source3/smbd/globals.h
@@ -375,6 +375,7 @@ struct smbXsrv_connection {
 			 */
 			int max_send;
 		} sessions;
+		struct smb_signing_state *signing_state;
 	} smb1;
 	struct {
 		struct {
@@ -764,8 +765,6 @@ struct smbd_server_connection {
 			uint32_t client_cap_high;
 		} unix_info;
 
-		struct smb_signing_state *signing_state;
-
 		struct notify_mid_map *notify_mid_maps;
 
 		struct {
diff --git a/source3/smbd/negprot.c b/source3/smbd/negprot.c
index 8c657e1..63b3d36 100644
--- a/source3/smbd/negprot.c
+++ b/source3/smbd/negprot.c
@@ -318,8 +318,8 @@ static void reply_nt1(struct smb_request *req, uint16 choice)
 		secword |= NEGOTIATE_SECURITY_CHALLENGE_RESPONSE;
 	}
 
-	signing_desired = smb_signing_is_desired(req->sconn->smb1.signing_state);
-	signing_required = smb_signing_is_mandatory(req->sconn->smb1.signing_state);
+	signing_desired = smb_signing_is_desired(xconn->smb1.signing_state);
+	signing_required = smb_signing_is_mandatory(xconn->smb1.signing_state);
 
 	if (signing_desired) {
 		secword |= NEGOTIATE_SECURITY_SIGNATURES_ENABLED;
diff --git a/source3/smbd/nttrans.c b/source3/smbd/nttrans.c
index 018a89d..57c4159 100644
--- a/source3/smbd/nttrans.c
+++ b/source3/smbd/nttrans.c
@@ -1339,12 +1339,14 @@ static void call_nt_transact_create(connection_struct *conn,
 
 void reply_ntcancel(struct smb_request *req)
 {
+	struct smbXsrv_connection *xconn = req->sconn->conn;
+
 	/*
 	 * Go through and cancel any pending change notifies.
 	 */
 
 	START_PROFILE(SMBntcancel);
-	srv_cancel_sign_response(req->sconn);
+	srv_cancel_sign_response(xconn);
 	remove_pending_change_notify_requests_by_mid(req->sconn, req->mid);
 	remove_pending_lock_requests_by_mid_smb1(req->sconn, req->mid);
 
diff --git a/source3/smbd/process.c b/source3/smbd/process.c
index c789157..188d619 100644
--- a/source3/smbd/process.c
+++ b/source3/smbd/process.c
@@ -231,7 +231,7 @@ bool srv_send_smb(struct smbd_server_connection *sconn, char *buffer,
 
 	if (do_signing) {
 		/* Sign the outgoing packet if required. */
-		srv_calculate_sign_mac(sconn, buf_out, seqnum);
+		srv_calculate_sign_mac(xconn, buf_out, seqnum);
 	}
 
 	if (do_encrypt) {
@@ -480,6 +480,7 @@ static NTSTATUS receive_smb_raw_talloc(TALLOC_CTX *mem_ctx,
 				       char **buffer, unsigned int timeout,
 				       size_t *p_unread, size_t *plen)
 {
+	struct smbXsrv_connection *xconn = sconn->conn;
 	char lenbuf[4];
 	size_t len;
 	int min_recv_size = lp_min_receive_file_size();
@@ -496,7 +497,7 @@ static NTSTATUS receive_smb_raw_talloc(TALLOC_CTX *mem_ctx,
 	if (CVAL(lenbuf,0) == 0 && min_recv_size &&
 	    (smb_len_large(lenbuf) > /* Could be a UNIX large writeX. */
 		(min_recv_size + STANDARD_WRITE_AND_X_HEADER_SIZE)) &&
-	    !srv_is_signing_active(sconn) &&
+	    !srv_is_signing_active(xconn) &&
 	    sconn->smb1.echo_handler.trusted_fde == NULL) {
 
 		return receive_smb_raw_talloc_partial_read(
@@ -540,6 +541,7 @@ static NTSTATUS receive_smb_talloc(TALLOC_CTX *mem_ctx,
 				   uint32_t *seqnum,
 				   bool trusted_channel)
 {
+	struct smbXsrv_connection *xconn = sconn->conn;
 	size_t len = 0;
 	NTSTATUS status;
 
@@ -569,7 +571,7 @@ static NTSTATUS receive_smb_talloc(TALLOC_CTX *mem_ctx,
 	}
 
 	/* Check the incoming SMB signature. */
-	if (!srv_check_sign_mac(sconn, *buffer, seqnum, trusted_channel)) {
+	if (!srv_check_sign_mac(xconn, *buffer, seqnum, trusted_channel)) {
 		DEBUG(0, ("receive_smb: SMB Signature verification failed on "
 			  "incoming packet!\n"));
 		return NT_STATUS_INVALID_NETWORK_RESPONSE;
@@ -3738,7 +3740,7 @@ void smbd_process(struct tevent_context *ev_ctx,
 		DEBUG(0,("Changed root to %s\n", lp_root_directory(talloc_tos())));
 	}
 
-	if (!srv_init_signing(sconn)) {
+	if (!srv_init_signing(conn)) {
 		exit_server("Failed to init smb_signing");
 	}
 
diff --git a/source3/smbd/proto.h b/source3/smbd/proto.h
index eab05c2..efabfad 100644
--- a/source3/smbd/proto.h
+++ b/source3/smbd/proto.h
@@ -48,17 +48,19 @@
 
 /* The following definitions come from smbd/signing.c  */
 
-bool srv_check_sign_mac(struct smbd_server_connection *conn,
+struct smbXsrv_connection;
+
+bool srv_check_sign_mac(struct smbXsrv_connection *conn,
 			const char *inbuf, uint32_t *seqnum, bool trusted_channel);
-void srv_calculate_sign_mac(struct smbd_server_connection *conn,
+void srv_calculate_sign_mac(struct smbXsrv_connection *conn,
 			    char *outbuf, uint32_t seqnum);
-void srv_cancel_sign_response(struct smbd_server_connection *conn);
-bool srv_init_signing(struct smbd_server_connection *conn);
-void srv_set_signing_negotiated(struct smbd_server_connection *conn,
+void srv_cancel_sign_response(struct smbXsrv_connection *conn);
+bool srv_init_signing(struct smbXsrv_connection *conn);
+void srv_set_signing_negotiated(struct smbXsrv_connection *conn,
 			        bool allowed, bool mandatory);
-bool srv_is_signing_active(struct smbd_server_connection *conn);
-bool srv_is_signing_negotiated(struct smbd_server_connection *conn);
-void srv_set_signing(struct smbd_server_connection *conn,
+bool srv_is_signing_active(struct smbXsrv_connection *conn);
+bool srv_is_signing_negotiated(struct smbXsrv_connection *conn);
+void srv_set_signing(struct smbXsrv_connection *conn,
 		     const DATA_BLOB user_session_key,
 		     const DATA_BLOB response);
 
diff --git a/source3/smbd/reply.c b/source3/smbd/reply.c
index 4d5c9ea..f7783e6 100644
--- a/source3/smbd/reply.c
+++ b/source3/smbd/reply.c
@@ -3212,7 +3212,7 @@ static void send_file_readbraw(connection_struct *conn,
 
 	if ( !req_is_in_chain(req) && (nread > 0) && (fsp->base_fsp == NULL) &&
 	    (fsp->wcp == NULL) &&
-	    lp_use_sendfile(SNUM(conn), req->sconn->smb1.signing_state) ) {
+	    lp_use_sendfile(SNUM(conn), xconn->smb1.signing_state) ) {
 		ssize_t sendfile_read = -1;
 		char header[4];
 		DATA_BLOB header_blob;
@@ -3323,6 +3323,7 @@ void reply_readbraw(struct smb_request *req)
 {
 	connection_struct *conn = req->conn;
 	struct smbd_server_connection *sconn = req->sconn;
+	struct smbXsrv_connection *xconn = sconn->conn;
 	ssize_t maxcount,mincount;
 	size_t nread = 0;
 	off_t startpos;
@@ -3332,7 +3333,7 @@ void reply_readbraw(struct smb_request *req)
 
 	START_PROFILE(SMBreadbraw);
 
-	if (srv_is_signing_active(sconn) || req->encrypted) {
+	if (srv_is_signing_active(xconn) || req->encrypted) {
 		exit_server_cleanly("reply_readbraw: SMB signing/sealing is active - "
 			"raw reads/writes are disallowed.");
 	}
@@ -3726,7 +3727,7 @@ static void send_file_readX(connection_struct *conn, struct smb_request *req,
 	    !req->encrypted &&
 	    (fsp->base_fsp == NULL) &&
 	    (fsp->wcp == NULL) &&
-	    lp_use_sendfile(SNUM(conn), req->sconn->smb1.signing_state) ) {
+	    lp_use_sendfile(SNUM(conn), xconn->smb1.signing_state) ) {
 		uint8 headerbuf[smb_size + 12 * 2];
 		DATA_BLOB header;
 
@@ -3924,7 +3925,7 @@ static size_t calc_max_read_pdu(const struct smb_request *req)
 		return xconn->smb1.sessions.max_send;
 	}
 
-	if (srv_is_signing_active(req->sconn)) {
+	if (srv_is_signing_active(xconn)) {
 		return 0x1FFFF;
 	}
 
@@ -4147,7 +4148,7 @@ void reply_writebraw(struct smb_request *req)
 	 */
 	SCVAL(discard_const_p(uint8_t, req->inbuf),smb_com,SMBwritec);
 
-	if (srv_is_signing_active(req->sconn)) {
+	if (srv_is_signing_active(xconn)) {
 		END_PROFILE(SMBwritebraw);
 		exit_server_cleanly("reply_writebraw: SMB signing is active - "
 				"raw reads/writes are disallowed.");
diff --git a/source3/smbd/service.c b/source3/smbd/service.c
index d3eabf8..e7a55a4 100644
--- a/source3/smbd/service.c
+++ b/source3/smbd/service.c
@@ -522,6 +522,7 @@ static NTSTATUS make_connection_snum(struct smbd_server_connection *sconn,
 					int snum, struct user_struct *vuser,
 					const char *pdev)
 {
+	struct smbXsrv_connection *xconn = sconn->conn;
 	struct smb_filename *smb_fname_cpath = NULL;
 	fstring dev;
 	int ret;
@@ -858,7 +859,7 @@ static NTSTATUS make_connection_snum(struct smbd_server_connection *sconn,
 		dbgtext( "%s (%s) ", get_remote_machine_name(),
 			 tsocket_address_string(conn->sconn->remote_address,
 						talloc_tos()) );
-		dbgtext( "%s", srv_is_signing_active(sconn) ? "signed " : "");
+		dbgtext( "%s", srv_is_signing_active(xconn) ? "signed " : "");
 		dbgtext( "connect to service %s ",
 			 lp_servicename(talloc_tos(), snum) );
 		dbgtext( "initially as user %s ",
diff --git a/source3/smbd/sesssetup.c b/source3/smbd/sesssetup.c
index 75da143..43be24a 100644
--- a/source3/smbd/sesssetup.c
+++ b/source3/smbd/sesssetup.c
@@ -331,7 +331,7 @@ static void reply_sesssetup_and_X_spnego(struct smb_request *req)
 				register_homes_share(session_info->unix_info->unix_name);
 		}
 
-		if (srv_is_signing_negotiated(sconn) &&
+		if (srv_is_signing_negotiated(xconn) &&
 		    action == 0 &&
 		    session->global->signing_key.length > 0)
 		{
@@ -339,7 +339,7 @@ static void reply_sesssetup_and_X_spnego(struct smb_request *req)
 			 * Try and turn on server signing on the first non-guest
 			 * sessionsetup.
 			 */
-			srv_set_signing(sconn,
+			srv_set_signing(xconn,
 				session->global->signing_key,
 				data_blob_null);
 		}
@@ -618,7 +618,7 @@ void reply_sesssetup_and_X(struct smb_request *req)
 	 * It finds out when it needs to turn into a noop
 	 * itself.
 	 */
-	srv_set_signing_negotiated(req->sconn,
+	srv_set_signing_negotiated(xconn,
 				   signing_allowed,
 				   signing_mandatory);
 
@@ -1029,7 +1029,7 @@ void reply_sesssetup_and_X(struct smb_request *req)
 			register_homes_share(session_info->unix_info->unix_name);
 	}
 
-	if (srv_is_signing_negotiated(sconn) &&
+	if (srv_is_signing_negotiated(xconn) &&
 	    action == 0 &&
 	    session->global->signing_key.length > 0)
 	{
@@ -1037,7 +1037,7 @@ void reply_sesssetup_and_X(struct smb_request *req)
 		 * Try and turn on server signing on the first non-guest
 		 * sessionsetup.
 		 */
-		srv_set_signing(sconn,
+		srv_set_signing(xconn,
 			session->global->signing_key,
 			nt_resp.data ? nt_resp : lm_resp);
 	}
diff --git a/source3/smbd/signing.c b/source3/smbd/signing.c
index b7683cd..9fac87b 100644
--- a/source3/smbd/signing.c
+++ b/source3/smbd/signing.c
@@ -29,7 +29,7 @@
  Called to validate an incoming packet from the client.
 ************************************************************/
 
-bool srv_check_sign_mac(struct smbd_server_connection *conn,
+bool srv_check_sign_mac(struct smbXsrv_connection *conn,
 			const char *inbuf, uint32_t *seqnum,
 			bool trusted_channel)
 {
@@ -75,7 +75,7 @@ bool srv_check_sign_mac(struct smbd_server_connection *conn,
  Called to sign an outgoing packet to the client.
 ************************************************************/
 
-void srv_calculate_sign_mac(struct smbd_server_connection *conn,
+void srv_calculate_sign_mac(struct smbXsrv_connection *conn,
 			    char *outbuf, uint32_t seqnum)
 {
 	uint8_t *outhdr;
@@ -96,7 +96,7 @@ void srv_calculate_sign_mac(struct smbd_server_connection *conn,
 /***********************************************************
  Called to indicate a oneway request
 ************************************************************/
-void srv_cancel_sign_response(struct smbd_server_connection *conn)
+void srv_cancel_sign_response(struct smbXsrv_connection *conn)
 {
 	smb_signing_cancel_reply(conn->smb1.signing_state, true);
 }
@@ -167,7 +167,7 @@ static void smbd_shm_signing_free(TALLOC_CTX *mem_ctx, void *ptr)
  Called by server negprot when signing has been negotiated.
 ************************************************************/
 
-bool srv_init_signing(struct smbd_server_connection *conn)
+bool srv_init_signing(struct smbXsrv_connection *conn)
 {
 	bool allowed = true;
 	bool desired;
@@ -229,7 +229,7 @@ bool srv_init_signing(struct smbd_server_connection *conn)
 	return true;
 }
 
-void srv_set_signing_negotiated(struct smbd_server_connection *conn,
+void srv_set_signing_negotiated(struct smbXsrv_connection *conn,
 				bool allowed, bool mandatory)
 {
 	smb_signing_set_negotiated(conn->smb1.signing_state,
@@ -241,7 +241,7 @@ void srv_set_signing_negotiated(struct smbd_server_connection *conn,
  reads/writes if it is.
 ************************************************************/
 
-bool srv_is_signing_active(struct smbd_server_connection *conn)
+bool srv_is_signing_active(struct smbXsrv_connection *conn)
 {
 	return smb_signing_is_active(conn->smb1.signing_state);
 }
@@ -252,7 +252,7 @@ bool srv_is_signing_active(struct smbd_server_connection *conn)
  in the negprot.
 ************************************************************/
 
-bool srv_is_signing_negotiated(struct smbd_server_connection *conn)
+bool srv_is_signing_negotiated(struct smbXsrv_connection *conn)
 {
 	return smb_signing_is_negotiated(conn->smb1.signing_state);
 }
@@ -261,7 +261,7 @@ bool srv_is_signing_negotiated(struct smbd_server_connection *conn)
  Turn on signing from this packet onwards.
 ************************************************************/
 
-void srv_set_signing(struct smbd_server_connection *conn,
+void srv_set_signing(struct smbXsrv_connection *conn,
 		     const DATA_BLOB user_session_key,
 		     const DATA_BLOB response)
 {
diff --git a/source3/smbd/trans2.c b/source3/smbd/trans2.c
index 76e410f..d00a6db 100644
--- a/source3/smbd/trans2.c
+++ b/source3/smbd/trans2.c
@@ -3074,6 +3074,7 @@ NTSTATUS smbd_do_qfsinfo(connection_struct *conn,
 			 char **ppdata,
 			 int *ret_data_len)
 {
+	struct smbXsrv_connection *xconn = conn->sconn->conn;
 	char *pdata, *end_data;
 	int data_len = 0, len;
 	const char *vname = volume_label(talloc_tos(), SNUM(conn));
@@ -3446,8 +3447,8 @@ cBytesSector=%u, cUnitTotal=%u, cUnitAvail=%d\n", (unsigned int)bsize, (unsigned
 		case SMB_QUERY_CIFS_UNIX_INFO:
 		{
 			bool large_write = lp_min_receive_file_size() &&
-					!srv_is_signing_active(conn->sconn);
-			bool large_read = !srv_is_signing_active(conn->sconn);
+					!srv_is_signing_active(xconn);
+			bool large_read = !srv_is_signing_active(xconn);
 			int encrypt_caps = 0;
 
 			if (!lp_unix_extensions()) {
-- 
1.8.1.2


From cf132e379cd34b6e7e0e86eb24e84f87b7918daa Mon Sep 17 00:00:00 2001
From: Stefan Metzmacher <metze@samba.org>
Date: Fri, 23 May 2014 09:26:26 +0200
Subject: [PATCH 19/94] s3:smbd: move sconn->smb1.echo_handler.* to
 xconn->smb1.echo_handler.*

This prepares the structures for multi-channel support.

Signed-off-by: Stefan Metzmacher <metze@samba.org>
Reviewed-by: Volker Lendecke <vl@samba.org>
---
 source3/smbd/globals.h | 66 ++++++++++++++++++------------------
 source3/smbd/msdfs.c   |  1 -
 source3/smbd/process.c | 92 ++++++++++++++++++++++++++------------------------
 source3/smbd/reply.c   |  4 +--
 source3/smbd/trans2.c  |  3 +-
 5 files changed, 84 insertions(+), 82 deletions(-)

diff --git a/source3/smbd/globals.h b/source3/smbd/globals.h
index 63bd731..ba90d47 100644
--- a/source3/smbd/globals.h
+++ b/source3/smbd/globals.h
@@ -115,7 +115,6 @@ DATA_BLOB negprot_spnego(TALLOC_CTX *ctx, struct smbd_server_connection *sconn);
 
 void smbd_lock_socket(struct smbd_server_connection *sconn);
 void smbd_unlock_socket(struct smbd_server_connection *sconn);
-void smbd_echo_init(struct smbd_server_connection *sconn);
 
 NTSTATUS smbd_do_locking(struct smb_request *req,
 			 files_struct *fsp,
@@ -355,6 +354,38 @@ struct smbXsrv_connection {
 
 	struct {
 		struct {
+			/*
+			 * fd for the fcntl lock and process shared
+			 * robust mutex to coordinate access to the
+			 * client socket. When the system supports
+			 * process shared robust mutexes, those are
+			 * used. If not, then the fcntl lock will be
+			 * used.
+			 */
+			int socket_lock_fd;
+#ifdef HAVE_ROBUST_MUTEXES
+			pthread_mutex_t *socket_mutex;
+#endif
+
+			/*
+			 * fd for the trusted pipe from
+			 * echo handler child
+			 */
+			int trusted_fd;
+
+			/*
+			 * fde for the trusted_fd
+			 */
+			struct tevent_fd *trusted_fde;
+
+			/*
+			 * Reference count for the fcntl lock to
+			 * allow recursive locks.
+			 */
+			int ref_count;
+		} echo_handler;
+
+		struct {
 			bool encrypted_passwords;
 			bool spnego;
 			struct auth4_context *auth_context;
@@ -726,39 +757,6 @@ struct smbd_server_connection {
 
 	struct {
 		struct {
-
-			/*
-			 * fd for the fcntl lock and process shared
-			 * robust mutex to coordinate access to the
-			 * client socket. When the system supports
-			 * process shared robust mutexes, those are
-			 * used. If not, then the fcntl lock will be
-			 * used.
-			 */
-			int socket_lock_fd;
-#ifdef HAVE_ROBUST_MUTEXES
-			pthread_mutex_t *socket_mutex;
-#endif
-
-			/*
-			 * fd for the trusted pipe from
-			 * echo handler child
-			 */
-			int trusted_fd;
-
-			/*
-			 * fde for the trusted_fd
-			 */
-			struct tevent_fd *trusted_fde;
-
-			/*
-			 * Reference count for the fcntl lock to
-			 * allow recursive locks.
-			 */
-			int ref_count;
-		} echo_handler;
-
-		struct {
 			uint16_t client_major;
 			uint16_t client_minor;
 			uint32_t client_cap_low;
diff --git a/source3/smbd/msdfs.c b/source3/smbd/msdfs.c
index 681ebc7..835eabc 100644
--- a/source3/smbd/msdfs.c
+++ b/source3/smbd/msdfs.c
@@ -246,7 +246,6 @@ static NTSTATUS create_conn_struct_as_root(TALLOC_CTX *ctx,
 
 	sconn->ev_ctx = ev;
 	sconn->msg_ctx = msg;
-	smbd_echo_init(sconn);
 
 	conn = conn_new(sconn);
 	if (conn == NULL) {
diff --git a/source3/smbd/process.c b/source3/smbd/process.c
index 188d619..daf1947 100644
--- a/source3/smbd/process.c
+++ b/source3/smbd/process.c
@@ -62,23 +62,23 @@ static struct pending_message_list *get_deferred_open_message_smb(
 	struct smbd_server_connection *sconn, uint64_t mid);
 static bool smb_splice_chain(uint8_t **poutbuf, const uint8_t *andx_buf);
 
-void smbd_echo_init(struct smbd_server_connection *sconn)
+static void smbd_echo_init(struct smbXsrv_connection *xconn)
 {
-	sconn->smb1.echo_handler.trusted_fd = -1;
-	sconn->smb1.echo_handler.socket_lock_fd = -1;
+	xconn->smb1.echo_handler.trusted_fd = -1;
+	xconn->smb1.echo_handler.socket_lock_fd = -1;
 #ifdef HAVE_ROBUST_MUTEXES
-	sconn->smb1.echo_handler.socket_mutex = NULL;
+	xconn->smb1.echo_handler.socket_mutex = NULL;
 #endif
 }
 
-static bool smbd_echo_active(struct smbd_server_connection *sconn)
+static bool smbd_echo_active(struct smbXsrv_connection *xconn)
 {
-	if (sconn->smb1.echo_handler.socket_lock_fd != -1) {
+	if (xconn->smb1.echo_handler.socket_lock_fd != -1) {
 		return true;
 	}
 
 #ifdef HAVE_ROBUST_MUTEXES
-	if (sconn->smb1.echo_handler.socket_mutex != NULL) {
+	if (xconn->smb1.echo_handler.socket_mutex != NULL) {
 		return true;
 	}
 #endif
@@ -88,25 +88,27 @@ static bool smbd_echo_active(struct smbd_server_connection *sconn)
 
 static bool smbd_lock_socket_internal(struct smbd_server_connection *sconn)
 {
-	if (!smbd_echo_active(sconn)) {
+	struct smbXsrv_connection *xconn = sconn->conn;
+
+	if (!smbd_echo_active(xconn)) {
 		return true;
 	}
 
-	sconn->smb1.echo_handler.ref_count++;
+	xconn->smb1.echo_handler.ref_count++;
 
-	if (sconn->smb1.echo_handler.ref_count > 1) {
+	if (xconn->smb1.echo_handler.ref_count > 1) {
 		return true;
 	}
 
 	DEBUG(10,("pid[%d] wait for socket lock\n", (int)getpid()));
 
 #ifdef HAVE_ROBUST_MUTEXES
-	if (sconn->smb1.echo_handler.socket_mutex != NULL) {
+	if (xconn->smb1.echo_handler.socket_mutex != NULL) {
 		int ret = EINTR;
 
 		while (ret == EINTR) {
 			ret = pthread_mutex_lock(
-				sconn->smb1.echo_handler.socket_mutex);
+				xconn->smb1.echo_handler.socket_mutex);
 			if (ret == 0) {
 				break;
 			}
@@ -119,12 +121,12 @@ static bool smbd_lock_socket_internal(struct smbd_server_connection *sconn)
 	}
 #endif
 
-	if (sconn->smb1.echo_handler.socket_lock_fd != -1) {
+	if (xconn->smb1.echo_handler.socket_lock_fd != -1) {
 		bool ok;
 
 		do {
 			ok = fcntl_lock(
-				sconn->smb1.echo_handler.socket_lock_fd,
+				xconn->smb1.echo_handler.socket_lock_fd,
 				F_SETLKW, 0, 0, F_WRLCK);
 		} while (!ok && (errno == EINTR));
 
@@ -148,23 +150,25 @@ void smbd_lock_socket(struct smbd_server_connection *sconn)
 
 static bool smbd_unlock_socket_internal(struct smbd_server_connection *sconn)
 {
-	if (!smbd_echo_active(sconn)) {
+	struct smbXsrv_connection *xconn = sconn->conn;
+
+	if (!smbd_echo_active(xconn)) {
 		return true;
 	}
 
-	sconn->smb1.echo_handler.ref_count--;
+	xconn->smb1.echo_handler.ref_count--;
 
-	if (sconn->smb1.echo_handler.ref_count > 0) {
+	if (xconn->smb1.echo_handler.ref_count > 0) {
 		return true;
 	}
 
 #ifdef HAVE_ROBUST_MUTEXES
-	if (sconn->smb1.echo_handler.socket_mutex != NULL) {
+	if (xconn->smb1.echo_handler.socket_mutex != NULL) {
 		int ret = EINTR;
 
 		while (ret == EINTR) {
 			ret = pthread_mutex_unlock(
-				sconn->smb1.echo_handler.socket_mutex);
+				xconn->smb1.echo_handler.socket_mutex);
 			if (ret == 0) {
 				break;
 			}
@@ -177,12 +181,12 @@ static bool smbd_unlock_socket_internal(struct smbd_server_connection *sconn)
 	}
 #endif
 
-	if (sconn->smb1.echo_handler.socket_lock_fd != -1) {
+	if (xconn->smb1.echo_handler.socket_lock_fd != -1) {
 		bool ok;
 
 		do {
 			ok = fcntl_lock(
-				sconn->smb1.echo_handler.socket_lock_fd,
+				xconn->smb1.echo_handler.socket_lock_fd,
 				F_SETLKW, 0, 0, F_UNLCK);
 		} while (!ok && (errno == EINTR));
 
@@ -498,7 +502,7 @@ static NTSTATUS receive_smb_raw_talloc(TALLOC_CTX *mem_ctx,
 	    (smb_len_large(lenbuf) > /* Could be a UNIX large writeX. */
 		(min_recv_size + STANDARD_WRITE_AND_X_HEADER_SIZE)) &&
 	    !srv_is_signing_active(xconn) &&
-	    sconn->smb1.echo_handler.trusted_fde == NULL) {
+	    xconn->smb1.echo_handler.trusted_fde == NULL) {
 
 		return receive_smb_raw_talloc_partial_read(
 			mem_ctx, lenbuf, sconn, sock, buffer, timeout,
@@ -2457,13 +2461,13 @@ static void smbd_server_connection_read_handler(
 	bool from_client = false;
 
 	if (async_echo) {
-		if (fd_is_readable(sconn->smb1.echo_handler.trusted_fd)) {
+		if (fd_is_readable(xconn->smb1.echo_handler.trusted_fd)) {
 			/*
 			 * This is the super-ugly hack to prefer the packets
 			 * forwarded by the echo handler over the ones by the
 			 * client directly
 			 */
-			fd = sconn->smb1.echo_handler.trusted_fd;
+			fd = xconn->smb1.echo_handler.trusted_fd;
 		}
 	}
 
@@ -2548,8 +2552,8 @@ static void smbd_server_echo_handler(struct tevent_context *ev,
 		/*
 		 * we're not supposed to do any io
 		 */
-		TEVENT_FD_NOT_READABLE(conn->smb1.echo_handler.trusted_fde);
-		TEVENT_FD_NOT_WRITEABLE(conn->smb1.echo_handler.trusted_fde);
+		TEVENT_FD_NOT_READABLE(xconn->smb1.echo_handler.trusted_fde);
+		TEVENT_FD_NOT_WRITEABLE(xconn->smb1.echo_handler.trusted_fde);
 		return;
 	}
 
@@ -2559,7 +2563,7 @@ static void smbd_server_echo_handler(struct tevent_context *ev,
 	}
 	if (flags & TEVENT_FD_READ) {
 		smbd_server_connection_read_handler(
-			conn, conn->smb1.echo_handler.trusted_fd);
+			conn, xconn->smb1.echo_handler.trusted_fd);
 		return;
 	}
 }
@@ -3199,6 +3203,7 @@ static void smbd_echo_got_packet(struct tevent_req *req)
  */
 bool fork_echo_handler(struct smbd_server_connection *sconn)
 {
+	struct smbXsrv_connection *xconn = sconn->conn;
 	int listener_pipe[2];
 	int res;
 	pid_t child;
@@ -3216,9 +3221,9 @@ bool fork_echo_handler(struct smbd_server_connection *sconn)
 	if (use_mutex) {
 		pthread_mutexattr_t a;
 
-		sconn->smb1.echo_handler.socket_mutex =
+		xconn->smb1.echo_handler.socket_mutex =
 			anonymous_shared_allocate(sizeof(pthread_mutex_t));
-		if (sconn->smb1.echo_handler.socket_mutex == NULL) {
+		if (xconn->smb1.echo_handler.socket_mutex == NULL) {
 			DEBUG(1, ("Could not create mutex shared memory: %s\n",
 				  strerror(errno)));
 			goto fail;
@@ -3251,7 +3256,7 @@ bool fork_echo_handler(struct smbd_server_connection *sconn)
 			pthread_mutexattr_destroy(&a);
 			goto fail;
 		}
-		res = pthread_mutex_init(sconn->smb1.echo_handler.socket_mutex,
+		res = pthread_mutex_init(xconn->smb1.echo_handler.socket_mutex,
 					 &a);
 		pthread_mutexattr_destroy(&a);
 		if (res != 0) {
@@ -3263,9 +3268,9 @@ bool fork_echo_handler(struct smbd_server_connection *sconn)
 #endif
 
 	if (!use_mutex) {
-		sconn->smb1.echo_handler.socket_lock_fd =
+		xconn->smb1.echo_handler.socket_lock_fd =
 			create_unlink_tmp(lp_lock_directory());
-		if (sconn->smb1.echo_handler.socket_lock_fd == -1) {
+		if (xconn->smb1.echo_handler.socket_lock_fd == -1) {
 			DEBUG(1, ("Could not create lock fd: %s\n",
 				  strerror(errno)));
 			goto fail;
@@ -3292,7 +3297,7 @@ bool fork_echo_handler(struct smbd_server_connection *sconn)
 	}
 	close(listener_pipe[1]);
 	listener_pipe[1] = -1;
-	sconn->smb1.echo_handler.trusted_fd = listener_pipe[0];
+	xconn->smb1.echo_handler.trusted_fd = listener_pipe[0];
 
 	DEBUG(10,("fork_echo_handler: main[%d] echo_child[%d]\n", (int)getpid(), (int)child));
 
@@ -3300,13 +3305,13 @@ bool fork_echo_handler(struct smbd_server_connection *sconn)
 	 * Without smb signing this is the same as the normal smbd
 	 * listener. This needs to change once signing comes in.
 	 */
-	sconn->smb1.echo_handler.trusted_fde = tevent_add_fd(sconn->ev_ctx,
+	xconn->smb1.echo_handler.trusted_fde = tevent_add_fd(sconn->ev_ctx,
 					sconn,
-					sconn->smb1.echo_handler.trusted_fd,
+					xconn->smb1.echo_handler.trusted_fd,
 					TEVENT_FD_READ,
 					smbd_server_echo_handler,
 					sconn);
-	if (sconn->smb1.echo_handler.trusted_fde == NULL) {
+	if (xconn->smb1.echo_handler.trusted_fde == NULL) {
 		DEBUG(1, ("event_add_fd failed\n"));
 		goto fail;
 	}
@@ -3320,17 +3325,16 @@ fail:
 	if (listener_pipe[1] != -1) {
 		close(listener_pipe[1]);
 	}
-	if (sconn->smb1.echo_handler.socket_lock_fd != -1) {
-		close(sconn->smb1.echo_handler.socket_lock_fd);
+	if (xconn->smb1.echo_handler.socket_lock_fd != -1) {
+		close(xconn->smb1.echo_handler.socket_lock_fd);
 	}
-
 #ifdef HAVE_ROBUST_MUTEXES
-	if (sconn->smb1.echo_handler.socket_mutex != NULL) {
-		pthread_mutex_destroy(sconn->smb1.echo_handler.socket_mutex);
-		anonymous_shared_free(sconn->smb1.echo_handler.socket_mutex);
+	if (xconn->smb1.echo_handler.socket_mutex != NULL) {
+		pthread_mutex_destroy(xconn->smb1.echo_handler.socket_mutex);
+		anonymous_shared_free(xconn->smb1.echo_handler.socket_mutex);
 	}
 #endif
-	smbd_echo_init(sconn);
+	smbd_echo_init(xconn);
 
 	return false;
 }
@@ -3559,6 +3563,7 @@ void smbd_process(struct tevent_context *ev_ctx,
 	conn->ev_ctx = ev_ctx;
 	conn->msg_ctx = msg_ctx;
 	conn->transport.sock = sock_fd;
+	smbd_echo_init(conn);
 
 	sconn = talloc_zero(conn, struct smbd_server_connection);
 	if (!sconn) {
@@ -3575,7 +3580,6 @@ void smbd_process(struct tevent_context *ev_ctx,
 
 	sconn->ev_ctx = ev_ctx;
 	sconn->msg_ctx = msg_ctx;
-	smbd_echo_init(sconn);
 
 	if (!interactive) {
 		smbd_setup_sig_term_handler(sconn);
diff --git a/source3/smbd/reply.c b/source3/smbd/reply.c
index f7783e6..6dac6db 100644
--- a/source3/smbd/reply.c
+++ b/source3/smbd/reply.c
@@ -3344,7 +3344,7 @@ void reply_readbraw(struct smb_request *req)
 		return;
 	}
 
-	if (sconn->smb1.echo_handler.trusted_fde) {
+	if (xconn->smb1.echo_handler.trusted_fde) {
 		DEBUG(2,("SMBreadbraw rejected with NOT_SUPPORTED because of "
 			 "'async smb echo handler = yes'\n"));
 		reply_readbraw_error(sconn);
@@ -4161,7 +4161,7 @@ void reply_writebraw(struct smb_request *req)
 		return;
 	}
 
-	if (req->sconn->smb1.echo_handler.trusted_fde) {
+	if (xconn->smb1.echo_handler.trusted_fde) {
 		DEBUG(2,("SMBwritebraw rejected with NOT_SUPPORTED because of "
 			 "'async smb echo handler = yes'\n"));
 		reply_nterror(req, NT_STATUS_NOT_SUPPORTED);
diff --git a/source3/smbd/trans2.c b/source3/smbd/trans2.c
index d00a6db..ca0ef73 100644
--- a/source3/smbd/trans2.c
+++ b/source3/smbd/trans2.c
@@ -3709,6 +3709,7 @@ static void call_trans2setfsinfo(connection_struct *conn,
 				 unsigned int max_data_bytes)
 {
 	struct smbd_server_connection *sconn = req->sconn;
+	struct smbXsrv_connection *xconn = sconn->conn;
 	char *pdata = *ppdata;
 	char *params = *pparams;
 	uint16 info_level;
@@ -3819,7 +3820,7 @@ static void call_trans2setfsinfo(connection_struct *conn,
 					return;
 				}
 
-				if (req->sconn->smb1.echo_handler.trusted_fde) {
+				if (xconn->smb1.echo_handler.trusted_fde) {
 					DEBUG( 2,("call_trans2setfsinfo: "
 						"request transport encryption disabled"
 						"with 'fork echo handler = yes'\n"));
-- 
1.8.1.2


From 13b517db97f0a2cba2076b538de3b42a7c169f5e Mon Sep 17 00:00:00 2001
From: Stefan Metzmacher <metze@samba.org>
Date: Tue, 10 Jun 2014 12:57:38 +0200
Subject: [PATCH 20/94] s3:smbd: move sconn->smb1.unix_info.* to
 xconn->smb1.unix_info.*

This prepares the structures for multi-channel support.

Signed-off-by: Stefan Metzmacher <metze@samba.org>
Reviewed-by: Volker Lendecke <vl@samba.org>
---
 source3/smbd/globals.h | 14 +++++++-------
 source3/smbd/trans2.c  | 22 +++++++++++-----------
 2 files changed, 18 insertions(+), 18 deletions(-)

diff --git a/source3/smbd/globals.h b/source3/smbd/globals.h
index ba90d47..5ce0fd3 100644
--- a/source3/smbd/globals.h
+++ b/source3/smbd/globals.h
@@ -407,6 +407,13 @@ struct smbXsrv_connection {
 			int max_send;
 		} sessions;
 		struct smb_signing_state *signing_state;
+
+		struct {
+			uint16_t client_major;
+			uint16_t client_minor;
+			uint32_t client_cap_low;
+			uint32_t client_cap_high;
+		} unix_info;
 	} smb1;
 	struct {
 		struct {
@@ -756,13 +763,6 @@ struct smbd_server_connection {
 	} oplocks;
 
 	struct {
-		struct {
-			uint16_t client_major;
-			uint16_t client_minor;
-			uint32_t client_cap_low;
-			uint32_t client_cap_high;
-		} unix_info;
-
 		struct notify_mid_map *notify_mid_maps;
 
 		struct {
diff --git a/source3/smbd/trans2.c b/source3/smbd/trans2.c
index ca0ef73..e6cb64f 100644
--- a/source3/smbd/trans2.c
+++ b/source3/smbd/trans2.c
@@ -3766,31 +3766,31 @@ static void call_trans2setfsinfo(connection_struct *conn,
 					NT_STATUS_INVALID_PARAMETER);
 				return;
 			}
-			sconn->smb1.unix_info.client_major = SVAL(pdata,0);
-			sconn->smb1.unix_info.client_minor = SVAL(pdata,2);
-			sconn->smb1.unix_info.client_cap_low = IVAL(pdata,4);
-			sconn->smb1.unix_info.client_cap_high = IVAL(pdata,8);
+			xconn->smb1.unix_info.client_major = SVAL(pdata,0);
+			xconn->smb1.unix_info.client_minor = SVAL(pdata,2);
+			xconn->smb1.unix_info.client_cap_low = IVAL(pdata,4);
+			xconn->smb1.unix_info.client_cap_high = IVAL(pdata,8);
 			/* Just print these values for now. */
 			DEBUG(10, ("call_trans2setfsinfo: set unix_info info. "
 				   "major = %u, minor = %u cap_low = 0x%x, "
 				   "cap_high = 0x%xn",
-				   (unsigned int)sconn->
+				   (unsigned int)xconn->
 				   smb1.unix_info.client_major,
-				   (unsigned int)sconn->
+				   (unsigned int)xconn->
 				   smb1.unix_info.client_minor,
-				   (unsigned int)sconn->
+				   (unsigned int)xconn->
 				   smb1.unix_info.client_cap_low,
-				   (unsigned int)sconn->
+				   (unsigned int)xconn->
 				   smb1.unix_info.client_cap_high));
 
 			/* Here is where we must switch to posix pathname processing... */
-			if (sconn->smb1.unix_info.client_cap_low & CIFS_UNIX_POSIX_PATHNAMES_CAP) {
+			if (xconn->smb1.unix_info.client_cap_low & CIFS_UNIX_POSIX_PATHNAMES_CAP) {
 				lp_set_posix_pathnames();
 				mangle_change_to_posix();
 			}
 
-			if ((sconn->smb1.unix_info.client_cap_low & CIFS_UNIX_FCNTL_LOCKS_CAP) &&
-			    !(sconn->smb1.unix_info.client_cap_low & CIFS_UNIX_POSIX_PATH_OPERATIONS_CAP)) {
+			if ((xconn->smb1.unix_info.client_cap_low & CIFS_UNIX_FCNTL_LOCKS_CAP) &&
+			    !(xconn->smb1.unix_info.client_cap_low & CIFS_UNIX_POSIX_PATH_OPERATIONS_CAP)) {
 				/* Client that knows how to do posix locks,
 				 * but not posix open/mkdir operations. Set a
 				 * default type for read/write checks. */
-- 
1.8.1.2


From e74492a85ab24d9efdf64eceeead4ce6d032984e Mon Sep 17 00:00:00 2001
From: Stefan Metzmacher <metze@samba.org>
Date: Wed, 11 Jun 2014 12:36:14 +0200
Subject: [PATCH 21/94] s3:smbd: move xconn->msg_state to xconn->smb1.msg_state

Signed-off-by: Stefan Metzmacher <metze@samba.org>
Reviewed-by: Volker Lendecke <vl@samba.org>
---
 source3/smbd/globals.h |  4 ++--
 source3/smbd/message.c | 38 +++++++++++++++++++++-----------------
 2 files changed, 23 insertions(+), 19 deletions(-)

diff --git a/source3/smbd/globals.h b/source3/smbd/globals.h
index 5ce0fd3..8399b8e 100644
--- a/source3/smbd/globals.h
+++ b/source3/smbd/globals.h
@@ -414,6 +414,8 @@ struct smbXsrv_connection {
 			uint32_t client_cap_low;
 			uint32_t client_cap_high;
 		} unix_info;
+
+		struct msg_state *msg_state;
 	} smb1;
 	struct {
 		struct {
@@ -434,8 +436,6 @@ struct smbXsrv_connection {
 		} server;
 	} smb2;
 
-	struct msg_state *msg_state;
-
 	/*
 	 * Link into libasys for asynchronous operations
 	 */
diff --git a/source3/smbd/message.c b/source3/smbd/message.c
index 0dca2d8..9d8c703 100644
--- a/source3/smbd/message.c
+++ b/source3/smbd/message.c
@@ -194,6 +194,7 @@ void reply_sends(struct smb_request *req)
 
 void reply_sendstrt(struct smb_request *req)
 {
+	struct smbXsrv_connection *xconn = req->sconn->conn;
 	const uint8_t *p;
 
 	START_PROFILE(SMBsendstrt);
@@ -204,11 +205,11 @@ void reply_sendstrt(struct smb_request *req)
 		return;
 	}
 
-	TALLOC_FREE(req->sconn->conn->msg_state);
+	TALLOC_FREE(xconn->smb1.msg_state);
 
-	req->sconn->conn->msg_state = talloc_zero(NULL, struct msg_state);
+	xconn->smb1.msg_state = talloc_zero(xconn, struct msg_state);
 
-	if (req->sconn->conn->msg_state == NULL) {
+	if (xconn->smb1.msg_state == NULL) {
 		reply_nterror(req, NT_STATUS_NO_MEMORY);
 		END_PROFILE(SMBsendstrt);
 		return;
@@ -216,17 +217,17 @@ void reply_sendstrt(struct smb_request *req)
 
 	p = req->buf+1;
 	p += srvstr_pull_req_talloc(
-		req->sconn->conn->msg_state, req,
-		&req->sconn->conn->msg_state->from, p,
+		xconn->smb1.msg_state, req,
+		&xconn->smb1.msg_state->from, p,
 		STR_ASCII|STR_TERMINATE) + 1;
 	p += srvstr_pull_req_talloc(
-		req->sconn->conn->msg_state, req,
-		&req->sconn->conn->msg_state->to, p,
+		xconn->smb1.msg_state, req,
+		&xconn->smb1.msg_state->to, p,
 		STR_ASCII|STR_TERMINATE) + 1;
 
 	DEBUG(3, ("SMBsendstrt (from %s to %s)\n",
-		  req->sconn->conn->msg_state->from,
-		  req->sconn->conn->msg_state->to));
+		  xconn->smb1.msg_state->from,
+		  xconn->smb1.msg_state->to));
 
 	reply_outbuf(req, 0, 0);
 
@@ -241,6 +242,7 @@ void reply_sendstrt(struct smb_request *req)
 
 void reply_sendtxt(struct smb_request *req)
 {
+	struct smbXsrv_connection *xconn = req->sconn->conn;
 	int len;
 	const char *msg;
 	char *tmp;
@@ -254,7 +256,7 @@ void reply_sendtxt(struct smb_request *req)
 		return;
 	}
 
-	if ((req->sconn->conn->msg_state == NULL) || (req->buflen < 3)) {
+	if ((xconn->smb1.msg_state == NULL) || (req->buflen < 3)) {
 		reply_nterror(req, NT_STATUS_INVALID_PARAMETER);
 		END_PROFILE(SMBsendtxt);
 		return;
@@ -262,12 +264,12 @@ void reply_sendtxt(struct smb_request *req)
 
 	msg = (const char *)req->buf + 1;
 
-	old_len = talloc_get_size(req->sconn->conn->msg_state->msg);
+	old_len = talloc_get_size(xconn->smb1.msg_state->msg);
 
 	len = MIN(SVAL(msg, 0), smbreq_bufrem(req, msg+2));
 
-	tmp = talloc_realloc(req->sconn->conn->msg_state,
-			     req->sconn->conn->msg_state->msg,
+	tmp = talloc_realloc(xconn->smb1.msg_state,
+			     xconn->smb1.msg_state->msg,
 			     char, old_len + len);
 
 	if (tmp == NULL) {
@@ -276,9 +278,9 @@ void reply_sendtxt(struct smb_request *req)
 		return;
 	}
 
-	req->sconn->conn->msg_state->msg = tmp;
+	xconn->smb1.msg_state->msg = tmp;
 
-	memcpy(&req->sconn->conn->msg_state->msg[old_len], msg+2, len);
+	memcpy(&xconn->smb1.msg_state->msg[old_len], msg+2, len);
 
 	DEBUG( 3, ( "SMBsendtxt\n" ) );
 
@@ -295,6 +297,8 @@ void reply_sendtxt(struct smb_request *req)
 
 void reply_sendend(struct smb_request *req)
 {
+
+	struct smbXsrv_connection *xconn = req->sconn->conn;
 	START_PROFILE(SMBsendend);
 
 	if (! (*lp_message_command(talloc_tos()))) {
@@ -305,9 +309,9 @@ void reply_sendend(struct smb_request *req)
 
 	DEBUG(3,("SMBsendend\n"));
 
-	msg_deliver(req->sconn->conn->msg_state);
+	msg_deliver(xconn->smb1.msg_state);
 
-	TALLOC_FREE(req->sconn->conn->msg_state);
+	TALLOC_FREE(xconn->smb1.msg_state);
 
 	reply_outbuf(req, 0, 0);
 
-- 
1.8.1.2


From 038c8f7d9a22c801eabfef2d6222595bd3e34f38 Mon Sep 17 00:00:00 2001
From: Stefan Metzmacher <metze@samba.org>
Date: Thu, 12 Jun 2014 08:32:47 +0200
Subject: [PATCH 22/94] s3:smbd: move libasys state from smbXsrv_connection to
 smbd_server_connection

This doesn't belong to a connection, it's state used within the VFS stack.
And smbd_server_connection is the legacy structure we're using for
global VFS state.

Signed-off-by: Stefan Metzmacher <metze@samba.org>
Reviewed-by: Volker Lendecke <vl@samba.org>
---
 source3/modules/vfs_default.c | 14 +++++++-------
 source3/smbd/globals.h        | 12 ++++++------
 2 files changed, 13 insertions(+), 13 deletions(-)

diff --git a/source3/modules/vfs_default.c b/source3/modules/vfs_default.c
index 78c249d..b8f4117 100644
--- a/source3/modules/vfs_default.c
+++ b/source3/modules/vfs_default.c
@@ -650,7 +650,7 @@ static void vfswrap_asys_finished(struct tevent_context *ev,
 				  struct tevent_fd *fde,
 				  uint16_t flags, void *p);
 
-static bool vfswrap_init_asys_ctx(struct smbXsrv_connection *conn)
+static bool vfswrap_init_asys_ctx(struct smbd_server_connection *conn)
 {
 	int ret;
 	int fd;
@@ -709,11 +709,11 @@ static struct tevent_req *vfswrap_pread_send(struct vfs_handle_struct *handle,
 	if (req == NULL) {
 		return NULL;
 	}
-	if (!vfswrap_init_asys_ctx(handle->conn->sconn->conn)) {
+	if (!vfswrap_init_asys_ctx(handle->conn->sconn)) {
 		tevent_req_oom(req);
 		return tevent_req_post(req, ev);
 	}
-	state->asys_ctx = handle->conn->sconn->conn->asys_ctx;
+	state->asys_ctx = handle->conn->sconn->asys_ctx;
 	state->req = req;
 
 	ret = asys_pread(state->asys_ctx, fsp->fh->fd, data, n, offset, req);
@@ -741,11 +741,11 @@ static struct tevent_req *vfswrap_pwrite_send(struct vfs_handle_struct *handle,
 	if (req == NULL) {
 		return NULL;
 	}
-	if (!vfswrap_init_asys_ctx(handle->conn->sconn->conn)) {
+	if (!vfswrap_init_asys_ctx(handle->conn->sconn)) {
 		tevent_req_oom(req);
 		return tevent_req_post(req, ev);
 	}
-	state->asys_ctx = handle->conn->sconn->conn->asys_ctx;
+	state->asys_ctx = handle->conn->sconn->asys_ctx;
 	state->req = req;
 
 	ret = asys_pwrite(state->asys_ctx, fsp->fh->fd, data, n, offset, req);
@@ -771,11 +771,11 @@ static struct tevent_req *vfswrap_fsync_send(struct vfs_handle_struct *handle,
 	if (req == NULL) {
 		return NULL;
 	}
-	if (!vfswrap_init_asys_ctx(handle->conn->sconn->conn)) {
+	if (!vfswrap_init_asys_ctx(handle->conn->sconn)) {
 		tevent_req_oom(req);
 		return tevent_req_post(req, ev);
 	}
-	state->asys_ctx = handle->conn->sconn->conn->asys_ctx;
+	state->asys_ctx = handle->conn->sconn->asys_ctx;
 	state->req = req;
 
 	ret = asys_fsync(state->asys_ctx, fsp->fh->fd, req);
diff --git a/source3/smbd/globals.h b/source3/smbd/globals.h
index 8399b8e..5a5ed28 100644
--- a/source3/smbd/globals.h
+++ b/source3/smbd/globals.h
@@ -436,12 +436,6 @@ struct smbXsrv_connection {
 		} server;
 	} smb2;
 
-	/*
-	 * Link into libasys for asynchronous operations
-	 */
-	struct asys_context *asys_ctx;
-	struct tevent_fd *asys_fde;
-
 	uint64_t smbd_idle_profstamp;
 
 	/*
@@ -850,6 +844,12 @@ struct smbd_server_connection {
 		uint32_t max_write;
 	} smb2;
 
+	/*
+	 * Link into libasys for asynchronous operations
+	 */
+	struct asys_context *asys_ctx;
+	struct tevent_fd *asys_fde;
+
 	struct smbXsrv_connection *conn;
 };
 
-- 
1.8.1.2


From d35564141a7c19dd1a8a2298be57eaf95177f466 Mon Sep 17 00:00:00 2001
From: Stefan Metzmacher <metze@samba.org>
Date: Fri, 23 May 2014 09:36:24 +0200
Subject: [PATCH 23/94] s3:smb2_server: use xconn->transport.fde also for smb2

This prepares the structures for multi-channel support.

Signed-off-by: Stefan Metzmacher <metze@samba.org>
Reviewed-by: Volker Lendecke <vl@samba.org>
---
 source3/smbd/globals.h     |  1 -
 source3/smbd/smb2_server.c | 22 +++++++++++-----------
 2 files changed, 11 insertions(+), 12 deletions(-)

diff --git a/source3/smbd/globals.h b/source3/smbd/globals.h
index 5a5ed28..f55d24a 100644
--- a/source3/smbd/globals.h
+++ b/source3/smbd/globals.h
@@ -788,7 +788,6 @@ struct smbd_server_connection {
 		} request_read_state;
 		struct smbd_smb2_send_queue *send_queue;
 		size_t send_queue_len;
-		struct tevent_fd *fde;
 		bool negprot_2ff;
 		struct {
 			/* The event that makes us process our blocking lock queue */
diff --git a/source3/smbd/smb2_server.c b/source3/smbd/smb2_server.c
index bce6034..a9db177 100644
--- a/source3/smbd/smb2_server.c
+++ b/source3/smbd/smb2_server.c
@@ -216,13 +216,13 @@ static NTSTATUS smbd_initialize_smb2(struct smbd_server_connection *sconn)
 		return NT_STATUS_NO_MEMORY;
 	}
 
-	sconn->smb2.fde = tevent_add_fd(sconn->ev_ctx,
+	xconn->transport.fde = tevent_add_fd(sconn->ev_ctx,
 					sconn,
 					xconn->transport.sock,
 					TEVENT_FD_READ,
 					smbd_smb2_connection_handler,
 					sconn);
-	if (sconn->smb2.fde == NULL) {
+	if (xconn->transport.fde == NULL) {
 		return NT_STATUS_NO_MEMORY;
 	}
 
@@ -2994,7 +2994,7 @@ static NTSTATUS smbd_smb2_request_next_incoming(struct smbd_server_connection *s
 	state->req->sconn = sconn;
 	state->min_recv_size = lp_min_receive_file_size();
 
-	TEVENT_FD_READABLE(sconn->smb2.fde);
+	TEVENT_FD_READABLE(xconn->transport.fde);
 
 	return NT_STATUS_OK;
 }
@@ -3106,7 +3106,7 @@ static NTSTATUS smbd_smb2_flush_send_queue(struct smbd_server_connection *sconn)
 	bool retry;
 
 	if (sconn->smb2.send_queue == NULL) {
-		TEVENT_FD_NOT_WRITEABLE(sconn->smb2.fde);
+		TEVENT_FD_NOT_WRITEABLE(xconn->transport.fde);
 		return NT_STATUS_OK;
 	}
 
@@ -3161,7 +3161,7 @@ static NTSTATUS smbd_smb2_flush_send_queue(struct smbd_server_connection *sconn)
 		err = socket_error_from_errno(ret, errno, &retry);
 		if (retry) {
 			/* retry later */
-			TEVENT_FD_WRITEABLE(sconn->smb2.fde);
+			TEVENT_FD_WRITEABLE(xconn->transport.fde);
 			return NT_STATUS_OK;
 		}
 		if (err != 0) {
@@ -3196,7 +3196,7 @@ static NTSTATUS smbd_smb2_flush_send_queue(struct smbd_server_connection *sconn)
 
 		if (e->count > 0) {
 			/* we have more to write */
-			TEVENT_FD_WRITEABLE(sconn->smb2.fde);
+			TEVENT_FD_WRITEABLE(xconn->transport.fde);
 			return NT_STATUS_OK;
 		}
 
@@ -3225,8 +3225,8 @@ static NTSTATUS smbd_smb2_io_handler(struct smbd_server_connection *sconn,
 		/*
 		 * we're not supposed to do any io
 		 */
-		TEVENT_FD_NOT_READABLE(sconn->smb2.fde);
-		TEVENT_FD_NOT_WRITEABLE(sconn->smb2.fde);
+		TEVENT_FD_NOT_READABLE(xconn->transport.fde);
+		TEVENT_FD_NOT_WRITEABLE(xconn->transport.fde);
 		return NT_STATUS_OK;
 	}
 
@@ -3242,7 +3242,7 @@ static NTSTATUS smbd_smb2_io_handler(struct smbd_server_connection *sconn,
 	}
 
 	if (state->req == NULL) {
-		TEVENT_FD_NOT_READABLE(sconn->smb2.fde);
+		TEVENT_FD_NOT_READABLE(xconn->transport.fde);
 		return NT_STATUS_OK;
 	}
 
@@ -3262,7 +3262,7 @@ again:
 	err = socket_error_from_errno(ret, errno, &retry);
 	if (retry) {
 		/* retry later */
-		TEVENT_FD_READABLE(sconn->smb2.fde);
+		TEVENT_FD_READABLE(xconn->transport.fde);
 		return NT_STATUS_OK;
 	}
 	if (err != 0) {
@@ -3276,7 +3276,7 @@ again:
 		state->vector.iov_base = (void *)base;
 		state->vector.iov_len -= ret;
 		/* we have more to read */
-		TEVENT_FD_READABLE(sconn->smb2.fde);
+		TEVENT_FD_READABLE(xconn->transport.fde);
 		return NT_STATUS_OK;
 	}
 
-- 
1.8.1.2


From 00eaf39e1c5292983ab7516423fa5e3ba080d745 Mon Sep 17 00:00:00 2001
From: Stefan Metzmacher <metze@samba.org>
Date: Fri, 23 May 2014 09:41:57 +0200
Subject: [PATCH 24/94] s3:smb2_server: move sconn->smb2.request_read_state to
 xconn->smb2.request_read_state

This prepares the structures for multi-channel support.

Signed-off-by: Stefan Metzmacher <metze@samba.org>
Reviewed-by: Volker Lendecke <vl@samba.org>
---
 source3/smbd/globals.h     | 27 ++++++++++++++-------------
 source3/smbd/smb2_server.c |  4 ++--
 2 files changed, 16 insertions(+), 15 deletions(-)

diff --git a/source3/smbd/globals.h b/source3/smbd/globals.h
index f55d24a..9d32a7e 100644
--- a/source3/smbd/globals.h
+++ b/source3/smbd/globals.h
@@ -418,6 +418,20 @@ struct smbXsrv_connection {
 		struct msg_state *msg_state;
 	} smb1;
 	struct {
+		struct smbd_smb2_request_read_state {
+			struct smbd_smb2_request *req;
+			struct {
+				uint8_t nbt[NBT_HDR_SIZE];
+				bool done;
+			} hdr;
+			struct iovec vector;
+			bool doing_receivefile;
+			size_t min_recv_size;
+			size_t pktfull;
+			size_t pktlen;
+			uint8_t *pktbuf;
+		} request_read_state;
+
 		struct {
 			uint32_t capabilities;
 			struct GUID guid;
@@ -773,19 +787,6 @@ struct smbd_server_connection {
 		} locks;
 	} smb1;
 	struct {
-		struct smbd_smb2_request_read_state {
-			struct smbd_smb2_request *req;
-			struct {
-				uint8_t nbt[NBT_HDR_SIZE];
-				bool done;
-			} hdr;
-			struct iovec vector;
-			bool doing_receivefile;
-			size_t min_recv_size;
-			size_t pktfull;
-			size_t pktlen;
-			uint8_t *pktbuf;
-		} request_read_state;
 		struct smbd_smb2_send_queue *send_queue;
 		size_t send_queue_len;
 		bool negprot_2ff;
diff --git a/source3/smbd/smb2_server.c b/source3/smbd/smb2_server.c
index a9db177..3b0b10a 100644
--- a/source3/smbd/smb2_server.c
+++ b/source3/smbd/smb2_server.c
@@ -2953,8 +2953,8 @@ static bool is_smb2_recvfile_write(struct smbd_smb2_request_read_state *state)
 
 static NTSTATUS smbd_smb2_request_next_incoming(struct smbd_server_connection *sconn)
 {
-	struct smbd_smb2_request_read_state *state = &sconn->smb2.request_read_state;
 	struct smbXsrv_connection *xconn = sconn->conn;
+	struct smbd_smb2_request_read_state *state = &xconn->smb2.request_read_state;
 	size_t max_send_queue_len;
 	size_t cur_send_queue_len;
 
@@ -3212,7 +3212,7 @@ static NTSTATUS smbd_smb2_io_handler(struct smbd_server_connection *sconn,
 				     uint16_t fde_flags)
 {
 	struct smbXsrv_connection *xconn = sconn->conn;
-	struct smbd_smb2_request_read_state *state = &sconn->smb2.request_read_state;
+	struct smbd_smb2_request_read_state *state = &xconn->smb2.request_read_state;
 	struct smbd_smb2_request *req = NULL;
 	size_t min_recvfile_size = UINT32_MAX;
 	int ret;
-- 
1.8.1.2


From 2ade42429b6ddab391e294f9a2838d0b7773c1ac Mon Sep 17 00:00:00 2001
From: Stefan Metzmacher <metze@samba.org>
Date: Fri, 23 May 2014 09:53:21 +0200
Subject: [PATCH 25/94] s3:smb2_server: move sconn->smb2.send_queue* to
 xconn->smb2.send_queue*

This prepares the structures for multi-channel support.

Signed-off-by: Stefan Metzmacher <metze@samba.org>
Reviewed-by: Volker Lendecke <vl@samba.org>
---
 source3/smbd/globals.h     |  4 ++--
 source3/smbd/smb2_server.c | 41 +++++++++++++++++++----------------------
 2 files changed, 21 insertions(+), 24 deletions(-)

diff --git a/source3/smbd/globals.h b/source3/smbd/globals.h
index 9d32a7e..fb27649 100644
--- a/source3/smbd/globals.h
+++ b/source3/smbd/globals.h
@@ -431,6 +431,8 @@ struct smbXsrv_connection {
 			size_t pktlen;
 			uint8_t *pktbuf;
 		} request_read_state;
+		struct smbd_smb2_send_queue *send_queue;
+		size_t send_queue_len;
 
 		struct {
 			uint32_t capabilities;
@@ -787,8 +789,6 @@ struct smbd_server_connection {
 		} locks;
 	} smb1;
 	struct {
-		struct smbd_smb2_send_queue *send_queue;
-		size_t send_queue_len;
 		bool negprot_2ff;
 		struct {
 			/* The event that makes us process our blocking lock queue */
diff --git a/source3/smbd/smb2_server.c b/source3/smbd/smb2_server.c
index 3b0b10a..ba00911 100644
--- a/source3/smbd/smb2_server.c
+++ b/source3/smbd/smb2_server.c
@@ -204,8 +204,6 @@ static NTSTATUS smbd_initialize_smb2(struct smbd_server_connection *sconn)
 
 	TALLOC_FREE(xconn->transport.fde);
 
-	sconn->smb2.send_queue = NULL;
-
 	sconn->smb2.seqnum_low = 0;
 	sconn->smb2.seqnum_range = 1;
 	sconn->smb2.credits_granted = 1;
@@ -1263,8 +1261,8 @@ static NTSTATUS smb2_send_async_interim_response(const struct smbd_smb2_request
 	nreq->queue_entry.mem_ctx = nreq;
 	nreq->queue_entry.vector = nreq->out.vector;
 	nreq->queue_entry.count = nreq->out.vector_count;
-	DLIST_ADD_END(nreq->sconn->smb2.send_queue, &nreq->queue_entry, NULL);
-	nreq->sconn->smb2.send_queue_len++;
+	DLIST_ADD_END(conn->smb2.send_queue, &nreq->queue_entry, NULL);
+	conn->smb2.send_queue_len++;
 
 	status = smbd_smb2_flush_send_queue(sconn);
 	if (!NT_STATUS_IS_OK(status)) {
@@ -1424,6 +1422,7 @@ static void smbd_smb2_request_pending_timer(struct tevent_context *ev,
 		talloc_get_type_abort(private_data,
 		struct smbd_smb2_request);
 	struct smbd_server_connection *sconn = req->sconn;
+	struct smbXsrv_connection *xconn = sconn->conn;
 	struct smbd_smb2_request_pending_state *state = NULL;
 	uint8_t *outhdr = NULL;
 	const uint8_t *inhdr = NULL;
@@ -1565,11 +1564,10 @@ static void smbd_smb2_request_pending_timer(struct tevent_context *ev,
 
 	if (req->do_encryption) {
 		struct smbXsrv_session *x = req->session;
-		struct smbXsrv_connection *conn = x->connection;
 		DATA_BLOB encryption_key = x->global->encryption_key;
 
 		status = smb2_signing_encrypt_pdu(encryption_key,
-					conn->protocol,
+					xconn->protocol,
 					&state->vector[1+SMBD_SMB2_TF_IOV_OFS],
 					SMBD_SMB2_NUM_IOV_PER_REQ);
 		if (!NT_STATUS_IS_OK(status)) {
@@ -1579,11 +1577,10 @@ static void smbd_smb2_request_pending_timer(struct tevent_context *ev,
 		}
 	} else if (req->do_signing) {
 		struct smbXsrv_session *x = req->session;
-		struct smbXsrv_connection *conn = x->connection;
 		DATA_BLOB signing_key = x->global->channels[0].signing_key;
 
 		status = smb2_signing_sign_pdu(signing_key,
-					conn->protocol,
+					xconn->protocol,
 					&state->vector[1+SMBD_SMB2_HDR_IOV_OFS],
 					SMBD_SMB2_NUM_IOV_PER_REQ - 1);
 		if (!NT_STATUS_IS_OK(status)) {
@@ -1596,8 +1593,8 @@ static void smbd_smb2_request_pending_timer(struct tevent_context *ev,
 	state->queue_entry.mem_ctx = state;
 	state->queue_entry.vector = state->vector;
 	state->queue_entry.count = ARRAY_SIZE(state->vector);
-	DLIST_ADD_END(sconn->smb2.send_queue, &state->queue_entry, NULL);
-	sconn->smb2.send_queue_len++;
+	DLIST_ADD_END(xconn->smb2.send_queue, &state->queue_entry, NULL);
+	xconn->smb2.send_queue_len++;
 
 	status = smbd_smb2_flush_send_queue(sconn);
 	if (!NT_STATUS_IS_OK(status)) {
@@ -2512,8 +2509,8 @@ static NTSTATUS smbd_smb2_request_reply(struct smbd_smb2_request *req)
 	req->queue_entry.mem_ctx = req;
 	req->queue_entry.vector = req->out.vector;
 	req->queue_entry.count = req->out.vector_count;
-	DLIST_ADD_END(req->sconn->smb2.send_queue, &req->queue_entry, NULL);
-	req->sconn->smb2.send_queue_len++;
+	DLIST_ADD_END(conn->smb2.send_queue, &req->queue_entry, NULL);
+	conn->smb2.send_queue_len++;
 
 	status = smbd_smb2_flush_send_queue(sconn);
 	if (!NT_STATUS_IS_OK(status)) {
@@ -2841,8 +2838,8 @@ static NTSTATUS smbd_smb2_send_break(struct smbd_server_connection *sconn,
 	state->queue_entry.mem_ctx = state;
 	state->queue_entry.vector = state->vector;
 	state->queue_entry.count = ARRAY_SIZE(state->vector);
-	DLIST_ADD_END(state->sconn->smb2.send_queue, &state->queue_entry, NULL);
-	state->sconn->smb2.send_queue_len++;
+	DLIST_ADD_END(conn->smb2.send_queue, &state->queue_entry, NULL);
+	conn->smb2.send_queue_len++;
 
 	status = smbd_smb2_flush_send_queue(sconn);
 	if (!NT_STATUS_IS_OK(status)) {
@@ -2974,7 +2971,7 @@ static NTSTATUS smbd_smb2_request_next_incoming(struct smbd_server_connection *s
 	}
 
 	max_send_queue_len = MAX(1, sconn->smb2.max_credits/16);
-	cur_send_queue_len = sconn->smb2.send_queue_len;
+	cur_send_queue_len = xconn->smb2.send_queue_len;
 
 	if (cur_send_queue_len > max_send_queue_len) {
 		/*
@@ -3105,13 +3102,13 @@ static NTSTATUS smbd_smb2_flush_send_queue(struct smbd_server_connection *sconn)
 	int err;
 	bool retry;
 
-	if (sconn->smb2.send_queue == NULL) {
+	if (xconn->smb2.send_queue == NULL) {
 		TEVENT_FD_NOT_WRITEABLE(xconn->transport.fde);
 		return NT_STATUS_OK;
 	}
 
-	while (sconn->smb2.send_queue != NULL) {
-		struct smbd_smb2_send_queue *e = sconn->smb2.send_queue;
+	while (xconn->smb2.send_queue != NULL) {
+		struct smbd_smb2_send_queue *e = xconn->smb2.send_queue;
 
 		if (e->sendfile_header != NULL) {
 			size_t size = 0;
@@ -3143,8 +3140,8 @@ static NTSTATUS smbd_smb2_flush_send_queue(struct smbd_server_connection *sconn)
 			e->sendfile_header->length = size;
 			e->count = 0;
 
-			sconn->smb2.send_queue_len--;
-			DLIST_REMOVE(sconn->smb2.send_queue, e);
+			xconn->smb2.send_queue_len--;
+			DLIST_REMOVE(xconn->smb2.send_queue, e);
 			/*
 			 * This triggers the sendfile path via
 			 * the destructor.
@@ -3200,8 +3197,8 @@ static NTSTATUS smbd_smb2_flush_send_queue(struct smbd_server_connection *sconn)
 			return NT_STATUS_OK;
 		}
 
-		sconn->smb2.send_queue_len--;
-		DLIST_REMOVE(sconn->smb2.send_queue, e);
+		xconn->smb2.send_queue_len--;
+		DLIST_REMOVE(xconn->smb2.send_queue, e);
 		talloc_free(e->mem_ctx);
 	}
 
-- 
1.8.1.2


From 8d88aff00d2f497bb6896bc2d0dbf80d07753dcd Mon Sep 17 00:00:00 2001
From: Stefan Metzmacher <metze@samba.org>
Date: Fri, 23 May 2014 10:07:21 +0200
Subject: [PATCH 26/94] s3:smb2_server: move sconn->smb2.negprot_2ff to
 xconn->smb2.allow_2ff

This prepares the structures for multi-channel support.

Signed-off-by: Stefan Metzmacher <metze@samba.org>
Reviewed-by: Volker Lendecke <vl@samba.org>
---
 source3/smbd/globals.h      | 2 +-
 source3/smbd/smb2_negprot.c | 8 +++++---
 2 files changed, 6 insertions(+), 4 deletions(-)

diff --git a/source3/smbd/globals.h b/source3/smbd/globals.h
index fb27649..3dd0397 100644
--- a/source3/smbd/globals.h
+++ b/source3/smbd/globals.h
@@ -434,6 +434,7 @@ struct smbXsrv_connection {
 		struct smbd_smb2_send_queue *send_queue;
 		size_t send_queue_len;
 
+		bool allow_2ff;
 		struct {
 			uint32_t capabilities;
 			struct GUID guid;
@@ -789,7 +790,6 @@ struct smbd_server_connection {
 		} locks;
 	} smb1;
 	struct {
-		bool negprot_2ff;
 		struct {
 			/* The event that makes us process our blocking lock queue */
 			struct tevent_timer *brl_timeout;
diff --git a/source3/smbd/smb2_negprot.c b/source3/smbd/smb2_negprot.c
index deeb19d..c7d88d9 100644
--- a/source3/smbd/smb2_negprot.c
+++ b/source3/smbd/smb2_negprot.c
@@ -78,7 +78,8 @@ void reply_smb2002(struct smb_request *req, uint16_t choice)
  */
 void reply_smb20ff(struct smb_request *req, uint16_t choice)
 {
-	req->sconn->smb2.negprot_2ff = true;
+	struct smbXsrv_connection *xconn = req->sconn->conn;
+	xconn->smb2.allow_2ff = true;
 	reply_smb20xx(req, SMB2_DIALECT_REVISION_2FF);
 }
 
@@ -169,6 +170,7 @@ enum protocol_types smbd_smb2_protocol_dialect_match(const uint8_t *indyn,
 
 NTSTATUS smbd_smb2_request_process_negprot(struct smbd_smb2_request *req)
 {
+	struct smbXsrv_connection *xconn = req->sconn->conn;
 	NTSTATUS status;
 	const uint8_t *inbody;
 	const uint8_t *indyn = NULL;
@@ -234,8 +236,8 @@ NTSTATUS smbd_smb2_request_process_negprot(struct smbd_smb2_request *req)
 
 		dialect = SVAL(indyn, c*2);
 		if (dialect == SMB2_DIALECT_REVISION_2FF) {
-			if (req->sconn->smb2.negprot_2ff) {
-				req->sconn->smb2.negprot_2ff = false;
+			if (xconn->smb2.allow_2ff) {
+				xconn->smb2.allow_2ff = false;
 				protocol = PROTOCOL_SMB2_10;
 				break;
 			}
-- 
1.8.1.2


From 0dcd875869b398a37f9c3cc3b8a5b95dd0a58237 Mon Sep 17 00:00:00 2001
From: Stefan Metzmacher <metze@samba.org>
Date: Fri, 23 May 2014 10:22:34 +0200
Subject: [PATCH 27/94] s3:smb2_server: use xconn->smb2.server.max_* instead of
 sconn->smb2.max_*

We don't need this twice...

Signed-off-by: Stefan Metzmacher <metze@samba.org>
Reviewed-by: Volker Lendecke <vl@samba.org>
---
 source3/smbd/globals.h      | 3 ---
 source3/smbd/smb2_find.c    | 5 +++--
 source3/smbd/smb2_getinfo.c | 9 +++++----
 source3/smbd/smb2_negprot.c | 4 ----
 source3/smbd/smb2_notify.c  | 3 ++-
 source3/smbd/smb2_read.c    | 5 +++--
 source3/smbd/smb2_setinfo.c | 5 +++--
 source3/smbd/smb2_write.c   | 5 +++--
 8 files changed, 19 insertions(+), 20 deletions(-)

diff --git a/source3/smbd/globals.h b/source3/smbd/globals.h
index 3dd0397..129edbe 100644
--- a/source3/smbd/globals.h
+++ b/source3/smbd/globals.h
@@ -839,9 +839,6 @@ struct smbd_server_connection {
 		 */
 		struct bitmap *credits_bitmap;
 		bool supports_multicredit;
-		uint32_t max_trans;
-		uint32_t max_read;
-		uint32_t max_write;
 	} smb2;
 
 	/*
diff --git a/source3/smbd/smb2_find.c b/source3/smbd/smb2_find.c
index 6bc44a5..2d310a5 100644
--- a/source3/smbd/smb2_find.c
+++ b/source3/smbd/smb2_find.c
@@ -207,6 +207,7 @@ static struct tevent_req *smbd_smb2_find_send(TALLOC_CTX *mem_ctx,
 					      uint32_t in_output_buffer_length,
 					      const char *in_file_name)
 {
+	struct smbXsrv_connection *xconn = smb2req->sconn->conn;
 	struct tevent_req *req;
 	struct smbd_smb2_find_state *state;
 	struct smb_request *smbreq;
@@ -273,11 +274,11 @@ static struct tevent_req *smbd_smb2_find_send(TALLOC_CTX *mem_ctx,
 		return tevent_req_post(req, ev);
 	}
 
-	if (in_output_buffer_length > smb2req->sconn->smb2.max_trans) {
+	if (in_output_buffer_length > xconn->smb2.server.max_trans) {
 		DEBUG(2,("smbd_smb2_find_send: "
 			 "client ignored max trans:%s: 0x%08X: 0x%08X\n",
 			 __location__, in_output_buffer_length,
-			 smb2req->sconn->smb2.max_trans));
+			 xconn->smb2.server.max_trans));
 		tevent_req_nterror(req, NT_STATUS_INVALID_PARAMETER);
 		return tevent_req_post(req, ev);
 	}
diff --git a/source3/smbd/smb2_getinfo.c b/source3/smbd/smb2_getinfo.c
index ec6181a..275792c 100644
--- a/source3/smbd/smb2_getinfo.c
+++ b/source3/smbd/smb2_getinfo.c
@@ -44,6 +44,7 @@ static NTSTATUS smbd_smb2_getinfo_recv(struct tevent_req *req,
 static void smbd_smb2_request_getinfo_done(struct tevent_req *subreq);
 NTSTATUS smbd_smb2_request_process_getinfo(struct smbd_smb2_request *req)
 {
+	struct smbXsrv_connection *xconn = req->sconn->conn;
 	NTSTATUS status;
 	const uint8_t *inbody;
 	uint8_t in_info_type;
@@ -90,18 +91,18 @@ NTSTATUS smbd_smb2_request_process_getinfo(struct smbd_smb2_request *req)
 	in_input_buffer.data = SMBD_SMB2_IN_DYN_PTR(req);
 	in_input_buffer.length = in_input_buffer_length;
 
-	if (in_input_buffer.length > req->sconn->smb2.max_trans) {
+	if (in_input_buffer.length > xconn->smb2.server.max_trans) {
 		DEBUG(2,("smbd_smb2_request_process_getinfo: "
 			 "client ignored max trans: %s: 0x%08X: 0x%08X\n",
 			 __location__, (unsigned)in_input_buffer.length,
-			 (unsigned)req->sconn->smb2.max_trans));
+			 (unsigned)xconn->smb2.server.max_trans));
 		return smbd_smb2_request_error(req, NT_STATUS_INVALID_PARAMETER);
 	}
-	if (in_output_buffer_length > req->sconn->smb2.max_trans) {
+	if (in_output_buffer_length > xconn->smb2.server.max_trans) {
 		DEBUG(2,("smbd_smb2_request_process_getinfo: "
 			 "client ignored max trans: %s: 0x%08X: 0x%08X\n",
 			 __location__, in_output_buffer_length,
-			 req->sconn->smb2.max_trans));
+			 xconn->smb2.server.max_trans));
 		return smbd_smb2_request_error(req, NT_STATUS_INVALID_PARAMETER);
 	}
 
diff --git a/source3/smbd/smb2_negprot.c b/source3/smbd/smb2_negprot.c
index c7d88d9..c2e1c71 100644
--- a/source3/smbd/smb2_negprot.c
+++ b/source3/smbd/smb2_negprot.c
@@ -394,10 +394,6 @@ NTSTATUS smbd_smb2_request_process_negprot(struct smbd_smb2_request *req)
 		conn->smb2.server.max_trans = max_trans;
 		conn->smb2.server.max_read  = max_read;
 		conn->smb2.server.max_write = max_write;
-
-		req->sconn->smb2.max_trans = max_trans;
-		req->sconn->smb2.max_read  = max_read;
-		req->sconn->smb2.max_write = max_write;
 	}
 
 	return smbd_smb2_request_done(req, outbody, &outdyn);
diff --git a/source3/smbd/smb2_notify.c b/source3/smbd/smb2_notify.c
index 7351d9f..3e33d42 100644
--- a/source3/smbd/smb2_notify.c
+++ b/source3/smbd/smb2_notify.c
@@ -48,6 +48,7 @@ static NTSTATUS smbd_smb2_notify_recv(struct tevent_req *req,
 static void smbd_smb2_request_notify_done(struct tevent_req *subreq);
 NTSTATUS smbd_smb2_request_process_notify(struct smbd_smb2_request *req)
 {
+	struct smbXsrv_connection *xconn = req->sconn->conn;
 	NTSTATUS status;
 	const uint8_t *inbody;
 	uint16_t in_flags;
@@ -74,7 +75,7 @@ NTSTATUS smbd_smb2_request_process_notify(struct smbd_smb2_request *req)
 	 * 0x00010000 is what Windows 7 uses,
 	 * Windows 2008 uses 0x00080000
 	 */
-	if (in_output_buffer_length > req->sconn->smb2.max_trans) {
+	if (in_output_buffer_length > xconn->smb2.server.max_trans) {
 		return smbd_smb2_request_error(req, NT_STATUS_INVALID_PARAMETER);
 	}
 
diff --git a/source3/smbd/smb2_read.c b/source3/smbd/smb2_read.c
index b68a1b6..1679944 100644
--- a/source3/smbd/smb2_read.c
+++ b/source3/smbd/smb2_read.c
@@ -43,6 +43,7 @@ static NTSTATUS smbd_smb2_read_recv(struct tevent_req *req,
 static void smbd_smb2_request_read_done(struct tevent_req *subreq);
 NTSTATUS smbd_smb2_request_process_read(struct smbd_smb2_request *req)
 {
+	struct smbXsrv_connection *xconn = req->sconn->conn;
 	NTSTATUS status;
 	const uint8_t *inbody;
 	uint32_t in_length;
@@ -68,10 +69,10 @@ NTSTATUS smbd_smb2_request_process_read(struct smbd_smb2_request *req)
 	in_remaining_bytes	= IVAL(inbody, 0x28);
 
 	/* check the max read size */
-	if (in_length > req->sconn->smb2.max_read) {
+	if (in_length > xconn->smb2.server.max_read) {
 		DEBUG(2,("smbd_smb2_request_process_read: "
 			 "client ignored max read: %s: 0x%08X: 0x%08X\n",
-			__location__, in_length, req->sconn->smb2.max_read));
+			__location__, in_length, xconn->smb2.server.max_read));
 		return smbd_smb2_request_error(req, NT_STATUS_INVALID_PARAMETER);
 	}
 
diff --git a/source3/smbd/smb2_setinfo.c b/source3/smbd/smb2_setinfo.c
index 97872da..1b5e2a8 100644
--- a/source3/smbd/smb2_setinfo.c
+++ b/source3/smbd/smb2_setinfo.c
@@ -39,6 +39,7 @@ static NTSTATUS smbd_smb2_setinfo_recv(struct tevent_req *req);
 static void smbd_smb2_request_setinfo_done(struct tevent_req *subreq);
 NTSTATUS smbd_smb2_request_process_setinfo(struct smbd_smb2_request *req)
 {
+	struct smbXsrv_connection *xconn = req->sconn->conn;
 	NTSTATUS status;
 	const uint8_t *inbody;
 	uint8_t in_info_type;
@@ -81,11 +82,11 @@ NTSTATUS smbd_smb2_request_process_setinfo(struct smbd_smb2_request *req)
 	in_input_buffer.data = SMBD_SMB2_IN_DYN_PTR(req);
 	in_input_buffer.length = in_input_buffer_length;
 
-	if (in_input_buffer.length > req->sconn->smb2.max_trans) {
+	if (in_input_buffer.length > xconn->smb2.server.max_trans) {
 		DEBUG(2,("smbd_smb2_request_process_setinfo: "
 			 "client ignored max trans: %s: 0x%08X: 0x%08X\n",
 			 __location__, (unsigned)in_input_buffer.length,
-			 (unsigned)req->sconn->smb2.max_trans));
+			 (unsigned)xconn->smb2.server.max_trans));
 		return smbd_smb2_request_error(req, NT_STATUS_INVALID_PARAMETER);
 	}
 
diff --git a/source3/smbd/smb2_write.c b/source3/smbd/smb2_write.c
index c61254f..40a5937 100644
--- a/source3/smbd/smb2_write.c
+++ b/source3/smbd/smb2_write.c
@@ -38,6 +38,7 @@ static NTSTATUS smbd_smb2_write_recv(struct tevent_req *req,
 static void smbd_smb2_request_write_done(struct tevent_req *subreq);
 NTSTATUS smbd_smb2_request_process_write(struct smbd_smb2_request *req)
 {
+	struct smbXsrv_connection *xconn = req->sconn->conn;
 	NTSTATUS status;
 	const uint8_t *inbody;
 	uint16_t in_data_offset;
@@ -82,10 +83,10 @@ NTSTATUS smbd_smb2_request_process_write(struct smbd_smb2_request *req)
 	}
 
 	/* check the max write size */
-	if (in_data_length > req->sconn->smb2.max_write) {
+	if (in_data_length > xconn->smb2.server.max_write) {
 		DEBUG(2,("smbd_smb2_request_process_write : "
 			"client ignored max write :%s: 0x%08X: 0x%08X\n",
-			__location__, in_data_length, req->sconn->smb2.max_write));
+			__location__, in_data_length, xconn->smb2.server.max_write));
 		return smbd_smb2_request_error(req, NT_STATUS_INVALID_PARAMETER);
 	}
 
-- 
1.8.1.2


From 6b5352c7940f95b7da3ff1cc16f28e7a04a0dc5d Mon Sep 17 00:00:00 2001
From: Stefan Metzmacher <metze@samba.org>
Date: Thu, 12 Jun 2014 08:36:08 +0200
Subject: [PATCH 28/94] s3:smb2_negprot: remove unused variable 'conn' we
 already have 'xconn'...

We don't need two helper variables for the same thing.

Signed-off-by: Stefan Metzmacher <metze@samba.org>
Reviewed-by: Volker Lendecke <vl@samba.org>
---
 source3/smbd/smb2_negprot.c | 36 +++++++++++++++++-------------------
 1 file changed, 17 insertions(+), 19 deletions(-)

diff --git a/source3/smbd/smb2_negprot.c b/source3/smbd/smb2_negprot.c
index c2e1c71..9d84f1a 100644
--- a/source3/smbd/smb2_negprot.c
+++ b/source3/smbd/smb2_negprot.c
@@ -366,34 +366,32 @@ NTSTATUS smbd_smb2_request_process_negprot(struct smbd_smb2_request *req)
 	req->sconn->using_smb2 = true;
 
 	if (dialect != SMB2_DIALECT_REVISION_2FF) {
-		struct smbXsrv_connection *conn = req->sconn->conn;
-
-		status = smbXsrv_connection_init_tables(conn, protocol);
+		status = smbXsrv_connection_init_tables(xconn, protocol);
 		if (!NT_STATUS_IS_OK(status)) {
 			return smbd_smb2_request_error(req, status);
 		}
 
-		conn->smb2.client.capabilities = in_capabilities;
-		conn->smb2.client.security_mode = in_security_mode;
-		conn->smb2.client.guid = in_guid;
-		conn->smb2.client.num_dialects = dialect_count;
-		conn->smb2.client.dialects = talloc_array(conn,
-							  uint16_t,
-							  dialect_count);
-		if (conn->smb2.client.dialects == NULL) {
+		xconn->smb2.client.capabilities = in_capabilities;
+		xconn->smb2.client.security_mode = in_security_mode;
+		xconn->smb2.client.guid = in_guid;
+		xconn->smb2.client.num_dialects = dialect_count;
+		xconn->smb2.client.dialects = talloc_array(xconn,
+							   uint16_t,
+							   dialect_count);
+		if (xconn->smb2.client.dialects == NULL) {
 			return smbd_smb2_request_error(req, NT_STATUS_NO_MEMORY);
 		}
 		for (c=0; c < dialect_count; c++) {
-			conn->smb2.client.dialects[c] = SVAL(indyn, c*2);
+			xconn->smb2.client.dialects[c] = SVAL(indyn, c*2);
 		}
 
-		conn->smb2.server.capabilities = capabilities;
-		conn->smb2.server.security_mode = security_mode;
-		conn->smb2.server.guid = out_guid;
-		conn->smb2.server.dialect = dialect;
-		conn->smb2.server.max_trans = max_trans;
-		conn->smb2.server.max_read  = max_read;
-		conn->smb2.server.max_write = max_write;
+		xconn->smb2.server.capabilities = capabilities;
+		xconn->smb2.server.security_mode = security_mode;
+		xconn->smb2.server.guid = out_guid;
+		xconn->smb2.server.dialect = dialect;
+		xconn->smb2.server.max_trans = max_trans;
+		xconn->smb2.server.max_read  = max_read;
+		xconn->smb2.server.max_write = max_write;
 	}
 
 	return smbd_smb2_request_done(req, outbody, &outdyn);
-- 
1.8.1.2


From aa10189cab71bee1e8e8501fd2d8b0d01847fd6f Mon Sep 17 00:00:00 2001
From: Stefan Metzmacher <metze@samba.org>
Date: Fri, 23 May 2014 12:10:23 +0200
Subject: [PATCH 29/94] s3:smb2_server: move sconn->smb2.*credits* to
 xconn->smb2.credits.*

This prepares the structures for multi-channel support.

Signed-off-by: Stefan Metzmacher <metze@samba.org>
Reviewed-by: Volker Lendecke <vl@samba.org>
---
 source3/smbd/globals.h      |  89 +++++++++++++++++++------------------
 source3/smbd/smb2_negprot.c |   2 +-
 source3/smbd/smb2_server.c  | 105 +++++++++++++++++++++++---------------------
 3 files changed, 103 insertions(+), 93 deletions(-)

diff --git a/source3/smbd/globals.h b/source3/smbd/globals.h
index 129edbe..28b4588 100644
--- a/source3/smbd/globals.h
+++ b/source3/smbd/globals.h
@@ -434,6 +434,52 @@ struct smbXsrv_connection {
 		struct smbd_smb2_send_queue *send_queue;
 		size_t send_queue_len;
 
+		struct {
+			/*
+			 * seq_low is the lowest sequence number
+			 * we will accept.
+			 */
+			uint64_t seq_low;
+			/*
+			 * seqnum_range is the range of credits we have
+			 * granted from the sequence windows starting
+			 * at seqnum_low.
+			 *
+			 * This gets incremented when new credits are
+			 * granted and gets decremented when the
+			 * lowest sequence number is consumed
+			 * (when seq_low gets incremented).
+			 */
+			uint16_t seq_range;
+			/*
+			 * The number of credits we have currently granted
+			 * to the client.
+			 *
+			 * This gets incremented when new credits are
+			 * granted and gets decremented when any credit
+			 * is comsumed.
+			 *
+			 * Note: the decrementing is different compared
+			 *       to seqnum_range.
+			 */
+			uint16_t granted;
+			/*
+			 * The maximum number of credits we will ever
+			 * grant to the client.
+			 *
+			 * Typically we will only grant 1/16th of
+			 * max_credits.
+			 *
+			 * This is the "server max credits" parameter.
+			 */
+			uint16_t max;
+			/*
+			 * a bitmap of size max_credits
+			 */
+			struct bitmap *bitmap;
+			bool multicredit;
+		} credits;
+
 		bool allow_2ff;
 		struct {
 			uint32_t capabilities;
@@ -796,49 +842,6 @@ struct smbd_server_connection {
 			bool blocking_lock_unlock_state;
 		} locks;
 		struct smbd_smb2_request *requests;
-		/*
-		 * seqnum_low is the lowest sequence number
-		 * we will accept.
-		 */
-		uint64_t seqnum_low;
-		/*
-		 * seqnum_range is the range of credits we have
-		 * granted from the sequence windows starting
-		 * at seqnum_low.
-		 *
-		 * This gets incremented when new credits are
-		 * granted and gets decremented when the
-		 * lowest sequence number is consumed
-		 * (when seqnum_low gets incremented).
-		 */
-		uint16_t seqnum_range;
-		/*
-		 * credits_grantedThe number of credits we have currently granted
-		 * to the client.
-		 *
-		 * This gets incremented when new credits are
-		 * granted and gets decremented when any credit
-		 * is comsumed.
-		 *
-		 * Note: the decrementing is different compared
-		 *       to seqnum_range.
-		 */
-		uint16_t credits_granted;
-		/*
-		 * The maximum number of credits we will ever
-		 * grant to the client.
-		 *
-		 * Typically we will only grant 1/16th of
-		 * max_credits.
-		 *
-		 * This is the "server max credits" parameter.
-		 */
-		uint16_t max_credits;
-		/*
-		 * a bitmap of size max_credits
-		 */
-		struct bitmap *credits_bitmap;
-		bool supports_multicredit;
 	} smb2;
 
 	/*
diff --git a/source3/smbd/smb2_negprot.c b/source3/smbd/smb2_negprot.c
index 9d84f1a..fde9514 100644
--- a/source3/smbd/smb2_negprot.c
+++ b/source3/smbd/smb2_negprot.c
@@ -300,7 +300,7 @@ NTSTATUS smbd_smb2_request_process_negprot(struct smbd_smb2_request *req)
 		/* largeMTU is not supported over NBT (tcp port 139) */
 		if (p != NBT_SMB_PORT) {
 			capabilities |= SMB2_CAP_LARGE_MTU;
-			req->sconn->smb2.supports_multicredit = true;
+			xconn->smb2.credits.multicredit = true;
 
 			/* Windows 2012R2 allows up to 8 MB */
 			max_limit = 0x800000; /* 8MB */
diff --git a/source3/smbd/smb2_server.c b/source3/smbd/smb2_server.c
index ba00911..b48d181 100644
--- a/source3/smbd/smb2_server.c
+++ b/source3/smbd/smb2_server.c
@@ -204,13 +204,13 @@ static NTSTATUS smbd_initialize_smb2(struct smbd_server_connection *sconn)
 
 	TALLOC_FREE(xconn->transport.fde);
 
-	sconn->smb2.seqnum_low = 0;
-	sconn->smb2.seqnum_range = 1;
-	sconn->smb2.credits_granted = 1;
-	sconn->smb2.max_credits = lp_smb2_max_credits();
-	sconn->smb2.credits_bitmap = bitmap_talloc(sconn,
-						   sconn->smb2.max_credits);
-	if (sconn->smb2.credits_bitmap == NULL) {
+	xconn->smb2.credits.seq_low = 0;
+	xconn->smb2.credits.seq_range = 1;
+	xconn->smb2.credits.granted = 1;
+	xconn->smb2.credits.max = lp_smb2_max_credits();
+	xconn->smb2.credits.bitmap = bitmap_talloc(xconn,
+						   xconn->smb2.credits.max);
+	if (xconn->smb2.credits.bitmap == NULL) {
 		return NT_STATUS_NO_MEMORY;
 	}
 
@@ -578,34 +578,38 @@ static NTSTATUS smbd_smb2_request_create(struct smbd_server_connection *sconn,
 static bool smb2_validate_sequence_number(struct smbd_server_connection *sconn,
 					  uint64_t message_id, uint64_t seq_id)
 {
-	struct bitmap *credits_bm = sconn->smb2.credits_bitmap;
+	struct smbXsrv_connection *xconn = sconn->conn;
+	struct bitmap *credits_bm = xconn->smb2.credits.bitmap;
 	unsigned int offset;
+	uint64_t seq_tmp;
 
-	if (seq_id < sconn->smb2.seqnum_low) {
+	seq_tmp = xconn->smb2.credits.seq_low;
+	if (seq_id < seq_tmp) {
 		DEBUG(0,("smb2_validate_sequence_number: bad message_id "
 			"%llu (sequence id %llu) "
 			"(granted = %u, low = %llu, range = %u)\n",
 			(unsigned long long)message_id,
 			(unsigned long long)seq_id,
-			(unsigned int)sconn->smb2.credits_granted,
-			(unsigned long long)sconn->smb2.seqnum_low,
-			(unsigned int)sconn->smb2.seqnum_range));
+			(unsigned int)xconn->smb2.credits.granted,
+			(unsigned long long)xconn->smb2.credits.seq_low,
+			(unsigned int)xconn->smb2.credits.seq_range));
 		return false;
 	}
 
-	if (seq_id >= sconn->smb2.seqnum_low + sconn->smb2.seqnum_range) {
+	seq_tmp += xconn->smb2.credits.seq_range;
+	if (seq_id >= seq_tmp) {
 		DEBUG(0,("smb2_validate_sequence_number: bad message_id "
 			"%llu (sequence id %llu) "
 			"(granted = %u, low = %llu, range = %u)\n",
 			(unsigned long long)message_id,
 			(unsigned long long)seq_id,
-			(unsigned int)sconn->smb2.credits_granted,
-			(unsigned long long)sconn->smb2.seqnum_low,
-			(unsigned int)sconn->smb2.seqnum_range));
+			(unsigned int)xconn->smb2.credits.granted,
+			(unsigned long long)xconn->smb2.credits.seq_low,
+			(unsigned int)xconn->smb2.credits.seq_range));
 		return false;
 	}
 
-	offset = seq_id % sconn->smb2.max_credits;
+	offset = seq_id % xconn->smb2.credits.max;
 
 	if (bitmap_query(credits_bm, offset)) {
 		DEBUG(0,("smb2_validate_sequence_number: duplicate message_id "
@@ -614,9 +618,9 @@ static bool smb2_validate_sequence_number(struct smbd_server_connection *sconn,
 			"(bm offset %u)\n",
 			(unsigned long long)message_id,
 			(unsigned long long)seq_id,
-			(unsigned int)sconn->smb2.credits_granted,
-			(unsigned long long)sconn->smb2.seqnum_low,
-			(unsigned int)sconn->smb2.seqnum_range,
+			(unsigned int)xconn->smb2.credits.granted,
+			(unsigned long long)xconn->smb2.credits.seq_low,
+			(unsigned int)xconn->smb2.credits.seq_range,
 			offset));
 		return false;
 	}
@@ -624,7 +628,7 @@ static bool smb2_validate_sequence_number(struct smbd_server_connection *sconn,
 	/* Mark the message_ids as seen in the bitmap. */
 	bitmap_set(credits_bm, offset);
 
-	if (seq_id != sconn->smb2.seqnum_low) {
+	if (seq_id != xconn->smb2.credits.seq_low) {
 		return true;
 	}
 
@@ -635,13 +639,13 @@ static bool smb2_validate_sequence_number(struct smbd_server_connection *sconn,
 	while (bitmap_query(credits_bm, offset)) {
 		DEBUG(10,("smb2_validate_sequence_number: clearing "
 			  "id %llu (position %u) from bitmap\n",
-			  (unsigned long long)(sconn->smb2.seqnum_low),
+			  (unsigned long long)(xconn->smb2.credits.seq_low),
 			  offset));
 		bitmap_clear(credits_bm, offset);
 
-		sconn->smb2.seqnum_low += 1;
-		sconn->smb2.seqnum_range -= 1;
-		offset = sconn->smb2.seqnum_low % sconn->smb2.max_credits;
+		xconn->smb2.credits.seq_low += 1;
+		xconn->smb2.credits.seq_range -= 1;
+		offset = xconn->smb2.credits.seq_low % xconn->smb2.credits.max;
 	}
 
 	return true;
@@ -650,6 +654,7 @@ static bool smb2_validate_sequence_number(struct smbd_server_connection *sconn,
 static bool smb2_validate_message_id(struct smbd_server_connection *sconn,
 				const uint8_t *inhdr)
 {
+	struct smbXsrv_connection *xconn = sconn->conn;
 	uint64_t message_id = BVAL(inhdr, SMB2_HDR_MESSAGE_ID);
 	uint16_t opcode = SVAL(inhdr, SMB2_HDR_OPCODE);
 	uint16_t credit_charge = 1;
@@ -660,7 +665,7 @@ static bool smb2_validate_message_id(struct smbd_server_connection *sconn,
 		return true;
 	}
 
-	if (sconn->smb2.supports_multicredit) {
+	if (xconn->smb2.credits.multicredit) {
 		credit_charge = SVAL(inhdr, SMB2_HDR_CREDIT_CHARGE);
 		credit_charge = MAX(credit_charge, 1);
 	}
@@ -670,20 +675,20 @@ static bool smb2_validate_message_id(struct smbd_server_connection *sconn,
 		   "seqnum low/range: %llu/%llu\n",
 		   (unsigned long long) message_id,
 		   (unsigned long long) credit_charge,
-		   (unsigned long long) sconn->smb2.credits_granted,
-		   (unsigned long long) sconn->smb2.seqnum_low,
-		   (unsigned long long) sconn->smb2.seqnum_range));
+		   (unsigned long long) xconn->smb2.credits.granted,
+		   (unsigned long long) xconn->smb2.credits.seq_low,
+		   (unsigned long long) xconn->smb2.credits.seq_range));
 
-	if (sconn->smb2.credits_granted < credit_charge) {
+	if (xconn->smb2.credits.granted < credit_charge) {
 		DEBUG(0, ("smb2_validate_message_id: client used more "
 			  "credits than granted, mid %llu, charge %llu, "
 			  "credits_granted %llu, "
 			  "seqnum low/range: %llu/%llu\n",
 			  (unsigned long long) message_id,
 			  (unsigned long long) credit_charge,
-			  (unsigned long long) sconn->smb2.credits_granted,
-			  (unsigned long long) sconn->smb2.seqnum_low,
-			  (unsigned long long) sconn->smb2.seqnum_range));
+			  (unsigned long long) xconn->smb2.credits.granted,
+			  (unsigned long long) xconn->smb2.credits.seq_low,
+			  (unsigned long long) xconn->smb2.credits.seq_range));
 		return false;
 	}
 
@@ -711,7 +716,7 @@ static bool smb2_validate_message_id(struct smbd_server_connection *sconn,
 	}
 
 	/* substract used credits */
-	sconn->smb2.credits_granted -= credit_charge;
+	xconn->smb2.credits.granted -= credit_charge;
 
 	return true;
 }
@@ -760,6 +765,7 @@ static void smb2_set_operation_credit(struct smbd_server_connection *sconn,
 			const struct iovec *in_vector,
 			struct iovec *out_vector)
 {
+	struct smbXsrv_connection *xconn = sconn->conn;
 	const uint8_t *inhdr = (const uint8_t *)in_vector->iov_base;
 	uint8_t *outhdr = (uint8_t *)out_vector->iov_base;
 	uint16_t credit_charge = 1;
@@ -785,10 +791,10 @@ static void smb2_set_operation_credit(struct smbd_server_connection *sconn,
 	 *       Which means we would grant more credits
 	 *       for client which use multi credit requests.
 	 */
-	current_max_credits = sconn->smb2.max_credits / 16;
+	current_max_credits = xconn->smb2.credits.max / 16;
 	current_max_credits = MAX(current_max_credits, 1);
 
-	if (sconn->smb2.supports_multicredit) {
+	if (xconn->smb2.credits.multicredit) {
 		credit_charge = SVAL(inhdr, SMB2_HDR_CREDIT_CHARGE);
 		credit_charge = MAX(credit_charge, 1);
 	}
@@ -798,9 +804,9 @@ static void smb2_set_operation_credit(struct smbd_server_connection *sconn,
 	out_flags = IVAL(outhdr, SMB2_HDR_FLAGS);
 	out_status = NT_STATUS(IVAL(outhdr, SMB2_HDR_STATUS));
 
-	SMB_ASSERT(sconn->smb2.max_credits >= sconn->smb2.credits_granted);
+	SMB_ASSERT(xconn->smb2.credits.max >= xconn->smb2.credits.granted);
 
-	if (sconn->smb2.max_credits < credit_charge) {
+	if (xconn->smb2.credits.max < credit_charge) {
 		smbd_server_connection_terminate(sconn,
 			"client error: credit charge > max credits\n");
 		return;
@@ -842,7 +848,7 @@ static void smb2_set_operation_credit(struct smbd_server_connection *sconn,
 		additional_credits = MIN(additional_credits, additional_max);
 
 		credits_granted = credit_charge + additional_credits;
-	} else if (sconn->smb2.credits_granted == 0) {
+	} else if (xconn->smb2.credits.granted == 0) {
 		/*
 		 * Make sure the client has always at least one credit
 		 */
@@ -864,19 +870,19 @@ static void smb2_set_operation_credit(struct smbd_server_connection *sconn,
 	 *    this makes sure the client consumes the lowest sequence
 	 *    number, before we can grant additional credits.
 	 */
-	credits_possible = UINT64_MAX - sconn->smb2.seqnum_low;
+	credits_possible = UINT64_MAX - xconn->smb2.credits.seq_low;
 	if (credits_possible > 0) {
 		/* remove UINT64_MAX */
 		credits_possible -= 1;
 	}
 	credits_possible = MIN(credits_possible, current_max_credits);
-	credits_possible -= sconn->smb2.seqnum_range;
+	credits_possible -= xconn->smb2.credits.seq_range;
 
 	credits_granted = MIN(credits_granted, credits_possible);
 
 	SSVAL(outhdr, SMB2_HDR_CREDIT, credits_granted);
-	sconn->smb2.credits_granted += credits_granted;
-	sconn->smb2.seqnum_range += credits_granted;
+	xconn->smb2.credits.granted += credits_granted;
+	xconn->smb2.credits.seq_range += credits_granted;
 
 	DEBUG(10,("smb2_set_operation_credit: requested %u, charge %u, "
 		"granted %u, current possible/max %u/%u, "
@@ -886,10 +892,10 @@ static void smb2_set_operation_credit(struct smbd_server_connection *sconn,
 		(unsigned int)credits_granted,
 		(unsigned int)credits_possible,
 		(unsigned int)current_max_credits,
-		(unsigned int)sconn->smb2.credits_granted,
-		(unsigned int)sconn->smb2.max_credits,
-		(unsigned long long)sconn->smb2.seqnum_low,
-		(unsigned int)sconn->smb2.seqnum_range));
+		(unsigned int)xconn->smb2.credits.granted,
+		(unsigned int)xconn->smb2.credits.max,
+		(unsigned long long)xconn->smb2.credits.seq_low,
+		(unsigned int)xconn->smb2.credits.seq_range));
 }
 
 static void smb2_calculate_credits(const struct smbd_smb2_request *inreq,
@@ -1805,13 +1811,14 @@ static NTSTATUS smbd_smb2_request_check_session(struct smbd_smb2_request *req)
 NTSTATUS smbd_smb2_request_verify_creditcharge(struct smbd_smb2_request *req,
 						uint32_t data_length)
 {
+	struct smbXsrv_connection *xconn = req->sconn->conn;
 	uint16_t needed_charge;
 	uint16_t credit_charge = 1;
 	const uint8_t *inhdr;
 
 	inhdr = SMBD_SMB2_IN_HDR_PTR(req);
 
-	if (req->sconn->smb2.supports_multicredit) {
+	if (xconn->smb2.credits.multicredit) {
 		credit_charge = SVAL(inhdr, SMB2_HDR_CREDIT_CHARGE);
 		credit_charge = MAX(credit_charge, 1);
 	}
@@ -2970,7 +2977,7 @@ static NTSTATUS smbd_smb2_request_next_incoming(struct smbd_server_connection *s
 		return NT_STATUS_OK;
 	}
 
-	max_send_queue_len = MAX(1, sconn->smb2.max_credits/16);
+	max_send_queue_len = MAX(1, xconn->smb2.credits.max/16);
 	cur_send_queue_len = xconn->smb2.send_queue_len;
 
 	if (cur_send_queue_len > max_send_queue_len) {
-- 
1.8.1.2


From 898083697ffa43c0aa96eb21cf6ee3300c41523a Mon Sep 17 00:00:00 2001
From: Stefan Metzmacher <metze@samba.org>
Date: Tue, 10 Jun 2014 13:24:50 +0200
Subject: [PATCH 30/94] s3:smb2_server: move sconn->smb2.requests to
 xconn->smb2.requests

This prepares the structures for multi-channel support.

Signed-off-by: Stefan Metzmacher <metze@samba.org>
Reviewed-by: Volker Lendecke <vl@samba.org>
---
 source3/smbd/globals.h        |  3 ++-
 source3/smbd/smb2_create.c    |  3 ++-
 source3/smbd/smb2_glue.c      |  3 ++-
 source3/smbd/smb2_lock.c      |  9 ++++++---
 source3/smbd/smb2_server.c    | 10 ++++++----
 source3/smbd/smb2_sesssetup.c |  7 +++++--
 source3/smbd/smb2_tcon.c      |  3 ++-
 7 files changed, 25 insertions(+), 13 deletions(-)

diff --git a/source3/smbd/globals.h b/source3/smbd/globals.h
index 28b4588..d1877a1 100644
--- a/source3/smbd/globals.h
+++ b/source3/smbd/globals.h
@@ -497,6 +497,8 @@ struct smbXsrv_connection {
 			uint32_t max_read;
 			uint32_t max_write;
 		} server;
+
+		struct smbd_smb2_request *requests;
 	} smb2;
 
 	uint64_t smbd_idle_profstamp;
@@ -841,7 +843,6 @@ struct smbd_server_connection {
 			struct tevent_timer *brl_timeout;
 			bool blocking_lock_unlock_state;
 		} locks;
-		struct smbd_smb2_request *requests;
 	} smb2;
 
 	/*
diff --git a/source3/smbd/smb2_create.c b/source3/smbd/smb2_create.c
index d22df4d..0c27a01 100644
--- a/source3/smbd/smb2_create.c
+++ b/source3/smbd/smb2_create.c
@@ -1213,9 +1213,10 @@ bool get_deferred_open_message_state_smb2(struct smbd_smb2_request *smb2req,
 static struct smbd_smb2_request *find_open_smb2req(
 	struct smbd_server_connection *sconn, uint64_t mid)
 {
+	struct smbXsrv_connection *xconn = sconn->conn;
 	struct smbd_smb2_request *smb2req;
 
-	for (smb2req = sconn->smb2.requests; smb2req; smb2req = smb2req->next) {
+	for (smb2req = xconn->smb2.requests; smb2req; smb2req = smb2req->next) {
 		uint64_t message_id;
 		if (smb2req->subreq == NULL) {
 			/* This message has been processed. */
diff --git a/source3/smbd/smb2_glue.c b/source3/smbd/smb2_glue.c
index 54135b5..69ff7bb 100644
--- a/source3/smbd/smb2_glue.c
+++ b/source3/smbd/smb2_glue.c
@@ -77,9 +77,10 @@ size_t smbd_smb2_unread_bytes(struct smbd_smb2_request *req)
 void remove_smb2_chained_fsp(files_struct *fsp)
 {
 	struct smbd_server_connection *sconn = fsp->conn->sconn;
+	struct smbXsrv_connection *xconn = sconn->conn;
 	struct smbd_smb2_request *smb2req;
 
-	for (smb2req = sconn->smb2.requests; smb2req; smb2req = smb2req->next) {
+	for (smb2req = xconn->smb2.requests; smb2req; smb2req = smb2req->next) {
 		if (smb2req->compat_chain_fsp == fsp) {
 			smb2req->compat_chain_fsp = NULL;
 		}
diff --git a/source3/smbd/smb2_lock.c b/source3/smbd/smb2_lock.c
index 4c5b61e..8e1eb40 100644
--- a/source3/smbd/smb2_lock.c
+++ b/source3/smbd/smb2_lock.c
@@ -468,13 +468,14 @@ struct blocking_lock_record *get_pending_smb2req_blr(struct smbd_smb2_request *s
 
 static bool recalc_smb2_brl_timeout(struct smbd_server_connection *sconn)
 {
+	struct smbXsrv_connection *xconn = sconn->conn;
 	struct smbd_smb2_request *smb2req;
 	struct timeval next_timeout = timeval_zero();
 	int max_brl_timeout = lp_parm_int(-1, "brl", "recalctime", 5);
 
 	TALLOC_FREE(sconn->smb2.locks.brl_timeout);
 
-	for (smb2req = sconn->smb2.requests; smb2req; smb2req = smb2req->next) {
+	for (smb2req = xconn->smb2.requests; smb2req; smb2req = smb2req->next) {
 		struct blocking_lock_record *blr =
 			get_pending_smb2req_blr(smb2req);
 		if (!blr) {
@@ -774,9 +775,10 @@ static void reprocess_blocked_smb2_lock(struct smbd_smb2_request *smb2req,
 void process_blocking_lock_queue_smb2(
 	struct smbd_server_connection *sconn, struct timeval tv_curr)
 {
+	struct smbXsrv_connection *xconn = sconn->conn;
 	struct smbd_smb2_request *smb2req, *nextreq;
 
-	for (smb2req = sconn->smb2.requests; smb2req; smb2req = nextreq) {
+	for (smb2req = xconn->smb2.requests; smb2req; smb2req = nextreq) {
 		const uint8_t *inhdr;
 
 		nextreq = smb2req->next;
@@ -808,9 +810,10 @@ void cancel_pending_lock_requests_by_fid_smb2(files_struct *fsp,
 			enum file_close_type close_type)
 {
 	struct smbd_server_connection *sconn = fsp->conn->sconn;
+	struct smbXsrv_connection *xconn = sconn->conn;
 	struct smbd_smb2_request *smb2req, *nextreq;
 
-	for (smb2req = sconn->smb2.requests; smb2req; smb2req = nextreq) {
+	for (smb2req = xconn->smb2.requests; smb2req; smb2req = nextreq) {
 		struct smbd_smb2_lock_state *state = NULL;
 		files_struct *fsp_curr = NULL;
 		struct blocking_lock_record *blr = NULL;
diff --git a/source3/smbd/smb2_server.c b/source3/smbd/smb2_server.c
index b48d181..b963b94 100644
--- a/source3/smbd/smb2_server.c
+++ b/source3/smbd/smb2_server.c
@@ -938,6 +938,7 @@ DATA_BLOB smbd_smb2_generate_outbody(struct smbd_smb2_request *req, size_t size)
 
 static NTSTATUS smbd_smb2_request_setup_out(struct smbd_smb2_request *req)
 {
+	struct smbXsrv_connection *xconn = req->sconn->conn;
 	TALLOC_CTX *mem_ctx;
 	struct iovec *vector;
 	int count;
@@ -1035,7 +1036,7 @@ static NTSTATUS smbd_smb2_request_setup_out(struct smbd_smb2_request *req)
 	/* setup the length of the NBT packet */
 	smb2_setup_nbt_length(req->out.vector, req->out.vector_count);
 
-	DLIST_ADD_END(req->sconn->smb2.requests, req, struct smbd_smb2_request *);
+	DLIST_ADD_END(xconn->smb2.requests, req, struct smbd_smb2_request *);
 
 	return NT_STATUS_OK;
 }
@@ -1613,6 +1614,7 @@ static void smbd_smb2_request_pending_timer(struct tevent_context *ev,
 static NTSTATUS smbd_smb2_request_process_cancel(struct smbd_smb2_request *req)
 {
 	struct smbd_server_connection *sconn = req->sconn;
+	struct smbXsrv_connection *xconn = sconn->conn;
 	struct smbd_smb2_request *cur;
 	const uint8_t *inhdr;
 	uint32_t flags;
@@ -1630,10 +1632,10 @@ static NTSTATUS smbd_smb2_request_process_cancel(struct smbd_smb2_request *req)
 	 * we don't need the request anymore
 	 * cancel requests never have a response
 	 */
-	DLIST_REMOVE(req->sconn->smb2.requests, req);
+	DLIST_REMOVE(xconn->smb2.requests, req);
 	TALLOC_FREE(req);
 
-	for (cur = sconn->smb2.requests; cur; cur = cur->next) {
+	for (cur = xconn->smb2.requests; cur; cur = cur->next) {
 		const uint8_t *outhdr;
 		uint64_t message_id;
 		uint64_t async_id;
@@ -2511,7 +2513,7 @@ static NTSTATUS smbd_smb2_request_reply(struct smbd_smb2_request *req)
 	 * We're done with this request -
 	 * move it off the "being processed" queue.
 	 */
-	DLIST_REMOVE(req->sconn->smb2.requests, req);
+	DLIST_REMOVE(conn->smb2.requests, req);
 
 	req->queue_entry.mem_ctx = req;
 	req->queue_entry.vector = req->out.vector;
diff --git a/source3/smbd/smb2_sesssetup.c b/source3/smbd/smb2_sesssetup.c
index 391b100..d4f0d79 100644
--- a/source3/smbd/smb2_sesssetup.c
+++ b/source3/smbd/smb2_sesssetup.c
@@ -457,6 +457,7 @@ static int pp_self_ref_destructor(struct smbd_smb2_session_setup_state **pp_stat
 
 static int smbd_smb2_session_setup_state_destructor(struct smbd_smb2_session_setup_state *state)
 {
+	struct smbXsrv_connection *xconn;
 	struct smbd_smb2_request *preq;
 
 	/*
@@ -478,8 +479,9 @@ static int smbd_smb2_session_setup_state_destructor(struct smbd_smb2_session_set
 	 * Ensure that any outstanding requests don't also refer
 	 * to it.
 	 */
+	xconn = state->smb2req->sconn->conn;
 
-	for (preq = state->smb2req->sconn->smb2.requests; preq != NULL; preq = preq->next) {
+	for (preq = xconn->smb2.requests; preq != NULL; preq = preq->next) {
 		if (preq == state->smb2req) {
 			continue;
 		}
@@ -875,6 +877,7 @@ static struct tevent_req *smbd_smb2_logoff_send(TALLOC_CTX *mem_ctx,
 	struct smbd_smb2_logout_state *state;
 	struct tevent_req *subreq;
 	struct smbd_smb2_request *preq;
+	struct smbXsrv_connection *xconn = smb2req->sconn->conn;
 
 	req = tevent_req_create(mem_ctx, &state,
 			struct smbd_smb2_logout_state);
@@ -893,7 +896,7 @@ static struct tevent_req *smbd_smb2_logoff_send(TALLOC_CTX *mem_ctx,
 	 */
 	smb2req->session->status = NT_STATUS_USER_SESSION_DELETED;
 
-	for (preq = smb2req->sconn->smb2.requests; preq != NULL; preq = preq->next) {
+	for (preq = xconn->smb2.requests; preq != NULL; preq = preq->next) {
 		if (preq == smb2req) {
 			/* Can't cancel current request. */
 			continue;
diff --git a/source3/smbd/smb2_tcon.c b/source3/smbd/smb2_tcon.c
index 93f62fd..6289427 100644
--- a/source3/smbd/smb2_tcon.c
+++ b/source3/smbd/smb2_tcon.c
@@ -498,6 +498,7 @@ static struct tevent_req *smbd_smb2_tdis_send(TALLOC_CTX *mem_ctx,
 	struct smbd_smb2_tdis_state *state;
 	struct tevent_req *subreq;
 	struct smbd_smb2_request *preq;
+	struct smbXsrv_connection *xconn = smb2req->sconn->conn;
 
 	req = tevent_req_create(mem_ctx, &state,
 			struct smbd_smb2_tdis_state);
@@ -516,7 +517,7 @@ static struct tevent_req *smbd_smb2_tdis_send(TALLOC_CTX *mem_ctx,
 	 */
 	smb2req->tcon->status = NT_STATUS_NETWORK_NAME_DELETED;
 
-	for (preq = smb2req->sconn->smb2.requests; preq != NULL; preq = preq->next) {
+	for (preq = xconn->smb2.requests; preq != NULL; preq = preq->next) {
 		if (preq == smb2req) {
 			/* Can't cancel current request. */
 			continue;
-- 
1.8.1.2


From 8163254c662b8b9d8969a1dac5a4a98c6a9559ac Mon Sep 17 00:00:00 2001
From: Stefan Metzmacher <metze@samba.org>
Date: Tue, 10 Jun 2014 13:34:55 +0200
Subject: [PATCH 31/94] TODO/REVIEW s3:smb2_server: talloc smbd_smb2_request as
 child of smbXsrv_connection

This prepares the structures for multi-channel support.
Each request needs to response on the same connection,
where it arrived.

Signed-off-by: Stefan Metzmacher <metze@samba.org>
Reviewed-by: Volker Lendecke <vl@samba.org>

This gets my R-B for the code, but to me the commit message is a bit
unclear. Directing responses does not directly have to do with the
talloc hierarchy, or do we want to play with talloc_parent() searching
for the smbXsrv_connection? Maybe this is answered in later commits,
but reading this commit message in isolation is a bit misleading.
---
 source3/smbd/smb2_server.c | 8 +++++---
 1 file changed, 5 insertions(+), 3 deletions(-)

diff --git a/source3/smbd/smb2_server.c b/source3/smbd/smb2_server.c
index b963b94..16c6fd4 100644
--- a/source3/smbd/smb2_server.c
+++ b/source3/smbd/smb2_server.c
@@ -511,6 +511,7 @@ static NTSTATUS smbd_smb2_request_create(struct smbd_server_connection *sconn,
 					 uint8_t *inbuf, size_t size,
 					 struct smbd_smb2_request **_req)
 {
+	struct smbXsrv_connection *xconn = sconn->conn;
 	struct smbd_smb2_request *req;
 	uint32_t protocol_version;
 	const uint8_t *inhdr = NULL;
@@ -547,7 +548,7 @@ static NTSTATUS smbd_smb2_request_create(struct smbd_server_connection *sconn,
 		return NT_STATUS_INVALID_PARAMETER;
 	}
 
-	req = smbd_smb2_request_allocate(sconn);
+	req = smbd_smb2_request_allocate(xconn);
 	if (req == NULL) {
 		return NT_STATUS_NO_MEMORY;
 	}
@@ -1151,12 +1152,13 @@ static bool dup_smb2_vec4(TALLOC_CTX *ctx,
 
 static struct smbd_smb2_request *dup_smb2_req(const struct smbd_smb2_request *req)
 {
+	struct smbXsrv_connection *xconn = req->sconn->conn;
 	struct smbd_smb2_request *newreq = NULL;
 	struct iovec *outvec = NULL;
 	int count = req->out.vector_count;
 	int i;
 
-	newreq = smbd_smb2_request_allocate(req->sconn);
+	newreq = smbd_smb2_request_allocate(xconn);
 	if (!newreq) {
 		return NULL;
 	}
@@ -2993,7 +2995,7 @@ static NTSTATUS smbd_smb2_request_next_incoming(struct smbd_server_connection *s
 
 	/* ask for the next request */
 	ZERO_STRUCTP(state);
-	state->req = smbd_smb2_request_allocate(sconn);
+	state->req = smbd_smb2_request_allocate(xconn);
 	if (state->req == NULL) {
 		return NT_STATUS_NO_MEMORY;
 	}
-- 
1.8.1.2


From 420714cb72dd12727b1f944e43b17766e6cdafd4 Mon Sep 17 00:00:00 2001
From: Stefan Metzmacher <metze@samba.org>
Date: Tue, 10 Jun 2014 13:34:55 +0200
Subject: [PATCH 32/94] TODO/REVIEW s3:smb2_server: remember smbXsrv_connection
 for each smbd_smb2_request

This prepares the structures for multi-channel support.
Each request needs to response on the same connection,
where it arrived.

Signed-off-by: Stefan Metzmacher <metze@samba.org>
Reviewed-by: Volker Lendecke <vl@samba.org>

Ah, here we are. This commit message matches the content more. Can
we focus the last one more on the talloc hierarchy and possible later
tear-down procedures via talloc_free()? Feel free to push when that's in.
---
 source3/smbd/globals.h     | 1 +
 source3/smbd/smb2_server.c | 6 ++++--
 2 files changed, 5 insertions(+), 2 deletions(-)

diff --git a/source3/smbd/globals.h b/source3/smbd/globals.h
index d1877a1..b94ba29 100644
--- a/source3/smbd/globals.h
+++ b/source3/smbd/globals.h
@@ -624,6 +624,7 @@ struct smbd_smb2_request {
 	struct smbd_smb2_request *prev, *next;
 
 	struct smbd_server_connection *sconn;
+	struct smbXsrv_connection *xconn;
 
 	struct smbd_smb2_send_queue queue_entry;
 
diff --git a/source3/smbd/smb2_server.c b/source3/smbd/smb2_server.c
index 16c6fd4..7a0555d 100644
--- a/source3/smbd/smb2_server.c
+++ b/source3/smbd/smb2_server.c
@@ -553,6 +553,7 @@ static NTSTATUS smbd_smb2_request_create(struct smbd_server_connection *sconn,
 		return NT_STATUS_NO_MEMORY;
 	}
 	req->sconn = sconn;
+	req->xconn = xconn;
 
 	talloc_steal(req, inbuf);
 
@@ -1152,18 +1153,18 @@ static bool dup_smb2_vec4(TALLOC_CTX *ctx,
 
 static struct smbd_smb2_request *dup_smb2_req(const struct smbd_smb2_request *req)
 {
-	struct smbXsrv_connection *xconn = req->sconn->conn;
 	struct smbd_smb2_request *newreq = NULL;
 	struct iovec *outvec = NULL;
 	int count = req->out.vector_count;
 	int i;
 
-	newreq = smbd_smb2_request_allocate(xconn);
+	newreq = smbd_smb2_request_allocate(req->xconn);
 	if (!newreq) {
 		return NULL;
 	}
 
 	newreq->sconn = req->sconn;
+	newreq->xconn = req->xconn;
 	newreq->session = req->session;
 	newreq->do_encryption = req->do_encryption;
 	newreq->do_signing = req->do_signing;
@@ -3000,6 +3001,7 @@ static NTSTATUS smbd_smb2_request_next_incoming(struct smbd_server_connection *s
 		return NT_STATUS_NO_MEMORY;
 	}
 	state->req->sconn = sconn;
+	state->req->xconn = xconn;
 	state->min_recv_size = lp_min_receive_file_size();
 
 	TEVENT_FD_READABLE(xconn->transport.fde);
-- 
1.8.1.2


From 4a0f6a07ac4bbb0ebb62dc5b449a8d60673e97b6 Mon Sep 17 00:00:00 2001
From: Stefan Metzmacher <metze@samba.org>
Date: Tue, 10 Jun 2014 13:34:55 +0200
Subject: [PATCH 33/94] s3:smbd: remember smbXsrv_connection for each
 smb_request

This prepares the structures for multi-channel support.

Signed-off-by: Stefan Metzmacher <metze@samba.org>
Reviewed-by: Volker Lendecke <vl@samba.org>
---
 source3/include/vfs.h    |  1 +
 source3/smbd/process.c   | 17 +++++++++++++----
 source3/smbd/smb2_glue.c |  1 +
 3 files changed, 15 insertions(+), 4 deletions(-)

diff --git a/source3/include/vfs.h b/source3/include/vfs.h
index 474b1e3..6c0bf70 100644
--- a/source3/include/vfs.h
+++ b/source3/include/vfs.h
@@ -403,6 +403,7 @@ struct smb_request {
 	bool encrypted;
 	connection_struct *conn;
 	struct smbd_server_connection *sconn;
+	struct smbXsrv_connection *xconn;
 	struct smb_perfcount_data pcd;
 
 	/*
diff --git a/source3/smbd/process.c b/source3/smbd/process.c
index daf1947..63e997d 100644
--- a/source3/smbd/process.c
+++ b/source3/smbd/process.c
@@ -591,6 +591,7 @@ static NTSTATUS receive_smb_talloc(TALLOC_CTX *mem_ctx,
 
 static bool init_smb_request(struct smb_request *req,
 			     struct smbd_server_connection *sconn,
+			     struct smbXsrv_connection *xconn,
 			     const uint8 *inbuf,
 			     size_t unread_bytes, bool encrypted,
 			     uint32_t seqnum)
@@ -624,7 +625,8 @@ static bool init_smb_request(struct smb_request *req,
 	req->unread_bytes = unread_bytes;
 	req->encrypted = encrypted;
 	req->sconn = sconn;
-	status = smb1srv_tcon_lookup(sconn->conn, req->tid, now, &tcon);
+	req->xconn = xconn;
+	status = smb1srv_tcon_lookup(xconn, req->tid, now, &tcon);
 	if (NT_STATUS_IS_OK(status)) {
 		req->conn = tcon->compat;
 	} else {
@@ -1641,6 +1643,7 @@ static void construct_reply(struct smbd_server_connection *sconn,
 			    uint32_t seqnum, bool encrypted,
 			    struct smb_perfcount_data *deferred_pcd)
 {
+	struct smbXsrv_connection *xconn = sconn->conn;
 	connection_struct *conn;
 	struct smb_request *req;
 
@@ -1648,7 +1651,7 @@ static void construct_reply(struct smbd_server_connection *sconn,
 		smb_panic("could not allocate smb_request");
 	}
 
-	if (!init_smb_request(req, sconn, (uint8 *)inbuf, unread_bytes,
+	if (!init_smb_request(req, sconn, xconn, (uint8 *)inbuf, unread_bytes,
 			      encrypted, seqnum)) {
 		exit_server_cleanly("Invalid SMB request");
 	}
@@ -2338,6 +2341,7 @@ struct smb1_parse_chain_state {
 	TALLOC_CTX *mem_ctx;
 	const uint8_t *buf;
 	struct smbd_server_connection *sconn;
+	struct smbXsrv_connection *xconn;
 	bool encrypted;
 	uint32_t seqnum;
 
@@ -2368,7 +2372,7 @@ static bool smb1_parse_chain_cb(uint8_t cmd,
 		return false;
 	}
 
-	ok = init_smb_request(req, state->sconn, state->buf, 0,
+	ok = init_smb_request(req, state->sconn, state->xconn, state->buf, 0,
 			      state->encrypted, state->seqnum);
 	if (!ok) {
 		return false;
@@ -2389,12 +2393,14 @@ bool smb1_parse_chain(TALLOC_CTX *mem_ctx, const uint8_t *buf,
 		      bool encrypted, uint32_t seqnum,
 		      struct smb_request ***reqs, unsigned *num_reqs)
 {
+	struct smbXsrv_connection *xconn = sconn->conn;
 	struct smb1_parse_chain_state state;
 	unsigned i;
 
 	state.mem_ctx = mem_ctx;
 	state.buf = buf;
 	state.sconn = sconn;
+	state.xconn = xconn;
 	state.encrypted = encrypted;
 	state.seqnum = seqnum;
 	state.reqs = NULL;
@@ -2953,6 +2959,7 @@ struct smbd_echo_state {
 	struct tevent_context *ev;
 	struct iovec *pending;
 	struct smbd_server_connection *sconn;
+	struct smbXsrv_connection *xconn;
 	int parent_pipe;
 
 	struct tevent_fd *parent_fde;
@@ -3034,7 +3041,7 @@ static bool smbd_echo_reply(struct smbd_echo_state *state,
 		return false;
 	}
 
-	if (!init_smb_request(&req, state->sconn, inbuf, 0, false,
+	if (!init_smb_request(&req, state->sconn, state->xconn, inbuf, 0, false,
 			      seqnum)) {
 		return false;
 	}
@@ -3095,6 +3102,7 @@ static void smbd_echo_got_packet(struct tevent_req *req);
 static void smbd_echo_loop(struct smbd_server_connection *sconn,
 			   int parent_pipe)
 {
+	struct smbXsrv_connection *xconn = sconn->conn;
 	struct smbd_echo_state *state;
 	struct tevent_req *read_req;
 
@@ -3104,6 +3112,7 @@ static void smbd_echo_loop(struct smbd_server_connection *sconn,
 		return;
 	}
 	state->sconn = sconn;
+	state->xconn = xconn;
 	state->parent_pipe = parent_pipe;
 	state->ev = s3_tevent_context_init(state);
 	if (state->ev == NULL) {
diff --git a/source3/smbd/smb2_glue.c b/source3/smbd/smb2_glue.c
index 69ff7bb..1d1fd89 100644
--- a/source3/smbd/smb2_glue.c
+++ b/source3/smbd/smb2_glue.c
@@ -42,6 +42,7 @@ struct smb_request *smbd_smb2_fake_smb_request(struct smbd_smb2_request *req)
 	smbreq->tid = req->tcon->compat->cnum;
 	smbreq->conn = req->tcon->compat;
 	smbreq->sconn = req->sconn;
+	smbreq->xconn = req->xconn;
 	smbreq->smbpid = (uint16_t)IVAL(inhdr, SMB2_HDR_PID);
 	smbreq->flags2 = FLAGS2_UNICODE_STRINGS |
 			 FLAGS2_32_BIT_ERROR_CODES |
-- 
1.8.1.2


From 9eb09b72750856122bfd22471070490ad9f0d550 Mon Sep 17 00:00:00 2001
From: Stefan Metzmacher <metze@samba.org>
Date: Tue, 10 Jun 2014 15:05:41 +0200
Subject: [PATCH 34/94] s3:smbd: rename 'conn' to 'xconn' in smbd_process()

Signed-off-by: Stefan Metzmacher <metze@samba.org>
Reviewed-by: Volker Lendecke <vl@samba.org>
---
 source3/smbd/process.c | 46 +++++++++++++++++++++++-----------------------
 1 file changed, 23 insertions(+), 23 deletions(-)

diff --git a/source3/smbd/process.c b/source3/smbd/process.c
index 63e997d..d249898 100644
--- a/source3/smbd/process.c
+++ b/source3/smbd/process.c
@@ -3546,7 +3546,7 @@ void smbd_process(struct tevent_context *ev_ctx,
 		  bool interactive)
 {
 	TALLOC_CTX *frame = talloc_stackframe();
-	struct smbXsrv_connection *conn;
+	struct smbXsrv_connection *xconn;
 	struct smbd_server_connection *sconn;
 	struct sockaddr_storage ss_srv;
 	void *sp_srv = (void *)&ss_srv;
@@ -3563,29 +3563,29 @@ void smbd_process(struct tevent_context *ev_ctx,
 	int ret;
 	int tmp;
 
-	conn = talloc_zero(ev_ctx, struct smbXsrv_connection);
-	if (conn == NULL) {
+	xconn = talloc_zero(ev_ctx, struct smbXsrv_connection);
+	if (xconn == NULL) {
 		DEBUG(0,("talloc_zero(struct smbXsrv_connection)\n"));
 		exit_server_cleanly("talloc_zero(struct smbXsrv_connection).\n");
 	}
 
-	conn->ev_ctx = ev_ctx;
-	conn->msg_ctx = msg_ctx;
-	conn->transport.sock = sock_fd;
-	smbd_echo_init(conn);
+	xconn->ev_ctx = ev_ctx;
+	xconn->msg_ctx = msg_ctx;
+	xconn->transport.sock = sock_fd;
+	smbd_echo_init(xconn);
 
-	sconn = talloc_zero(conn, struct smbd_server_connection);
+	sconn = talloc_zero(xconn, struct smbd_server_connection);
 	if (!sconn) {
 		exit_server("failed to create smbd_server_connection");
 	}
 
-	conn->sconn = sconn;
-	sconn->conn = conn;
+	xconn->sconn = sconn;
+	sconn->conn = xconn;
 
 	/*
 	 * TODO: remove this...:-)
 	 */
-	global_smbXsrv_connection = conn;
+	global_smbXsrv_connection = xconn;
 
 	sconn->ev_ctx = ev_ctx;
 	sconn->msg_ctx = msg_ctx;
@@ -3753,7 +3753,7 @@ void smbd_process(struct tevent_context *ev_ctx,
 		DEBUG(0,("Changed root to %s\n", lp_root_directory(talloc_tos())));
 	}
 
-	if (!srv_init_signing(conn)) {
+	if (!srv_init_signing(xconn)) {
 		exit_server("Failed to init smb_signing");
 	}
 
@@ -3843,22 +3843,22 @@ void smbd_process(struct tevent_context *ev_ctx,
 	tmp = MAX(tmp, SMB_BUFFER_SIZE_MIN);
 	tmp = MIN(tmp, SMB_BUFFER_SIZE_MAX);
 
-	conn->smb1.negprot.max_recv = tmp;
+	xconn->smb1.negprot.max_recv = tmp;
 
-	conn->smb1.sessions.done_sesssetup = false;
-	conn->smb1.sessions.max_send = SMB_BUFFER_SIZE_MAX;
+	xconn->smb1.sessions.done_sesssetup = false;
+	xconn->smb1.sessions.max_send = SMB_BUFFER_SIZE_MAX;
 
 	if (!init_dptrs(sconn)) {
 		exit_server("init_dptrs() failed");
 	}
 
-	conn->transport.fde = tevent_add_fd(ev_ctx,
-					    sconn,
-					    sock_fd,
-					    TEVENT_FD_READ,
-					    smbd_server_connection_handler,
-					    sconn);
-	if (!conn->transport.fde) {
+	xconn->transport.fde = tevent_add_fd(ev_ctx,
+					     sconn,
+					     sock_fd,
+					     TEVENT_FD_READ,
+					     smbd_server_connection_handler,
+					     sconn);
+	if (!xconn->transport.fde) {
 		exit_server("failed to create smbd_server_connection fde");
 	}
 
@@ -3869,7 +3869,7 @@ void smbd_process(struct tevent_context *ev_ctx,
 
 	TALLOC_FREE(frame);
 
-	tevent_set_trace_callback(ev_ctx, smbd_tevent_trace_callback, conn);
+	tevent_set_trace_callback(ev_ctx, smbd_tevent_trace_callback, xconn);
 
 	while (True) {
 		frame = talloc_stackframe_pool(8192);
-- 
1.8.1.2


From 606d0bb3d288572c53b40513fd585356f0238cba Mon Sep 17 00:00:00 2001
From: Stefan Metzmacher <metze@samba.org>
Date: Tue, 10 Jun 2014 15:05:41 +0200
Subject: [PATCH 35/94] s3:smbd: rename 'conn' to 'xconn' in
 exit_server_common()

Signed-off-by: Stefan Metzmacher <metze@samba.org>
Reviewed-by: Volker Lendecke <vl@samba.org>
---
 source3/smbd/server_exit.c | 20 ++++++++++----------
 1 file changed, 10 insertions(+), 10 deletions(-)

diff --git a/source3/smbd/server_exit.c b/source3/smbd/server_exit.c
index 077b81a..c708cb6 100644
--- a/source3/smbd/server_exit.c
+++ b/source3/smbd/server_exit.c
@@ -85,12 +85,12 @@ static void exit_server_common(enum server_exit_reason how,
 static void exit_server_common(enum server_exit_reason how,
 	const char *reason)
 {
-	struct smbXsrv_connection *conn = global_smbXsrv_connection;
+	struct smbXsrv_connection *xconn = global_smbXsrv_connection;
 	struct smbd_server_connection *sconn = NULL;
 	struct messaging_context *msg_ctx = server_messaging_context();
 
-	if (conn != NULL) {
-		sconn = conn->sconn;
+	if (xconn != NULL) {
+		sconn = xconn->sconn;
 	}
 
 	if (!exit_firsttime)
@@ -102,18 +102,18 @@ static void exit_server_common(enum server_exit_reason how,
 	if (sconn) {
 		NTSTATUS status;
 
-		if (NT_STATUS_IS_OK(conn->transport.status)) {
+		if (NT_STATUS_IS_OK(xconn->transport.status)) {
 			switch (how) {
 			case SERVER_EXIT_ABNORMAL:
-				conn->transport.status = NT_STATUS_INTERNAL_ERROR;
+				xconn->transport.status = NT_STATUS_INTERNAL_ERROR;
 				break;
 			case SERVER_EXIT_NORMAL:
-				conn->transport.status = NT_STATUS_LOCAL_DISCONNECT;
+				xconn->transport.status = NT_STATUS_LOCAL_DISCONNECT;
 				break;
 			}
 		}
 
-		TALLOC_FREE(conn->smb1.negprot.auth_context);
+		TALLOC_FREE(xconn->smb1.negprot.auth_context);
 
 		if (lp_log_writeable_files_on_exit()) {
 			bool found = false;
@@ -124,7 +124,7 @@ static void exit_server_common(enum server_exit_reason how,
 		 * Note: this is a no-op for smb2 as
 		 * conn->tcon_table is empty
 		 */
-		status = smb1srv_tcon_disconnect_all(conn);
+		status = smb1srv_tcon_disconnect_all(xconn);
 		if (!NT_STATUS_IS_OK(status)) {
 			DEBUG(0,("Server exit (%s)\n",
 				(reason ? reason : "normal exit")));
@@ -135,7 +135,7 @@ static void exit_server_common(enum server_exit_reason how,
 			reason = "smb1srv_tcon_disconnect_all failed";
 		}
 
-		status = smbXsrv_session_logoff_all(conn);
+		status = smbXsrv_session_logoff_all(xconn);
 		if (!NT_STATUS_IS_OK(status)) {
 			DEBUG(0,("Server exit (%s)\n",
 				(reason ? reason : "normal exit")));
@@ -201,7 +201,7 @@ static void exit_server_common(enum server_exit_reason how,
 	 * because smbd_msg_ctx is not a talloc child of smbd_server_conn.
 	 */
 	sconn = NULL;
-	conn = NULL;
+	xconn = NULL;
 	TALLOC_FREE(global_smbXsrv_connection);
 	server_messaging_context_free();
 	server_event_context_free();
-- 
1.8.1.2


From 52b65a6b54a35ff13c3551e4cca5aabdcf351dd5 Mon Sep 17 00:00:00 2001
From: Stefan Metzmacher <metze@samba.org>
Date: Wed, 11 Jun 2014 03:06:00 +0200
Subject: [PATCH 36/94] s3:smbd: pass smbXsrv_connection to smbd_register_ips()

Signed-off-by: Stefan Metzmacher <metze@samba.org>
Reviewed-by: Volker Lendecke <vl@samba.org>
---
 source3/smbd/process.c | 19 ++++++++++---------
 1 file changed, 10 insertions(+), 9 deletions(-)

diff --git a/source3/smbd/process.c b/source3/smbd/process.c
index d249898..b8baa42 100644
--- a/source3/smbd/process.c
+++ b/source3/smbd/process.c
@@ -2575,7 +2575,7 @@ static void smbd_server_echo_handler(struct tevent_context *ev,
 }
 
 struct smbd_release_ip_state {
-	struct smbd_server_connection *sconn;
+	struct smbXsrv_connection *xconn;
 	struct tevent_immediate *im;
 	char addr[INET6_ADDRSTRLEN];
 };
@@ -2587,7 +2587,8 @@ static void smbd_release_ip_immediate(struct tevent_context *ctx,
 	struct smbd_release_ip_state *state =
 		talloc_get_type_abort(private_data,
 		struct smbd_release_ip_state);
-	struct smbXsrv_connection *xconn = state->sconn->conn;
+	struct smbXsrv_connection *xconn = state->xconn;
+	struct smbd_server_connection *sconn = xconn->sconn;
 
 	if (!NT_STATUS_EQUAL(xconn->transport.status, NT_STATUS_ADDRESS_CLOSED)) {
 		/*
@@ -2596,7 +2597,7 @@ static void smbd_release_ip_immediate(struct tevent_context *ctx,
 		return;
 	}
 
-	smbd_server_connection_terminate(state->sconn, "CTDB_SRVID_RELEASE_IP");
+	smbd_server_connection_terminate(sconn, "CTDB_SRVID_RELEASE_IP");
 }
 
 /****************************************************************************
@@ -2607,7 +2608,7 @@ static bool release_ip(const char *ip, void *priv)
 	struct smbd_release_ip_state *state =
 		talloc_get_type_abort(priv,
 		struct smbd_release_ip_state);
-	struct smbXsrv_connection *xconn = state->sconn->conn;
+	struct smbXsrv_connection *xconn = state->xconn;
 	const char *addr = state->addr;
 	const char *p = addr;
 
@@ -2647,7 +2648,7 @@ static bool release_ip(const char *ip, void *priv)
 		 * as we might be called from within ctdbd_migrate(),
 		 * we need to defer our action to the next event loop
 		 */
-		tevent_schedule_immediate(state->im, state->sconn->ev_ctx,
+		tevent_schedule_immediate(state->im, xconn->ev_ctx,
 					  smbd_release_ip_immediate, state);
 
 		/*
@@ -2660,7 +2661,7 @@ static bool release_ip(const char *ip, void *priv)
 	return false;
 }
 
-static NTSTATUS smbd_register_ips(struct smbd_server_connection *sconn,
+static NTSTATUS smbd_register_ips(struct smbXsrv_connection *xconn,
 				  struct sockaddr_storage *srv,
 				  struct sockaddr_storage *clnt)
 {
@@ -2672,11 +2673,11 @@ static NTSTATUS smbd_register_ips(struct smbd_server_connection *sconn,
 		return NT_STATUS_NO_MEMORY;
 	}
 
-	state = talloc_zero(sconn, struct smbd_release_ip_state);
+	state = talloc_zero(xconn, struct smbd_release_ip_state);
 	if (state == NULL) {
 		return NT_STATUS_NO_MEMORY;
 	}
-	state->sconn = sconn;
+	state->xconn = xconn;
 	state->im = tevent_create_immediate(state);
 	if (state->im == NULL) {
 		return NT_STATUS_NO_MEMORY;
@@ -3832,7 +3833,7 @@ void smbd_process(struct tevent_context *ev_ctx,
 		 */
 		NTSTATUS status;
 
-		status = smbd_register_ips(sconn, &ss_srv, &ss_clnt);
+		status = smbd_register_ips(xconn, &ss_srv, &ss_clnt);
 		if (!NT_STATUS_IS_OK(status)) {
 			DEBUG(0, ("ctdbd_register_ips failed: %s\n",
 				  nt_errstr(status)));
-- 
1.8.1.2


From 510da6b8d6f5f0900cace13bc87dc55058457fe5 Mon Sep 17 00:00:00 2001
From: Stefan Metzmacher <metze@samba.org>
Date: Wed, 11 Jun 2014 03:22:53 +0200
Subject: [PATCH 37/94] s3:smbd: use smbXsrv_connection in switch_message()

Signed-off-by: Stefan Metzmacher <metze@samba.org>
Reviewed-by: Volker Lendecke <vl@samba.org>
---
 source3/smbd/process.c | 12 ++++++------
 1 file changed, 6 insertions(+), 6 deletions(-)

diff --git a/source3/smbd/process.c b/source3/smbd/process.c
index b8baa42..c015733 100644
--- a/source3/smbd/process.c
+++ b/source3/smbd/process.c
@@ -1450,7 +1450,7 @@ static connection_struct *switch_message(uint8 type, struct smb_request *req)
 	int flags;
 	uint64_t session_tag;
 	connection_struct *conn = NULL;
-	struct smbd_server_connection *sconn = req->sconn;
+	struct smbXsrv_connection *xconn = req->xconn;
 	NTTIME now = timeval_to_nttime(&req->request_time);
 	struct smbXsrv_session *session = NULL;
 	NTSTATUS status;
@@ -1493,7 +1493,7 @@ static connection_struct *switch_message(uint8 type, struct smb_request *req)
 	 * Note: for now we only check for NT_STATUS_NETWORK_SESSION_EXPIRED
 	 * here, the main check is still in change_to_user()
 	 */
-	status = smb1srv_session_lookup(sconn->conn,
+	status = smb1srv_session_lookup(xconn,
 					session_tag,
 					now,
 					&session);
@@ -1511,10 +1511,10 @@ static connection_struct *switch_message(uint8 type, struct smb_request *req)
 		}
 	}
 
-	if (session_tag != sconn->conn->last_session_id) {
+	if (session_tag != xconn->last_session_id) {
 		struct user_struct *vuser = NULL;
 
-		sconn->conn->last_session_id = session_tag;
+		xconn->last_session_id = session_tag;
 		if (session) {
 			vuser = session->compat;
 		}
@@ -1609,7 +1609,7 @@ static connection_struct *switch_message(uint8 type, struct smb_request *req)
 			return conn;
 		}
 
-		raddr = tsocket_address_inet_addr_string(sconn->remote_address,
+		raddr = tsocket_address_inet_addr_string(xconn->remote_address,
 							 talloc_tos());
 		if (raddr == NULL) {
 			reply_nterror(req, NT_STATUS_NO_MEMORY);
@@ -1621,7 +1621,7 @@ static connection_struct *switch_message(uint8 type, struct smb_request *req)
 		 */
 
 		ok = allow_access(lp_hosts_deny(-1), lp_hosts_allow(-1),
-				  sconn->remote_hostname, raddr);
+				  xconn->remote_hostname, raddr);
 		TALLOC_FREE(raddr);
 
 		if (!ok) {
-- 
1.8.1.2


From 56b4f041e64071cbe75ba1e78ff3a897e1fb52e1 Mon Sep 17 00:00:00 2001
From: Stefan Metzmacher <metze@samba.org>
Date: Wed, 11 Jun 2014 08:34:05 +0200
Subject: [PATCH 38/94] s3:smbd: let construct_reply() also use
 smb_request_done()

This way it matches the code path in construct_reply_chain().

Signed-off-by: Stefan Metzmacher <metze@samba.org>
Reviewed-by: Volker Lendecke <vl@samba.org>
---
 source3/smbd/process.c | 21 ++++++---------------
 1 file changed, 6 insertions(+), 15 deletions(-)

diff --git a/source3/smbd/process.c b/source3/smbd/process.c
index c015733..c1685da 100644
--- a/source3/smbd/process.c
+++ b/source3/smbd/process.c
@@ -1644,7 +1644,6 @@ static void construct_reply(struct smbd_server_connection *sconn,
 			    struct smb_perfcount_data *deferred_pcd)
 {
 	struct smbXsrv_connection *xconn = sconn->conn;
-	connection_struct *conn;
 	struct smb_request *req;
 
 	if (!(req = talloc(talloc_tos(), struct smb_request))) {
@@ -1667,27 +1666,19 @@ static void construct_reply(struct smbd_server_connection *sconn,
 		SMB_PERFCOUNT_SET_MSGLEN_IN(&req->pcd, size);
 	}
 
-	conn = switch_message(req->cmd, req);
+	req->conn = switch_message(req->cmd, req);
 
 	if (req->outbuf == NULL) {
+		/*
+		 * Request has suspended itself, will come
+		 * back here.
+		 */
 		return;
 	}
-
 	if (CVAL(req->outbuf,0) == 0) {
 		show_msg((char *)req->outbuf);
 	}
-
-	if (!srv_send_smb(req->sconn,
-			(char *)req->outbuf,
-			true, req->seqnum+1,
-			IS_CONN_ENCRYPTED(conn)||req->encrypted,
-			&req->pcd)) {
-		exit_server_cleanly("construct_reply: srv_send_smb failed.");
-	}
-
-	TALLOC_FREE(req);
-
-	return;
+	smb_request_done(req);
 }
 
 static void construct_reply_chain(struct smbd_server_connection *sconn,
-- 
1.8.1.2


From 04fbfa18257370c9cb642a87a55bebbb6e0bc5f9 Mon Sep 17 00:00:00 2001
From: Stefan Metzmacher <metze@samba.org>
Date: Wed, 11 Jun 2014 10:40:42 +0200
Subject: [PATCH 39/94] s3:smbd: pass smbXsrv_connection to process_smb()

Signed-off-by: Stefan Metzmacher <metze@samba.org>
Reviewed-by: Volker Lendecke <vl@samba.org>
---
 source3/smbd/process.c | 12 ++++++++----
 1 file changed, 8 insertions(+), 4 deletions(-)

diff --git a/source3/smbd/process.c b/source3/smbd/process.c
index c1685da..e49e357 100644
--- a/source3/smbd/process.c
+++ b/source3/smbd/process.c
@@ -47,6 +47,7 @@ struct pending_message_list {
 	struct pending_message_list *next, *prev;
 	struct timeval request_time; /* When was this first issued? */
 	struct smbd_server_connection *sconn;
+	struct smbXsrv_connection *xconn;
 	struct tevent_timer *te;
 	struct smb_perfcount_data pcd;
 	uint32_t seqnum;
@@ -659,7 +660,7 @@ static bool init_smb_request(struct smb_request *req,
 	return true;
 }
 
-static void process_smb(struct smbd_server_connection *conn,
+static void process_smb(struct smbXsrv_connection *xconn,
 			uint8_t *inbuf, size_t nread, size_t unread_bytes,
 			uint32_t seqnum, bool encrypted,
 			struct smb_perfcount_data *deferred_pcd);
@@ -672,6 +673,7 @@ static void smbd_deferred_open_timer(struct tevent_context *ev,
 	struct pending_message_list *msg = talloc_get_type(private_data,
 					   struct pending_message_list);
 	struct smbd_server_connection *sconn = msg->sconn;
+	struct smbXsrv_connection *xconn = msg->xconn;
 	TALLOC_CTX *mem_ctx = talloc_tos();
 	uint64_t mid = (uint64_t)SVAL(msg->buf.data,smb_mid);
 	uint8_t *inbuf;
@@ -692,7 +694,7 @@ static void smbd_deferred_open_timer(struct tevent_context *ev,
 	 * re-processed in error. */
 	msg->processed = true;
 
-	process_smb(sconn, inbuf,
+	process_smb(xconn, inbuf,
 		    msg->buf.length, 0,
 		    msg->seqnum, msg->encrypted, &msg->pcd);
 
@@ -723,6 +725,7 @@ static bool push_queued_message(struct smb_request *req,
 		return False;
 	}
 	msg->sconn = req->sconn;
+	msg->xconn = req->xconn;
 
 	msg->buf = data_blob_talloc(msg, req->inbuf, msg_len);
 	if(msg->buf.data == NULL) {
@@ -1848,11 +1851,12 @@ error:
 /****************************************************************************
  Process an smb from the client
 ****************************************************************************/
-static void process_smb(struct smbd_server_connection *sconn,
+static void process_smb(struct smbXsrv_connection *xconn,
 			uint8_t *inbuf, size_t nread, size_t unread_bytes,
 			uint32_t seqnum, bool encrypted,
 			struct smb_perfcount_data *deferred_pcd)
 {
+	struct smbd_server_connection *sconn = xconn->sconn;
 	int msg_type = CVAL(inbuf,0);
 
 	DO_PROFILE_INC(smb_count);
@@ -2504,7 +2508,7 @@ static void smbd_server_connection_read_handler(
 	}
 
 process:
-	process_smb(sconn, inbuf, inbuf_len, unread_bytes,
+	process_smb(xconn, inbuf, inbuf_len, unread_bytes,
 		    seqnum, encrypted, NULL);
 }
 
-- 
1.8.1.2


From d27b54b8ff7a6122307a17675c4435de8c558a13 Mon Sep 17 00:00:00 2001
From: Stefan Metzmacher <metze@samba.org>
Date: Tue, 10 Jun 2014 20:20:58 +0200
Subject: [PATCH 40/94] s3:smbd: pass smbXsrv_connection to fd handlers

Signed-off-by: Stefan Metzmacher <metze@samba.org>
Reviewed-by: Volker Lendecke <vl@samba.org>
---
 source3/smbd/process.c | 36 ++++++++++++++++++------------------
 1 file changed, 18 insertions(+), 18 deletions(-)

diff --git a/source3/smbd/process.c b/source3/smbd/process.c
index e49e357..c8d6079 100644
--- a/source3/smbd/process.c
+++ b/source3/smbd/process.c
@@ -2441,15 +2441,15 @@ static bool fd_is_readable(int fd)
 }
 
 static void smbd_server_connection_write_handler(
-	struct smbd_server_connection *sconn)
+	struct smbXsrv_connection *xconn)
 {
 	/* TODO: make write nonblocking */
 }
 
 static void smbd_server_connection_read_handler(
-	struct smbd_server_connection *sconn, int fd)
+	struct smbXsrv_connection *xconn, int fd)
 {
-	struct smbXsrv_connection *xconn = sconn->conn;
+	struct smbd_server_connection *sconn = xconn->sconn;
 	uint8_t *inbuf = NULL;
 	size_t inbuf_len = 0;
 	size_t unread_bytes = 0;
@@ -2517,9 +2517,9 @@ static void smbd_server_connection_handler(struct tevent_context *ev,
 					   uint16_t flags,
 					   void *private_data)
 {
-	struct smbd_server_connection *conn = talloc_get_type(private_data,
-					      struct smbd_server_connection);
-	struct smbXsrv_connection *xconn = conn->conn;
+	struct smbXsrv_connection *xconn =
+		talloc_get_type_abort(private_data,
+		struct smbXsrv_connection);
 
 	if (!NT_STATUS_IS_OK(xconn->transport.status)) {
 		/*
@@ -2531,11 +2531,11 @@ static void smbd_server_connection_handler(struct tevent_context *ev,
 	}
 
 	if (flags & TEVENT_FD_WRITE) {
-		smbd_server_connection_write_handler(conn);
+		smbd_server_connection_write_handler(xconn);
 		return;
 	}
 	if (flags & TEVENT_FD_READ) {
-		smbd_server_connection_read_handler(conn, xconn->transport.sock);
+		smbd_server_connection_read_handler(xconn, xconn->transport.sock);
 		return;
 	}
 }
@@ -2545,9 +2545,9 @@ static void smbd_server_echo_handler(struct tevent_context *ev,
 				     uint16_t flags,
 				     void *private_data)
 {
-	struct smbd_server_connection *conn = talloc_get_type(private_data,
-					      struct smbd_server_connection);
-	struct smbXsrv_connection *xconn = conn->conn;
+	struct smbXsrv_connection *xconn =
+		talloc_get_type_abort(private_data,
+		struct smbXsrv_connection);
 
 	if (!NT_STATUS_IS_OK(xconn->transport.status)) {
 		/*
@@ -2559,12 +2559,12 @@ static void smbd_server_echo_handler(struct tevent_context *ev,
 	}
 
 	if (flags & TEVENT_FD_WRITE) {
-		smbd_server_connection_write_handler(conn);
+		smbd_server_connection_write_handler(xconn);
 		return;
 	}
 	if (flags & TEVENT_FD_READ) {
 		smbd_server_connection_read_handler(
-			conn, xconn->smb1.echo_handler.trusted_fd);
+			xconn, xconn->smb1.echo_handler.trusted_fd);
 		return;
 	}
 }
@@ -3310,12 +3310,12 @@ bool fork_echo_handler(struct smbd_server_connection *sconn)
 	 * Without smb signing this is the same as the normal smbd
 	 * listener. This needs to change once signing comes in.
 	 */
-	xconn->smb1.echo_handler.trusted_fde = tevent_add_fd(sconn->ev_ctx,
-					sconn,
+	xconn->smb1.echo_handler.trusted_fde = tevent_add_fd(xconn->ev_ctx,
+					xconn,
 					xconn->smb1.echo_handler.trusted_fd,
 					TEVENT_FD_READ,
 					smbd_server_echo_handler,
-					sconn);
+					xconn);
 	if (xconn->smb1.echo_handler.trusted_fde == NULL) {
 		DEBUG(1, ("event_add_fd failed\n"));
 		goto fail;
@@ -3849,11 +3849,11 @@ void smbd_process(struct tevent_context *ev_ctx,
 	}
 
 	xconn->transport.fde = tevent_add_fd(ev_ctx,
-					     sconn,
+					     xconn,
 					     sock_fd,
 					     TEVENT_FD_READ,
 					     smbd_server_connection_handler,
-					     sconn);
+					     xconn);
 	if (!xconn->transport.fde) {
 		exit_server("failed to create smbd_server_connection fde");
 	}
-- 
1.8.1.2


From 5030a1a0eafe1d125ad9c411fc919cf410402664 Mon Sep 17 00:00:00 2001
From: Stefan Metzmacher <metze@samba.org>
Date: Tue, 10 Jun 2014 20:20:58 +0200
Subject: [PATCH 41/94] s3:smb2_server: pass smbXsrv_connection to fd handlers

Signed-off-by: Stefan Metzmacher <metze@samba.org>
Reviewed-by: Volker Lendecke <vl@samba.org>
---
 source3/smbd/smb2_server.c | 19 ++++++++++---------
 1 file changed, 10 insertions(+), 9 deletions(-)

diff --git a/source3/smbd/smb2_server.c b/source3/smbd/smb2_server.c
index 7a0555d..7b17fdd 100644
--- a/source3/smbd/smb2_server.c
+++ b/source3/smbd/smb2_server.c
@@ -214,12 +214,12 @@ static NTSTATUS smbd_initialize_smb2(struct smbd_server_connection *sconn)
 		return NT_STATUS_NO_MEMORY;
 	}
 
-	xconn->transport.fde = tevent_add_fd(sconn->ev_ctx,
-					sconn,
+	xconn->transport.fde = tevent_add_fd(xconn->ev_ctx,
+					xconn,
 					xconn->transport.sock,
 					TEVENT_FD_READ,
 					smbd_smb2_connection_handler,
-					sconn);
+					xconn);
 	if (xconn->transport.fde == NULL) {
 		return NT_STATUS_NO_MEMORY;
 	}
@@ -3218,10 +3218,10 @@ static NTSTATUS smbd_smb2_flush_send_queue(struct smbd_server_connection *sconn)
 	return NT_STATUS_OK;
 }
 
-static NTSTATUS smbd_smb2_io_handler(struct smbd_server_connection *sconn,
+static NTSTATUS smbd_smb2_io_handler(struct smbXsrv_connection *xconn,
 				     uint16_t fde_flags)
 {
-	struct smbXsrv_connection *xconn = sconn->conn;
+	struct smbd_server_connection *sconn = xconn->sconn;
 	struct smbd_smb2_request_read_state *state = &xconn->smb2.request_read_state;
 	struct smbd_smb2_request *req = NULL;
 	size_t min_recvfile_size = UINT32_MAX;
@@ -3383,7 +3383,7 @@ got_full:
 	req->request_time = timeval_current();
 	now = timeval_to_nttime(&req->request_time);
 
-	status = smbd_smb2_inbuf_parse_compound(req->sconn->conn,
+	status = smbd_smb2_inbuf_parse_compound(xconn,
 						now,
 						state->pktbuf,
 						state->pktlen,
@@ -3452,12 +3452,13 @@ static void smbd_smb2_connection_handler(struct tevent_context *ev,
 					 uint16_t flags,
 					 void *private_data)
 {
-	struct smbd_server_connection *sconn =
+	struct smbXsrv_connection *xconn =
 		talloc_get_type_abort(private_data,
-		struct smbd_server_connection);
+		struct smbXsrv_connection);
+	struct smbd_server_connection *sconn = xconn->sconn;
 	NTSTATUS status;
 
-	status = smbd_smb2_io_handler(sconn, flags);
+	status = smbd_smb2_io_handler(xconn, flags);
 	if (!NT_STATUS_IS_OK(status)) {
 		smbd_server_connection_terminate(sconn, nt_errstr(status));
 		return;
-- 
1.8.1.2


From 6e67cfdc99f9b415debd84d0f7f86a024ddc5945 Mon Sep 17 00:00:00 2001
From: Stefan Metzmacher <metze@samba.org>
Date: Mon, 7 Jul 2014 12:25:48 +0200
Subject: [PATCH 42/94] s3:smb2_server: rename 'conn' variables to 'xconn'

This makes the code more consistent with other functions
using 'struct smbXsrv_connection *xconn' as local variables.

Signed-off-by: Stefan Metzmacher <metze@samba.org>
Reviewed-by: Volker Lendecke <vl@samba.org>
---
 source3/smbd/smb2_server.c | 54 +++++++++++++++++++++++-----------------------
 1 file changed, 27 insertions(+), 27 deletions(-)

diff --git a/source3/smbd/smb2_server.c b/source3/smbd/smb2_server.c
index 7b17fdd..e0d3ce9 100644
--- a/source3/smbd/smb2_server.c
+++ b/source3/smbd/smb2_server.c
@@ -294,7 +294,7 @@ static struct smbd_smb2_request *smbd_smb2_request_allocate(TALLOC_CTX *mem_ctx)
 	return req;
 }
 
-static NTSTATUS smbd_smb2_inbuf_parse_compound(struct smbXsrv_connection *conn,
+static NTSTATUS smbd_smb2_inbuf_parse_compound(struct smbXsrv_connection *xconn,
 					       NTTIME now,
 					       uint8_t *buf,
 					       size_t buflen,
@@ -351,19 +351,19 @@ static NTSTATUS smbd_smb2_inbuf_parse_compound(struct smbXsrv_connection *conn,
 			NTSTATUS status;
 			size_t enc_len;
 
-			if (conn->protocol < PROTOCOL_SMB2_24) {
+			if (xconn->protocol < PROTOCOL_SMB2_24) {
 				DEBUG(10, ("Got SMB2_TRANSFORM header, "
 					   "but dialect[0x%04X] is used\n",
-					   conn->smb2.server.dialect));
+					   xconn->smb2.server.dialect));
 				goto inval;
 			}
 
-			if (!(conn->smb2.server.capabilities & SMB2_CAP_ENCRYPTION)) {
+			if (!(xconn->smb2.server.capabilities & SMB2_CAP_ENCRYPTION)) {
 				DEBUG(10, ("Got SMB2_TRANSFORM header, "
 					   "but not negotiated "
 					   "client[0x%08X] server[0x%08X]\n",
-					   conn->smb2.client.capabilities,
-					   conn->smb2.server.capabilities));
+					   xconn->smb2.client.capabilities,
+					   xconn->smb2.server.capabilities));
 				goto inval;
 			}
 
@@ -387,7 +387,7 @@ static NTSTATUS smbd_smb2_inbuf_parse_compound(struct smbXsrv_connection *conn,
 				goto inval;
 			}
 
-			status = smb2srv_session_lookup(conn, uid, now, &s);
+			status = smb2srv_session_lookup(xconn, uid, now, &s);
 			if (s == NULL) {
 				DEBUG(1, ("invalid session[%llu] in "
 					  "SMB2_TRANSFORM header\n",
@@ -402,7 +402,7 @@ static NTSTATUS smbd_smb2_inbuf_parse_compound(struct smbXsrv_connection *conn,
 			tf_iov[1].iov_len = enc_len;
 
 			status = smb2_signing_decrypt_pdu(s->global->decryption_key,
-							  conn->protocol,
+							  xconn->protocol,
 							  tf_iov, 2);
 			if (!NT_STATUS_IS_OK(status)) {
 				TALLOC_FREE(iov_alloc);
@@ -1205,7 +1205,7 @@ static struct smbd_smb2_request *dup_smb2_req(const struct smbd_smb2_request *re
 static NTSTATUS smb2_send_async_interim_response(const struct smbd_smb2_request *req)
 {
 	struct smbd_server_connection *sconn = req->sconn;
-	struct smbXsrv_connection *conn = req->sconn->conn;
+	struct smbXsrv_connection *xconn = req->sconn->conn;
 	int first_idx = 1;
 	struct iovec *firsttf = NULL;
 	struct iovec *outhdr_v = NULL;
@@ -1252,7 +1252,7 @@ static NTSTATUS smb2_send_async_interim_response(const struct smbd_smb2_request
 	 */
 	if (firsttf->iov_len == SMB2_TF_HDR_SIZE) {
 		status = smb2_signing_encrypt_pdu(req->first_key,
-					conn->protocol,
+					xconn->protocol,
 					firsttf,
 					nreq->out.vector_count - first_idx);
 		if (!NT_STATUS_IS_OK(status)) {
@@ -1260,7 +1260,7 @@ static NTSTATUS smb2_send_async_interim_response(const struct smbd_smb2_request
 		}
 	} else if (req->last_key.length > 0) {
 		status = smb2_signing_sign_pdu(req->last_key,
-					       conn->protocol,
+					       xconn->protocol,
 					       outhdr_v,
 					       SMBD_SMB2_NUM_IOV_PER_REQ - 1);
 		if (!NT_STATUS_IS_OK(status)) {
@@ -1271,8 +1271,8 @@ static NTSTATUS smb2_send_async_interim_response(const struct smbd_smb2_request
 	nreq->queue_entry.mem_ctx = nreq;
 	nreq->queue_entry.vector = nreq->out.vector;
 	nreq->queue_entry.count = nreq->out.vector_count;
-	DLIST_ADD_END(conn->smb2.send_queue, &nreq->queue_entry, NULL);
-	conn->smb2.send_queue_len++;
+	DLIST_ADD_END(xconn->smb2.send_queue, &nreq->queue_entry, NULL);
+	xconn->smb2.send_queue_len++;
 
 	status = smbd_smb2_flush_send_queue(sconn);
 	if (!NT_STATUS_IS_OK(status)) {
@@ -1915,7 +1915,7 @@ NTSTATUS smbd_smb2_request_verify_sizes(struct smbd_smb2_request *req,
 
 NTSTATUS smbd_smb2_request_dispatch(struct smbd_smb2_request *req)
 {
-	struct smbXsrv_connection *conn = req->sconn->conn;
+	struct smbXsrv_connection *xconn = req->sconn->conn;
 	const struct smbd_smb2_dispatch_table *call = NULL;
 	const struct iovec *intf_v = SMBD_SMB2_IN_TF_IOV(req);
 	const uint8_t *inhdr;
@@ -1941,7 +1941,7 @@ NTSTATUS smbd_smb2_request_dispatch(struct smbd_smb2_request *req)
 		smb2_opcode_name(opcode),
 		(unsigned long long)mid));
 
-	if (conn->protocol >= PROTOCOL_SMB2_02) {
+	if (xconn->protocol >= PROTOCOL_SMB2_02) {
 		/*
 		 * once the protocol is negotiated
 		 * SMB2_OP_NEGPROT is not allowed anymore
@@ -2075,7 +2075,7 @@ NTSTATUS smbd_smb2_request_dispatch(struct smbd_smb2_request *req)
 		}
 
 		status = smb2_signing_check_pdu(signing_key,
-						conn->protocol,
+						xconn->protocol,
 						SMBD_SMB2_IN_HDR_IOV(req),
 						SMBD_SMB2_NUM_IOV_PER_REQ - 1);
 		if (!NT_STATUS_IS_OK(status)) {
@@ -2341,7 +2341,7 @@ NTSTATUS smbd_smb2_request_dispatch(struct smbd_smb2_request *req)
 static NTSTATUS smbd_smb2_request_reply(struct smbd_smb2_request *req)
 {
 	struct smbd_server_connection *sconn = req->sconn;
-	struct smbXsrv_connection *conn = req->sconn->conn;
+	struct smbXsrv_connection *xconn = req->sconn->conn;
 	int first_idx = 1;
 	struct iovec *firsttf = SMBD_SMB2_IDX_TF_IOV(req,out,first_idx);
 	struct iovec *outhdr = SMBD_SMB2_OUT_HDR_IOV(req);
@@ -2417,7 +2417,7 @@ static NTSTATUS smbd_smb2_request_reply(struct smbd_smb2_request *req)
 		 * with the last signing key we remembered.
 		 */
 		status = smb2_signing_sign_pdu(req->last_key,
-					       conn->protocol,
+					       xconn->protocol,
 					       lasthdr,
 					       SMBD_SMB2_NUM_IOV_PER_REQ - 1);
 		if (!NT_STATUS_IS_OK(status)) {
@@ -2482,7 +2482,7 @@ static NTSTATUS smbd_smb2_request_reply(struct smbd_smb2_request *req)
 	 */
 	if (firsttf->iov_len == SMB2_TF_HDR_SIZE) {
 		status = smb2_signing_encrypt_pdu(req->first_key,
-					conn->protocol,
+					xconn->protocol,
 					firsttf,
 					req->out.vector_count - first_idx);
 		if (!NT_STATUS_IS_OK(status)) {
@@ -2493,7 +2493,7 @@ static NTSTATUS smbd_smb2_request_reply(struct smbd_smb2_request *req)
 		DATA_BLOB signing_key = x->global->channels[0].signing_key;
 
 		status = smb2_signing_sign_pdu(signing_key,
-					       conn->protocol,
+					       xconn->protocol,
 					       outhdr,
 					       SMBD_SMB2_NUM_IOV_PER_REQ - 1);
 		if (!NT_STATUS_IS_OK(status)) {
@@ -2516,13 +2516,13 @@ static NTSTATUS smbd_smb2_request_reply(struct smbd_smb2_request *req)
 	 * We're done with this request -
 	 * move it off the "being processed" queue.
 	 */
-	DLIST_REMOVE(conn->smb2.requests, req);
+	DLIST_REMOVE(xconn->smb2.requests, req);
 
 	req->queue_entry.mem_ctx = req;
 	req->queue_entry.vector = req->out.vector;
 	req->queue_entry.count = req->out.vector_count;
-	DLIST_ADD_END(conn->smb2.send_queue, &req->queue_entry, NULL);
-	conn->smb2.send_queue_len++;
+	DLIST_ADD_END(xconn->smb2.send_queue, &req->queue_entry, NULL);
+	xconn->smb2.send_queue_len++;
 
 	status = smbd_smb2_flush_send_queue(sconn);
 	if (!NT_STATUS_IS_OK(status)) {
@@ -2749,7 +2749,7 @@ static NTSTATUS smbd_smb2_send_break(struct smbd_server_connection *sconn,
 				     size_t body_len)
 {
 	struct smbd_smb2_send_break_state *state;
-	struct smbXsrv_connection *conn = sconn->conn;
+	struct smbXsrv_connection *xconn = sconn->conn;
 	bool do_encryption = session->global->encryption_required;
 	uint64_t nonce_high = 0;
 	uint64_t nonce_low = 0;
@@ -2839,7 +2839,7 @@ static NTSTATUS smbd_smb2_send_break(struct smbd_server_connection *sconn,
 		DATA_BLOB encryption_key = session->global->encryption_key;
 
 		status = smb2_signing_encrypt_pdu(encryption_key,
-					conn->protocol,
+					xconn->protocol,
 					&state->vector[1+SMBD_SMB2_TF_IOV_OFS],
 					SMBD_SMB2_NUM_IOV_PER_REQ);
 		if (!NT_STATUS_IS_OK(status)) {
@@ -2850,8 +2850,8 @@ static NTSTATUS smbd_smb2_send_break(struct smbd_server_connection *sconn,
 	state->queue_entry.mem_ctx = state;
 	state->queue_entry.vector = state->vector;
 	state->queue_entry.count = ARRAY_SIZE(state->vector);
-	DLIST_ADD_END(conn->smb2.send_queue, &state->queue_entry, NULL);
-	conn->smb2.send_queue_len++;
+	DLIST_ADD_END(xconn->smb2.send_queue, &state->queue_entry, NULL);
+	xconn->smb2.send_queue_len++;
 
 	status = smbd_smb2_flush_send_queue(sconn);
 	if (!NT_STATUS_IS_OK(status)) {
-- 
1.8.1.2


From 013365021fee04bd9a2052b1c4486f5d4b46a32c Mon Sep 17 00:00:00 2001
From: Stefan Metzmacher <metze@samba.org>
Date: Wed, 11 Jun 2014 10:00:09 +0200
Subject: [PATCH 43/94] s3:smb2_server: pass smbXsrv_connection to
 smbd_smb2_flush_send_queue()

Signed-off-by: Stefan Metzmacher <metze@samba.org>
Reviewed-by: Volker Lendecke <vl@samba.org>
---
 source3/smbd/smb2_server.c | 21 +++++++++------------
 1 file changed, 9 insertions(+), 12 deletions(-)

diff --git a/source3/smbd/smb2_server.c b/source3/smbd/smb2_server.c
index e0d3ce9..8efd980 100644
--- a/source3/smbd/smb2_server.c
+++ b/source3/smbd/smb2_server.c
@@ -34,7 +34,7 @@ static void smbd_smb2_connection_handler(struct tevent_context *ev,
 					 struct tevent_fd *fde,
 					 uint16_t flags,
 					 void *private_data);
-static NTSTATUS smbd_smb2_flush_send_queue(struct smbd_server_connection *sconn);
+static NTSTATUS smbd_smb2_flush_send_queue(struct smbXsrv_connection *xconn);
 
 static const struct smbd_smb2_dispatch_table {
 	uint16_t opcode;
@@ -1204,8 +1204,7 @@ static struct smbd_smb2_request *dup_smb2_req(const struct smbd_smb2_request *re
 
 static NTSTATUS smb2_send_async_interim_response(const struct smbd_smb2_request *req)
 {
-	struct smbd_server_connection *sconn = req->sconn;
-	struct smbXsrv_connection *xconn = req->sconn->conn;
+	struct smbXsrv_connection *xconn = req->xconn;
 	int first_idx = 1;
 	struct iovec *firsttf = NULL;
 	struct iovec *outhdr_v = NULL;
@@ -1274,7 +1273,7 @@ static NTSTATUS smb2_send_async_interim_response(const struct smbd_smb2_request
 	DLIST_ADD_END(xconn->smb2.send_queue, &nreq->queue_entry, NULL);
 	xconn->smb2.send_queue_len++;
 
-	status = smbd_smb2_flush_send_queue(sconn);
+	status = smbd_smb2_flush_send_queue(xconn);
 	if (!NT_STATUS_IS_OK(status)) {
 		return status;
 	}
@@ -1606,7 +1605,7 @@ static void smbd_smb2_request_pending_timer(struct tevent_context *ev,
 	DLIST_ADD_END(xconn->smb2.send_queue, &state->queue_entry, NULL);
 	xconn->smb2.send_queue_len++;
 
-	status = smbd_smb2_flush_send_queue(sconn);
+	status = smbd_smb2_flush_send_queue(xconn);
 	if (!NT_STATUS_IS_OK(status)) {
 		smbd_server_connection_terminate(sconn,
 						 nt_errstr(status));
@@ -2340,8 +2339,7 @@ NTSTATUS smbd_smb2_request_dispatch(struct smbd_smb2_request *req)
 
 static NTSTATUS smbd_smb2_request_reply(struct smbd_smb2_request *req)
 {
-	struct smbd_server_connection *sconn = req->sconn;
-	struct smbXsrv_connection *xconn = req->sconn->conn;
+	struct smbXsrv_connection *xconn = req->xconn;
 	int first_idx = 1;
 	struct iovec *firsttf = SMBD_SMB2_IDX_TF_IOV(req,out,first_idx);
 	struct iovec *outhdr = SMBD_SMB2_OUT_HDR_IOV(req);
@@ -2524,7 +2522,7 @@ static NTSTATUS smbd_smb2_request_reply(struct smbd_smb2_request *req)
 	DLIST_ADD_END(xconn->smb2.send_queue, &req->queue_entry, NULL);
 	xconn->smb2.send_queue_len++;
 
-	status = smbd_smb2_flush_send_queue(sconn);
+	status = smbd_smb2_flush_send_queue(xconn);
 	if (!NT_STATUS_IS_OK(status)) {
 		return status;
 	}
@@ -2853,7 +2851,7 @@ static NTSTATUS smbd_smb2_send_break(struct smbd_server_connection *sconn,
 	DLIST_ADD_END(xconn->smb2.send_queue, &state->queue_entry, NULL);
 	xconn->smb2.send_queue_len++;
 
-	status = smbd_smb2_flush_send_queue(sconn);
+	status = smbd_smb2_flush_send_queue(xconn);
 	if (!NT_STATUS_IS_OK(status)) {
 		return status;
 	}
@@ -3108,9 +3106,8 @@ static int socket_error_from_errno(int ret,
 	return sys_errno;
 }
 
-static NTSTATUS smbd_smb2_flush_send_queue(struct smbd_server_connection *sconn)
+static NTSTATUS smbd_smb2_flush_send_queue(struct smbXsrv_connection *xconn)
 {
-	struct smbXsrv_connection *xconn = sconn->conn;
 	int ret;
 	int err;
 	bool retry;
@@ -3241,7 +3238,7 @@ static NTSTATUS smbd_smb2_io_handler(struct smbXsrv_connection *xconn,
 	}
 
 	if (fde_flags & TEVENT_FD_WRITE) {
-		status = smbd_smb2_flush_send_queue(sconn);
+		status = smbd_smb2_flush_send_queue(xconn);
 		if (!NT_STATUS_IS_OK(status)) {
 			return status;
 		}
-- 
1.8.1.2


From 6471326c115c5faef684b1b61c22cf2cbfb66668 Mon Sep 17 00:00:00 2001
From: Stefan Metzmacher <metze@samba.org>
Date: Wed, 11 Jun 2014 10:03:27 +0200
Subject: [PATCH 44/94] s3:smb2_server: pass smbXsrv_connection to
 smb2_validate_sequence_number()

Signed-off-by: Stefan Metzmacher <metze@samba.org>
Reviewed-by: Volker Lendecke <vl@samba.org>
---
 source3/smbd/smb2_server.c | 5 ++---
 1 file changed, 2 insertions(+), 3 deletions(-)

diff --git a/source3/smbd/smb2_server.c b/source3/smbd/smb2_server.c
index 8efd980..427f815 100644
--- a/source3/smbd/smb2_server.c
+++ b/source3/smbd/smb2_server.c
@@ -577,10 +577,9 @@ static NTSTATUS smbd_smb2_request_create(struct smbd_server_connection *sconn,
 	return NT_STATUS_OK;
 }
 
-static bool smb2_validate_sequence_number(struct smbd_server_connection *sconn,
+static bool smb2_validate_sequence_number(struct smbXsrv_connection *xconn,
 					  uint64_t message_id, uint64_t seq_id)
 {
-	struct smbXsrv_connection *xconn = sconn->conn;
 	struct bitmap *credits_bm = xconn->smb2.credits.bitmap;
 	unsigned int offset;
 	uint64_t seq_tmp;
@@ -711,7 +710,7 @@ static bool smb2_validate_message_id(struct smbd_server_connection *sconn,
 			   credit_charge,
 			   (unsigned long long)id));
 
-		ok = smb2_validate_sequence_number(sconn, message_id, id);
+		ok = smb2_validate_sequence_number(xconn, message_id, id);
 		if (!ok) {
 			return false;
 		}
-- 
1.8.1.2


From b9b547be602278ccd0d7f9bb1312b9efc5d6b1c5 Mon Sep 17 00:00:00 2001
From: Stefan Metzmacher <metze@samba.org>
Date: Wed, 11 Jun 2014 10:05:43 +0200
Subject: [PATCH 45/94] s3:smb2_server: pass smbXsrv_connection to
 smb2_validate_message_id()

Signed-off-by: Stefan Metzmacher <metze@samba.org>
Reviewed-by: Volker Lendecke <vl@samba.org>
---
 source3/smbd/smb2_server.c | 7 +++----
 1 file changed, 3 insertions(+), 4 deletions(-)

diff --git a/source3/smbd/smb2_server.c b/source3/smbd/smb2_server.c
index 427f815..1f7c3d4 100644
--- a/source3/smbd/smb2_server.c
+++ b/source3/smbd/smb2_server.c
@@ -652,10 +652,9 @@ static bool smb2_validate_sequence_number(struct smbXsrv_connection *xconn,
 	return true;
 }
 
-static bool smb2_validate_message_id(struct smbd_server_connection *sconn,
-				const uint8_t *inhdr)
+static bool smb2_validate_message_id(struct smbXsrv_connection *xconn,
+				     const uint8_t *inhdr)
 {
-	struct smbXsrv_connection *xconn = sconn->conn;
 	uint64_t message_id = BVAL(inhdr, SMB2_HDR_MESSAGE_ID);
 	uint16_t opcode = SVAL(inhdr, SMB2_HDR_OPCODE);
 	uint16_t credit_charge = 1;
@@ -754,7 +753,7 @@ static NTSTATUS smbd_smb2_request_validate(struct smbd_smb2_request *req)
 			return NT_STATUS_INVALID_PARAMETER;
 		}
 
-		if (!smb2_validate_message_id(req->sconn, inhdr)) {
+		if (!smb2_validate_message_id(req->xconn, inhdr)) {
 			return NT_STATUS_INVALID_PARAMETER;
 		}
 	}
-- 
1.8.1.2


From c567960f05956786cb5748f92282812b174750bb Mon Sep 17 00:00:00 2001
From: Stefan Metzmacher <metze@samba.org>
Date: Wed, 11 Jun 2014 10:08:26 +0200
Subject: [PATCH 46/94] s3:smb2_server: pass smbXsrv_connection to
 smb2_set_operation_credit()

Signed-off-by: Stefan Metzmacher <metze@samba.org>
Reviewed-by: Volker Lendecke <vl@samba.org>
---
 source3/smbd/smb2_server.c | 12 ++++++------
 1 file changed, 6 insertions(+), 6 deletions(-)

diff --git a/source3/smbd/smb2_server.c b/source3/smbd/smb2_server.c
index 1f7c3d4..eee08f1 100644
--- a/source3/smbd/smb2_server.c
+++ b/source3/smbd/smb2_server.c
@@ -761,11 +761,11 @@ static NTSTATUS smbd_smb2_request_validate(struct smbd_smb2_request *req)
 	return NT_STATUS_OK;
 }
 
-static void smb2_set_operation_credit(struct smbd_server_connection *sconn,
-			const struct iovec *in_vector,
-			struct iovec *out_vector)
+static void smb2_set_operation_credit(struct smbXsrv_connection *xconn,
+				      const struct iovec *in_vector,
+				      struct iovec *out_vector)
 {
-	struct smbXsrv_connection *xconn = sconn->conn;
+	struct smbd_server_connection *sconn = xconn->sconn;
 	const uint8_t *inhdr = (const uint8_t *)in_vector->iov_base;
 	uint8_t *outhdr = (uint8_t *)out_vector->iov_base;
 	uint16_t credit_charge = 1;
@@ -911,7 +911,7 @@ static void smb2_calculate_credits(const struct smbd_smb2_request *inreq,
 		struct iovec *outhdr_v = SMBD_SMB2_IDX_HDR_IOV(outreq,out,idx);
 		uint8_t *outhdr = (uint8_t *)outhdr_v->iov_base;
 
-		smb2_set_operation_credit(outreq->sconn, inhdr_v, outhdr_v);
+		smb2_set_operation_credit(outreq->xconn, inhdr_v, outhdr_v);
 
 		/* To match Windows, count up what we
 		   just granted. */
@@ -1552,7 +1552,7 @@ static void smbd_smb2_request_pending_timer(struct tevent_context *ev,
 	/* Ensure we correctly go through crediting. Grant
 	   the credits now, and zero credits on the final
 	   response. */
-	smb2_set_operation_credit(req->sconn,
+	smb2_set_operation_credit(req->xconn,
 			SMBD_SMB2_IN_HDR_IOV(req),
 			&state->vector[1+SMBD_SMB2_HDR_IOV_OFS]);
 
-- 
1.8.1.2


From d8fd9d5c8af0e04b8f89b15bd453bba4749ea19c Mon Sep 17 00:00:00 2001
From: Stefan Metzmacher <metze@samba.org>
Date: Wed, 11 Jun 2014 10:42:01 +0200
Subject: [PATCH 47/94] s3:smb2_server: pass smbXsrv_connection to
 smbd_smb2_first_negprot()

Signed-off-by: Stefan Metzmacher <metze@samba.org>
Reviewed-by: Volker Lendecke <vl@samba.org>
---
 source3/smbd/globals.h      | 2 +-
 source3/smbd/process.c      | 2 +-
 source3/smbd/smb2_negprot.c | 2 +-
 source3/smbd/smb2_server.c  | 3 ++-
 4 files changed, 5 insertions(+), 4 deletions(-)

diff --git a/source3/smbd/globals.h b/source3/smbd/globals.h
index b94ba29..9ec58f3 100644
--- a/source3/smbd/globals.h
+++ b/source3/smbd/globals.h
@@ -224,7 +224,7 @@ bool smbd_is_smb2_header(const uint8_t *inbuf, size_t size);
 
 void reply_smb2002(struct smb_request *req, uint16_t choice);
 void reply_smb20ff(struct smb_request *req, uint16_t choice);
-void smbd_smb2_first_negprot(struct smbd_server_connection *sconn,
+void smbd_smb2_first_negprot(struct smbXsrv_connection *xconn,
 			     uint8_t *inbuf, size_t size);
 
 DATA_BLOB smbd_smb2_generate_outbody(struct smbd_smb2_request *req, size_t size);
diff --git a/source3/smbd/process.c b/source3/smbd/process.c
index c8d6079..b970f00 100644
--- a/source3/smbd/process.c
+++ b/source3/smbd/process.c
@@ -1878,7 +1878,7 @@ static void process_smb(struct smbXsrv_connection *xconn,
 		/* At this point we're not really using smb2,
 		 * we make the decision here.. */
 		if (smbd_is_smb2_header(inbuf, nread)) {
-			smbd_smb2_first_negprot(sconn, inbuf, nread);
+			smbd_smb2_first_negprot(xconn, inbuf, nread);
 			return;
 		} else if (nread >= smb_size && valid_smb_header(sconn, inbuf)
 				&& CVAL(inbuf, smb_com) != 0x72) {
diff --git a/source3/smbd/smb2_negprot.c b/source3/smbd/smb2_negprot.c
index fde9514..3f76f3f 100644
--- a/source3/smbd/smb2_negprot.c
+++ b/source3/smbd/smb2_negprot.c
@@ -59,7 +59,7 @@ static void reply_smb20xx(struct smb_request *req, uint16_t dialect)
 
 	req->outbuf = NULL;
 
-	smbd_smb2_first_negprot(req->sconn, smb2_inbuf, len);
+	smbd_smb2_first_negprot(req->xconn, smb2_inbuf, len);
 	return;
 }
 
diff --git a/source3/smbd/smb2_server.c b/source3/smbd/smb2_server.c
index eee08f1..59cb6d8 100644
--- a/source3/smbd/smb2_server.c
+++ b/source3/smbd/smb2_server.c
@@ -3005,9 +3005,10 @@ static NTSTATUS smbd_smb2_request_next_incoming(struct smbd_server_connection *s
 	return NT_STATUS_OK;
 }
 
-void smbd_smb2_first_negprot(struct smbd_server_connection *sconn,
+void smbd_smb2_first_negprot(struct smbXsrv_connection *xconn,
 			     uint8_t *inbuf, size_t size)
 {
+	struct smbd_server_connection *sconn = xconn->sconn;
 	NTSTATUS status;
 	struct smbd_smb2_request *req = NULL;
 
-- 
1.8.1.2


From 3f89a97a4ee74436d83e0e30a38f7cf28a1e850b Mon Sep 17 00:00:00 2001
From: Stefan Metzmacher <metze@samba.org>
Date: Wed, 11 Jun 2014 10:45:56 +0200
Subject: [PATCH 48/94] s3:smb2_server: pass smbXsrv_connection to
 smbd_initialize_smb2()

Signed-off-by: Stefan Metzmacher <metze@samba.org>
Reviewed-by: Volker Lendecke <vl@samba.org>
---
 source3/smbd/smb2_server.c | 6 ++----
 1 file changed, 2 insertions(+), 4 deletions(-)

diff --git a/source3/smbd/smb2_server.c b/source3/smbd/smb2_server.c
index 59cb6d8..af2330d 100644
--- a/source3/smbd/smb2_server.c
+++ b/source3/smbd/smb2_server.c
@@ -198,10 +198,8 @@ bool smbd_is_smb2_header(const uint8_t *inbuf, size_t size)
 	return true;
 }
 
-static NTSTATUS smbd_initialize_smb2(struct smbd_server_connection *sconn)
+static NTSTATUS smbd_initialize_smb2(struct smbXsrv_connection *xconn)
 {
-	struct smbXsrv_connection *xconn = sconn->conn;
-
 	TALLOC_FREE(xconn->transport.fde);
 
 	xconn->smb2.credits.seq_low = 0;
@@ -3015,7 +3013,7 @@ void smbd_smb2_first_negprot(struct smbXsrv_connection *xconn,
 	DEBUG(10,("smbd_smb2_first_negprot: packet length %u\n",
 		 (unsigned int)size));
 
-	status = smbd_initialize_smb2(sconn);
+	status = smbd_initialize_smb2(xconn);
 	if (!NT_STATUS_IS_OK(status)) {
 		smbd_server_connection_terminate(sconn, nt_errstr(status));
 		return;
-- 
1.8.1.2


From d072554ef6e04da97ff0caf8c6ffb656b7d7cd7e Mon Sep 17 00:00:00 2001
From: Stefan Metzmacher <metze@samba.org>
Date: Wed, 11 Jun 2014 10:51:04 +0200
Subject: [PATCH 49/94] s3:smb2_server: pass smbXsrv_connection to
 smbd_smb2_request_create()

Signed-off-by: Stefan Metzmacher <metze@samba.org>
Reviewed-by: Volker Lendecke <vl@samba.org>
---
 source3/smbd/smb2_server.c | 8 ++++----
 1 file changed, 4 insertions(+), 4 deletions(-)

diff --git a/source3/smbd/smb2_server.c b/source3/smbd/smb2_server.c
index af2330d..e0d906f 100644
--- a/source3/smbd/smb2_server.c
+++ b/source3/smbd/smb2_server.c
@@ -505,11 +505,11 @@ inval:
 	return NT_STATUS_INVALID_PARAMETER;
 }
 
-static NTSTATUS smbd_smb2_request_create(struct smbd_server_connection *sconn,
+static NTSTATUS smbd_smb2_request_create(struct smbXsrv_connection *xconn,
 					 uint8_t *inbuf, size_t size,
 					 struct smbd_smb2_request **_req)
 {
-	struct smbXsrv_connection *xconn = sconn->conn;
+	struct smbd_server_connection *sconn = xconn->sconn;
 	struct smbd_smb2_request *req;
 	uint32_t protocol_version;
 	const uint8_t *inhdr = NULL;
@@ -558,7 +558,7 @@ static NTSTATUS smbd_smb2_request_create(struct smbd_server_connection *sconn,
 	req->request_time = timeval_current();
 	now = timeval_to_nttime(&req->request_time);
 
-	status = smbd_smb2_inbuf_parse_compound(sconn->conn,
+	status = smbd_smb2_inbuf_parse_compound(xconn,
 						now,
 						inbuf + NBT_HDR_SIZE,
 						size - NBT_HDR_SIZE,
@@ -3019,7 +3019,7 @@ void smbd_smb2_first_negprot(struct smbXsrv_connection *xconn,
 		return;
 	}
 
-	status = smbd_smb2_request_create(sconn, inbuf, size, &req);
+	status = smbd_smb2_request_create(xconn, inbuf, size, &req);
 	if (!NT_STATUS_IS_OK(status)) {
 		smbd_server_connection_terminate(sconn, nt_errstr(status));
 		return;
-- 
1.8.1.2


From ed7c8f1457fd20d364b08092fc52543fd6407890 Mon Sep 17 00:00:00 2001
From: Stefan Metzmacher <metze@samba.org>
Date: Wed, 11 Jun 2014 10:59:00 +0200
Subject: [PATCH 50/94] s3:smb2_server: pass smbXsrv_connection to
 smbd_smb2_request_next_incoming()

Signed-off-by: Stefan Metzmacher <metze@samba.org>
Reviewed-by: Volker Lendecke <vl@samba.org>
---
 source3/smbd/smb2_server.c | 13 +++++++------
 1 file changed, 7 insertions(+), 6 deletions(-)

diff --git a/source3/smbd/smb2_server.c b/source3/smbd/smb2_server.c
index e0d906f..c7506cb 100644
--- a/source3/smbd/smb2_server.c
+++ b/source3/smbd/smb2_server.c
@@ -2526,7 +2526,7 @@ static NTSTATUS smbd_smb2_request_reply(struct smbd_smb2_request *req)
 	return NT_STATUS_OK;
 }
 
-static NTSTATUS smbd_smb2_request_next_incoming(struct smbd_server_connection *sconn);
+static NTSTATUS smbd_smb2_request_next_incoming(struct smbXsrv_connection *xconn);
 
 void smbd_smb2_request_dispatch_immediate(struct tevent_context *ctx,
 					struct tevent_immediate *im,
@@ -2535,6 +2535,7 @@ void smbd_smb2_request_dispatch_immediate(struct tevent_context *ctx,
 	struct smbd_smb2_request *req = talloc_get_type_abort(private_data,
 					struct smbd_smb2_request);
 	struct smbd_server_connection *sconn = req->sconn;
+	struct smbXsrv_connection *xconn = req->xconn;
 	NTSTATUS status;
 
 	TALLOC_FREE(im);
@@ -2551,7 +2552,7 @@ void smbd_smb2_request_dispatch_immediate(struct tevent_context *ctx,
 		return;
 	}
 
-	status = smbd_smb2_request_next_incoming(sconn);
+	status = smbd_smb2_request_next_incoming(xconn);
 	if (!NT_STATUS_IS_OK(status)) {
 		smbd_server_connection_terminate(sconn, nt_errstr(status));
 		return;
@@ -2954,9 +2955,9 @@ static bool is_smb2_recvfile_write(struct smbd_smb2_request_read_state *state)
 	return true;
 }
 
-static NTSTATUS smbd_smb2_request_next_incoming(struct smbd_server_connection *sconn)
+static NTSTATUS smbd_smb2_request_next_incoming(struct smbXsrv_connection *xconn)
 {
-	struct smbXsrv_connection *xconn = sconn->conn;
+	struct smbd_server_connection *sconn = xconn->sconn;
 	struct smbd_smb2_request_read_state *state = &xconn->smb2.request_read_state;
 	size_t max_send_queue_len;
 	size_t cur_send_queue_len;
@@ -3043,7 +3044,7 @@ void smbd_smb2_first_negprot(struct smbXsrv_connection *xconn,
 		return;
 	}
 
-	status = smbd_smb2_request_next_incoming(sconn);
+	status = smbd_smb2_request_next_incoming(xconn);
 	if (!NT_STATUS_IS_OK(status)) {
 		smbd_server_connection_terminate(sconn, nt_errstr(status));
 		return;
@@ -3433,7 +3434,7 @@ got_full:
 		check_log_size();
 	}
 
-	status = smbd_smb2_request_next_incoming(sconn);
+	status = smbd_smb2_request_next_incoming(xconn);
 	if (!NT_STATUS_IS_OK(status)) {
 		return status;
 	}
-- 
1.8.1.2


From 03ed1ba8c33bfeb7d361bb3653d856709491f805 Mon Sep 17 00:00:00 2001
From: Stefan Metzmacher <metze@samba.org>
Date: Wed, 11 Jun 2014 11:19:24 +0200
Subject: [PATCH 51/94] s3:smb2_server: use req->xconn in
 smbd_smb2_request_setup_out()

Signed-off-by: Stefan Metzmacher <metze@samba.org>
Reviewed-by: Volker Lendecke <vl@samba.org>
---
 source3/smbd/smb2_server.c | 2 +-
 1 file changed, 1 insertion(+), 1 deletion(-)

diff --git a/source3/smbd/smb2_server.c b/source3/smbd/smb2_server.c
index c7506cb..a6f0833 100644
--- a/source3/smbd/smb2_server.c
+++ b/source3/smbd/smb2_server.c
@@ -936,7 +936,7 @@ DATA_BLOB smbd_smb2_generate_outbody(struct smbd_smb2_request *req, size_t size)
 
 static NTSTATUS smbd_smb2_request_setup_out(struct smbd_smb2_request *req)
 {
-	struct smbXsrv_connection *xconn = req->sconn->conn;
+	struct smbXsrv_connection *xconn = req->xconn;
 	TALLOC_CTX *mem_ctx;
 	struct iovec *vector;
 	int count;
-- 
1.8.1.2


From 29799b1bc169b6fba361b85861b128dddf98cca2 Mon Sep 17 00:00:00 2001
From: Stefan Metzmacher <metze@samba.org>
Date: Wed, 11 Jun 2014 11:22:16 +0200
Subject: [PATCH 52/94] s3:smb2_server: talloc smbd_smb2_request_pending_state
 on smbXsrv_connection

Signed-off-by: Stefan Metzmacher <metze@samba.org>
Reviewed-by: Volker Lendecke <vl@samba.org>
---
 source3/smbd/smb2_server.c | 2 +-
 1 file changed, 1 insertion(+), 1 deletion(-)

diff --git a/source3/smbd/smb2_server.c b/source3/smbd/smb2_server.c
index a6f0833..b24d47c 100644
--- a/source3/smbd/smb2_server.c
+++ b/source3/smbd/smb2_server.c
@@ -1469,7 +1469,7 @@ static void smbd_smb2_request_pending_timer(struct tevent_context *ev,
 	 * of this fact sometime when refactoring. JRA.
 	 */
 
-	state = talloc_zero(req->sconn, struct smbd_smb2_request_pending_state);
+	state = talloc_zero(req->xconn, struct smbd_smb2_request_pending_state);
 	if (state == NULL) {
 		smbd_server_connection_terminate(req->sconn,
 						 nt_errstr(NT_STATUS_NO_MEMORY));
-- 
1.8.1.2


From c36559eab3a0033b4e542d0660ab8e2663078b8f Mon Sep 17 00:00:00 2001
From: Stefan Metzmacher <metze@samba.org>
Date: Wed, 11 Jun 2014 11:22:16 +0200
Subject: [PATCH 53/94] s3:smb2_server: remove unused 'sconn' from
 smbd_smb2_request_pending_state

Signed-off-by: Stefan Metzmacher <metze@samba.org>
Reviewed-by: Volker Lendecke <vl@samba.org>
---
 source3/smbd/smb2_server.c | 7 ++-----
 1 file changed, 2 insertions(+), 5 deletions(-)

diff --git a/source3/smbd/smb2_server.c b/source3/smbd/smb2_server.c
index b24d47c..b34bea8 100644
--- a/source3/smbd/smb2_server.c
+++ b/source3/smbd/smb2_server.c
@@ -1278,7 +1278,6 @@ static NTSTATUS smb2_send_async_interim_response(const struct smbd_smb2_request
 }
 
 struct smbd_smb2_request_pending_state {
-        struct smbd_server_connection *sconn;
 	struct smbd_smb2_send_queue queue_entry;
         uint8_t buf[NBT_HDR_SIZE + SMB2_TF_HDR_SIZE + SMB2_HDR_BODY + 0x08 + 1];
         struct iovec vector[1 + SMBD_SMB2_NUM_IOV_PER_REQ];
@@ -1426,8 +1425,7 @@ static void smbd_smb2_request_pending_timer(struct tevent_context *ev,
 	struct smbd_smb2_request *req =
 		talloc_get_type_abort(private_data,
 		struct smbd_smb2_request);
-	struct smbd_server_connection *sconn = req->sconn;
-	struct smbXsrv_connection *xconn = sconn->conn;
+	struct smbXsrv_connection *xconn = req->xconn;
 	struct smbd_smb2_request_pending_state *state = NULL;
 	uint8_t *outhdr = NULL;
 	const uint8_t *inhdr = NULL;
@@ -1475,7 +1473,6 @@ static void smbd_smb2_request_pending_timer(struct tevent_context *ev,
 						 nt_errstr(NT_STATUS_NO_MEMORY));
 		return;
 	}
-	state->sconn = req->sconn;
 
 	tf = state->buf + NBT_HDR_SIZE;
 	tf_len = SMB2_TF_HDR_SIZE;
@@ -1603,7 +1600,7 @@ static void smbd_smb2_request_pending_timer(struct tevent_context *ev,
 
 	status = smbd_smb2_flush_send_queue(xconn);
 	if (!NT_STATUS_IS_OK(status)) {
-		smbd_server_connection_terminate(sconn,
+		smbd_server_connection_terminate(req->sconn,
 						 nt_errstr(status));
 		return;
 	}
-- 
1.8.1.2


From 4bebb2839098fe6bb833a7cc32e351969d589cb4 Mon Sep 17 00:00:00 2001
From: Stefan Metzmacher <metze@samba.org>
Date: Wed, 11 Jun 2014 11:29:01 +0200
Subject: [PATCH 54/94] s3:smb2_server: use req->xconn in
 smbd_smb2_request_process_cancel()

Signed-off-by: Stefan Metzmacher <metze@samba.org>
Reviewed-by: Volker Lendecke <vl@samba.org>
---
 source3/smbd/smb2_server.c | 3 +--
 1 file changed, 1 insertion(+), 2 deletions(-)

diff --git a/source3/smbd/smb2_server.c b/source3/smbd/smb2_server.c
index b34bea8..6df8f0e 100644
--- a/source3/smbd/smb2_server.c
+++ b/source3/smbd/smb2_server.c
@@ -1608,8 +1608,7 @@ static void smbd_smb2_request_pending_timer(struct tevent_context *ev,
 
 static NTSTATUS smbd_smb2_request_process_cancel(struct smbd_smb2_request *req)
 {
-	struct smbd_server_connection *sconn = req->sconn;
-	struct smbXsrv_connection *xconn = sconn->conn;
+	struct smbXsrv_connection *xconn = req->xconn;
 	struct smbd_smb2_request *cur;
 	const uint8_t *inhdr;
 	uint32_t flags;
-- 
1.8.1.2


From f1d731ffc7e80203ad83dfe7877f6266a4093382 Mon Sep 17 00:00:00 2001
From: Stefan Metzmacher <metze@samba.org>
Date: Wed, 11 Jun 2014 11:30:23 +0200
Subject: [PATCH 55/94] s3:smb2_server: use req->xconn in
 smbd_smb2_request_check_session()

Signed-off-by: Stefan Metzmacher <metze@samba.org>
Reviewed-by: Volker Lendecke <vl@samba.org>
---
 source3/smbd/smb2_server.c | 6 +++---
 1 file changed, 3 insertions(+), 3 deletions(-)

diff --git a/source3/smbd/smb2_server.c b/source3/smbd/smb2_server.c
index 6df8f0e..04f1e4e 100644
--- a/source3/smbd/smb2_server.c
+++ b/source3/smbd/smb2_server.c
@@ -1752,7 +1752,7 @@ static NTSTATUS smbd_smb2_request_check_session(struct smbd_smb2_request *req)
 	req->last_session_id = 0;
 
 	/* lookup an existing session */
-	status = smb2srv_session_lookup(req->sconn->conn,
+	status = smb2srv_session_lookup(req->xconn,
 					in_session_id, now,
 					&session);
 	if (session) {
@@ -1794,8 +1794,8 @@ static NTSTATUS smbd_smb2_request_check_session(struct smbd_smb2_request *req)
 		return NT_STATUS_INVALID_HANDLE;
 	}
 
-	if (in_session_id != req->sconn->conn->last_session_id) {
-		req->sconn->conn->last_session_id = in_session_id;
+	if (in_session_id != req->xconn->last_session_id) {
+		req->xconn->last_session_id = in_session_id;
 		set_current_user_info(session_info->unix_info->sanitized_username,
 				      session_info->unix_info->unix_name,
 				      session_info->info->domain_name);
-- 
1.8.1.2


From 25ed7abb6196c4afd3939e1304d7909f288bb83c Mon Sep 17 00:00:00 2001
From: Stefan Metzmacher <metze@samba.org>
Date: Wed, 11 Jun 2014 11:31:04 +0200
Subject: [PATCH 56/94] s3:smb2_server: use req->xconn in
 smbd_smb2_request_verify_creditcharge()

Signed-off-by: Stefan Metzmacher <metze@samba.org>
Reviewed-by: Volker Lendecke <vl@samba.org>
---
 source3/smbd/smb2_server.c | 2 +-
 1 file changed, 1 insertion(+), 1 deletion(-)

diff --git a/source3/smbd/smb2_server.c b/source3/smbd/smb2_server.c
index 04f1e4e..5181623 100644
--- a/source3/smbd/smb2_server.c
+++ b/source3/smbd/smb2_server.c
@@ -1807,7 +1807,7 @@ static NTSTATUS smbd_smb2_request_check_session(struct smbd_smb2_request *req)
 NTSTATUS smbd_smb2_request_verify_creditcharge(struct smbd_smb2_request *req,
 						uint32_t data_length)
 {
-	struct smbXsrv_connection *xconn = req->sconn->conn;
+	struct smbXsrv_connection *xconn = req->xconn;
 	uint16_t needed_charge;
 	uint16_t credit_charge = 1;
 	const uint8_t *inhdr;
-- 
1.8.1.2


From 275c9d3c739d9fc769e10d05a3448c3656faf50d Mon Sep 17 00:00:00 2001
From: Stefan Metzmacher <metze@samba.org>
Date: Wed, 11 Jun 2014 11:31:37 +0200
Subject: [PATCH 57/94] s3:smb2_server: use req->xconn in
 smbd_smb2_request_dispatch()

Signed-off-by: Stefan Metzmacher <metze@samba.org>
Reviewed-by: Volker Lendecke <vl@samba.org>
---
 source3/smbd/smb2_server.c | 2 +-
 1 file changed, 1 insertion(+), 1 deletion(-)

diff --git a/source3/smbd/smb2_server.c b/source3/smbd/smb2_server.c
index 5181623..4b79e14 100644
--- a/source3/smbd/smb2_server.c
+++ b/source3/smbd/smb2_server.c
@@ -1906,7 +1906,7 @@ NTSTATUS smbd_smb2_request_verify_sizes(struct smbd_smb2_request *req,
 
 NTSTATUS smbd_smb2_request_dispatch(struct smbd_smb2_request *req)
 {
-	struct smbXsrv_connection *xconn = req->sconn->conn;
+	struct smbXsrv_connection *xconn = req->xconn;
 	const struct smbd_smb2_dispatch_table *call = NULL;
 	const struct iovec *intf_v = SMBD_SMB2_IN_TF_IOV(req);
 	const uint8_t *inhdr;
-- 
1.8.1.2


From 1aaba2be3402846b681ee56859d7143e92b4e493 Mon Sep 17 00:00:00 2001
From: Stefan Metzmacher <metze@samba.org>
Date: Wed, 11 Jun 2014 11:40:04 +0200
Subject: [PATCH 58/94] s3:smb2_server: use req->xconn in
 smbd_smb2_request_error_ex()

Signed-off-by: Stefan Metzmacher <metze@samba.org>
Reviewed-by: Volker Lendecke <vl@samba.org>
---
 source3/smbd/smb2_server.c | 2 +-
 1 file changed, 1 insertion(+), 1 deletion(-)

diff --git a/source3/smbd/smb2_server.c b/source3/smbd/smb2_server.c
index 4b79e14..9d694ea 100644
--- a/source3/smbd/smb2_server.c
+++ b/source3/smbd/smb2_server.c
@@ -2663,7 +2663,7 @@ NTSTATUS smbd_smb2_request_error_ex(struct smbd_smb2_request *req,
 				    DATA_BLOB *info,
 				    const char *location)
 {
-	struct smbXsrv_connection *xconn = req->sconn->conn;
+	struct smbXsrv_connection *xconn = req->xconn;
 	DATA_BLOB body;
 	DATA_BLOB _dyn;
 	uint8_t *outhdr = SMBD_SMB2_OUT_HDR_PTR(req);
-- 
1.8.1.2


From f4a1cbfc0b36aa89d2cbb0c6ef661aa86b37b843 Mon Sep 17 00:00:00 2001
From: Stefan Metzmacher <metze@samba.org>
Date: Wed, 11 Jun 2014 11:40:36 +0200
Subject: [PATCH 59/94] s3:smb2_server: remove unused 'sconn' from
 smbd_smb2_send_break_state

Signed-off-by: Stefan Metzmacher <metze@samba.org>
Reviewed-by: Volker Lendecke <vl@samba.org>
---
 source3/smbd/smb2_server.c | 2 --
 1 file changed, 2 deletions(-)

diff --git a/source3/smbd/smb2_server.c b/source3/smbd/smb2_server.c
index 9d694ea..e8a783a 100644
--- a/source3/smbd/smb2_server.c
+++ b/source3/smbd/smb2_server.c
@@ -2724,7 +2724,6 @@ NTSTATUS smbd_smb2_request_error_ex(struct smbd_smb2_request *req,
 
 
 struct smbd_smb2_send_break_state {
-	struct smbd_server_connection *sconn;
 	struct smbd_smb2_send_queue queue_entry;
 	uint8_t nbt_hdr[NBT_HDR_SIZE];
 	uint8_t tf[SMB2_TF_HDR_SIZE];
@@ -2759,7 +2758,6 @@ static NTSTATUS smbd_smb2_send_break(struct smbd_server_connection *sconn,
 		return NT_STATUS_NO_MEMORY;
 	}
 	talloc_set_name_const(state, "struct smbd_smb2_send_break_state");
-	state->sconn = sconn;
 
 	if (do_encryption) {
 		nonce_high = session->nonce_high;
-- 
1.8.1.2


From 357dd7c1761e2532c5980ca8886db104d6a89c7c Mon Sep 17 00:00:00 2001
From: Stefan Metzmacher <metze@samba.org>
Date: Wed, 11 Jun 2014 11:41:27 +0200
Subject: [PATCH 60/94] s3:smb2_server: use req->xconn in
 is_smb2_recvfile_write()

Signed-off-by: Stefan Metzmacher <metze@samba.org>
Reviewed-by: Volker Lendecke <vl@samba.org>
---
 source3/smbd/smb2_server.c | 2 +-
 1 file changed, 1 insertion(+), 1 deletion(-)

diff --git a/source3/smbd/smb2_server.c b/source3/smbd/smb2_server.c
index e8a783a..3de78b0 100644
--- a/source3/smbd/smb2_server.c
+++ b/source3/smbd/smb2_server.c
@@ -2919,7 +2919,7 @@ static bool is_smb2_recvfile_write(struct smbd_smb2_request_read_state *state)
 	file_id_persistent	= BVAL(body, 0x10);
 	file_id_volatile	= BVAL(body, 0x18);
 
-	status = smb2srv_open_lookup(state->req->sconn->conn,
+	status = smb2srv_open_lookup(state->req->xconn,
 				     file_id_persistent,
 				     file_id_volatile,
 				     0, /* now */
-- 
1.8.1.2


From 802e632711d8bf44786ebe73bbf06b964a1949b8 Mon Sep 17 00:00:00 2001
From: Stefan Metzmacher <metze@samba.org>
Date: Thu, 12 Jun 2014 08:38:48 +0200
Subject: [PATCH 61/94] s3:smb2_*: make use of smb2req->xconn where possible

We need to use the connection that is used by the current request.

Signed-off-by: Stefan Metzmacher <metze@samba.org>
Reviewed-by: Volker Lendecke <vl@samba.org>
---
 source3/smbd/smb2_create.c           |  2 +-
 source3/smbd/smb2_find.c             |  2 +-
 source3/smbd/smb2_getinfo.c          |  2 +-
 source3/smbd/smb2_ioctl_network_fs.c |  2 +-
 source3/smbd/smb2_negprot.c          |  4 ++--
 source3/smbd/smb2_notify.c           |  2 +-
 source3/smbd/smb2_read.c             |  4 ++--
 source3/smbd/smb2_sesssetup.c        | 10 +++++-----
 source3/smbd/smb2_setinfo.c          |  2 +-
 source3/smbd/smb2_tcon.c             |  4 ++--
 source3/smbd/smb2_write.c            |  2 +-
 11 files changed, 18 insertions(+), 18 deletions(-)

diff --git a/source3/smbd/smb2_create.c b/source3/smbd/smb2_create.c
index 0c27a01..7565201 100644
--- a/source3/smbd/smb2_create.c
+++ b/source3/smbd/smb2_create.c
@@ -820,7 +820,7 @@ static struct tevent_req *smbd_smb2_create_send(TALLOC_CTX *mem_ctx,
 			DATA_BLOB new_cookie = data_blob_null;
 			NTTIME now = timeval_to_nttime(&smb2req->request_time);
 
-			status = smb2srv_open_recreate(smb2req->sconn->conn,
+			status = smb2srv_open_recreate(smb2req->xconn,
 						smb1req->conn->session_info,
 						persistent_id, create_guid,
 						now, &op);
diff --git a/source3/smbd/smb2_find.c b/source3/smbd/smb2_find.c
index 2d310a5..7597713 100644
--- a/source3/smbd/smb2_find.c
+++ b/source3/smbd/smb2_find.c
@@ -207,7 +207,7 @@ static struct tevent_req *smbd_smb2_find_send(TALLOC_CTX *mem_ctx,
 					      uint32_t in_output_buffer_length,
 					      const char *in_file_name)
 {
-	struct smbXsrv_connection *xconn = smb2req->sconn->conn;
+	struct smbXsrv_connection *xconn = smb2req->xconn;
 	struct tevent_req *req;
 	struct smbd_smb2_find_state *state;
 	struct smb_request *smbreq;
diff --git a/source3/smbd/smb2_getinfo.c b/source3/smbd/smb2_getinfo.c
index 275792c..87bb1c5 100644
--- a/source3/smbd/smb2_getinfo.c
+++ b/source3/smbd/smb2_getinfo.c
@@ -44,7 +44,7 @@ static NTSTATUS smbd_smb2_getinfo_recv(struct tevent_req *req,
 static void smbd_smb2_request_getinfo_done(struct tevent_req *subreq);
 NTSTATUS smbd_smb2_request_process_getinfo(struct smbd_smb2_request *req)
 {
-	struct smbXsrv_connection *xconn = req->sconn->conn;
+	struct smbXsrv_connection *xconn = req->xconn;
 	NTSTATUS status;
 	const uint8_t *inbody;
 	uint8_t in_info_type;
diff --git a/source3/smbd/smb2_ioctl_network_fs.c b/source3/smbd/smb2_ioctl_network_fs.c
index b2dfb21..30d02a2 100644
--- a/source3/smbd/smb2_ioctl_network_fs.c
+++ b/source3/smbd/smb2_ioctl_network_fs.c
@@ -506,7 +506,7 @@ struct tevent_req *smb2_ioctl_network_fs(uint32_t ctl_code,
 		break;
 	case FSCTL_VALIDATE_NEGOTIATE_INFO:
 		status = fsctl_validate_neg_info(state, ev,
-						 state->smbreq->sconn->conn,
+						 state->smbreq->xconn,
 						 &state->in_input,
 						 state->in_max_output,
 						 &state->out_output,
diff --git a/source3/smbd/smb2_negprot.c b/source3/smbd/smb2_negprot.c
index 3f76f3f..feb5a4d 100644
--- a/source3/smbd/smb2_negprot.c
+++ b/source3/smbd/smb2_negprot.c
@@ -78,7 +78,7 @@ void reply_smb2002(struct smb_request *req, uint16_t choice)
  */
 void reply_smb20ff(struct smb_request *req, uint16_t choice)
 {
-	struct smbXsrv_connection *xconn = req->sconn->conn;
+	struct smbXsrv_connection *xconn = req->xconn;
 	xconn->smb2.allow_2ff = true;
 	reply_smb20xx(req, SMB2_DIALECT_REVISION_2FF);
 }
@@ -170,7 +170,7 @@ enum protocol_types smbd_smb2_protocol_dialect_match(const uint8_t *indyn,
 
 NTSTATUS smbd_smb2_request_process_negprot(struct smbd_smb2_request *req)
 {
-	struct smbXsrv_connection *xconn = req->sconn->conn;
+	struct smbXsrv_connection *xconn = req->xconn;
 	NTSTATUS status;
 	const uint8_t *inbody;
 	const uint8_t *indyn = NULL;
diff --git a/source3/smbd/smb2_notify.c b/source3/smbd/smb2_notify.c
index 3e33d42..39be829 100644
--- a/source3/smbd/smb2_notify.c
+++ b/source3/smbd/smb2_notify.c
@@ -48,7 +48,7 @@ static NTSTATUS smbd_smb2_notify_recv(struct tevent_req *req,
 static void smbd_smb2_request_notify_done(struct tevent_req *subreq);
 NTSTATUS smbd_smb2_request_process_notify(struct smbd_smb2_request *req)
 {
-	struct smbXsrv_connection *xconn = req->sconn->conn;
+	struct smbXsrv_connection *xconn = req->xconn;
 	NTSTATUS status;
 	const uint8_t *inbody;
 	uint16_t in_flags;
diff --git a/source3/smbd/smb2_read.c b/source3/smbd/smb2_read.c
index 1679944..b8ae743 100644
--- a/source3/smbd/smb2_read.c
+++ b/source3/smbd/smb2_read.c
@@ -43,7 +43,7 @@ static NTSTATUS smbd_smb2_read_recv(struct tevent_req *req,
 static void smbd_smb2_request_read_done(struct tevent_req *subreq);
 NTSTATUS smbd_smb2_request_process_read(struct smbd_smb2_request *req)
 {
-	struct smbXsrv_connection *xconn = req->sconn->conn;
+	struct smbXsrv_connection *xconn = req->xconn;
 	NTSTATUS status;
 	const uint8_t *inbody;
 	uint32_t in_length;
@@ -186,7 +186,7 @@ static int smb2_sendfile_send_data(struct smbd_smb2_read_state *state)
 	uint64_t in_offset = state->in_offset;
 	files_struct *fsp = state->fsp;
 	const DATA_BLOB *hdr = state->smb2req->queue_entry.sendfile_header;
-	struct smbXsrv_connection *xconn = state->smb2req->sconn->conn;
+	struct smbXsrv_connection *xconn = state->smb2req->xconn;
 	ssize_t nread;
 	ssize_t ret;
 	int saved_errno;
diff --git a/source3/smbd/smb2_sesssetup.c b/source3/smbd/smb2_sesssetup.c
index d4f0d79..3a80afd 100644
--- a/source3/smbd/smb2_sesssetup.c
+++ b/source3/smbd/smb2_sesssetup.c
@@ -479,7 +479,7 @@ static int smbd_smb2_session_setup_state_destructor(struct smbd_smb2_session_set
 	 * Ensure that any outstanding requests don't also refer
 	 * to it.
 	 */
-	xconn = state->smb2req->sconn->conn;
+	xconn = state->smb2req->xconn;
 
 	for (preq = xconn->smb2.requests; preq != NULL; preq = preq->next) {
 		if (preq == state->smb2req) {
@@ -549,7 +549,7 @@ static struct tevent_req *smbd_smb2_session_setup_send(TALLOC_CTX *mem_ctx,
 	state->in_security_buffer = in_security_buffer;
 
 	if (in_flags & SMB2_SESSION_FLAG_BINDING) {
-		if (smb2req->sconn->conn->protocol < PROTOCOL_SMB2_22) {
+		if (smb2req->xconn->protocol < PROTOCOL_SMB2_22) {
 			tevent_req_nterror(req, NT_STATUS_REQUEST_NOT_ACCEPTED);
 			return tevent_req_post(req, ev);
 		}
@@ -565,13 +565,13 @@ static struct tevent_req *smbd_smb2_session_setup_send(TALLOC_CTX *mem_ctx,
 
 	if (state->in_session_id == 0) {
 		/* create a new session */
-		status = smbXsrv_session_create(state->smb2req->sconn->conn,
+		status = smbXsrv_session_create(state->smb2req->xconn,
 					        now, &state->session);
 		if (tevent_req_nterror(req, status)) {
 			return tevent_req_post(req, ev);
 		}
 	} else {
-		status = smb2srv_session_lookup(state->smb2req->sconn->conn,
+		status = smb2srv_session_lookup(state->smb2req->xconn,
 						state->in_session_id, now,
 						&state->session);
 		if (NT_STATUS_EQUAL(status, NT_STATUS_NETWORK_SESSION_EXPIRED)) {
@@ -877,7 +877,7 @@ static struct tevent_req *smbd_smb2_logoff_send(TALLOC_CTX *mem_ctx,
 	struct smbd_smb2_logout_state *state;
 	struct tevent_req *subreq;
 	struct smbd_smb2_request *preq;
-	struct smbXsrv_connection *xconn = smb2req->sconn->conn;
+	struct smbXsrv_connection *xconn = smb2req->xconn;
 
 	req = tevent_req_create(mem_ctx, &state,
 			struct smbd_smb2_logout_state);
diff --git a/source3/smbd/smb2_setinfo.c b/source3/smbd/smb2_setinfo.c
index 1b5e2a8..031c0be 100644
--- a/source3/smbd/smb2_setinfo.c
+++ b/source3/smbd/smb2_setinfo.c
@@ -39,7 +39,7 @@ static NTSTATUS smbd_smb2_setinfo_recv(struct tevent_req *req);
 static void smbd_smb2_request_setinfo_done(struct tevent_req *subreq);
 NTSTATUS smbd_smb2_request_process_setinfo(struct smbd_smb2_request *req)
 {
-	struct smbXsrv_connection *xconn = req->sconn->conn;
+	struct smbXsrv_connection *xconn = req->xconn;
 	NTSTATUS status;
 	const uint8_t *inbody;
 	uint8_t in_info_type;
diff --git a/source3/smbd/smb2_tcon.c b/source3/smbd/smb2_tcon.c
index 6289427..5d78b4f 100644
--- a/source3/smbd/smb2_tcon.c
+++ b/source3/smbd/smb2_tcon.c
@@ -175,7 +175,7 @@ static NTSTATUS smbd_smb2_tree_connect(struct smbd_smb2_request *req,
 				       uint32_t *out_maximal_access,
 				       uint32_t *out_tree_id)
 {
-	struct smbXsrv_connection *conn = req->sconn->conn;
+	struct smbXsrv_connection *conn = req->xconn;
 	const char *share = in_path;
 	char *service = NULL;
 	int snum = -1;
@@ -498,7 +498,7 @@ static struct tevent_req *smbd_smb2_tdis_send(TALLOC_CTX *mem_ctx,
 	struct smbd_smb2_tdis_state *state;
 	struct tevent_req *subreq;
 	struct smbd_smb2_request *preq;
-	struct smbXsrv_connection *xconn = smb2req->sconn->conn;
+	struct smbXsrv_connection *xconn = smb2req->xconn;
 
 	req = tevent_req_create(mem_ctx, &state,
 			struct smbd_smb2_tdis_state);
diff --git a/source3/smbd/smb2_write.c b/source3/smbd/smb2_write.c
index 40a5937..35ee020 100644
--- a/source3/smbd/smb2_write.c
+++ b/source3/smbd/smb2_write.c
@@ -38,7 +38,7 @@ static NTSTATUS smbd_smb2_write_recv(struct tevent_req *req,
 static void smbd_smb2_request_write_done(struct tevent_req *subreq);
 NTSTATUS smbd_smb2_request_process_write(struct smbd_smb2_request *req)
 {
-	struct smbXsrv_connection *xconn = req->sconn->conn;
+	struct smbXsrv_connection *xconn = req->xconn;
 	NTSTATUS status;
 	const uint8_t *inbody;
 	uint16_t in_data_offset;
-- 
1.8.1.2


From 828a9373a102bfb0f1a7ad79ba93a5f31841da52 Mon Sep 17 00:00:00 2001
From: Stefan Metzmacher <metze@samba.org>
Date: Wed, 11 Jun 2014 12:15:48 +0200
Subject: [PATCH 62/94] s3:smb2_server: pass smbXsrv_connection to
 smbd_server_connection_terminate*()

Signed-off-by: Stefan Metzmacher <metze@samba.org>
Reviewed-by: Volker Lendecke <vl@samba.org>
---
 source3/smbd/globals.h         |  6 +++---
 source3/smbd/process.c         |  3 +--
 source3/smbd/smb2_break.c      | 14 ++++++++------
 source3/smbd/smb2_close.c      |  6 +++---
 source3/smbd/smb2_create.c     | 14 +++++++-------
 source3/smbd/smb2_find.c       |  6 +++---
 source3/smbd/smb2_flush.c      |  6 +++---
 source3/smbd/smb2_getinfo.c    |  8 ++++----
 source3/smbd/smb2_ioctl.c      |  8 ++++----
 source3/smbd/smb2_lock.c       |  6 +++---
 source3/smbd/smb2_notify.c     |  6 +++---
 source3/smbd/smb2_read.c       |  6 +++---
 source3/smbd/smb2_server.c     | 33 +++++++++++++++------------------
 source3/smbd/smb2_sesssetup.c  | 12 ++++++------
 source3/smbd/smb2_setinfo.c    |  6 +++---
 source3/smbd/smb2_tcon.c       | 12 ++++++------
 source3/smbd/smb2_write.c      |  6 +++---
 source3/smbd/smbXsrv_session.c |  2 +-
 18 files changed, 79 insertions(+), 81 deletions(-)

diff --git a/source3/smbd/globals.h b/source3/smbd/globals.h
index 9ec58f3..bf168ac 100644
--- a/source3/smbd/globals.h
+++ b/source3/smbd/globals.h
@@ -213,11 +213,11 @@ NTSTATUS smbd_calculate_access_mask(connection_struct *conn,
 
 void smbd_notify_cancel_by_smbreq(const struct smb_request *smbreq);
 
-void smbd_server_connection_terminate_ex(struct smbd_server_connection *sconn,
+void smbd_server_connection_terminate_ex(struct smbXsrv_connection *xconn,
 					 const char *reason,
 					 const char *location);
-#define smbd_server_connection_terminate(sconn, reason) \
-	smbd_server_connection_terminate_ex(sconn, reason, __location__)
+#define smbd_server_connection_terminate(xconn, reason) \
+	smbd_server_connection_terminate_ex(xconn, reason, __location__)
 
 const char *smb2_opcode_name(uint16_t opcode);
 bool smbd_is_smb2_header(const uint8_t *inbuf, size_t size);
diff --git a/source3/smbd/process.c b/source3/smbd/process.c
index b970f00..02d43d5 100644
--- a/source3/smbd/process.c
+++ b/source3/smbd/process.c
@@ -2583,7 +2583,6 @@ static void smbd_release_ip_immediate(struct tevent_context *ctx,
 		talloc_get_type_abort(private_data,
 		struct smbd_release_ip_state);
 	struct smbXsrv_connection *xconn = state->xconn;
-	struct smbd_server_connection *sconn = xconn->sconn;
 
 	if (!NT_STATUS_EQUAL(xconn->transport.status, NT_STATUS_ADDRESS_CLOSED)) {
 		/*
@@ -2592,7 +2591,7 @@ static void smbd_release_ip_immediate(struct tevent_context *ctx,
 		return;
 	}
 
-	smbd_server_connection_terminate(sconn, "CTDB_SRVID_RELEASE_IP");
+	smbd_server_connection_terminate(xconn, "CTDB_SRVID_RELEASE_IP");
 }
 
 /****************************************************************************
diff --git a/source3/smbd/smb2_break.c b/source3/smbd/smb2_break.c
index 6446b79..5492186 100644
--- a/source3/smbd/smb2_break.c
+++ b/source3/smbd/smb2_break.c
@@ -94,7 +94,7 @@ static void smbd_smb2_request_oplock_break_done(struct tevent_req *subreq)
 	if (!NT_STATUS_IS_OK(status)) {
 		error = smbd_smb2_request_error(req, status);
 		if (!NT_STATUS_IS_OK(error)) {
-			smbd_server_connection_terminate(req->sconn,
+			smbd_server_connection_terminate(req->xconn,
 							 nt_errstr(error));
 			return;
 		}
@@ -110,7 +110,7 @@ static void smbd_smb2_request_oplock_break_done(struct tevent_req *subreq)
 	if (outbody.data == NULL) {
 		error = smbd_smb2_request_error(req, NT_STATUS_NO_MEMORY);
 		if (!NT_STATUS_IS_OK(error)) {
-			smbd_server_connection_terminate(req->sconn,
+			smbd_server_connection_terminate(req->xconn,
 							 nt_errstr(error));
 			return;
 		}
@@ -129,7 +129,7 @@ static void smbd_smb2_request_oplock_break_done(struct tevent_req *subreq)
 
 	error = smbd_smb2_request_done(req, outbody, NULL);
 	if (!NT_STATUS_IS_OK(error)) {
-		smbd_server_connection_terminate(req->sconn,
+		smbd_server_connection_terminate(req->xconn,
 						 nt_errstr(error));
 		return;
 	}
@@ -233,11 +233,12 @@ void send_break_message_smb2(files_struct *fsp, int level)
 				SMB2_OPLOCK_LEVEL_II :
 				SMB2_OPLOCK_LEVEL_NONE;
 	NTSTATUS status;
+	struct smbXsrv_connection *xconn = fsp->conn->sconn->conn;
 	struct smbXsrv_session *session = NULL;
 	struct timeval tv = timeval_current();
 	NTTIME now = timeval_to_nttime(&tv);
 
-	status = smb2srv_session_lookup(fsp->conn->sconn->conn,
+	status = smb2srv_session_lookup(xconn,
 					fsp->vuid,
 					now,
 					&session);
@@ -266,7 +267,8 @@ void send_break_message_smb2(files_struct *fsp, int level)
 					     fsp->op,
 					     smb2_oplock_level);
 	if (!NT_STATUS_IS_OK(status)) {
-		smbd_server_connection_terminate(fsp->conn->sconn,
-				 nt_errstr(status));
+		smbd_server_connection_terminate(xconn,
+						 nt_errstr(status));
+		return;
 	}
 }
diff --git a/source3/smbd/smb2_close.c b/source3/smbd/smb2_close.c
index 5334e1d..0e1475d 100644
--- a/source3/smbd/smb2_close.c
+++ b/source3/smbd/smb2_close.c
@@ -108,7 +108,7 @@ static void smbd_smb2_request_close_done(struct tevent_req *subreq)
 	if (!NT_STATUS_IS_OK(status)) {
 		error = smbd_smb2_request_error(req, status);
 		if (!NT_STATUS_IS_OK(error)) {
-			smbd_server_connection_terminate(req->sconn,
+			smbd_server_connection_terminate(req->xconn,
 							 nt_errstr(error));
 			return;
 		}
@@ -119,7 +119,7 @@ static void smbd_smb2_request_close_done(struct tevent_req *subreq)
 	if (outbody.data == NULL) {
 		error = smbd_smb2_request_error(req, NT_STATUS_NO_MEMORY);
 		if (!NT_STATUS_IS_OK(error)) {
-			smbd_server_connection_terminate(req->sconn,
+			smbd_server_connection_terminate(req->xconn,
 							 nt_errstr(error));
 			return;
 		}
@@ -143,7 +143,7 @@ static void smbd_smb2_request_close_done(struct tevent_req *subreq)
 
 	error = smbd_smb2_request_done(req, outbody, NULL);
 	if (!NT_STATUS_IS_OK(error)) {
-		smbd_server_connection_terminate(req->sconn,
+		smbd_server_connection_terminate(req->xconn,
 						 nt_errstr(error));
 		return;
 	}
diff --git a/source3/smbd/smb2_create.c b/source3/smbd/smb2_create.c
index 7565201..6709f51 100644
--- a/source3/smbd/smb2_create.c
+++ b/source3/smbd/smb2_create.c
@@ -297,7 +297,7 @@ static void smbd_smb2_request_create_done(struct tevent_req *tsubreq)
 	if (!NT_STATUS_IS_OK(status)) {
 		error = smbd_smb2_request_error(smb2req, status);
 		if (!NT_STATUS_IS_OK(error)) {
-			smbd_server_connection_terminate(smb2req->sconn,
+			smbd_server_connection_terminate(smb2req->xconn,
 							 nt_errstr(error));
 			return;
 		}
@@ -308,7 +308,7 @@ static void smbd_smb2_request_create_done(struct tevent_req *tsubreq)
 	if (!NT_STATUS_IS_OK(status)) {
 		error = smbd_smb2_request_error(smb2req, status);
 		if (!NT_STATUS_IS_OK(error)) {
-			smbd_server_connection_terminate(smb2req->sconn,
+			smbd_server_connection_terminate(smb2req->xconn,
 							 nt_errstr(error));
 			return;
 		}
@@ -323,7 +323,7 @@ static void smbd_smb2_request_create_done(struct tevent_req *tsubreq)
 	if (outbody.data == NULL) {
 		error = smbd_smb2_request_error(smb2req, NT_STATUS_NO_MEMORY);
 		if (!NT_STATUS_IS_OK(error)) {
-			smbd_server_connection_terminate(smb2req->sconn,
+			smbd_server_connection_terminate(smb2req->xconn,
 							 nt_errstr(error));
 			return;
 		}
@@ -368,7 +368,7 @@ static void smbd_smb2_request_create_done(struct tevent_req *tsubreq)
 
 	error = smbd_smb2_request_done(smb2req, outbody, &outdyn);
 	if (!NT_STATUS_IS_OK(error)) {
-		smbd_server_connection_terminate(smb2req->sconn,
+		smbd_server_connection_terminate(smb2req->xconn,
 						 nt_errstr(error));
 		return;
 	}
@@ -1318,7 +1318,6 @@ static void smbd_smb2_create_request_dispatch_immediate(struct tevent_context *c
 {
 	struct smbd_smb2_request *smb2req = talloc_get_type_abort(private_data,
 					struct smbd_smb2_request);
-	struct smbd_server_connection *sconn = smb2req->sconn;
 	uint64_t mid = get_mid_from_smb2req(smb2req);
 	NTSTATUS status;
 
@@ -1328,7 +1327,8 @@ static void smbd_smb2_create_request_dispatch_immediate(struct tevent_context *c
 
 	status = smbd_smb2_request_dispatch(smb2req);
 	if (!NT_STATUS_IS_OK(status)) {
-		smbd_server_connection_terminate(sconn, nt_errstr(status));
+		smbd_server_connection_terminate(smb2req->xconn,
+						 nt_errstr(status));
 		return;
 	}
 }
@@ -1375,7 +1375,7 @@ bool schedule_deferred_open_message_smb2(
 
 	state->im = tevent_create_immediate(smb2req);
 	if (!state->im) {
-		smbd_server_connection_terminate(smb2req->sconn,
+		smbd_server_connection_terminate(smb2req->xconn,
 			nt_errstr(NT_STATUS_NO_MEMORY));
 		return false;
 	}
diff --git a/source3/smbd/smb2_find.c b/source3/smbd/smb2_find.c
index 7597713..45b0890 100644
--- a/source3/smbd/smb2_find.c
+++ b/source3/smbd/smb2_find.c
@@ -153,7 +153,7 @@ static void smbd_smb2_request_find_done(struct tevent_req *subreq)
 	if (!NT_STATUS_IS_OK(status)) {
 		error = smbd_smb2_request_error(req, status);
 		if (!NT_STATUS_IS_OK(error)) {
-			smbd_server_connection_terminate(req->sconn,
+			smbd_server_connection_terminate(req->xconn,
 							 nt_errstr(error));
 			return;
 		}
@@ -166,7 +166,7 @@ static void smbd_smb2_request_find_done(struct tevent_req *subreq)
 	if (outbody.data == NULL) {
 		error = smbd_smb2_request_error(req, NT_STATUS_NO_MEMORY);
 		if (!NT_STATUS_IS_OK(error)) {
-			smbd_server_connection_terminate(req->sconn,
+			smbd_server_connection_terminate(req->xconn,
 							 nt_errstr(error));
 			return;
 		}
@@ -186,7 +186,7 @@ static void smbd_smb2_request_find_done(struct tevent_req *subreq)
 
 	error = smbd_smb2_request_done(req, outbody, &outdyn);
 	if (!NT_STATUS_IS_OK(error)) {
-		smbd_server_connection_terminate(req->sconn,
+		smbd_server_connection_terminate(req->xconn,
 						 nt_errstr(error));
 		return;
 	}
diff --git a/source3/smbd/smb2_flush.c b/source3/smbd/smb2_flush.c
index 0d4c1a5..04a8710 100644
--- a/source3/smbd/smb2_flush.c
+++ b/source3/smbd/smb2_flush.c
@@ -77,7 +77,7 @@ static void smbd_smb2_request_flush_done(struct tevent_req *subreq)
 	if (!NT_STATUS_IS_OK(status)) {
 		error = smbd_smb2_request_error(req, status);
 		if (!NT_STATUS_IS_OK(error)) {
-			smbd_server_connection_terminate(req->sconn,
+			smbd_server_connection_terminate(req->xconn,
 							 nt_errstr(error));
 			return;
 		}
@@ -88,7 +88,7 @@ static void smbd_smb2_request_flush_done(struct tevent_req *subreq)
 	if (outbody.data == NULL) {
 		error = smbd_smb2_request_error(req, NT_STATUS_NO_MEMORY);
 		if (!NT_STATUS_IS_OK(error)) {
-			smbd_server_connection_terminate(req->sconn,
+			smbd_server_connection_terminate(req->xconn,
 							 nt_errstr(error));
 			return;
 		}
@@ -100,7 +100,7 @@ static void smbd_smb2_request_flush_done(struct tevent_req *subreq)
 
 	error = smbd_smb2_request_done(req, outbody, NULL);
 	if (!NT_STATUS_IS_OK(error)) {
-		smbd_server_connection_terminate(req->sconn,
+		smbd_server_connection_terminate(req->xconn,
 						 nt_errstr(error));
 		return;
 	}
diff --git a/source3/smbd/smb2_getinfo.c b/source3/smbd/smb2_getinfo.c
index 87bb1c5..64669b8 100644
--- a/source3/smbd/smb2_getinfo.c
+++ b/source3/smbd/smb2_getinfo.c
@@ -153,7 +153,7 @@ static void smbd_smb2_request_getinfo_done(struct tevent_req *subreq)
 	if (!NT_STATUS_IS_OK(status)) {
 		error = smbd_smb2_request_error(req, status);
 		if (!NT_STATUS_IS_OK(error)) {
-			smbd_server_connection_terminate(req->sconn,
+			smbd_server_connection_terminate(req->xconn,
 							 nt_errstr(error));
 			return;
 		}
@@ -170,7 +170,7 @@ static void smbd_smb2_request_getinfo_done(struct tevent_req *subreq)
 						&out_output_buffer,
 						__location__);
 		if (!NT_STATUS_IS_OK(error)) {
-			smbd_server_connection_terminate(req->sconn,
+			smbd_server_connection_terminate(req->xconn,
 							 nt_errstr(error));
 			return;
 		}
@@ -183,7 +183,7 @@ static void smbd_smb2_request_getinfo_done(struct tevent_req *subreq)
 	if (outbody.data == NULL) {
 		error = smbd_smb2_request_error(req, NT_STATUS_NO_MEMORY);
 		if (!NT_STATUS_IS_OK(error)) {
-			smbd_server_connection_terminate(req->sconn,
+			smbd_server_connection_terminate(req->xconn,
 							 nt_errstr(error));
 			return;
 		}
@@ -200,7 +200,7 @@ static void smbd_smb2_request_getinfo_done(struct tevent_req *subreq)
 
 	error = smbd_smb2_request_done_ex(req, call_status, outbody, &outdyn, __location__);
 	if (!NT_STATUS_IS_OK(error)) {
-		smbd_server_connection_terminate(req->sconn,
+		smbd_server_connection_terminate(req->xconn,
 						 nt_errstr(error));
 		return;
 	}
diff --git a/source3/smbd/smb2_ioctl.c b/source3/smbd/smb2_ioctl.c
index 6a19fcb..993682f 100644
--- a/source3/smbd/smb2_ioctl.c
+++ b/source3/smbd/smb2_ioctl.c
@@ -293,7 +293,7 @@ static void smbd_smb2_request_ioctl_done(struct tevent_req *subreq)
 	TALLOC_FREE(subreq);
 	if (disconnect) {
 		error = status;
-		smbd_server_connection_terminate(req->sconn,
+		smbd_server_connection_terminate(req->xconn,
 						 nt_errstr(error));
 		return;
 	}
@@ -308,7 +308,7 @@ static void smbd_smb2_request_ioctl_done(struct tevent_req *subreq)
 				       out_output_buffer.length)) {
 		error = smbd_smb2_request_error(req, status);
 		if (!NT_STATUS_IS_OK(error)) {
-			smbd_server_connection_terminate(req->sconn,
+			smbd_server_connection_terminate(req->xconn,
 							 nt_errstr(error));
 			return;
 		}
@@ -322,7 +322,7 @@ static void smbd_smb2_request_ioctl_done(struct tevent_req *subreq)
 	if (outbody.data == NULL) {
 		error = smbd_smb2_request_error(req, NT_STATUS_NO_MEMORY);
 		if (!NT_STATUS_IS_OK(error)) {
-			smbd_server_connection_terminate(req->sconn,
+			smbd_server_connection_terminate(req->xconn,
 							 nt_errstr(error));
 			return;
 		}
@@ -357,7 +357,7 @@ static void smbd_smb2_request_ioctl_done(struct tevent_req *subreq)
 	error = smbd_smb2_request_done_ex(req, status, outbody, &outdyn,
 					  __location__);
 	if (!NT_STATUS_IS_OK(error)) {
-		smbd_server_connection_terminate(req->sconn,
+		smbd_server_connection_terminate(req->xconn,
 						 nt_errstr(error));
 		return;
 	}
diff --git a/source3/smbd/smb2_lock.c b/source3/smbd/smb2_lock.c
index 8e1eb40..0713892 100644
--- a/source3/smbd/smb2_lock.c
+++ b/source3/smbd/smb2_lock.c
@@ -139,7 +139,7 @@ static void smbd_smb2_request_lock_done(struct tevent_req *subreq)
 	if (!NT_STATUS_IS_OK(status)) {
 		error = smbd_smb2_request_error(smb2req, status);
 		if (!NT_STATUS_IS_OK(error)) {
-			smbd_server_connection_terminate(smb2req->sconn,
+			smbd_server_connection_terminate(smb2req->xconn,
 							 nt_errstr(error));
 			return;
 		}
@@ -150,7 +150,7 @@ static void smbd_smb2_request_lock_done(struct tevent_req *subreq)
 	if (outbody.data == NULL) {
 		error = smbd_smb2_request_error(smb2req, NT_STATUS_NO_MEMORY);
 		if (!NT_STATUS_IS_OK(error)) {
-			smbd_server_connection_terminate(smb2req->sconn,
+			smbd_server_connection_terminate(smb2req->xconn,
 							 nt_errstr(error));
 			return;
 		}
@@ -162,7 +162,7 @@ static void smbd_smb2_request_lock_done(struct tevent_req *subreq)
 
 	error = smbd_smb2_request_done(smb2req, outbody, NULL);
 	if (!NT_STATUS_IS_OK(error)) {
-		smbd_server_connection_terminate(smb2req->sconn,
+		smbd_server_connection_terminate(smb2req->xconn,
 						 nt_errstr(error));
 		return;
 	}
diff --git a/source3/smbd/smb2_notify.c b/source3/smbd/smb2_notify.c
index 39be829..7b92927 100644
--- a/source3/smbd/smb2_notify.c
+++ b/source3/smbd/smb2_notify.c
@@ -122,7 +122,7 @@ static void smbd_smb2_request_notify_done(struct tevent_req *subreq)
 	if (!NT_STATUS_IS_OK(status)) {
 		error = smbd_smb2_request_error(req, status);
 		if (!NT_STATUS_IS_OK(error)) {
-			smbd_server_connection_terminate(req->sconn,
+			smbd_server_connection_terminate(req->xconn,
 							 nt_errstr(error));
 			return;
 		}
@@ -135,7 +135,7 @@ static void smbd_smb2_request_notify_done(struct tevent_req *subreq)
 	if (outbody.data == NULL) {
 		error = smbd_smb2_request_error(req, NT_STATUS_NO_MEMORY);
 		if (!NT_STATUS_IS_OK(error)) {
-			smbd_server_connection_terminate(req->sconn,
+			smbd_server_connection_terminate(req->xconn,
 							 nt_errstr(error));
 			return;
 		}
@@ -152,7 +152,7 @@ static void smbd_smb2_request_notify_done(struct tevent_req *subreq)
 
 	error = smbd_smb2_request_done(req, outbody, &outdyn);
 	if (!NT_STATUS_IS_OK(error)) {
-		smbd_server_connection_terminate(req->sconn,
+		smbd_server_connection_terminate(req->xconn,
 						 nt_errstr(error));
 		return;
 	}
diff --git a/source3/smbd/smb2_read.c b/source3/smbd/smb2_read.c
index b8ae743..892fa15 100644
--- a/source3/smbd/smb2_read.c
+++ b/source3/smbd/smb2_read.c
@@ -120,7 +120,7 @@ static void smbd_smb2_request_read_done(struct tevent_req *subreq)
 	if (!NT_STATUS_IS_OK(status)) {
 		error = smbd_smb2_request_error(req, status);
 		if (!NT_STATUS_IS_OK(error)) {
-			smbd_server_connection_terminate(req->sconn,
+			smbd_server_connection_terminate(req->xconn,
 							 nt_errstr(error));
 			return;
 		}
@@ -133,7 +133,7 @@ static void smbd_smb2_request_read_done(struct tevent_req *subreq)
 	if (outbody.data == NULL) {
 		error = smbd_smb2_request_error(req, NT_STATUS_NO_MEMORY);
 		if (!NT_STATUS_IS_OK(error)) {
-			smbd_server_connection_terminate(req->sconn,
+			smbd_server_connection_terminate(req->xconn,
 							 nt_errstr(error));
 			return;
 		}
@@ -154,7 +154,7 @@ static void smbd_smb2_request_read_done(struct tevent_req *subreq)
 
 	error = smbd_smb2_request_done(req, outbody, &outdyn);
 	if (!NT_STATUS_IS_OK(error)) {
-		smbd_server_connection_terminate(req->sconn,
+		smbd_server_connection_terminate(req->xconn,
 						 nt_errstr(error));
 		return;
 	}
diff --git a/source3/smbd/smb2_server.c b/source3/smbd/smb2_server.c
index 3de78b0..77aa4f1 100644
--- a/source3/smbd/smb2_server.c
+++ b/source3/smbd/smb2_server.c
@@ -763,7 +763,6 @@ static void smb2_set_operation_credit(struct smbXsrv_connection *xconn,
 				      const struct iovec *in_vector,
 				      struct iovec *out_vector)
 {
-	struct smbd_server_connection *sconn = xconn->sconn;
 	const uint8_t *inhdr = (const uint8_t *)in_vector->iov_base;
 	uint8_t *outhdr = (uint8_t *)out_vector->iov_base;
 	uint16_t credit_charge = 1;
@@ -805,7 +804,7 @@ static void smb2_set_operation_credit(struct smbXsrv_connection *xconn,
 	SMB_ASSERT(xconn->smb2.credits.max >= xconn->smb2.credits.granted);
 
 	if (xconn->smb2.credits.max < credit_charge) {
-		smbd_server_connection_terminate(sconn,
+		smbd_server_connection_terminate(xconn,
 			"client error: credit charge > max credits\n");
 		return;
 	}
@@ -1039,7 +1038,7 @@ static NTSTATUS smbd_smb2_request_setup_out(struct smbd_smb2_request *req)
 	return NT_STATUS_OK;
 }
 
-void smbd_server_connection_terminate_ex(struct smbd_server_connection *sconn,
+void smbd_server_connection_terminate_ex(struct smbXsrv_connection *xconn,
 					 const char *reason,
 					 const char *location)
 {
@@ -1469,7 +1468,7 @@ static void smbd_smb2_request_pending_timer(struct tevent_context *ev,
 
 	state = talloc_zero(req->xconn, struct smbd_smb2_request_pending_state);
 	if (state == NULL) {
-		smbd_server_connection_terminate(req->sconn,
+		smbd_server_connection_terminate(xconn,
 						 nt_errstr(NT_STATUS_NO_MEMORY));
 		return;
 	}
@@ -1573,7 +1572,7 @@ static void smbd_smb2_request_pending_timer(struct tevent_context *ev,
 					&state->vector[1+SMBD_SMB2_TF_IOV_OFS],
 					SMBD_SMB2_NUM_IOV_PER_REQ);
 		if (!NT_STATUS_IS_OK(status)) {
-			smbd_server_connection_terminate(req->sconn,
+			smbd_server_connection_terminate(xconn,
 						nt_errstr(status));
 			return;
 		}
@@ -1586,7 +1585,7 @@ static void smbd_smb2_request_pending_timer(struct tevent_context *ev,
 					&state->vector[1+SMBD_SMB2_HDR_IOV_OFS],
 					SMBD_SMB2_NUM_IOV_PER_REQ - 1);
 		if (!NT_STATUS_IS_OK(status)) {
-			smbd_server_connection_terminate(req->sconn,
+			smbd_server_connection_terminate(xconn,
 						nt_errstr(status));
 			return;
 		}
@@ -1600,7 +1599,7 @@ static void smbd_smb2_request_pending_timer(struct tevent_context *ev,
 
 	status = smbd_smb2_flush_send_queue(xconn);
 	if (!NT_STATUS_IS_OK(status)) {
-		smbd_server_connection_terminate(req->sconn,
+		smbd_server_connection_terminate(xconn,
 						 nt_errstr(status));
 		return;
 	}
@@ -2530,7 +2529,6 @@ void smbd_smb2_request_dispatch_immediate(struct tevent_context *ctx,
 {
 	struct smbd_smb2_request *req = talloc_get_type_abort(private_data,
 					struct smbd_smb2_request);
-	struct smbd_server_connection *sconn = req->sconn;
 	struct smbXsrv_connection *xconn = req->xconn;
 	NTSTATUS status;
 
@@ -2544,13 +2542,13 @@ void smbd_smb2_request_dispatch_immediate(struct tevent_context *ctx,
 
 	status = smbd_smb2_request_dispatch(req);
 	if (!NT_STATUS_IS_OK(status)) {
-		smbd_server_connection_terminate(sconn, nt_errstr(status));
+		smbd_server_connection_terminate(xconn, nt_errstr(status));
 		return;
 	}
 
 	status = smbd_smb2_request_next_incoming(xconn);
 	if (!NT_STATUS_IS_OK(status)) {
-		smbd_server_connection_terminate(sconn, nt_errstr(status));
+		smbd_server_connection_terminate(xconn, nt_errstr(status));
 		return;
 	}
 }
@@ -3010,37 +3008,37 @@ void smbd_smb2_first_negprot(struct smbXsrv_connection *xconn,
 
 	status = smbd_initialize_smb2(xconn);
 	if (!NT_STATUS_IS_OK(status)) {
-		smbd_server_connection_terminate(sconn, nt_errstr(status));
+		smbd_server_connection_terminate(xconn, nt_errstr(status));
 		return;
 	}
 
 	status = smbd_smb2_request_create(xconn, inbuf, size, &req);
 	if (!NT_STATUS_IS_OK(status)) {
-		smbd_server_connection_terminate(sconn, nt_errstr(status));
+		smbd_server_connection_terminate(xconn, nt_errstr(status));
 		return;
 	}
 
 	status = smbd_smb2_request_validate(req);
 	if (!NT_STATUS_IS_OK(status)) {
-		smbd_server_connection_terminate(sconn, nt_errstr(status));
+		smbd_server_connection_terminate(xconn, nt_errstr(status));
 		return;
 	}
 
 	status = smbd_smb2_request_setup_out(req);
 	if (!NT_STATUS_IS_OK(status)) {
-		smbd_server_connection_terminate(sconn, nt_errstr(status));
+		smbd_server_connection_terminate(xconn, nt_errstr(status));
 		return;
 	}
 
 	status = smbd_smb2_request_dispatch(req);
 	if (!NT_STATUS_IS_OK(status)) {
-		smbd_server_connection_terminate(sconn, nt_errstr(status));
+		smbd_server_connection_terminate(xconn, nt_errstr(status));
 		return;
 	}
 
 	status = smbd_smb2_request_next_incoming(xconn);
 	if (!NT_STATUS_IS_OK(status)) {
-		smbd_server_connection_terminate(sconn, nt_errstr(status));
+		smbd_server_connection_terminate(xconn, nt_errstr(status));
 		return;
 	}
 
@@ -3444,12 +3442,11 @@ static void smbd_smb2_connection_handler(struct tevent_context *ev,
 	struct smbXsrv_connection *xconn =
 		talloc_get_type_abort(private_data,
 		struct smbXsrv_connection);
-	struct smbd_server_connection *sconn = xconn->sconn;
 	NTSTATUS status;
 
 	status = smbd_smb2_io_handler(xconn, flags);
 	if (!NT_STATUS_IS_OK(status)) {
-		smbd_server_connection_terminate(sconn, nt_errstr(status));
+		smbd_server_connection_terminate(xconn, nt_errstr(status));
 		return;
 	}
 }
diff --git a/source3/smbd/smb2_sesssetup.c b/source3/smbd/smb2_sesssetup.c
index 3a80afd..efeac18 100644
--- a/source3/smbd/smb2_sesssetup.c
+++ b/source3/smbd/smb2_sesssetup.c
@@ -129,7 +129,7 @@ static void smbd_smb2_request_sesssetup_done(struct tevent_req *subreq)
 		status = nt_status_squash(status);
 		error = smbd_smb2_request_error(smb2req, status);
 		if (!NT_STATUS_IS_OK(error)) {
-			smbd_server_connection_terminate(smb2req->sconn,
+			smbd_server_connection_terminate(smb2req->xconn,
 							 nt_errstr(error));
 			return;
 		}
@@ -144,7 +144,7 @@ static void smbd_smb2_request_sesssetup_done(struct tevent_req *subreq)
 	if (outbody.data == NULL) {
 		error = smbd_smb2_request_error(smb2req, NT_STATUS_NO_MEMORY);
 		if (!NT_STATUS_IS_OK(error)) {
-			smbd_server_connection_terminate(smb2req->sconn,
+			smbd_server_connection_terminate(smb2req->xconn,
 							 nt_errstr(error));
 			return;
 		}
@@ -166,7 +166,7 @@ static void smbd_smb2_request_sesssetup_done(struct tevent_req *subreq)
 	error = smbd_smb2_request_done_ex(smb2req, status, outbody, &outdyn,
 					   __location__);
 	if (!NT_STATUS_IS_OK(error)) {
-		smbd_server_connection_terminate(smb2req->sconn,
+		smbd_server_connection_terminate(smb2req->xconn,
 						 nt_errstr(error));
 		return;
 	}
@@ -833,7 +833,7 @@ static void smbd_smb2_request_logoff_done(struct tevent_req *subreq)
 	if (!NT_STATUS_IS_OK(status)) {
 		error = smbd_smb2_request_error(smb2req, status);
 		if (!NT_STATUS_IS_OK(error)) {
-			smbd_server_connection_terminate(smb2req->sconn,
+			smbd_server_connection_terminate(smb2req->xconn,
 							nt_errstr(error));
 			return;
 		}
@@ -844,7 +844,7 @@ static void smbd_smb2_request_logoff_done(struct tevent_req *subreq)
 	if (outbody.data == NULL) {
 		error = smbd_smb2_request_error(smb2req, NT_STATUS_NO_MEMORY);
 		if (!NT_STATUS_IS_OK(error)) {
-			smbd_server_connection_terminate(smb2req->sconn,
+			smbd_server_connection_terminate(smb2req->xconn,
 							nt_errstr(error));
 			return;
 		}
@@ -856,7 +856,7 @@ static void smbd_smb2_request_logoff_done(struct tevent_req *subreq)
 
 	error = smbd_smb2_request_done(smb2req, outbody, NULL);
 	if (!NT_STATUS_IS_OK(error)) {
-		smbd_server_connection_terminate(smb2req->sconn,
+		smbd_server_connection_terminate(smb2req->xconn,
 						nt_errstr(error));
 		return;
 	}
diff --git a/source3/smbd/smb2_setinfo.c b/source3/smbd/smb2_setinfo.c
index 031c0be..3722697 100644
--- a/source3/smbd/smb2_setinfo.c
+++ b/source3/smbd/smb2_setinfo.c
@@ -128,7 +128,7 @@ static void smbd_smb2_request_setinfo_done(struct tevent_req *subreq)
 	if (!NT_STATUS_IS_OK(status)) {
 		error = smbd_smb2_request_error(req, status);
 		if (!NT_STATUS_IS_OK(error)) {
-			smbd_server_connection_terminate(req->sconn,
+			smbd_server_connection_terminate(req->xconn,
 							 nt_errstr(error));
 			return;
 		}
@@ -139,7 +139,7 @@ static void smbd_smb2_request_setinfo_done(struct tevent_req *subreq)
 	if (outbody.data == NULL) {
 		error = smbd_smb2_request_error(req, NT_STATUS_NO_MEMORY);
 		if (!NT_STATUS_IS_OK(error)) {
-			smbd_server_connection_terminate(req->sconn,
+			smbd_server_connection_terminate(req->xconn,
 							 nt_errstr(error));
 			return;
 		}
@@ -150,7 +150,7 @@ static void smbd_smb2_request_setinfo_done(struct tevent_req *subreq)
 
 	error = smbd_smb2_request_done(req, outbody, NULL);
 	if (!NT_STATUS_IS_OK(error)) {
-		smbd_server_connection_terminate(req->sconn,
+		smbd_server_connection_terminate(req->xconn,
 						 nt_errstr(error));
 		return;
 	}
diff --git a/source3/smbd/smb2_tcon.c b/source3/smbd/smb2_tcon.c
index 5d78b4f..c39ed98 100644
--- a/source3/smbd/smb2_tcon.c
+++ b/source3/smbd/smb2_tcon.c
@@ -126,7 +126,7 @@ static void smbd_smb2_request_tcon_done(struct tevent_req *subreq)
 	if (!NT_STATUS_IS_OK(status)) {
 		error = smbd_smb2_request_error(req, status);
 		if (!NT_STATUS_IS_OK(error)) {
-			smbd_server_connection_terminate(req->sconn,
+			smbd_server_connection_terminate(req->xconn,
 							 nt_errstr(error));
 			return;
 		}
@@ -139,7 +139,7 @@ static void smbd_smb2_request_tcon_done(struct tevent_req *subreq)
 	if (outbody.data == NULL) {
 		error = smbd_smb2_request_error(req, NT_STATUS_NO_MEMORY);
 		if (!NT_STATUS_IS_OK(error)) {
-			smbd_server_connection_terminate(req->sconn,
+			smbd_server_connection_terminate(req->xconn,
 							 nt_errstr(error));
 			return;
 		}
@@ -161,7 +161,7 @@ static void smbd_smb2_request_tcon_done(struct tevent_req *subreq)
 
 	error = smbd_smb2_request_done(req, outbody, NULL);
 	if (!NT_STATUS_IS_OK(error)) {
-		smbd_server_connection_terminate(req->sconn,
+		smbd_server_connection_terminate(req->xconn,
 						 nt_errstr(error));
 		return;
 	}
@@ -454,7 +454,7 @@ static void smbd_smb2_request_tdis_done(struct tevent_req *subreq)
 	if (!NT_STATUS_IS_OK(status)) {
 		error = smbd_smb2_request_error(smb2req, status);
 		if (!NT_STATUS_IS_OK(error)) {
-			smbd_server_connection_terminate(smb2req->sconn,
+			smbd_server_connection_terminate(smb2req->xconn,
 							nt_errstr(error));
 			return;
 		}
@@ -465,7 +465,7 @@ static void smbd_smb2_request_tdis_done(struct tevent_req *subreq)
 	if (outbody.data == NULL) {
 		error = smbd_smb2_request_error(smb2req, NT_STATUS_NO_MEMORY);
 		if (!NT_STATUS_IS_OK(error)) {
-			smbd_server_connection_terminate(smb2req->sconn,
+			smbd_server_connection_terminate(smb2req->xconn,
 							nt_errstr(error));
 			return;
 		}
@@ -477,7 +477,7 @@ static void smbd_smb2_request_tdis_done(struct tevent_req *subreq)
 
 	error = smbd_smb2_request_done(smb2req, outbody, NULL);
 	if (!NT_STATUS_IS_OK(error)) {
-		smbd_server_connection_terminate(smb2req->sconn,
+		smbd_server_connection_terminate(smb2req->xconn,
 						nt_errstr(error));
 		return;
 	}
diff --git a/source3/smbd/smb2_write.c b/source3/smbd/smb2_write.c
index 35ee020..508c0b3 100644
--- a/source3/smbd/smb2_write.c
+++ b/source3/smbd/smb2_write.c
@@ -134,7 +134,7 @@ static void smbd_smb2_request_write_done(struct tevent_req *subreq)
 	if (!NT_STATUS_IS_OK(status)) {
 		error = smbd_smb2_request_error(req, status);
 		if (!NT_STATUS_IS_OK(error)) {
-			smbd_server_connection_terminate(req->sconn,
+			smbd_server_connection_terminate(req->xconn,
 							 nt_errstr(error));
 			return;
 		}
@@ -145,7 +145,7 @@ static void smbd_smb2_request_write_done(struct tevent_req *subreq)
 	if (outbody.data == NULL) {
 		error = smbd_smb2_request_error(req, NT_STATUS_NO_MEMORY);
 		if (!NT_STATUS_IS_OK(error)) {
-			smbd_server_connection_terminate(req->sconn,
+			smbd_server_connection_terminate(req->xconn,
 							 nt_errstr(error));
 			return;
 		}
@@ -163,7 +163,7 @@ static void smbd_smb2_request_write_done(struct tevent_req *subreq)
 
 	error = smbd_smb2_request_done(req, outbody, &outdyn);
 	if (!NT_STATUS_IS_OK(error)) {
-		smbd_server_connection_terminate(req->sconn, nt_errstr(error));
+		smbd_server_connection_terminate(req->xconn, nt_errstr(error));
 		return;
 	}
 }
diff --git a/source3/smbd/smbXsrv_session.c b/source3/smbd/smbXsrv_session.c
index 560fa3c..bd08f6b 100644
--- a/source3/smbd/smbXsrv_session.c
+++ b/source3/smbd/smbXsrv_session.c
@@ -341,7 +341,7 @@ next:
 	subreq = messaging_read_send(table, conn->ev_ctx, conn->msg_ctx,
 				     MSG_SMBXSRV_SESSION_CLOSE);
 	if (subreq == NULL) {
-		smbd_server_connection_terminate(conn->sconn,
+		smbd_server_connection_terminate(conn,
 						 "msg_read_send() failed");
 		return;
 	}
-- 
1.8.1.2


From a848620774ba22c10c4c67aed9a532ebf3720723 Mon Sep 17 00:00:00 2001
From: Stefan Metzmacher <metze@samba.org>
Date: Tue, 24 Jun 2014 19:28:51 +0200
Subject: [PATCH 63/94] s3:smb2_negprot: remove NBT_HDR_SIZE from
 smbd_smb2_first_negprot()

This will simplify future features like multi-channel and rdma.

Signed-off-by: Stefan Metzmacher <metze@samba.org>
Reviewed-by: Volker Lendecke <vl@samba.org>
---
 source3/smbd/globals.h      |  2 +-
 source3/smbd/process.c      |  4 +++-
 source3/smbd/smb2_negprot.c | 12 ++++++------
 source3/smbd/smb2_server.c  | 20 ++++++++++++--------
 4 files changed, 22 insertions(+), 16 deletions(-)

diff --git a/source3/smbd/globals.h b/source3/smbd/globals.h
index bf168ac..b69dcd0 100644
--- a/source3/smbd/globals.h
+++ b/source3/smbd/globals.h
@@ -225,7 +225,7 @@ bool smbd_is_smb2_header(const uint8_t *inbuf, size_t size);
 void reply_smb2002(struct smb_request *req, uint16_t choice);
 void reply_smb20ff(struct smb_request *req, uint16_t choice);
 void smbd_smb2_first_negprot(struct smbXsrv_connection *xconn,
-			     uint8_t *inbuf, size_t size);
+			     const uint8_t *inpdu, size_t size);
 
 DATA_BLOB smbd_smb2_generate_outbody(struct smbd_smb2_request *req, size_t size);
 
diff --git a/source3/smbd/process.c b/source3/smbd/process.c
index 02d43d5..31a6e01 100644
--- a/source3/smbd/process.c
+++ b/source3/smbd/process.c
@@ -1878,7 +1878,9 @@ static void process_smb(struct smbXsrv_connection *xconn,
 		/* At this point we're not really using smb2,
 		 * we make the decision here.. */
 		if (smbd_is_smb2_header(inbuf, nread)) {
-			smbd_smb2_first_negprot(xconn, inbuf, nread);
+			const uint8_t *inpdu = inbuf + NBT_HDR_SIZE;
+			size_t pdulen = nread - NBT_HDR_SIZE;
+			smbd_smb2_first_negprot(xconn, inpdu, pdulen);
 			return;
 		} else if (nread >= smb_size && valid_smb_header(sconn, inbuf)
 				&& CVAL(inbuf, smb_com) != 0x72) {
diff --git a/source3/smbd/smb2_negprot.c b/source3/smbd/smb2_negprot.c
index feb5a4d..e9d17e2 100644
--- a/source3/smbd/smb2_negprot.c
+++ b/source3/smbd/smb2_negprot.c
@@ -33,19 +33,19 @@ extern fstring remote_proto;
  */
 static void reply_smb20xx(struct smb_request *req, uint16_t dialect)
 {
-	uint8_t *smb2_inbuf;
+	uint8_t *smb2_inpdu;
 	uint8_t *smb2_hdr;
 	uint8_t *smb2_body;
 	uint8_t *smb2_dyn;
-	size_t len = 4 + SMB2_HDR_BODY + 0x24 + 2;
+	size_t len = SMB2_HDR_BODY + 0x24 + 2;
 
-	smb2_inbuf = talloc_zero_array(talloc_tos(), uint8_t, len);
-	if (smb2_inbuf == NULL) {
+	smb2_inpdu = talloc_zero_array(talloc_tos(), uint8_t, len);
+	if (smb2_inpdu == NULL) {
 		DEBUG(0, ("Could not push spnego blob\n"));
 		reply_nterror(req, NT_STATUS_NO_MEMORY);
 		return;
 	}
-	smb2_hdr = smb2_inbuf + 4;
+	smb2_hdr = smb2_inpdu;
 	smb2_body = smb2_hdr + SMB2_HDR_BODY;
 	smb2_dyn = smb2_body + 0x24;
 
@@ -59,7 +59,7 @@ static void reply_smb20xx(struct smb_request *req, uint16_t dialect)
 
 	req->outbuf = NULL;
 
-	smbd_smb2_first_negprot(req->xconn, smb2_inbuf, len);
+	smbd_smb2_first_negprot(req->xconn, smb2_inpdu, len);
 	return;
 }
 
diff --git a/source3/smbd/smb2_server.c b/source3/smbd/smb2_server.c
index 77aa4f1..d9e6a76 100644
--- a/source3/smbd/smb2_server.c
+++ b/source3/smbd/smb2_server.c
@@ -506,24 +506,25 @@ inval:
 }
 
 static NTSTATUS smbd_smb2_request_create(struct smbXsrv_connection *xconn,
-					 uint8_t *inbuf, size_t size,
+					 const uint8_t *_inpdu, size_t size,
 					 struct smbd_smb2_request **_req)
 {
 	struct smbd_server_connection *sconn = xconn->sconn;
 	struct smbd_smb2_request *req;
 	uint32_t protocol_version;
+	uint8_t *inpdu = NULL;
 	const uint8_t *inhdr = NULL;
 	uint16_t cmd;
 	uint32_t next_command_ofs;
 	NTSTATUS status;
 	NTTIME now;
 
-	if (size < (4 + SMB2_HDR_BODY + 2)) {
+	if (size < (SMB2_HDR_BODY + 2)) {
 		DEBUG(0,("Invalid SMB2 packet length count %ld\n", (long)size));
 		return NT_STATUS_INVALID_PARAMETER;
 	}
 
-	inhdr = inbuf + 4;
+	inhdr = _inpdu;
 
 	protocol_version = IVAL(inhdr, SMB2_HDR_PROTOCOL_ID);
 	if (protocol_version != SMB2_MAGIC) {
@@ -553,15 +554,18 @@ static NTSTATUS smbd_smb2_request_create(struct smbXsrv_connection *xconn,
 	req->sconn = sconn;
 	req->xconn = xconn;
 
-	talloc_steal(req, inbuf);
+	inpdu = talloc_memdup(req, _inpdu, size);
+	if (inpdu == NULL) {
+		return NT_STATUS_NO_MEMORY;
+	}
 
 	req->request_time = timeval_current();
 	now = timeval_to_nttime(&req->request_time);
 
 	status = smbd_smb2_inbuf_parse_compound(xconn,
 						now,
-						inbuf + NBT_HDR_SIZE,
-						size - NBT_HDR_SIZE,
+						inpdu,
+						size,
 						req, &req->in.vector,
 						&req->in.vector_count);
 	if (!NT_STATUS_IS_OK(status)) {
@@ -2997,7 +3001,7 @@ static NTSTATUS smbd_smb2_request_next_incoming(struct smbXsrv_connection *xconn
 }
 
 void smbd_smb2_first_negprot(struct smbXsrv_connection *xconn,
-			     uint8_t *inbuf, size_t size)
+			     const uint8_t *inpdu, size_t size)
 {
 	struct smbd_server_connection *sconn = xconn->sconn;
 	NTSTATUS status;
@@ -3012,7 +3016,7 @@ void smbd_smb2_first_negprot(struct smbXsrv_connection *xconn,
 		return;
 	}
 
-	status = smbd_smb2_request_create(xconn, inbuf, size, &req);
+	status = smbd_smb2_request_create(xconn, inpdu, size, &req);
 	if (!NT_STATUS_IS_OK(status)) {
 		smbd_server_connection_terminate(xconn, nt_errstr(status));
 		return;
-- 
1.8.1.2


From 2944c7e66035bd0a6e0aee024620556e733bbfdc Mon Sep 17 00:00:00 2001
From: Stefan Metzmacher <metze@samba.org>
Date: Wed, 11 Jun 2014 12:41:26 +0200
Subject: [PATCH 64/94] s3:smbd: pass smbXsrv_connection to negprot_spnego()

Signed-off-by: Stefan Metzmacher <metze@samba.org>
Reviewed-by: Volker Lendecke <vl@samba.org>
---
 source3/smbd/globals.h      |  2 +-
 source3/smbd/negprot.c      | 11 +++++------
 source3/smbd/smb2_negprot.c |  2 +-
 3 files changed, 7 insertions(+), 8 deletions(-)

diff --git a/source3/smbd/globals.h b/source3/smbd/globals.h
index b69dcd0..a8555e6 100644
--- a/source3/smbd/globals.h
+++ b/source3/smbd/globals.h
@@ -111,7 +111,7 @@ extern bool exit_firsttime;
 struct tstream_context;
 struct smbd_smb2_request;
 
-DATA_BLOB negprot_spnego(TALLOC_CTX *ctx, struct smbd_server_connection *sconn);
+DATA_BLOB negprot_spnego(TALLOC_CTX *ctx, struct smbXsrv_connection *xconn);
 
 void smbd_lock_socket(struct smbd_server_connection *sconn);
 void smbd_unlock_socket(struct smbd_server_connection *sconn);
diff --git a/source3/smbd/negprot.c b/source3/smbd/negprot.c
index 63b3d36..e617d20 100644
--- a/source3/smbd/negprot.c
+++ b/source3/smbd/negprot.c
@@ -160,9 +160,8 @@ static void reply_lanman2(struct smb_request *req, uint16 choice)
  Generate the spnego negprot reply blob. Return the number of bytes used.
 ****************************************************************************/
 
-DATA_BLOB negprot_spnego(TALLOC_CTX *ctx, struct smbd_server_connection *sconn)
+DATA_BLOB negprot_spnego(TALLOC_CTX *ctx, struct smbXsrv_connection *xconn)
 {
-	struct smbXsrv_connection *xconn = sconn->conn;
 	DATA_BLOB blob = data_blob_null;
 	DATA_BLOB blob_out = data_blob_null;
 	nstring dos_name;
@@ -175,7 +174,7 @@ DATA_BLOB negprot_spnego(TALLOC_CTX *ctx, struct smbd_server_connection *sconn)
 
 	/* See if we can get an SPNEGO blob */
 	status = auth_generic_prepare(talloc_tos(),
-				      sconn->remote_address,
+				      xconn->remote_address,
 				      &gensec_security);
 	if (NT_STATUS_IS_OK(status)) {
 		status = gensec_start_mech_by_oid(gensec_security, GENSEC_OID_SPNEGO);
@@ -254,7 +253,7 @@ static void reply_nt1(struct smb_request *req, uint16 choice)
 	struct timespec ts;
 	ssize_t ret;
 	struct smbd_server_connection *sconn = req->sconn;
-	struct smbXsrv_connection *xconn = sconn->conn;
+	struct smbXsrv_connection *xconn = req->xconn;
 	bool signing_desired = false;
 	bool signing_required = false;
 
@@ -333,7 +332,7 @@ static void reply_nt1(struct smb_request *req, uint16 choice)
 	SSVAL(req->outbuf,smb_vwv0,choice);
 	SCVAL(req->outbuf,smb_vwv1,secword);
 
-	smbXsrv_connection_init_tables(req->sconn->conn, PROTOCOL_NT1);
+	smbXsrv_connection_init_tables(xconn, PROTOCOL_NT1);
 
 	SSVAL(req->outbuf,smb_vwv1+1, lp_max_mux()); /* maxmpx */
 	SSVAL(req->outbuf,smb_vwv2+1, 1); /* num vcs */
@@ -380,7 +379,7 @@ static void reply_nt1(struct smb_request *req, uint16 choice)
 		}
 		DEBUG(3,("not using SPNEGO\n"));
 	} else {
-		DATA_BLOB spnego_blob = negprot_spnego(req, req->sconn);
+		DATA_BLOB spnego_blob = negprot_spnego(req, xconn);
 
 		if (spnego_blob.data == NULL) {
 			reply_nterror(req, NT_STATUS_NO_MEMORY);
diff --git a/source3/smbd/smb2_negprot.c b/source3/smbd/smb2_negprot.c
index e9d17e2..6524658 100644
--- a/source3/smbd/smb2_negprot.c
+++ b/source3/smbd/smb2_negprot.c
@@ -259,7 +259,7 @@ NTSTATUS smbd_smb2_request_process_negprot(struct smbd_smb2_request *req)
 	DEBUG(3,("Selected protocol %s\n", remote_proto));
 
 	/* negprot_spnego() returns a the server guid in the first 16 bytes */
-	negprot_spnego_blob = negprot_spnego(req, req->sconn);
+	negprot_spnego_blob = negprot_spnego(req, xconn);
 	if (negprot_spnego_blob.data == NULL) {
 		return smbd_smb2_request_error(req, NT_STATUS_NO_MEMORY);
 	}
-- 
1.8.1.2


From d5b6cdbd2d1a3cf013552a344f06c2899a069b33 Mon Sep 17 00:00:00 2001
From: Stefan Metzmacher <metze@samba.org>
Date: Wed, 11 Jun 2014 12:42:56 +0200
Subject: [PATCH 65/94] s3:smbd: pass smbXsrv_connection to get_challenge()

Signed-off-by: Stefan Metzmacher <metze@samba.org>
Reviewed-by: Volker Lendecke <vl@samba.org>
---
 source3/smbd/negprot.c | 24 ++++++++++--------------
 1 file changed, 10 insertions(+), 14 deletions(-)

diff --git a/source3/smbd/negprot.c b/source3/smbd/negprot.c
index e617d20..95cbd8f 100644
--- a/source3/smbd/negprot.c
+++ b/source3/smbd/negprot.c
@@ -31,9 +31,8 @@
 
 extern fstring remote_proto;
 
-static void get_challenge(struct smbd_server_connection *sconn, uint8 buff[8])
+static void get_challenge(struct smbXsrv_connection *xconn, uint8 buff[8])
 {
-	struct smbXsrv_connection *xconn = sconn->conn;
 	NTSTATUS nt_status;
 
 	/* We might be called more than once, multiple negprots are
@@ -65,8 +64,7 @@ static void reply_lanman1(struct smb_request *req, uint16 choice)
 {
 	int secword=0;
 	time_t t = time(NULL);
-	struct smbd_server_connection *sconn = req->sconn;
-	struct smbXsrv_connection *xconn = sconn->conn;
+	struct smbXsrv_connection *xconn = req->xconn;
 	uint16_t raw;
 	if (lp_async_smb_echo_handler()) {
 		raw = 0;
@@ -87,11 +85,11 @@ static void reply_lanman1(struct smb_request *req, uint16 choice)
 	SSVAL(req->outbuf,smb_vwv1,secword);
 	/* Create a token value and add it to the outgoing packet. */
 	if (xconn->smb1.negprot.encrypted_passwords) {
-		get_challenge(sconn, (uint8 *)smb_buf(req->outbuf));
+		get_challenge(xconn, (uint8 *)smb_buf(req->outbuf));
 		SSVAL(req->outbuf,smb_vwv11, 8);
 	}
 
-	smbXsrv_connection_init_tables(req->sconn->conn, PROTOCOL_LANMAN1);
+	smbXsrv_connection_init_tables(xconn, PROTOCOL_LANMAN1);
 
 	/* Reply, SMBlockread, SMBwritelock supported. */
 	SCVAL(req->outbuf,smb_flg, FLAG_REPLY|FLAG_SUPPORT_LOCKREAD);
@@ -116,8 +114,7 @@ static void reply_lanman2(struct smb_request *req, uint16 choice)
 {
 	int secword=0;
 	time_t t = time(NULL);
-	struct smbd_server_connection *sconn = req->sconn;
-	struct smbXsrv_connection *xconn = sconn->conn;
+	struct smbXsrv_connection *xconn = req->xconn;
 	uint16_t raw;
 	if (lp_async_smb_echo_handler()) {
 		raw = 0;
@@ -140,11 +137,11 @@ static void reply_lanman2(struct smb_request *req, uint16 choice)
 
 	/* Create a token value and add it to the outgoing packet. */
 	if (xconn->smb1.negprot.encrypted_passwords) {
-		get_challenge(sconn, (uint8 *)smb_buf(req->outbuf));
+		get_challenge(xconn, (uint8 *)smb_buf(req->outbuf));
 		SSVAL(req->outbuf,smb_vwv11, 8);
 	}
 
-	smbXsrv_connection_init_tables(req->sconn->conn, PROTOCOL_LANMAN2);
+	smbXsrv_connection_init_tables(xconn, PROTOCOL_LANMAN2);
 
 	/* Reply, SMBlockread, SMBwritelock supported. */
 	SCVAL(req->outbuf,smb_flg,FLAG_REPLY|FLAG_SUPPORT_LOCKREAD);
@@ -252,7 +249,6 @@ static void reply_nt1(struct smb_request *req, uint16 choice)
 	bool negotiate_spnego = False;
 	struct timespec ts;
 	ssize_t ret;
-	struct smbd_server_connection *sconn = req->sconn;
 	struct smbXsrv_connection *xconn = req->xconn;
 	bool signing_desired = false;
 	bool signing_required = false;
@@ -351,7 +347,7 @@ static void reply_nt1(struct smb_request *req, uint16 choice)
 			uint8 chal[8];
 			/* note that we do not send a challenge at all if
 			   we are using plaintext */
-			get_challenge(sconn, chal);
+			get_challenge(xconn, chal);
 			ret = message_push_blob(
 				&req->outbuf, data_blob_const(chal, sizeof(chal)));
 			if (ret == -1) {
@@ -519,8 +515,8 @@ void reply_negprot(struct smb_request *req)
 	char **cliprotos;
 	int i;
 	size_t converted_size;
-	struct smbd_server_connection *sconn = req->sconn;
-	struct smbXsrv_connection *xconn = sconn->conn;
+	struct smbXsrv_connection *xconn = req->xconn;
+	struct smbd_server_connection *sconn = xconn->sconn;
 
 	START_PROFILE(SMBnegprot);
 
-- 
1.8.1.2


From c4d7439facef71dc4d001833c579eb8380912e61 Mon Sep 17 00:00:00 2001
From: Stefan Metzmacher <metze@samba.org>
Date: Wed, 11 Jun 2014 12:54:47 +0200
Subject: [PATCH 66/94] s3:smbd: pass smbXsrv_connection to reply_special()

Signed-off-by: Stefan Metzmacher <metze@samba.org>
Reviewed-by: Volker Lendecke <vl@samba.org>
---
 source3/smbd/process.c | 2 +-
 source3/smbd/proto.h   | 2 +-
 source3/smbd/reply.c   | 6 +++---
 3 files changed, 5 insertions(+), 5 deletions(-)

diff --git a/source3/smbd/process.c b/source3/smbd/process.c
index 31a6e01..5f07275 100644
--- a/source3/smbd/process.c
+++ b/source3/smbd/process.c
@@ -1870,7 +1870,7 @@ static void process_smb(struct smbXsrv_connection *xconn,
 		/*
 		 * NetBIOS session request, keepalive, etc.
 		 */
-		reply_special(sconn, (char *)inbuf, nread);
+		reply_special(xconn, (char *)inbuf, nread);
 		goto done;
 	}
 
diff --git a/source3/smbd/proto.h b/source3/smbd/proto.h
index efabfad..74c9950 100644
--- a/source3/smbd/proto.h
+++ b/source3/smbd/proto.h
@@ -836,7 +836,7 @@ bool check_fsp(connection_struct *conn, struct smb_request *req,
 	       files_struct *fsp);
 bool check_fsp_ntquota_handle(connection_struct *conn, struct smb_request *req,
 			      files_struct *fsp);
-void reply_special(struct smbd_server_connection *sconn, char *inbuf, size_t inbuf_len);
+void reply_special(struct smbXsrv_connection *xconn, char *inbuf, size_t inbuf_size);
 void reply_tcon(struct smb_request *req);
 void reply_tcon_and_X(struct smb_request *req);
 void reply_unknown_new(struct smb_request *req, uint8 type);
diff --git a/source3/smbd/reply.c b/source3/smbd/reply.c
index 6dac6db..03e8ec3 100644
--- a/source3/smbd/reply.c
+++ b/source3/smbd/reply.c
@@ -532,9 +532,9 @@ static void reply_called_name_not_present(char *outbuf)
  Reply to a (netbios-level) special message. 
 ****************************************************************************/
 
-void reply_special(struct smbd_server_connection *sconn, char *inbuf, size_t inbuf_size)
+void reply_special(struct smbXsrv_connection *xconn, char *inbuf, size_t inbuf_size)
 {
-	struct smbXsrv_connection *xconn = sconn->conn;
+	struct smbd_server_connection *sconn = xconn->sconn;
 	int msg_type = CVAL(inbuf,0);
 	int msg_flags = CVAL(inbuf,1);
 	/*
@@ -627,7 +627,7 @@ void reply_special(struct smbd_server_connection *sconn, char *inbuf, size_t inb
 			if (sconn->remote_hostname == NULL) {
 				exit_server_cleanly("could not copy remote name");
 			}
-			sconn->conn->remote_hostname = sconn->remote_hostname;
+			xconn->remote_hostname = sconn->remote_hostname;
 		}
 
 		DEBUG(2,("netbios connect: local=%s remote=%s, name type = %x\n",
-- 
1.8.1.2


From 5731fb876158461a11a5c52a57a2a13c2cce5301 Mon Sep 17 00:00:00 2001
From: Stefan Metzmacher <metze@samba.org>
Date: Wed, 11 Jun 2014 12:55:24 +0200
Subject: [PATCH 67/94] s3:smbd: pass smbXsrv_connection to
 netbios_session_retarget()

Signed-off-by: Stefan Metzmacher <metze@samba.org>
Reviewed-by: Volker Lendecke <vl@samba.org>
---
 source3/smbd/reply.c | 6 +++---
 1 file changed, 3 insertions(+), 3 deletions(-)

diff --git a/source3/smbd/reply.c b/source3/smbd/reply.c
index 03e8ec3..5d72917 100644
--- a/source3/smbd/reply.c
+++ b/source3/smbd/reply.c
@@ -428,10 +428,10 @@ bool check_fsp_ntquota_handle(connection_struct *conn, struct smb_request *req,
 	return true;
 }
 
-static bool netbios_session_retarget(struct smbd_server_connection *sconn,
+static bool netbios_session_retarget(struct smbXsrv_connection *xconn,
 				     const char *name, int name_type)
 {
-	struct smbXsrv_connection *xconn = sconn->conn;
+	struct smbd_server_connection *sconn = xconn->sconn;
 	char *trim_name;
 	char *trim_name_type;
 	const char *retarget_parm;
@@ -593,7 +593,7 @@ void reply_special(struct smbXsrv_connection *xconn, char *inbuf, size_t inbuf_s
 		DEBUG(2,("netbios connect: name1=%s0x%x name2=%s0x%x\n",
 			 name1, name_type1, name2, name_type2));
 
-		if (netbios_session_retarget(sconn, name1, name_type1)) {
+		if (netbios_session_retarget(xconn, name1, name_type1)) {
 			exit_server_cleanly("retargeted client");
 		}
 
-- 
1.8.1.2


From b073047807d36d6e8b734678b944a5a8f3cfde85 Mon Sep 17 00:00:00 2001
From: Stefan Metzmacher <metze@samba.org>
Date: Wed, 11 Jun 2014 13:36:34 +0200
Subject: [PATCH 68/94] s3:smbd: pass smbXsrv_connection to smbd_do_qfsinfo()

Signed-off-by: Stefan Metzmacher <metze@samba.org>
Reviewed-by: Volker Lendecke <vl@samba.org>
---
 source3/smbd/globals.h      | 3 ++-
 source3/smbd/smb2_getinfo.c | 2 +-
 source3/smbd/trans2.c       | 6 +++---
 3 files changed, 6 insertions(+), 5 deletions(-)

diff --git a/source3/smbd/globals.h b/source3/smbd/globals.h
index a8555e6..97bd968 100644
--- a/source3/smbd/globals.h
+++ b/source3/smbd/globals.h
@@ -153,7 +153,8 @@ NTSTATUS smbd_do_setfilepathinfo(connection_struct *conn,
 				char **ppdata, int total_data,
 				int *ret_data_size);
 
-NTSTATUS smbd_do_qfsinfo(connection_struct *conn,
+NTSTATUS smbd_do_qfsinfo(struct smbXsrv_connection *xconn,
+			 connection_struct *conn,
 			 TALLOC_CTX *mem_ctx,
 			 uint16_t info_level,
 			 uint16_t flags2,
diff --git a/source3/smbd/smb2_getinfo.c b/source3/smbd/smb2_getinfo.c
index 64669b8..3139a32 100644
--- a/source3/smbd/smb2_getinfo.c
+++ b/source3/smbd/smb2_getinfo.c
@@ -427,7 +427,7 @@ static struct tevent_req *smbd_smb2_getinfo_send(TALLOC_CTX *mem_ctx,
 		/* the levels directly map to the passthru levels */
 		file_info_level = in_file_info_class + 1000;
 
-		status = smbd_do_qfsinfo(conn, state,
+		status = smbd_do_qfsinfo(smb2req->xconn, conn, state,
 					 file_info_level,
 					 STR_UNICODE,
 					 in_output_buffer_length,
diff --git a/source3/smbd/trans2.c b/source3/smbd/trans2.c
index e6cb64f..a183794 100644
--- a/source3/smbd/trans2.c
+++ b/source3/smbd/trans2.c
@@ -3064,7 +3064,8 @@ static void samba_extended_info_version(struct smb_extended_info *extended_info)
 		  "%s", samba_version_string());
 }
 
-NTSTATUS smbd_do_qfsinfo(connection_struct *conn,
+NTSTATUS smbd_do_qfsinfo(struct smbXsrv_connection *xconn,
+			 connection_struct *conn,
 			 TALLOC_CTX *mem_ctx,
 			 uint16_t info_level,
 			 uint16_t flags2,
@@ -3074,7 +3075,6 @@ NTSTATUS smbd_do_qfsinfo(connection_struct *conn,
 			 char **ppdata,
 			 int *ret_data_len)
 {
-	struct smbXsrv_connection *xconn = conn->sconn->conn;
 	char *pdata, *end_data;
 	int data_len = 0, len;
 	const char *vname = volume_label(talloc_tos(), SNUM(conn));
@@ -3677,7 +3677,7 @@ static void call_trans2qfsinfo(connection_struct *conn,
 
 	DEBUG(3,("call_trans2qfsinfo: level = %d\n", info_level));
 
-	status = smbd_do_qfsinfo(conn, req,
+	status = smbd_do_qfsinfo(req->xconn, conn, req,
 				 info_level,
 				 req->flags2,
 				 max_data_bytes,
-- 
1.8.1.2


From 726fef144b27dbe8ba09fb8c42935ec9fdb8969b Mon Sep 17 00:00:00 2001
From: Stefan Metzmacher <metze@samba.org>
Date: Wed, 11 Jun 2014 13:54:43 +0200
Subject: [PATCH 69/94] s3:smbd: use req->xconn in vfs_[p]write_data()

Signed-off-by: Stefan Metzmacher <metze@samba.org>
Reviewed-by: Volker Lendecke <vl@samba.org>
---
 source3/smbd/vfs.c | 4 ++--
 1 file changed, 2 insertions(+), 2 deletions(-)

diff --git a/source3/smbd/vfs.c b/source3/smbd/vfs.c
index 82e0cdf..c219b1f 100644
--- a/source3/smbd/vfs.c
+++ b/source3/smbd/vfs.c
@@ -406,7 +406,7 @@ ssize_t vfs_write_data(struct smb_request *req,
 	ssize_t ret;
 
 	if (req && req->unread_bytes) {
-		int sockfd = req->sconn->conn->transport.sock;
+		int sockfd = req->xconn->transport.sock;
 		int old_flags;
 		SMB_ASSERT(req->unread_bytes == N);
 		/* VFS_RECVFILE must drain the socket
@@ -450,7 +450,7 @@ ssize_t vfs_pwrite_data(struct smb_request *req,
 	ssize_t ret;
 
 	if (req && req->unread_bytes) {
-		int sockfd = req->sconn->conn->transport.sock;
+		int sockfd = req->xconn->transport.sock;
 		SMB_ASSERT(req->unread_bytes == N);
 		/* VFS_RECVFILE must drain the socket
 		 * before returning. */
-- 
1.8.1.2


From 06f2ec4935f3903033757f91f97c645b0e39055b Mon Sep 17 00:00:00 2001
From: Stefan Metzmacher <metze@samba.org>
Date: Wed, 11 Jun 2014 13:57:30 +0200
Subject: [PATCH 70/94] s3:smbd: remove unused 'sconn' in
 call_trans2setfsinfo()

Signed-off-by: Stefan Metzmacher <metze@samba.org>
Reviewed-by: Volker Lendecke <vl@samba.org>
---
 source3/smbd/trans2.c | 3 +--
 1 file changed, 1 insertion(+), 2 deletions(-)

diff --git a/source3/smbd/trans2.c b/source3/smbd/trans2.c
index a183794..77ba9aa 100644
--- a/source3/smbd/trans2.c
+++ b/source3/smbd/trans2.c
@@ -3708,8 +3708,7 @@ static void call_trans2setfsinfo(connection_struct *conn,
 				 char **ppdata, int total_data,
 				 unsigned int max_data_bytes)
 {
-	struct smbd_server_connection *sconn = req->sconn;
-	struct smbXsrv_connection *xconn = sconn->conn;
+	struct smbXsrv_connection *xconn = req->xconn;
 	char *pdata = *ppdata;
 	char *params = *pparams;
 	uint16 info_level;
-- 
1.8.1.2


From 35764fad09feeddb3b10fd306bbc55845bf70a71 Mon Sep 17 00:00:00 2001
From: Stefan Metzmacher <metze@samba.org>
Date: Wed, 11 Jun 2014 14:22:13 +0200
Subject: [PATCH 71/94] s3:smbd: use req->xconn to send_trans2_replies()

Signed-off-by: Stefan Metzmacher <metze@samba.org>
Reviewed-by: Volker Lendecke <vl@samba.org>
---
 source3/smbd/trans2.c | 4 ++--
 1 file changed, 2 insertions(+), 2 deletions(-)

diff --git a/source3/smbd/trans2.c b/source3/smbd/trans2.c
index 77ba9aa..9a02a74 100644
--- a/source3/smbd/trans2.c
+++ b/source3/smbd/trans2.c
@@ -859,8 +859,8 @@ void send_trans2_replies(connection_struct *conn,
 	int alignment_offset = 1; /* JRA. This used to be 3. Set to 1 to make netmon parse ok. */
 	int data_alignment_offset = 0;
 	bool overflow = False;
-	struct smbd_server_connection *sconn = req->sconn;
-	struct smbXsrv_connection *xconn = sconn->conn;
+	struct smbXsrv_connection *xconn = req->xconn;
+	struct smbd_server_connection *sconn = xconn->sconn;
 	int max_send = xconn->smb1.sessions.max_send;
 
 	/* Modify the data_to_send and datasize and set the error if
-- 
1.8.1.2


From a5534dc804fe27b1d04dd821daf25b73de56b51a Mon Sep 17 00:00:00 2001
From: Stefan Metzmacher <metze@samba.org>
Date: Wed, 11 Jun 2014 14:25:07 +0200
Subject: [PATCH 72/94] s3:smbd: use req->xconn in files.c

Signed-off-by: Stefan Metzmacher <metze@samba.org>
Reviewed-by: Volker Lendecke <vl@samba.org>
---
 source3/smbd/files.c | 9 +++++----
 1 file changed, 5 insertions(+), 4 deletions(-)

diff --git a/source3/smbd/files.c b/source3/smbd/files.c
index 2a0f6ce..a9e8357 100644
--- a/source3/smbd/files.c
+++ b/source3/smbd/files.c
@@ -94,10 +94,11 @@ NTSTATUS file_new(struct smb_request *req, connection_struct *conn,
 	GetTimeOfDay(&fsp->open_time);
 
 	if (req) {
+		struct smbXsrv_connection *xconn = req->xconn;
 		struct smbXsrv_open *op = NULL;
 		NTTIME now = timeval_to_nttime(&fsp->open_time);
 
-		status = smbXsrv_open_create(sconn->conn,
+		status = smbXsrv_open_create(xconn,
 					     conn->session_info,
 					     now, &op);
 		if (!NT_STATUS_IS_OK(status)) {
@@ -558,13 +559,13 @@ files_struct *file_fsp(struct smb_request *req, uint16 fid)
 		return req->chain_fsp;
 	}
 
-	if (req->sconn->conn == NULL) {
+	if (req->xconn == NULL) {
 		return NULL;
 	}
 
 	now = timeval_to_nttime(&req->request_time);
 
-	status = smb1srv_open_lookup(req->sconn->conn,
+	status = smb1srv_open_lookup(req->xconn,
 				     fid, now, &op);
 	if (!NT_STATUS_IS_OK(status)) {
 		return NULL;
@@ -594,7 +595,7 @@ struct files_struct *file_fsp_get(struct smbd_smb2_request *smb2req,
 
 	now = timeval_to_nttime(&smb2req->request_time);
 
-	status = smb2srv_open_lookup(smb2req->sconn->conn,
+	status = smb2srv_open_lookup(smb2req->xconn,
 				     persistent_id, volatile_id,
 				     now, &op);
 	if (!NT_STATUS_IS_OK(status)) {
-- 
1.8.1.2


From 0e5dd68e22e43a50137b070d373d6aac23239790 Mon Sep 17 00:00:00 2001
From: Stefan Metzmacher <metze@samba.org>
Date: Wed, 11 Jun 2014 14:26:08 +0200
Subject: [PATCH 73/94] s3:smbd: use req->xconn in send_trans_reply()

Signed-off-by: Stefan Metzmacher <metze@samba.org>
Reviewed-by: Volker Lendecke <vl@samba.org>
---
 source3/smbd/ipc.c | 4 ++--
 1 file changed, 2 insertions(+), 2 deletions(-)

diff --git a/source3/smbd/ipc.c b/source3/smbd/ipc.c
index 1c8033d..9a07ba0 100644
--- a/source3/smbd/ipc.c
+++ b/source3/smbd/ipc.c
@@ -107,8 +107,8 @@ void send_trans_reply(connection_struct *conn,
 
 	int ldata  = rdata  ? rdata_len : 0;
 	int lparam = rparam ? rparam_len : 0;
-	struct smbd_server_connection *sconn = req->sconn;
-	struct smbXsrv_connection *xconn = sconn->conn;
+	struct smbXsrv_connection *xconn = req->xconn;
+	struct smbd_server_connection *sconn = xconn->sconn;
 	int max_send = xconn->smb1.sessions.max_send;
 	/* HACK: make sure we send at least 128 byte in one go */
 	int hdr_overhead = SMB_BUFFER_SIZE_MIN - 128;
-- 
1.8.1.2


From f69bd6b59a817e6887017b77c87ffbaba0c27467 Mon Sep 17 00:00:00 2001
From: Stefan Metzmacher <metze@samba.org>
Date: Wed, 11 Jun 2014 14:26:57 +0200
Subject: [PATCH 74/94] s3:smbd: use req->xconn in nttrans.c

Signed-off-by: Stefan Metzmacher <metze@samba.org>
Reviewed-by: Volker Lendecke <vl@samba.org>
---
 source3/smbd/nttrans.c | 11 ++++++-----
 1 file changed, 6 insertions(+), 5 deletions(-)

diff --git a/source3/smbd/nttrans.c b/source3/smbd/nttrans.c
index 57c4159..2521a1e 100644
--- a/source3/smbd/nttrans.c
+++ b/source3/smbd/nttrans.c
@@ -67,8 +67,8 @@ static void send_nt_replies(connection_struct *conn,
 	int params_sent_thistime, data_sent_thistime, total_sent_thistime;
 	int alignment_offset = 1;
 	int data_alignment_offset = 0;
-	struct smbd_server_connection *sconn = req->sconn;
-	struct smbXsrv_connection *xconn = sconn->conn;
+	struct smbXsrv_connection *xconn = req->xconn;
+	struct smbd_server_connection *sconn = xconn->sconn;
 	int max_send = xconn->smb1.sessions.max_send;
 
 	/*
@@ -1339,7 +1339,8 @@ static void call_nt_transact_create(connection_struct *conn,
 
 void reply_ntcancel(struct smb_request *req)
 {
-	struct smbXsrv_connection *xconn = req->sconn->conn;
+	struct smbXsrv_connection *xconn = req->xconn;
+	struct smbd_server_connection *sconn = xconn->sconn;
 
 	/*
 	 * Go through and cancel any pending change notifies.
@@ -1347,8 +1348,8 @@ void reply_ntcancel(struct smb_request *req)
 
 	START_PROFILE(SMBntcancel);
 	srv_cancel_sign_response(xconn);
-	remove_pending_change_notify_requests_by_mid(req->sconn, req->mid);
-	remove_pending_lock_requests_by_mid_smb1(req->sconn, req->mid);
+	remove_pending_change_notify_requests_by_mid(sconn, req->mid);
+	remove_pending_lock_requests_by_mid_smb1(sconn, req->mid);
 
 	DEBUG(3,("reply_ntcancel: cancel called on mid = %llu.\n",
 		(unsigned long long)req->mid));
-- 
1.8.1.2


From 8fc5286d8d3afac7eb2d704a2c7d489c6dd30a71 Mon Sep 17 00:00:00 2001
From: Stefan Metzmacher <metze@samba.org>
Date: Wed, 11 Jun 2014 14:28:13 +0200
Subject: [PATCH 75/94] s3:smbd: remove unused 'sconn' from
 is_encrypted_packet()

Signed-off-by: Stefan Metzmacher <metze@samba.org>
Reviewed-by: Volker Lendecke <vl@samba.org>
---
 source3/smbd/process.c | 4 ++--
 source3/smbd/proto.h   | 3 +--
 source3/smbd/reply.c   | 2 +-
 source3/smbd/seal.c    | 3 +--
 4 files changed, 5 insertions(+), 7 deletions(-)

diff --git a/source3/smbd/process.c b/source3/smbd/process.c
index 5f07275..8caa7d2 100644
--- a/source3/smbd/process.c
+++ b/source3/smbd/process.c
@@ -299,7 +299,7 @@ int srv_set_message(char *buf,
 static bool valid_smb_header(struct smbd_server_connection *sconn,
 			     const uint8_t *inbuf)
 {
-	if (is_encrypted_packet(sconn, inbuf)) {
+	if (is_encrypted_packet(inbuf)) {
 		return true;
 	}
 	/*
@@ -564,7 +564,7 @@ static NTSTATUS receive_smb_talloc(TALLOC_CTX *mem_ctx,
 		return status;
 	}
 
-	if (is_encrypted_packet(sconn, (uint8_t *)*buffer)) {
+	if (is_encrypted_packet((uint8_t *)*buffer)) {
 		status = srv_decrypt_buffer(sconn, *buffer);
 		if (!NT_STATUS_IS_OK(status)) {
 			DEBUG(0, ("receive_smb_talloc: SMB decryption failed on "
diff --git a/source3/smbd/proto.h b/source3/smbd/proto.h
index 74c9950..3dea578 100644
--- a/source3/smbd/proto.h
+++ b/source3/smbd/proto.h
@@ -927,8 +927,7 @@ void reply_getattrE(struct smb_request *req);
 
 /* The following definitions come from smbd/seal.c  */
 
-bool is_encrypted_packet(struct smbd_server_connection *sconn,
-			 const uint8_t *inbuf);
+bool is_encrypted_packet(const uint8_t *inbuf);
 void srv_free_enc_buffer(struct smbd_server_connection *sconn, char *buf);
 NTSTATUS srv_decrypt_buffer(struct smbd_server_connection *sconn, char *buf);
 NTSTATUS srv_encrypt_buffer(struct smbd_server_connection *sconn, char *buf,
diff --git a/source3/smbd/reply.c b/source3/smbd/reply.c
index 5d72917..6b0acb6 100644
--- a/source3/smbd/reply.c
+++ b/source3/smbd/reply.c
@@ -4627,7 +4627,7 @@ bool is_valid_writeX_buffer(struct smbd_server_connection *sconn,
 	struct files_struct *fsp = NULL;
 	NTSTATUS status;
 
-	if (is_encrypted_packet(sconn, inbuf)) {
+	if (is_encrypted_packet(inbuf)) {
 		/* Can't do this on encrypted
 		 * connections. */
 		return false;
diff --git a/source3/smbd/seal.c b/source3/smbd/seal.c
index 273013b..7ab992e 100644
--- a/source3/smbd/seal.c
+++ b/source3/smbd/seal.c
@@ -45,8 +45,7 @@ static uint16_t srv_enc_ctx(const struct smb_trans_enc_state *es)
  Is this an incoming encrypted packet ?
 ******************************************************************************/
 
-bool is_encrypted_packet(struct smbd_server_connection *sconn,
-			 const uint8_t *inbuf)
+bool is_encrypted_packet(const uint8_t *inbuf)
 {
 	NTSTATUS status;
 	uint16_t enc_num;
-- 
1.8.1.2


From 33b4fba9c7f3970458261e83fec39c01e4df8d3d Mon Sep 17 00:00:00 2001
From: Stefan Metzmacher <metze@samba.org>
Date: Wed, 11 Jun 2014 14:29:52 +0200
Subject: [PATCH 76/94] s3:smbd: remove unused 'sconn' from valid_smb_header()

Signed-off-by: Stefan Metzmacher <metze@samba.org>
Reviewed-by: Volker Lendecke <vl@samba.org>
---
 source3/smbd/process.c | 9 ++++-----
 1 file changed, 4 insertions(+), 5 deletions(-)

diff --git a/source3/smbd/process.c b/source3/smbd/process.c
index 8caa7d2..8a57b53 100644
--- a/source3/smbd/process.c
+++ b/source3/smbd/process.c
@@ -296,8 +296,7 @@ int srv_set_message(char *buf,
 	return (smb_size + num_words*2 + num_bytes);
 }
 
-static bool valid_smb_header(struct smbd_server_connection *sconn,
-			     const uint8_t *inbuf)
+static bool valid_smb_header(const uint8_t *inbuf)
 {
 	if (is_encrypted_packet(inbuf)) {
 		return true;
@@ -1882,7 +1881,7 @@ static void process_smb(struct smbXsrv_connection *xconn,
 			size_t pdulen = nread - NBT_HDR_SIZE;
 			smbd_smb2_first_negprot(xconn, inpdu, pdulen);
 			return;
-		} else if (nread >= smb_size && valid_smb_header(sconn, inbuf)
+		} else if (nread >= smb_size && valid_smb_header(inbuf)
 				&& CVAL(inbuf, smb_com) != 0x72) {
 			/* This is a non-negprot SMB1 packet.
 			   Disable SMB2 from now on. */
@@ -1892,7 +1891,7 @@ static void process_smb(struct smbXsrv_connection *xconn,
 
 	/* Make sure this is an SMB packet. smb_size contains NetBIOS header
 	 * so subtract 4 from it. */
-	if ((nread < (smb_size - 4)) || !valid_smb_header(sconn, inbuf)) {
+	if ((nread < (smb_size - 4)) || !valid_smb_header(inbuf)) {
 		DEBUG(2,("Non-SMB packet of length %d. Terminating server\n",
 			 smb_len(inbuf)));
 
@@ -3033,7 +3032,7 @@ static bool smbd_echo_reply(struct smbd_echo_state *state,
 		DEBUG(10, ("Got short packet: %d bytes\n", (int)inbuf_len));
 		return false;
 	}
-	if (!valid_smb_header(state->sconn, inbuf)) {
+	if (!valid_smb_header(inbuf)) {
 		DEBUG(10, ("Got invalid SMB header\n"));
 		return false;
 	}
-- 
1.8.1.2


From 28c6d3b6df916e708a6294af944b3f723c1dfc9f Mon Sep 17 00:00:00 2001
From: Stefan Metzmacher <metze@samba.org>
Date: Wed, 11 Jun 2014 14:35:55 +0200
Subject: [PATCH 77/94] s3:smbd: pass smbXsrv_connection to
 is_valid_writeX_buffer()

Signed-off-by: Stefan Metzmacher <metze@samba.org>
Reviewed-by: Volker Lendecke <vl@samba.org>
---
 source3/smbd/process.c | 3 ++-
 source3/smbd/proto.h   | 2 +-
 source3/smbd/reply.c   | 4 ++--
 3 files changed, 5 insertions(+), 4 deletions(-)

diff --git a/source3/smbd/process.c b/source3/smbd/process.c
index 8a57b53..66cb95f 100644
--- a/source3/smbd/process.c
+++ b/source3/smbd/process.c
@@ -370,6 +370,7 @@ static NTSTATUS receive_smb_raw_talloc_partial_read(TALLOC_CTX *mem_ctx,
 						    size_t *p_unread,
 						    size_t *len_ret)
 {
+	struct smbXsrv_connection *xconn = sconn->conn;
 	/* Size of a WRITEX call (+4 byte len). */
 	char writeX_header[4 + STANDARD_WRITE_AND_X_HEADER_SIZE];
 	ssize_t len = smb_len_large(lenbuf); /* Could be a UNIX large writeX. */
@@ -398,7 +399,7 @@ static NTSTATUS receive_smb_raw_talloc_partial_read(TALLOC_CTX *mem_ctx,
 	 * valid writeX call.
 	 */
 
-	if (is_valid_writeX_buffer(sconn, (uint8_t *)writeX_header)) {
+	if (is_valid_writeX_buffer(xconn, (uint8_t *)writeX_header)) {
 		/*
 		 * If the data offset is beyond what
 		 * we've read, drain the extra bytes.
diff --git a/source3/smbd/proto.h b/source3/smbd/proto.h
index 3dea578..a44d1bd 100644
--- a/source3/smbd/proto.h
+++ b/source3/smbd/proto.h
@@ -869,7 +869,7 @@ void error_to_writebrawerr(struct smb_request *req);
 void reply_writebraw(struct smb_request *req);
 void reply_writeunlock(struct smb_request *req);
 void reply_write(struct smb_request *req);
-bool is_valid_writeX_buffer(struct smbd_server_connection *sconn,
+bool is_valid_writeX_buffer(struct smbXsrv_connection *xconn,
 			    const uint8_t *inbuf);
 void reply_write_and_X(struct smb_request *req);
 void reply_lseek(struct smb_request *req);
diff --git a/source3/smbd/reply.c b/source3/smbd/reply.c
index 6b0acb6..516145a 100644
--- a/source3/smbd/reply.c
+++ b/source3/smbd/reply.c
@@ -4616,7 +4616,7 @@ strict_unlock:
 						(2*14) + /* word count (including bcc) */ \
 						1 /* pad byte */)
 
-bool is_valid_writeX_buffer(struct smbd_server_connection *sconn,
+bool is_valid_writeX_buffer(struct smbXsrv_connection *xconn,
 			    const uint8_t *inbuf)
 {
 	size_t numtowrite;
@@ -4645,7 +4645,7 @@ bool is_valid_writeX_buffer(struct smbd_server_connection *sconn,
 	}
 
 	fnum = SVAL(inbuf, smb_vwv2);
-	status = smb1srv_open_lookup(sconn->conn,
+	status = smb1srv_open_lookup(xconn,
 				     fnum,
 				     0, /* now */
 				     &op);
-- 
1.8.1.2


From 48182283a50bfb85e733b1583fd3d50d7499126d Mon Sep 17 00:00:00 2001
From: Stefan Metzmacher <metze@samba.org>
Date: Wed, 11 Jun 2014 14:51:05 +0200
Subject: [PATCH 78/94] s3:smbd: pass smbXsrv_connection to
 reply_readbraw_error()

Signed-off-by: Stefan Metzmacher <metze@samba.org>
Reviewed-by: Volker Lendecke <vl@samba.org>
---
 source3/smbd/reply.c | 24 +++++++++++-------------
 1 file changed, 11 insertions(+), 13 deletions(-)

diff --git a/source3/smbd/reply.c b/source3/smbd/reply.c
index 516145a..a46431d 100644
--- a/source3/smbd/reply.c
+++ b/source3/smbd/reply.c
@@ -3162,9 +3162,9 @@ void sendfile_short_send(files_struct *fsp,
  Return a readbraw error (4 bytes of zero).
 ****************************************************************************/
 
-static void reply_readbraw_error(struct smbd_server_connection *sconn)
+static void reply_readbraw_error(struct smbXsrv_connection *xconn)
 {
-	struct smbXsrv_connection *xconn = sconn->conn;
+	struct smbd_server_connection *sconn = xconn->sconn;
 	char header[4];
 
 	SIVAL(header,0,0);
@@ -3198,8 +3198,7 @@ static void send_file_readbraw(connection_struct *conn,
 			       size_t nread,
 			       ssize_t mincount)
 {
-	struct smbd_server_connection *sconn = req->sconn;
-	struct smbXsrv_connection *xconn = sconn->conn;
+	struct smbXsrv_connection *xconn = req->xconn;
 	char *outbuf = NULL;
 	ssize_t ret=0;
 
@@ -3282,7 +3281,7 @@ normal_readbraw:
 	if (!outbuf) {
 		DEBUG(0,("send_file_readbraw: talloc_array failed for size %u.\n",
 			(unsigned)(nread+4)));
-		reply_readbraw_error(sconn);
+		reply_readbraw_error(xconn);
 		return;
 	}
 
@@ -3322,8 +3321,7 @@ normal_readbraw:
 void reply_readbraw(struct smb_request *req)
 {
 	connection_struct *conn = req->conn;
-	struct smbd_server_connection *sconn = req->sconn;
-	struct smbXsrv_connection *xconn = sconn->conn;
+	struct smbXsrv_connection *xconn = req->xconn;
 	ssize_t maxcount,mincount;
 	size_t nread = 0;
 	off_t startpos;
@@ -3339,7 +3337,7 @@ void reply_readbraw(struct smb_request *req)
 	}
 
 	if (req->wct < 8) {
-		reply_readbraw_error(sconn);
+		reply_readbraw_error(xconn);
 		END_PROFILE(SMBreadbraw);
 		return;
 	}
@@ -3347,7 +3345,7 @@ void reply_readbraw(struct smb_request *req)
 	if (xconn->smb1.echo_handler.trusted_fde) {
 		DEBUG(2,("SMBreadbraw rejected with NOT_SUPPORTED because of "
 			 "'async smb echo handler = yes'\n"));
-		reply_readbraw_error(sconn);
+		reply_readbraw_error(xconn);
 		END_PROFILE(SMBreadbraw);
 		return;
 	}
@@ -3375,7 +3373,7 @@ void reply_readbraw(struct smb_request *req)
 		DEBUG(3,("reply_readbraw: fnum %d not valid "
 			"- cache prime?\n",
 			(int)SVAL(req->vwv+0, 0)));
-		reply_readbraw_error(sconn);
+		reply_readbraw_error(xconn);
 		END_PROFILE(SMBreadbraw);
 		return;
 	}
@@ -3386,7 +3384,7 @@ void reply_readbraw(struct smb_request *req)
 				(fsp->access_mask & FILE_EXECUTE)))) {
 		DEBUG(3,("reply_readbraw: fnum %d not readable.\n",
 				(int)SVAL(req->vwv+0, 0)));
-		reply_readbraw_error(sconn);
+		reply_readbraw_error(xconn);
 		END_PROFILE(SMBreadbraw);
 		return;
 	}
@@ -3405,7 +3403,7 @@ void reply_readbraw(struct smb_request *req)
 			DEBUG(0,("reply_readbraw: negative 64 bit "
 				"readraw offset (%.0f) !\n",
 				(double)startpos ));
-			reply_readbraw_error(sconn);
+			reply_readbraw_error(xconn);
 			END_PROFILE(SMBreadbraw);
 			return;
 		}
@@ -3422,7 +3420,7 @@ void reply_readbraw(struct smb_request *req)
 	    &lock);
 
 	if (!SMB_VFS_STRICT_LOCK(conn, fsp, &lock)) {
-		reply_readbraw_error(sconn);
+		reply_readbraw_error(xconn);
 		END_PROFILE(SMBreadbraw);
 		return;
 	}
-- 
1.8.1.2


From c2653d8cc6c0bb80c6f5835de463cee155a00a30 Mon Sep 17 00:00:00 2001
From: Stefan Metzmacher <metze@samba.org>
Date: Wed, 11 Jun 2014 15:11:05 +0200
Subject: [PATCH 79/94] s3:smbd: use req->xconn in sesssetup.c

Signed-off-by: Stefan Metzmacher <metze@samba.org>
Reviewed-by: Volker Lendecke <vl@samba.org>
---
 source3/smbd/sesssetup.c | 16 ++++++++--------
 1 file changed, 8 insertions(+), 8 deletions(-)

diff --git a/source3/smbd/sesssetup.c b/source3/smbd/sesssetup.c
index 43be24a..b5e8d87 100644
--- a/source3/smbd/sesssetup.c
+++ b/source3/smbd/sesssetup.c
@@ -131,8 +131,8 @@ static void reply_sesssetup_and_X_spnego(struct smb_request *req)
 	enum remote_arch_types ra_type = get_remote_arch();
 	uint64_t vuid = req->vuid;
 	NTSTATUS status = NT_STATUS_OK;
-	struct smbd_server_connection *sconn = req->sconn;
-	struct smbXsrv_connection *xconn = sconn->conn;
+	struct smbXsrv_connection *xconn = req->xconn;
+	struct smbd_server_connection *sconn = xconn->sconn;
 	uint16_t action = 0;
 	NTTIME now = timeval_to_nttime(&req->request_time);
 	struct smbXsrv_session *session = NULL;
@@ -203,7 +203,7 @@ static void reply_sesssetup_and_X_spnego(struct smb_request *req)
 	}
 
 	if (vuid != 0) {
-		status = smb1srv_session_lookup(sconn->conn,
+		status = smb1srv_session_lookup(xconn,
 						vuid, now,
 						&session);
 		if (NT_STATUS_EQUAL(status, NT_STATUS_USER_SESSION_DELETED)) {
@@ -226,7 +226,7 @@ static void reply_sesssetup_and_X_spnego(struct smb_request *req)
 
 	if (session == NULL) {
 		/* create a new session */
-		status = smbXsrv_session_create(sconn->conn,
+		status = smbXsrv_session_create(xconn,
 					        now, &session);
 		if (!NT_STATUS_IS_OK(status)) {
 			reply_nterror(req, nt_status_squash(status));
@@ -235,7 +235,7 @@ static void reply_sesssetup_and_X_spnego(struct smb_request *req)
 	}
 
 	if (!session->gensec) {
-		status = auth_generic_prepare(session, sconn->remote_address,
+		status = auth_generic_prepare(session, xconn->remote_address,
 					      &session->gensec);
 		if (!NT_STATUS_IS_OK(status)) {
 			TALLOC_FREE(session);
@@ -592,8 +592,8 @@ void reply_sesssetup_and_X(struct smb_request *req)
 	NTTIME now = timeval_to_nttime(&req->request_time);
 	struct smbXsrv_session *session = NULL;
 	NTSTATUS nt_status;
-	struct smbd_server_connection *sconn = req->sconn;
-	struct smbXsrv_connection *xconn = sconn->conn;
+	struct smbXsrv_connection *xconn = req->xconn;
+	struct smbd_server_connection *sconn = xconn->sconn;
 	bool doencrypt = xconn->smb1.negprot.encrypted_passwords;
 	bool signing_allowed = false;
 	bool signing_mandatory = false;
@@ -942,7 +942,7 @@ void reply_sesssetup_and_X(struct smb_request *req)
 	/* register the name and uid as being validated, so further connections
 	   to a uid can get through without a password, on the same VC */
 
-	nt_status = smbXsrv_session_create(sconn->conn,
+	nt_status = smbXsrv_session_create(xconn,
 					   now, &session);
 	if (!NT_STATUS_IS_OK(nt_status)) {
 		data_blob_free(&nt_resp);
-- 
1.8.1.2


From 701598c92ec258f8868c7fd58155b6e9c26e78ef Mon Sep 17 00:00:00 2001
From: Stefan Metzmacher <metze@samba.org>
Date: Wed, 11 Jun 2014 15:12:32 +0200
Subject: [PATCH 80/94] s3:smbd: use req->xconn in reply.c

Signed-off-by: Stefan Metzmacher <metze@samba.org>
Reviewed-by: Volker Lendecke <vl@samba.org>
---
 source3/smbd/reply.c | 33 ++++++++++++++++-----------------
 1 file changed, 16 insertions(+), 17 deletions(-)

diff --git a/source3/smbd/reply.c b/source3/smbd/reply.c
index a46431d..d3b6765 100644
--- a/source3/smbd/reply.c
+++ b/source3/smbd/reply.c
@@ -693,8 +693,8 @@ void reply_tcon(struct smb_request *req)
 	const uint8_t *p;
 	const char *p2;
 	TALLOC_CTX *ctx = talloc_tos();
-	struct smbd_server_connection *sconn = req->sconn;
-	struct smbXsrv_connection *xconn = sconn->conn;
+	struct smbXsrv_connection *xconn = req->xconn;
+	struct smbd_server_connection *sconn = xconn->sconn;
 	NTTIME now = timeval_to_nttime(&req->request_time);
 
 	START_PROFILE(SMBtcon);
@@ -771,8 +771,8 @@ void reply_tcon_and_X(struct smb_request *req)
 	NTTIME now = timeval_to_nttime(&req->request_time);
 	bool session_key_updated = false;
 	uint16_t optional_support = 0;
-	struct smbd_server_connection *sconn = req->sconn;
-	struct smbXsrv_connection *xconn = sconn->conn;
+	struct smbXsrv_connection *xconn = req->xconn;
+	struct smbd_server_connection *sconn = xconn->sconn;
 
 	START_PROFILE(SMBtconX);
 
@@ -862,7 +862,7 @@ void reply_tcon_and_X(struct smb_request *req)
 
 	DEBUG(4,("Client requested device type [%s] for share [%s]\n", client_devicetype, service));
 
-	nt_status = smb1srv_session_lookup(req->sconn->conn,
+	nt_status = smb1srv_session_lookup(xconn,
 					   req->vuid, now, &session);
 	if (NT_STATUS_EQUAL(nt_status, NT_STATUS_USER_SESSION_DELETED)) {
 		reply_force_doserror(req, ERRSRV, ERRbaduid);
@@ -1555,8 +1555,8 @@ void reply_search(struct smb_request *req)
 	TALLOC_CTX *ctx = talloc_tos();
 	bool ask_sharemode = lp_parm_bool(SNUM(conn), "smbd", "search ask sharemode", true);
 	struct dptr_struct *dirptr = NULL;
-	struct smbd_server_connection *sconn = req->sconn;
-	struct smbXsrv_connection *xconn = sconn->conn;
+	struct smbXsrv_connection *xconn = req->xconn;
+	struct smbd_server_connection *sconn = xconn->sconn;
 
 	START_PROFILE(SMBsearch);
 
@@ -3476,8 +3476,7 @@ void reply_lockread(struct smb_request *req)
 	files_struct *fsp;
 	struct byte_range_lock *br_lck = NULL;
 	char *p = NULL;
-	struct smbd_server_connection *sconn = req->sconn;
-	struct smbXsrv_connection *xconn = sconn->conn;
+	struct smbXsrv_connection *xconn = req->xconn;
 
 	START_PROFILE(SMBlockread);
 
@@ -3586,8 +3585,7 @@ void reply_read(struct smb_request *req)
 	off_t startpos;
 	files_struct *fsp;
 	struct lock_struct lock;
-	struct smbd_server_connection *sconn = req->sconn;
-	struct smbXsrv_connection *xconn = sconn->conn;
+	struct smbXsrv_connection *xconn = req->xconn;
 
 	START_PROFILE(SMBread);
 
@@ -3701,7 +3699,7 @@ static void send_file_readX(connection_struct *conn, struct smb_request *req,
 			    files_struct *fsp, off_t startpos,
 			    size_t smb_maxcnt)
 {
-	struct smbXsrv_connection *xconn = req->sconn->conn;
+	struct smbXsrv_connection *xconn = req->xconn;
 	ssize_t nread = -1;
 	struct lock_struct lock;
 	int saved_errno = 0;
@@ -3900,9 +3898,9 @@ nosendfile_read:
 
 static size_t calc_max_read_pdu(const struct smb_request *req)
 {
-	struct smbXsrv_connection *xconn = req->sconn->conn;
+	struct smbXsrv_connection *xconn = req->xconn;
 
-	if (req->sconn->conn->protocol < PROTOCOL_NT1) {
+	if (xconn->protocol < PROTOCOL_NT1) {
 		return xconn->smb1.sessions.max_send;
 	}
 
@@ -3946,6 +3944,7 @@ static size_t calc_read_size(const struct smb_request *req,
 			     size_t upper_size,
 			     size_t lower_size)
 {
+	struct smbXsrv_connection *xconn = req->xconn;
 	size_t max_pdu = calc_max_read_pdu(req);
 	size_t total_size = 0;
 	size_t hdr_len = MIN_SMB_SIZE + VWV(12);
@@ -3961,7 +3960,7 @@ static size_t calc_read_size(const struct smb_request *req,
 		upper_size = 0;
 	}
 
-	if (req->sconn->conn->protocol < PROTOCOL_NT1) {
+	if (xconn->protocol < PROTOCOL_NT1) {
 		upper_size = 0;
 	}
 
@@ -4124,7 +4123,7 @@ static NTSTATUS read_smb_length(int fd, char *inbuf, unsigned int timeout,
 void reply_writebraw(struct smb_request *req)
 {
 	connection_struct *conn = req->conn;
-	struct smbXsrv_connection *xconn = req->sconn->conn;
+	struct smbXsrv_connection *xconn = req->xconn;
 	char *buf = NULL;
 	ssize_t nwritten=0;
 	ssize_t total_written=0;
@@ -4716,7 +4715,7 @@ bool is_valid_writeX_buffer(struct smbXsrv_connection *xconn,
 void reply_write_and_X(struct smb_request *req)
 {
 	connection_struct *conn = req->conn;
-	struct smbXsrv_connection *xconn = req->sconn->conn;
+	struct smbXsrv_connection *xconn = req->xconn;
 	files_struct *fsp;
 	struct lock_struct lock;
 	off_t startpos;
-- 
1.8.1.2


From 906f0dd07f957f8cabf4107ef13b13568d38728b Mon Sep 17 00:00:00 2001
From: Stefan Metzmacher <metze@samba.org>
Date: Wed, 11 Jun 2014 12:36:14 +0200
Subject: [PATCH 81/94] s3:smbd: use req->xconn in message.c

Signed-off-by: Stefan Metzmacher <metze@samba.org>
Reviewed-by: Volker Lendecke <vl@samba.org>
---
 source3/smbd/message.c | 7 +++----
 1 file changed, 3 insertions(+), 4 deletions(-)

diff --git a/source3/smbd/message.c b/source3/smbd/message.c
index 9d8c703..a1ead4a 100644
--- a/source3/smbd/message.c
+++ b/source3/smbd/message.c
@@ -194,7 +194,7 @@ void reply_sends(struct smb_request *req)
 
 void reply_sendstrt(struct smb_request *req)
 {
-	struct smbXsrv_connection *xconn = req->sconn->conn;
+	struct smbXsrv_connection *xconn = req->xconn;
 	const uint8_t *p;
 
 	START_PROFILE(SMBsendstrt);
@@ -242,7 +242,7 @@ void reply_sendstrt(struct smb_request *req)
 
 void reply_sendtxt(struct smb_request *req)
 {
-	struct smbXsrv_connection *xconn = req->sconn->conn;
+	struct smbXsrv_connection *xconn = req->xconn;
 	int len;
 	const char *msg;
 	char *tmp;
@@ -297,8 +297,7 @@ void reply_sendtxt(struct smb_request *req)
 
 void reply_sendend(struct smb_request *req)
 {
-
-	struct smbXsrv_connection *xconn = req->sconn->conn;
+	struct smbXsrv_connection *xconn = req->xconn;
 	START_PROFILE(SMBsendend);
 
 	if (! (*lp_message_command(talloc_tos()))) {
-- 
1.8.1.2


From 8b34c79013af5f571d0d7587d71e08330577b6ea Mon Sep 17 00:00:00 2001
From: Stefan Metzmacher <metze@samba.org>
Date: Thu, 12 Jun 2014 08:51:44 +0200
Subject: [PATCH 82/94] s3:smbd: use req->xconn in smb_request_done()

Signed-off-by: Stefan Metzmacher <metze@samba.org>
Reviewed-by: Volker Lendecke <vl@samba.org>
---
 source3/smbd/process.c | 2 +-
 1 file changed, 1 insertion(+), 1 deletion(-)

diff --git a/source3/smbd/process.c b/source3/smbd/process.c
index 66cb95f..77ecdb9 100644
--- a/source3/smbd/process.c
+++ b/source3/smbd/process.c
@@ -1764,7 +1764,7 @@ void smb_request_done(struct smb_request *req)
 
 		next->vuid = SVAL(req->outbuf, smb_uid);
 		next->tid  = SVAL(req->outbuf, smb_tid);
-		status = smb1srv_tcon_lookup(req->sconn->conn, req->tid,
+		status = smb1srv_tcon_lookup(req->xconn, req->tid,
 					     now, &tcon);
 		if (NT_STATUS_IS_OK(status)) {
 			req->conn = tcon->compat;
-- 
1.8.1.2


From 13203f70bfd84947320b989c2c3bcb276ea89651 Mon Sep 17 00:00:00 2001
From: Stefan Metzmacher <metze@samba.org>
Date: Thu, 12 Jun 2014 08:48:16 +0200
Subject: [PATCH 83/94] s3:smbd: pass smbXsrv_connection to
 smbd_[un]lock_socket_internal()

Signed-off-by: Stefan Metzmacher <metze@samba.org>
Reviewed-by: Volker Lendecke <vl@samba.org>
---
 source3/smbd/process.c | 22 +++++++++++-----------
 1 file changed, 11 insertions(+), 11 deletions(-)

diff --git a/source3/smbd/process.c b/source3/smbd/process.c
index 77ecdb9..0f03d88 100644
--- a/source3/smbd/process.c
+++ b/source3/smbd/process.c
@@ -87,10 +87,8 @@ static bool smbd_echo_active(struct smbXsrv_connection *xconn)
 	return false;
 }
 
-static bool smbd_lock_socket_internal(struct smbd_server_connection *sconn)
+static bool smbd_lock_socket_internal(struct smbXsrv_connection *xconn)
 {
-	struct smbXsrv_connection *xconn = sconn->conn;
-
 	if (!smbd_echo_active(xconn)) {
 		return true;
 	}
@@ -144,15 +142,15 @@ static bool smbd_lock_socket_internal(struct smbd_server_connection *sconn)
 
 void smbd_lock_socket(struct smbd_server_connection *sconn)
 {
-	if (!smbd_lock_socket_internal(sconn)) {
+	struct smbXsrv_connection *xconn = sconn->conn;
+
+	if (!smbd_lock_socket_internal(xconn)) {
 		exit_server_cleanly("failed to lock socket");
 	}
 }
 
-static bool smbd_unlock_socket_internal(struct smbd_server_connection *sconn)
+static bool smbd_unlock_socket_internal(struct smbXsrv_connection *xconn)
 {
-	struct smbXsrv_connection *xconn = sconn->conn;
-
 	if (!smbd_echo_active(xconn)) {
 		return true;
 	}
@@ -204,7 +202,9 @@ static bool smbd_unlock_socket_internal(struct smbd_server_connection *sconn)
 
 void smbd_unlock_socket(struct smbd_server_connection *sconn)
 {
-	if (!smbd_unlock_socket_internal(sconn)) {
+	struct smbXsrv_connection *xconn = sconn->conn;
+
+	if (!smbd_unlock_socket_internal(xconn)) {
 		exit_server_cleanly("failed to unlock socket");
 	}
 }
@@ -2882,7 +2882,7 @@ static void smbd_echo_read_waited(struct tevent_req *subreq)
 		return;
 	}
 
-	ok = smbd_lock_socket_internal(sconn);
+	ok = smbd_lock_socket_internal(xconn);
 	if (!ok) {
 		tevent_req_nterror(req, map_nt_error_from_unix(errno));
 		DEBUG(0, ("%s: failed to lock socket\n", __location__));
@@ -2893,7 +2893,7 @@ static void smbd_echo_read_waited(struct tevent_req *subreq)
 		DEBUG(10,("echo_handler[%d] the parent smbd was faster\n",
 			  (int)getpid()));
 
-		ok = smbd_unlock_socket_internal(sconn);
+		ok = smbd_unlock_socket_internal(xconn);
 		if (!ok) {
 			tevent_req_nterror(req, map_nt_error_from_unix(errno));
 			DEBUG(1, ("%s: failed to unlock socket\n",
@@ -2927,7 +2927,7 @@ static void smbd_echo_read_waited(struct tevent_req *subreq)
 		return;
 	}
 
-	ok = smbd_unlock_socket_internal(sconn);
+	ok = smbd_unlock_socket_internal(xconn);
 	if (!ok) {
 		tevent_req_nterror(req, map_nt_error_from_unix(errno));
 		DEBUG(1, ("%s: failed to unlock socket\n", __location__));
-- 
1.8.1.2


From bc9c9e491c083775652bfff9ecd6099251a3e172 Mon Sep 17 00:00:00 2001
From: Stefan Metzmacher <metze@samba.org>
Date: Thu, 12 Jun 2014 10:40:12 +0200
Subject: [PATCH 84/94] s3:smbXsrv_tcon: don't pass smbXsrv_connection to
 smbXsrv_tcon_create()

A tcon can be used on multiple connections in future,
so better pass the few needed parameters explicitly
and let the caller figure out where to get them.

Signed-off-by: Stefan Metzmacher <metze@samba.org>
Reviewed-by: Volker Lendecke <vl@samba.org>
---
 source3/smbd/smbXsrv_tcon.c | 23 +++++++++++++++--------
 1 file changed, 15 insertions(+), 8 deletions(-)

diff --git a/source3/smbd/smbXsrv_tcon.c b/source3/smbd/smbXsrv_tcon.c
index 2cbd761..48a938e 100644
--- a/source3/smbd/smbXsrv_tcon.c
+++ b/source3/smbd/smbXsrv_tcon.c
@@ -697,8 +697,9 @@ static int smbXsrv_tcon_destructor(struct smbXsrv_tcon *tcon)
 	return 0;
 }
 
-static NTSTATUS smbXsrv_tcon_create(struct smbXsrv_connection *conn,
-				    struct smbXsrv_tcon_table *table,
+static NTSTATUS smbXsrv_tcon_create(struct smbXsrv_tcon_table *table,
+				    enum protocol_types protocol,
+				    struct server_id server_id,
 				    NTTIME now,
 				    struct smbXsrv_tcon **_tcon)
 {
@@ -729,7 +730,7 @@ static NTSTATUS smbXsrv_tcon_create(struct smbXsrv_connection *conn,
 	}
 	tcon->global = global;
 
-	if (conn->protocol >= PROTOCOL_SMB2_02) {
+	if (protocol >= PROTOCOL_SMB2_02) {
 		uint64_t id = global->tcon_global_id;
 		uint8_t key_buf[SMBXSRV_TCON_LOCAL_TDB_KEY_SIZE];
 		TDB_DATA key;
@@ -770,7 +771,7 @@ static NTSTATUS smbXsrv_tcon_create(struct smbXsrv_connection *conn,
 
 	global->creation_time = now;
 
-	global->server_id = messaging_server_id(conn->msg_ctx);
+	global->server_id = server_id;
 
 	ptr = tcon;
 	val = make_tdb_data((uint8_t const *)&ptr, sizeof(ptr));
@@ -1087,8 +1088,11 @@ NTSTATUS smb1srv_tcon_create(struct smbXsrv_connection *conn,
 			     NTTIME now,
 			     struct smbXsrv_tcon **_tcon)
 {
-	return smbXsrv_tcon_create(conn, conn->tcon_table, now,
-				   _tcon);
+	struct server_id id = messaging_server_id(conn->msg_ctx);
+
+	return smbXsrv_tcon_create(conn->tcon_table,
+				   conn->protocol,
+				   id, now, _tcon);
 }
 
 NTSTATUS smb1srv_tcon_lookup(struct smbXsrv_connection *conn,
@@ -1139,8 +1143,11 @@ NTSTATUS smb2srv_tcon_create(struct smbXsrv_session *session,
 			     NTTIME now,
 			     struct smbXsrv_tcon **_tcon)
 {
-	return smbXsrv_tcon_create(session->connection, session->tcon_table,
-				   now, _tcon);
+	struct server_id id = messaging_server_id(session->connection->msg_ctx);
+
+	return smbXsrv_tcon_create(session->tcon_table,
+				   PROTOCOL_SMB2_02,
+				   id, now, _tcon);
 }
 
 NTSTATUS smb2srv_tcon_lookup(struct smbXsrv_session *session,
-- 
1.8.1.2


From e492f7d673b86c19eac09d8bf085fad325cacc26 Mon Sep 17 00:00:00 2001
From: Stefan Metzmacher <metze@samba.org>
Date: Thu, 12 Jun 2014 09:19:29 +0200
Subject: [PATCH 85/94] s3:smb2_sesssetup: split out
 smbd_smb2_session_setup_auth_return()

Signed-off-by: Stefan Metzmacher <metze@samba.org>
Reviewed-by: Volker Lendecke <vl@samba.org>
---
 source3/smbd/smb2_sesssetup.c | 46 +++++++++++--------------------------------
 1 file changed, 12 insertions(+), 34 deletions(-)

diff --git a/source3/smbd/smb2_sesssetup.c b/source3/smbd/smb2_sesssetup.c
index efeac18..a8e4c38 100644
--- a/source3/smbd/smb2_sesssetup.c
+++ b/source3/smbd/smb2_sesssetup.c
@@ -501,6 +501,7 @@ static int smbd_smb2_session_setup_state_destructor(struct smbd_smb2_session_set
 
 static void smbd_smb2_session_setup_gensec_done(struct tevent_req *subreq);
 static void smbd_smb2_session_setup_previous_done(struct tevent_req *subreq);
+static void smbd_smb2_session_setup_auth_return(struct tevent_req *req);
 
 /************************************************************************
  We have to tag the state->session pointer with memory talloc'ed
@@ -679,37 +680,7 @@ static void smbd_smb2_session_setup_gensec_done(struct tevent_req *subreq)
 		return;
 	}
 
-	if (state->session->global->auth_session_info != NULL) {
-		status = smbd_smb2_reauth_generic_return(state->session,
-							 state->smb2req,
-							 state->session_info,
-							 &state->out_session_flags,
-							 &state->out_session_id);
-		if (tevent_req_nterror(req, status)) {
-			return;
-		}
-		/* we want to keep the session */
-		state->session = NULL;
-		TALLOC_FREE(state->pp_self_ref);
-		tevent_req_done(req);
-		return;
-	}
-
-	status = smbd_smb2_auth_generic_return(state->session,
-					       state->smb2req,
-					       state->in_security_mode,
-					       state->session_info,
-					       &state->out_session_flags,
-					       &state->out_session_id);
-	if (tevent_req_nterror(req, status)) {
-		return;
-	}
-
-	/* we want to keep the session */
-	state->session = NULL;
-	TALLOC_FREE(state->pp_self_ref);
-	tevent_req_done(req);
-	return;
+	smbd_smb2_session_setup_auth_return(req);
 }
 
 static void smbd_smb2_session_setup_previous_done(struct tevent_req *subreq)
@@ -717,9 +688,6 @@ static void smbd_smb2_session_setup_previous_done(struct tevent_req *subreq)
 	struct tevent_req *req =
 		tevent_req_callback_data(subreq,
 		struct tevent_req);
-	struct smbd_smb2_session_setup_state *state =
-		tevent_req_data(req,
-		struct smbd_smb2_session_setup_state);
 	NTSTATUS status;
 
 	status = smb2srv_session_close_previous_recv(subreq);
@@ -728,6 +696,16 @@ static void smbd_smb2_session_setup_previous_done(struct tevent_req *subreq)
 		return;
 	}
 
+	smbd_smb2_session_setup_auth_return(req);
+}
+
+static void smbd_smb2_session_setup_auth_return(struct tevent_req *req)
+{
+	struct smbd_smb2_session_setup_state *state =
+		tevent_req_data(req,
+		struct smbd_smb2_session_setup_state);
+	NTSTATUS status;
+
 	if (state->session->global->auth_session_info != NULL) {
 		status = smbd_smb2_reauth_generic_return(state->session,
 							 state->smb2req,
-- 
1.8.1.2


From f6dc6c64d57f6afd22f622ccdbdcfcbc74a012d6 Mon Sep 17 00:00:00 2001
From: Stefan Metzmacher <metze@samba.org>
Date: Thu, 12 Jun 2014 10:47:51 +0200
Subject: [PATCH 86/94] s3:smb2_sesssetup: make use of smb2req->[s|x]conn

We should use stuff relative to the current request.

Signed-off-by: Stefan Metzmacher <metze@samba.org>
Reviewed-by: Volker Lendecke <vl@samba.org>
---
 source3/smbd/smb2_sesssetup.c | 22 +++++++++++-----------
 1 file changed, 11 insertions(+), 11 deletions(-)

diff --git a/source3/smbd/smb2_sesssetup.c b/source3/smbd/smb2_sesssetup.c
index a8e4c38..9004e69 100644
--- a/source3/smbd/smb2_sesssetup.c
+++ b/source3/smbd/smb2_sesssetup.c
@@ -183,7 +183,7 @@ static NTSTATUS smbd_smb2_auth_generic_return(struct smbXsrv_session *session,
 	bool guest = false;
 	uint8_t session_key[16];
 	struct smbXsrv_session *x = session;
-	struct smbXsrv_connection *conn = session->connection;
+	struct smbXsrv_connection *xconn = smb2req->xconn;
 
 	if ((in_security_mode & SMB2_NEGOTIATE_SIGNING_REQUIRED) ||
 	    lp_server_signing() == SMB_SIGNING_REQUIRED) {
@@ -208,11 +208,11 @@ static NTSTATUS smbd_smb2_auth_generic_return(struct smbXsrv_session *session,
 		return NT_STATUS_ACCESS_DENIED;
 	}
 
-	if (!(conn->smb2.server.capabilities & SMB2_CAP_ENCRYPTION)) {
+	if (!(xconn->smb2.server.capabilities & SMB2_CAP_ENCRYPTION)) {
 		if (x->global->encryption_required) {
 			DEBUG(1,("reject session with dialect[0x%04X] "
 				 "as encryption is required\n",
-				 conn->smb2.server.dialect));
+				 xconn->smb2.server.dialect));
 			return NT_STATUS_ACCESS_DENIED;
 		}
 	}
@@ -233,7 +233,7 @@ static NTSTATUS smbd_smb2_auth_generic_return(struct smbXsrv_session *session,
 		return NT_STATUS_NO_MEMORY;
 	}
 
-	if (conn->protocol >= PROTOCOL_SMB2_24) {
+	if (xconn->protocol >= PROTOCOL_SMB2_24) {
 		const DATA_BLOB label = data_blob_string_const_null("SMB2AESCMAC");
 		const DATA_BLOB context = data_blob_string_const_null("SmbSign");
 
@@ -243,7 +243,7 @@ static NTSTATUS smbd_smb2_auth_generic_return(struct smbXsrv_session *session,
 				    x->global->signing_key.data);
 	}
 
-	if (conn->protocol >= PROTOCOL_SMB2_24) {
+	if (xconn->protocol >= PROTOCOL_SMB2_24) {
 		const DATA_BLOB label = data_blob_string_const_null("SMB2AESCCM");
 		const DATA_BLOB context = data_blob_string_const_null("ServerIn ");
 
@@ -261,7 +261,7 @@ static NTSTATUS smbd_smb2_auth_generic_return(struct smbXsrv_session *session,
 				    x->global->decryption_key.data);
 	}
 
-	if (conn->protocol >= PROTOCOL_SMB2_24) {
+	if (xconn->protocol >= PROTOCOL_SMB2_24) {
 		const DATA_BLOB label = data_blob_string_const_null("SMB2AESCCM");
 		const DATA_BLOB context = data_blob_string_const_null("ServerOut");
 
@@ -289,7 +289,7 @@ static NTSTATUS smbd_smb2_auth_generic_return(struct smbXsrv_session *session,
 		return NT_STATUS_NO_MEMORY;
 	}
 
-	if (conn->protocol >= PROTOCOL_SMB2_24) {
+	if (xconn->protocol >= PROTOCOL_SMB2_24) {
 		const DATA_BLOB label = data_blob_string_const_null("SMB2APP");
 		const DATA_BLOB context = data_blob_string_const_null("SmbRpc");
 
@@ -382,7 +382,7 @@ static NTSTATUS smbd_smb2_reauth_generic_return(struct smbXsrv_session *session,
 {
 	NTSTATUS status;
 	struct smbXsrv_session *x = session;
-	struct smbXsrv_connection *conn = session->connection;
+	struct smbXsrv_connection *xconn = smb2req->xconn;
 
 	data_blob_clear_free(&session_info->session_key);
 	session_info->session_key = data_blob_dup_talloc(session_info,
@@ -419,7 +419,7 @@ static NTSTATUS smbd_smb2_reauth_generic_return(struct smbXsrv_session *session,
 		return NT_STATUS_LOGON_FAILURE;
 	}
 
-	conn_clear_vuid_caches(conn->sconn, session->compat->vuid);
+	conn_clear_vuid_caches(xconn->sconn, session->compat->vuid);
 
 	*out_session_id = session->global->session_wire_id;
 
@@ -596,7 +596,7 @@ static struct tevent_req *smbd_smb2_session_setup_send(TALLOC_CTX *mem_ctx,
 
 	if (state->session->gensec == NULL) {
 		status = auth_generic_prepare(state->session,
-					      state->session->connection->remote_address,
+					      state->smb2req->xconn->remote_address,
 					      &state->session->gensec);
 		if (tevent_req_nterror(req, status)) {
 			return tevent_req_post(req, ev);
@@ -667,7 +667,7 @@ static void smbd_smb2_session_setup_gensec_done(struct tevent_req *subreq)
 	      state->in_previous_session_id))
 	{
 		subreq = smb2srv_session_close_previous_send(state, state->ev,
-						state->session->connection,
+						state->smb2req->xconn,
 						state->session_info,
 						state->in_previous_session_id,
 						state->session->global->session_wire_id);
-- 
1.8.1.2


From 8142a3e4b18049151ea47ecf2dd52b7d45a85c22 Mon Sep 17 00:00:00 2001
From: Stefan Metzmacher <metze@samba.org>
Date: Thu, 12 Jun 2014 09:49:28 +0200
Subject: [PATCH 87/94] s3:smbd: remember the time of the session setup
 auth_time

This is the time of the last reauth.

Signed-off-by: Stefan Metzmacher <metze@samba.org>
Reviewed-by: Volker Lendecke <vl@samba.org>
---
 source3/librpc/idl/smbXsrv.idl | 1 +
 source3/smbd/sesssetup.c       | 3 +++
 source3/smbd/smb2_sesssetup.c  | 2 ++
 3 files changed, 6 insertions(+)

diff --git a/source3/librpc/idl/smbXsrv.idl b/source3/librpc/idl/smbXsrv.idl
index 36710dd..ca5c3f3 100644
--- a/source3/librpc/idl/smbXsrv.idl
+++ b/source3/librpc/idl/smbXsrv.idl
@@ -98,6 +98,7 @@ interface smbXsrv
 		 * auth_session is NULL until the
 		 * session is valid for the first time.
 		 */
+		NTTIME					auth_time;
 		uint32					auth_session_info_seqnum;
 		auth_session_info			*auth_session_info;
 		uint16					connection_dialect;
diff --git a/source3/smbd/sesssetup.c b/source3/smbd/sesssetup.c
index b5e8d87..f00a55c 100644
--- a/source3/smbd/sesssetup.c
+++ b/source3/smbd/sesssetup.c
@@ -354,6 +354,7 @@ static void reply_sesssetup_and_X_spnego(struct smb_request *req)
 		session->global->auth_session_info_seqnum += 1;
 		session->global->channels[0].auth_session_info_seqnum =
 			session->global->auth_session_info_seqnum;
+		session->global->auth_time = now;
 		if (client_caps & CAP_DYNAMIC_REAUTH) {
 			session->global->expiration_time =
 				gensec_expire_time(session->gensec);
@@ -441,6 +442,7 @@ static void reply_sesssetup_and_X_spnego(struct smb_request *req)
 		session->global->auth_session_info_seqnum += 1;
 		session->global->channels[0].auth_session_info_seqnum =
 			session->global->auth_session_info_seqnum;
+		session->global->auth_time = now;
 		if (client_caps & CAP_DYNAMIC_REAUTH) {
 			session->global->expiration_time =
 				gensec_expire_time(session->gensec);
@@ -1052,6 +1054,7 @@ void reply_sesssetup_and_X(struct smb_request *req)
 	session->global->auth_session_info_seqnum += 1;
 	session->global->channels[0].auth_session_info_seqnum =
 		session->global->auth_session_info_seqnum;
+	session->global->auth_time = now;
 	session->global->expiration_time = GENSEC_EXPIRE_TIME_INFINITY;
 
 	nt_status = smbXsrv_session_update(session);
diff --git a/source3/smbd/smb2_sesssetup.c b/source3/smbd/smb2_sesssetup.c
index 9004e69..b31df84 100644
--- a/source3/smbd/smb2_sesssetup.c
+++ b/source3/smbd/smb2_sesssetup.c
@@ -341,6 +341,7 @@ static NTSTATUS smbd_smb2_auth_generic_return(struct smbXsrv_session *session,
 	session->global->auth_session_info_seqnum += 1;
 	session->global->channels[0].auth_session_info_seqnum =
 		session->global->auth_session_info_seqnum;
+	session->global->auth_time = timeval_to_nttime(&smb2req->request_time);
 	session->global->expiration_time = gensec_expire_time(session->gensec);
 
 	if (!session_claim(session)) {
@@ -409,6 +410,7 @@ static NTSTATUS smbd_smb2_reauth_generic_return(struct smbXsrv_session *session,
 	session->global->auth_session_info_seqnum += 1;
 	session->global->channels[0].auth_session_info_seqnum =
 		session->global->auth_session_info_seqnum;
+	session->global->auth_time = timeval_to_nttime(&smb2req->request_time);
 	session->global->expiration_time = gensec_expire_time(session->gensec);
 
 	status = smbXsrv_session_update(session);
-- 
1.8.1.2


From baf019b956b94b0db364cef6d5b7907e4afd7689 Mon Sep 17 00:00:00 2001
From: Stefan Metzmacher <metze@samba.org>
Date: Thu, 12 Jun 2014 08:40:25 +0200
Subject: [PATCH 88/94] s3:smbd: pass smbXsrv_connection explicitly to
 fake_sendfile()

In future (with multi-channel) a fsp can be used from multiple
connections, we need to make it explicit on which we want to reply.

Signed-off-by: Stefan Metzmacher <metze@samba.org>
Reviewed-by: Volker Lendecke <vl@samba.org>
---
 source3/smbd/proto.h     |  3 ++-
 source3/smbd/reply.c     | 10 +++++-----
 source3/smbd/smb2_read.c |  4 ++--
 3 files changed, 9 insertions(+), 8 deletions(-)

diff --git a/source3/smbd/proto.h b/source3/smbd/proto.h
index a44d1bd..3e15be0 100644
--- a/source3/smbd/proto.h
+++ b/source3/smbd/proto.h
@@ -856,7 +856,8 @@ NTSTATUS unlink_internals(connection_struct *conn, struct smb_request *req,
 			  uint32 dirtype, struct smb_filename *smb_fname,
 			  bool has_wild);
 void reply_unlink(struct smb_request *req);
-ssize_t fake_sendfile(files_struct *fsp, off_t startpos, size_t nread);
+ssize_t fake_sendfile(struct smbXsrv_connection *xconn, files_struct *fsp,
+		      off_t startpos, size_t nread);
 void sendfile_short_send(files_struct *fsp,
 				ssize_t nread,
 				size_t headersize,
diff --git a/source3/smbd/reply.c b/source3/smbd/reply.c
index d3b6765..fff32b3 100644
--- a/source3/smbd/reply.c
+++ b/source3/smbd/reply.c
@@ -3026,9 +3026,9 @@ static void fail_readraw(void)
  Fake (read/write) sendfile. Returns -1 on read or write fail.
 ****************************************************************************/
 
-ssize_t fake_sendfile(files_struct *fsp, off_t startpos, size_t nread)
+ssize_t fake_sendfile(struct smbXsrv_connection *xconn, files_struct *fsp,
+		      off_t startpos, size_t nread)
 {
-	struct smbXsrv_connection *xconn = fsp->conn->sconn->conn;
 	size_t bufsize;
 	size_t tosend = nread;
 	char *buf;
@@ -3239,7 +3239,7 @@ static void send_file_readbraw(connection_struct *conn,
 				set_use_sendfile(SNUM(conn), False);
 				DEBUG(0,("send_file_readbraw: sendfile not available. Faking..\n"));
 
-				if (fake_sendfile(fsp, startpos, nread) == -1) {
+				if (fake_sendfile(xconn, fsp, startpos, nread) == -1) {
 					DEBUG(0,("send_file_readbraw: "
 						 "fake_sendfile failed for "
 						 "file %s (%s).\n",
@@ -3774,7 +3774,7 @@ static void send_file_readX(connection_struct *conn, struct smb_request *req,
 				/* Ensure we don't do this again. */
 				set_use_sendfile(SNUM(conn), False);
 				DEBUG(0,("send_file_readX: sendfile not available. Faking..\n"));
-				nread = fake_sendfile(fsp, startpos,
+				nread = fake_sendfile(xconn, fsp, startpos,
 						      smb_maxcnt);
 				if (nread == -1) {
 					DEBUG(0,("send_file_readX: "
@@ -3850,7 +3850,7 @@ normal_read:
 			errno = saved_errno;
 			exit_server_cleanly("send_file_readX sendfile failed");
 		}
-		nread = fake_sendfile(fsp, startpos, smb_maxcnt);
+		nread = fake_sendfile(xconn, fsp, startpos, smb_maxcnt);
 		if (nread == -1) {
 			saved_errno = errno;
 			DEBUG(0,("send_file_readX: fake_sendfile failed for file "
diff --git a/source3/smbd/smb2_read.c b/source3/smbd/smb2_read.c
index 892fa15..636f068 100644
--- a/source3/smbd/smb2_read.c
+++ b/source3/smbd/smb2_read.c
@@ -217,7 +217,7 @@ static int smb2_sendfile_send_data(struct smbd_smb2_read_state *state)
 			 * Fake this up by doing read/write calls.
 			 */
 			set_use_sendfile(SNUM(fsp->conn), false);
-			nread = fake_sendfile(fsp, in_offset, in_length);
+			nread = fake_sendfile(xconn, fsp, in_offset, in_length);
 			if (nread == -1) {
 				saved_errno = errno;
 				DEBUG(0,("smb2_sendfile_send_data: fake_sendfile "
@@ -267,7 +267,7 @@ normal_read:
 			 smbXsrv_connection_dbg(xconn)));
 		exit_server_cleanly("smb2_sendfile_send_data: write_data failed");
 	}
-	nread = fake_sendfile(fsp, in_offset, in_length);
+	nread = fake_sendfile(xconn, fsp, in_offset, in_length);
 	if (nread == -1) {
 		saved_errno = errno;
 		DEBUG(0,("smb2_sendfile_send_data: fake_sendfile "
-- 
1.8.1.2


From 7d047a3722b75a1f5d2d6e25b1127ce297db5772 Mon Sep 17 00:00:00 2001
From: Stefan Metzmacher <metze@samba.org>
Date: Thu, 12 Jun 2014 08:42:16 +0200
Subject: [PATCH 89/94] s3:smbd: pass smbXsrv_connection explicitly to
 sendfile_short_send()

We now let the caller terminate the connection.

Signed-off-by: Stefan Metzmacher <metze@samba.org>
Reviewed-by: Volker Lendecke <vl@samba.org>
---
 source3/smbd/proto.h     |  9 +++++----
 source3/smbd/reply.c     | 44 ++++++++++++++++++++++++++++++--------------
 source3/smbd/smb2_read.c | 13 ++++++++++++-
 3 files changed, 47 insertions(+), 19 deletions(-)

diff --git a/source3/smbd/proto.h b/source3/smbd/proto.h
index 3e15be0..453d829 100644
--- a/source3/smbd/proto.h
+++ b/source3/smbd/proto.h
@@ -858,10 +858,11 @@ NTSTATUS unlink_internals(connection_struct *conn, struct smb_request *req,
 void reply_unlink(struct smb_request *req);
 ssize_t fake_sendfile(struct smbXsrv_connection *xconn, files_struct *fsp,
 		      off_t startpos, size_t nread);
-void sendfile_short_send(files_struct *fsp,
-				ssize_t nread,
-				size_t headersize,
-				size_t smb_maxcnt);
+ssize_t sendfile_short_send(struct smbXsrv_connection *xconn,
+			    files_struct *fsp,
+			    ssize_t nread,
+			    size_t headersize,
+			    size_t smb_maxcnt);
 void reply_readbraw(struct smb_request *req);
 void reply_lockread(struct smb_request *req);
 void reply_read(struct smb_request *req);
diff --git a/source3/smbd/reply.c b/source3/smbd/reply.c
index fff32b3..28044f1 100644
--- a/source3/smbd/reply.c
+++ b/source3/smbd/reply.c
@@ -3088,22 +3088,21 @@ ssize_t fake_sendfile(struct smbXsrv_connection *xconn, files_struct *fsp,
 
 /****************************************************************************
  Deal with the case of sendfile reading less bytes from the file than
- requested. Fill with zeros (all we can do).
+ requested. Fill with zeros (all we can do). Returns 0 on success
 ****************************************************************************/
 
-void sendfile_short_send(files_struct *fsp,
-				ssize_t nread,
-				size_t headersize,
-				size_t smb_maxcnt)
+ssize_t sendfile_short_send(struct smbXsrv_connection *xconn,
+			    files_struct *fsp,
+			    ssize_t nread,
+			    size_t headersize,
+			    size_t smb_maxcnt)
 {
-	struct smbXsrv_connection *xconn = fsp->conn->sconn->conn;
-
 #define SHORT_SEND_BUFSIZE 1024
 	if (nread < headersize) {
 		DEBUG(0,("sendfile_short_send: sendfile failed to send "
 			"header for file %s (%s). Terminating\n",
 			fsp_str_dbg(fsp), strerror(errno)));
-		exit_server_cleanly("sendfile_short_send failed");
+		return -1;
 	}
 
 	nread -= headersize;
@@ -3111,8 +3110,10 @@ void sendfile_short_send(files_struct *fsp,
 	if (nread < smb_maxcnt) {
 		char *buf = SMB_CALLOC_ARRAY(char, SHORT_SEND_BUFSIZE);
 		if (!buf) {
-			exit_server_cleanly("sendfile_short_send: "
-				"malloc failed");
+			DEBUG(0,("sendfile_short_send: malloc failed "
+				"for file %s (%s). Terminating\n",
+				fsp_str_dbg(fsp), strerror(errno)));
+			return -1;
 		}
 
 		DEBUG(0,("sendfile_short_send: filling truncated file %s "
@@ -3149,13 +3150,14 @@ void sendfile_short_send(files_struct *fsp,
 					  smbXsrv_connection_dbg(xconn),
 					  strerror(saved_errno)));
 				errno = saved_errno;
-				exit_server_cleanly("sendfile_short_send: "
-						    "write_data failed");
+				return -1;
 			}
 			nread += to_write;
 		}
 		SAFE_FREE(buf);
 	}
+
+	return 0;
 }
 
 /****************************************************************************
@@ -3270,7 +3272,11 @@ static void send_file_readbraw(connection_struct *conn,
 
 		/* Deal with possible short send. */
 		if (sendfile_read != 4+nread) {
-			sendfile_short_send(fsp, sendfile_read, 4, nread);
+			ret = sendfile_short_send(xconn, fsp,
+						  sendfile_read, 4, nread);
+			if (ret == -1) {
+				fail_readraw();
+			}
 		}
 		return;
 	}
@@ -3816,7 +3822,17 @@ static void send_file_readX(connection_struct *conn, struct smb_request *req,
 
 		/* Deal with possible short send. */
 		if (nread != smb_maxcnt + sizeof(headerbuf)) {
-			sendfile_short_send(fsp, nread, sizeof(headerbuf), smb_maxcnt);
+			ssize_t ret;
+
+			ret = sendfile_short_send(xconn, fsp, nread,
+						  sizeof(headerbuf), smb_maxcnt);
+			if (ret == -1) {
+				const char *r;
+				r = "send_file_readX: sendfile_short_send failed";
+				DEBUG(0,("%s for file %s (%s).\n",
+					 r, fsp_str_dbg(fsp), strerror(errno)));
+				exit_server_cleanly(r);
+			}
 		}
 		/* No outbuf here means successful sendfile. */
 		SMB_PERFCOUNT_SET_MSGLEN_OUT(&req->pcd, nread);
diff --git a/source3/smbd/smb2_read.c b/source3/smbd/smb2_read.c
index 636f068..f485fe9 100644
--- a/source3/smbd/smb2_read.c
+++ b/source3/smbd/smb2_read.c
@@ -282,7 +282,18 @@ normal_read:
   out:
 
 	if (nread < in_length) {
-		sendfile_short_send(fsp, nread, hdr->length, in_length);
+		ret = sendfile_short_send(xconn, fsp, nread,
+					  hdr->length, in_length);
+		if (ret == -1) {
+			saved_errno = errno;
+			DEBUG(0,("smb2_sendfile_send_data: sendfile_short_send "
+				 "failed for file %s (%s) for client %s. "
+				 "Terminating\n",
+				 fsp_str_dbg(fsp), strerror(saved_errno),
+				 smbXsrv_connection_dbg(xconn)));
+			exit_server_cleanly("smb2_sendfile_send_data: "
+				"sendfile_short_send failed");
+		}
 	}
 
 	init_strict_lock_struct(fsp,
-- 
1.8.1.2


From 097fd0ba0d08b02b1fb4e7c1600238859682d881 Mon Sep 17 00:00:00 2001
From: Volker Lendecke <vl@samba.org>
Date: Wed, 23 Jul 2014 13:26:19 +0000
Subject: [PATCH 90/94] smbd: Use %s/__func__

Signed-off-by: Volker Lendecke <vl@samba.org>
---
 source3/smbd/smb2_read.c | 3 ++-
 1 file changed, 2 insertions(+), 1 deletion(-)

diff --git a/source3/smbd/smb2_read.c b/source3/smbd/smb2_read.c
index f485fe9..0d08114 100644
--- a/source3/smbd/smb2_read.c
+++ b/source3/smbd/smb2_read.c
@@ -286,9 +286,10 @@ normal_read:
 					  hdr->length, in_length);
 		if (ret == -1) {
 			saved_errno = errno;
-			DEBUG(0,("smb2_sendfile_send_data: sendfile_short_send "
+			DEBUG(0,("%s: sendfile_short_send "
 				 "failed for file %s (%s) for client %s. "
 				 "Terminating\n",
+				 __func__,
 				 fsp_str_dbg(fsp), strerror(saved_errno),
 				 smbXsrv_connection_dbg(xconn)));
 			exit_server_cleanly("smb2_sendfile_send_data: "
-- 
1.8.1.2


From 31d7d0aa52e0b851afd0e84cbefe91efd7f48be7 Mon Sep 17 00:00:00 2001
From: Stefan Metzmacher <metze@samba.org>
Date: Fri, 11 Jul 2014 02:25:00 +0200
Subject: [PATCH 91/94] s3:smb2_server: propagate NTSTATUS from
 smb2_sendfile_send_data() to smbd_smb2_flush_send_queue()

Calling exit_server() from within a destructor is a bit ugly...

This will result in smbd_server_connection_terminate() instead of
directly calling exit_server(), which will be useful for multi-channel in
future.

Signed-off-by: Stefan Metzmacher <metze@samba.org>
Reviewed-by: Volker Lendecke <vl@samba.org>
---
 source3/smbd/globals.h     |  1 +
 source3/smbd/smb2_read.c   | 21 +++++++++++++--------
 source3/smbd/smb2_server.c |  6 ++++++
 3 files changed, 20 insertions(+), 8 deletions(-)

diff --git a/source3/smbd/globals.h b/source3/smbd/globals.h
index 97bd968..690fc82 100644
--- a/source3/smbd/globals.h
+++ b/source3/smbd/globals.h
@@ -615,6 +615,7 @@ struct smbd_smb2_send_queue {
 	struct smbd_smb2_send_queue *prev, *next;
 
 	DATA_BLOB *sendfile_header;
+	NTSTATUS *sendfile_status;
 	struct iovec *vector;
 	int count;
 
diff --git a/source3/smbd/smb2_read.c b/source3/smbd/smb2_read.c
index 0d08114..470e496 100644
--- a/source3/smbd/smb2_read.c
+++ b/source3/smbd/smb2_read.c
@@ -186,6 +186,7 @@ static int smb2_sendfile_send_data(struct smbd_smb2_read_state *state)
 	uint64_t in_offset = state->in_offset;
 	files_struct *fsp = state->fsp;
 	const DATA_BLOB *hdr = state->smb2req->queue_entry.sendfile_header;
+	NTSTATUS *pstatus = state->smb2req->queue_entry.sendfile_status;
 	struct smbXsrv_connection *xconn = state->smb2req->xconn;
 	ssize_t nread;
 	ssize_t ret;
@@ -225,8 +226,8 @@ static int smb2_sendfile_send_data(struct smbd_smb2_read_state *state)
 					 "Terminating\n",
 					 fsp_str_dbg(fsp), strerror(saved_errno),
 					 smbXsrv_connection_dbg(xconn)));
-				exit_server_cleanly("smb2_sendfile_send_data: "
-					"fake_sendfile failed");
+				*pstatus = map_nt_error_from_unix_common(saved_errno);
+				return 0;
 			}
 			goto out;
 		}
@@ -235,7 +236,8 @@ static int smb2_sendfile_send_data(struct smbd_smb2_read_state *state)
 			 "%s (%s) for client %s. Terminating\n",
 			 fsp_str_dbg(fsp), strerror(saved_errno),
 			 smbXsrv_connection_dbg(xconn)));
-		exit_server_cleanly("smb2_sendfile_send_data: sendfile failed");
+		*pstatus = map_nt_error_from_unix_common(saved_errno);
+		return 0;
 	} else if (nread == 0) {
 		/*
 		 * Some sendfile implementations return 0 to indicate
@@ -265,7 +267,8 @@ normal_read:
 			 "%s (%s) for client %s. Terminating\n",
 			 fsp_str_dbg(fsp), strerror(saved_errno),
 			 smbXsrv_connection_dbg(xconn)));
-		exit_server_cleanly("smb2_sendfile_send_data: write_data failed");
+		*pstatus = map_nt_error_from_unix_common(saved_errno);
+		return 0;
 	}
 	nread = fake_sendfile(xconn, fsp, in_offset, in_length);
 	if (nread == -1) {
@@ -275,8 +278,8 @@ normal_read:
 			 "Terminating\n",
 			 fsp_str_dbg(fsp), strerror(saved_errno),
 			 smbXsrv_connection_dbg(xconn)));
-		exit_server_cleanly("smb2_sendfile_send_data: "
-			"fake_sendfile failed");
+		*pstatus = map_nt_error_from_unix_common(saved_errno);
+		return 0;
 	}
 
   out:
@@ -292,8 +295,8 @@ normal_read:
 				 __func__,
 				 fsp_str_dbg(fsp), strerror(saved_errno),
 				 smbXsrv_connection_dbg(xconn)));
-			exit_server_cleanly("smb2_sendfile_send_data: "
-				"sendfile_short_send failed");
+			*pstatus = map_nt_error_from_unix_common(saved_errno);
+			return 0;
 		}
 	}
 
@@ -305,6 +308,8 @@ normal_read:
 				&lock);
 
 	SMB_VFS_STRICT_UNLOCK(fsp->conn, fsp, &lock);
+
+	*pstatus = NT_STATUS_OK;
 	return 0;
 }
 
diff --git a/source3/smbd/smb2_server.c b/source3/smbd/smb2_server.c
index d9e6a76..984d440 100644
--- a/source3/smbd/smb2_server.c
+++ b/source3/smbd/smb2_server.c
@@ -3115,6 +3115,7 @@ static NTSTATUS smbd_smb2_flush_send_queue(struct smbXsrv_connection *xconn)
 		struct smbd_smb2_send_queue *e = xconn->smb2.send_queue;
 
 		if (e->sendfile_header != NULL) {
+			NTSTATUS status = NT_STATUS_INTERNAL_ERROR;
 			size_t size = 0;
 			size_t i = 0;
 			uint8_t *buf;
@@ -3142,6 +3143,7 @@ static NTSTATUS smbd_smb2_flush_send_queue(struct smbXsrv_connection *xconn)
 
 			e->sendfile_header->data = buf;
 			e->sendfile_header->length = size;
+			e->sendfile_status = &status;
 			e->count = 0;
 
 			xconn->smb2.send_queue_len--;
@@ -3151,6 +3153,10 @@ static NTSTATUS smbd_smb2_flush_send_queue(struct smbXsrv_connection *xconn)
 			 * the destructor.
 			 */
 			talloc_free(e->mem_ctx);
+
+			if (!NT_STATUS_IS_OK(status)) {
+				return status;
+			}
 			continue;
 		}
 
-- 
1.8.1.2


From 94e6754d46daa0b2728e43e1e8c59016a76a09c2 Mon Sep 17 00:00:00 2001
From: Stefan Metzmacher <metze@samba.org>
Date: Thu, 12 Jun 2014 08:43:26 +0200
Subject: [PATCH 92/94] s3:smbd: pass smbXsrv_connection to smb1 encryption
 functions

These parameters are not really used currently, but may in future.

Signed-off-by: Stefan Metzmacher <metze@samba.org>
Reviewed-by: Volker Lendecke <vl@samba.org>
---
 source3/smbd/process.c | 8 ++++----
 source3/smbd/proto.h   | 8 ++++----
 source3/smbd/seal.c    | 8 ++++----
 3 files changed, 12 insertions(+), 12 deletions(-)

diff --git a/source3/smbd/process.c b/source3/smbd/process.c
index 0f03d88..780a67a 100644
--- a/source3/smbd/process.c
+++ b/source3/smbd/process.c
@@ -240,7 +240,7 @@ bool srv_send_smb(struct smbd_server_connection *sconn, char *buffer,
 	}
 
 	if (do_encrypt) {
-		NTSTATUS status = srv_encrypt_buffer(sconn, buffer, &buf_out);
+		NTSTATUS status = srv_encrypt_buffer(xconn, buffer, &buf_out);
 		if (!NT_STATUS_IS_OK(status)) {
 			DEBUG(0, ("send_smb: SMB encryption failed "
 				"on outgoing packet! Error %s\n",
@@ -265,12 +265,12 @@ bool srv_send_smb(struct smbd_server_connection *sconn, char *buffer,
 			 (int)ret, strerror(saved_errno)));
 		errno = saved_errno;
 
-		srv_free_enc_buffer(sconn, buf_out);
+		srv_free_enc_buffer(xconn, buf_out);
 		goto out;
 	}
 
 	SMB_PERFCOUNT_SET_MSGLEN_OUT(pcd, len);
-	srv_free_enc_buffer(sconn, buf_out);
+	srv_free_enc_buffer(xconn, buf_out);
 out:
 	SMB_PERFCOUNT_END(pcd);
 
@@ -565,7 +565,7 @@ static NTSTATUS receive_smb_talloc(TALLOC_CTX *mem_ctx,
 	}
 
 	if (is_encrypted_packet((uint8_t *)*buffer)) {
-		status = srv_decrypt_buffer(sconn, *buffer);
+		status = srv_decrypt_buffer(xconn, *buffer);
 		if (!NT_STATUS_IS_OK(status)) {
 			DEBUG(0, ("receive_smb_talloc: SMB decryption failed on "
 				"incoming packet! Error %s\n",
diff --git a/source3/smbd/proto.h b/source3/smbd/proto.h
index 453d829..22cd921 100644
--- a/source3/smbd/proto.h
+++ b/source3/smbd/proto.h
@@ -930,9 +930,9 @@ void reply_getattrE(struct smb_request *req);
 /* The following definitions come from smbd/seal.c  */
 
 bool is_encrypted_packet(const uint8_t *inbuf);
-void srv_free_enc_buffer(struct smbd_server_connection *sconn, char *buf);
-NTSTATUS srv_decrypt_buffer(struct smbd_server_connection *sconn, char *buf);
-NTSTATUS srv_encrypt_buffer(struct smbd_server_connection *sconn, char *buf,
+void srv_free_enc_buffer(struct smbXsrv_connection *xconn, char *buf);
+NTSTATUS srv_decrypt_buffer(struct smbXsrv_connection *xconn, char *buf);
+NTSTATUS srv_encrypt_buffer(struct smbXsrv_connection *xconn, char *buf,
 			    char **buf_out);
 NTSTATUS srv_request_encryption_setup(connection_struct *conn,
 					unsigned char **ppdata,
@@ -940,7 +940,7 @@ NTSTATUS srv_request_encryption_setup(connection_struct *conn,
 					unsigned char **pparam,
 					size_t *p_param_size);
 NTSTATUS srv_encryption_start(connection_struct *conn);
-void server_encryption_shutdown(struct smbd_server_connection *sconn);
+void server_encryption_shutdown(struct smbXsrv_connection *xconn);
 
 /* The following definitions come from smbd/sec_ctx.c  */
 
diff --git a/source3/smbd/seal.c b/source3/smbd/seal.c
index 7ab992e..e088d74 100644
--- a/source3/smbd/seal.c
+++ b/source3/smbd/seal.c
@@ -134,7 +134,7 @@ static NTSTATUS make_srv_encryption_context(const struct tsocket_address *remote
  Free an encryption-allocated buffer.
 ******************************************************************************/
 
-void srv_free_enc_buffer(struct smbd_server_connection *sconn, char *buf)
+void srv_free_enc_buffer(struct smbXsrv_connection *xconn, char *buf)
 {
 	/* We know this is an smb buffer, and we
 	 * didn't malloc, only copy, for a keepalive,
@@ -153,7 +153,7 @@ void srv_free_enc_buffer(struct smbd_server_connection *sconn, char *buf)
  Decrypt an incoming buffer.
 ******************************************************************************/
 
-NTSTATUS srv_decrypt_buffer(struct smbd_server_connection *sconn, char *buf)
+NTSTATUS srv_decrypt_buffer(struct smbXsrv_connection *xconn, char *buf)
 {
 	/* Ignore non-session messages. */
 	if(CVAL(buf,0)) {
@@ -171,7 +171,7 @@ NTSTATUS srv_decrypt_buffer(struct smbd_server_connection *sconn, char *buf)
  Encrypt an outgoing buffer. Return the encrypted pointer in buf_out.
 ******************************************************************************/
 
-NTSTATUS srv_encrypt_buffer(struct smbd_server_connection *sconn, char *buf,
+NTSTATUS srv_encrypt_buffer(struct smbXsrv_connection *xconn, char *buf,
 			    char **buf_out)
 {
 	*buf_out = buf;
@@ -302,7 +302,7 @@ NTSTATUS srv_encryption_start(connection_struct *conn)
  Shutdown all server contexts.
 ******************************************************************************/
 
-void server_encryption_shutdown(struct smbd_server_connection *sconn)
+void server_encryption_shutdown(struct smbXsrv_connection *xconn)
 {
 	TALLOC_FREE(partial_srv_trans_enc_ctx);
 	TALLOC_FREE(srv_trans_enc_ctx);
-- 
1.8.1.2


From 84f1fe5ca3893b02f657ba46fe119d5514e83b85 Mon Sep 17 00:00:00 2001
From: Stefan Metzmacher <metze@samba.org>
Date: Thu, 12 Jun 2014 08:43:26 +0200
Subject: [PATCH 93/94] s3:smb2_server: pass smbXsrv_connection to
 smbd_smb2_send_break()

Signed-off-by: Stefan Metzmacher <metze@samba.org>
Reviewed-by: Volker Lendecke <vl@samba.org>
---
 source3/smbd/smb2_server.c | 8 ++++----
 1 file changed, 4 insertions(+), 4 deletions(-)

diff --git a/source3/smbd/smb2_server.c b/source3/smbd/smb2_server.c
index 984d440..497c5dc 100644
--- a/source3/smbd/smb2_server.c
+++ b/source3/smbd/smb2_server.c
@@ -2734,14 +2734,13 @@ struct smbd_smb2_send_break_state {
 	uint8_t body[1];
 };
 
-static NTSTATUS smbd_smb2_send_break(struct smbd_server_connection *sconn,
+static NTSTATUS smbd_smb2_send_break(struct smbXsrv_connection *xconn,
 				     struct smbXsrv_session *session,
 				     struct smbXsrv_tcon *tcon,
 				     const uint8_t *body,
 				     size_t body_len)
 {
 	struct smbd_smb2_send_break_state *state;
-	struct smbXsrv_connection *xconn = sconn->conn;
 	bool do_encryption = session->global->encryption_required;
 	uint64_t nonce_high = 0;
 	uint64_t nonce_low = 0;
@@ -2755,7 +2754,7 @@ static NTSTATUS smbd_smb2_send_break(struct smbd_server_connection *sconn,
 	statelen = offsetof(struct smbd_smb2_send_break_state, body) +
 		body_len;
 
-	state = talloc_zero_size(sconn, statelen);
+	state = talloc_zero_size(xconn, statelen);
 	if (state == NULL) {
 		return NT_STATUS_NO_MEMORY;
 	}
@@ -2858,6 +2857,7 @@ NTSTATUS smbd_smb2_send_oplock_break(struct smbd_server_connection *sconn,
 				     struct smbXsrv_open *op,
 				     uint8_t oplock_level)
 {
+	struct smbXsrv_connection *xconn = sconn->conn;
 	uint8_t body[0x18];
 
 	SSVAL(body, 0x00, sizeof(body));
@@ -2867,7 +2867,7 @@ NTSTATUS smbd_smb2_send_oplock_break(struct smbd_server_connection *sconn,
 	SBVAL(body, 0x08, op->global->open_persistent_id);
 	SBVAL(body, 0x10, op->global->open_volatile_id);
 
-	return smbd_smb2_send_break(sconn, session, tcon, body, sizeof(body));
+	return smbd_smb2_send_break(xconn, session, tcon, body, sizeof(body));
 }
 
 static bool is_smb2_recvfile_write(struct smbd_smb2_request_read_state *state)
-- 
1.8.1.2


From 29dc7c06789e7a12f57a0f15bc4460a8ae318b5c Mon Sep 17 00:00:00 2001
From: Stefan Metzmacher <metze@samba.org>
Date: Thu, 12 Jun 2014 08:43:26 +0200
Subject: [PATCH 94/94] s3:smbd: pass smbXsrv_connection to
 smbd_[un]lock_socket()

Signed-off-by: Stefan Metzmacher <metze@samba.org>
Reviewed-by: Volker Lendecke <vl@samba.org>
---
 source3/smbd/globals.h |  4 ++--
 source3/smbd/process.c | 22 +++++++++-------------
 source3/smbd/reply.c   |  9 ++++-----
 3 files changed, 15 insertions(+), 20 deletions(-)

diff --git a/source3/smbd/globals.h b/source3/smbd/globals.h
index 690fc82..53b99e3 100644
--- a/source3/smbd/globals.h
+++ b/source3/smbd/globals.h
@@ -113,8 +113,8 @@ struct smbd_smb2_request;
 
 DATA_BLOB negprot_spnego(TALLOC_CTX *ctx, struct smbXsrv_connection *xconn);
 
-void smbd_lock_socket(struct smbd_server_connection *sconn);
-void smbd_unlock_socket(struct smbd_server_connection *sconn);
+void smbd_lock_socket(struct smbXsrv_connection *xconn);
+void smbd_unlock_socket(struct smbXsrv_connection *xconn);
 
 NTSTATUS smbd_do_locking(struct smb_request *req,
 			 files_struct *fsp,
diff --git a/source3/smbd/process.c b/source3/smbd/process.c
index 780a67a..7148462 100644
--- a/source3/smbd/process.c
+++ b/source3/smbd/process.c
@@ -140,10 +140,8 @@ static bool smbd_lock_socket_internal(struct smbXsrv_connection *xconn)
 	return true;
 }
 
-void smbd_lock_socket(struct smbd_server_connection *sconn)
+void smbd_lock_socket(struct smbXsrv_connection *xconn)
 {
-	struct smbXsrv_connection *xconn = sconn->conn;
-
 	if (!smbd_lock_socket_internal(xconn)) {
 		exit_server_cleanly("failed to lock socket");
 	}
@@ -200,10 +198,8 @@ static bool smbd_unlock_socket_internal(struct smbXsrv_connection *xconn)
 	return true;
 }
 
-void smbd_unlock_socket(struct smbd_server_connection *sconn)
+void smbd_unlock_socket(struct smbXsrv_connection *xconn)
 {
-	struct smbXsrv_connection *xconn = sconn->conn;
-
 	if (!smbd_unlock_socket_internal(xconn)) {
 		exit_server_cleanly("failed to unlock socket");
 	}
@@ -232,7 +228,7 @@ bool srv_send_smb(struct smbd_server_connection *sconn, char *buffer,
 		return true;
 	}
 
-	smbd_lock_socket(sconn);
+	smbd_lock_socket(xconn);
 
 	if (do_signing) {
 		/* Sign the outgoing packet if required. */
@@ -274,7 +270,7 @@ bool srv_send_smb(struct smbd_server_connection *sconn, char *buffer,
 out:
 	SMB_PERFCOUNT_END(pcd);
 
-	smbd_unlock_socket(sconn);
+	smbd_unlock_socket(xconn);
 	return (ret > 0);
 }
 
@@ -2477,11 +2473,11 @@ static void smbd_server_connection_read_handler(
 	from_client = (xconn->transport.sock == fd);
 
 	if (async_echo && from_client) {
-		smbd_lock_socket(sconn);
+		smbd_lock_socket(xconn);
 
 		if (!fd_is_readable(fd)) {
 			DEBUG(10,("the echo listener was faster\n"));
-			smbd_unlock_socket(sconn);
+			smbd_unlock_socket(xconn);
 			return;
 		}
 	}
@@ -2496,7 +2492,7 @@ static void smbd_server_connection_read_handler(
 				    !from_client /* trusted channel */);
 
 	if (async_echo && from_client) {
-		smbd_unlock_socket(sconn);
+		smbd_unlock_socket(xconn);
 	}
 
 	if (NT_STATUS_EQUAL(status, NT_STATUS_RETRY)) {
@@ -2726,9 +2722,9 @@ static bool keepalive_fn(const struct timeval *now, void *private_data)
 		return false;
 	}
 
-	smbd_lock_socket(sconn);
+	smbd_lock_socket(xconn);
 	ret = send_keepalive(xconn->transport.sock);
-	smbd_unlock_socket(sconn);
+	smbd_unlock_socket(xconn);
 
 	if (!ret) {
 		int saved_errno = errno;
diff --git a/source3/smbd/reply.c b/source3/smbd/reply.c
index 28044f1..c04c425 100644
--- a/source3/smbd/reply.c
+++ b/source3/smbd/reply.c
@@ -3166,12 +3166,11 @@ ssize_t sendfile_short_send(struct smbXsrv_connection *xconn,
 
 static void reply_readbraw_error(struct smbXsrv_connection *xconn)
 {
-	struct smbd_server_connection *sconn = xconn->sconn;
 	char header[4];
 
 	SIVAL(header,0,0);
 
-	smbd_lock_socket(sconn);
+	smbd_lock_socket(xconn);
 	if (write_data(xconn->transport.sock,header,4) != 4) {
 		int saved_errno = errno;
 		/*
@@ -3186,7 +3185,7 @@ static void reply_readbraw_error(struct smbXsrv_connection *xconn)
 
 		fail_readraw();
 	}
-	smbd_unlock_socket(sconn);
+	smbd_unlock_socket(xconn);
 }
 
 /****************************************************************************
@@ -4072,9 +4071,9 @@ void reply_read_and_X(struct smb_request *req)
 		/* NT_STATUS_RETRY - fall back to sync read. */
 	}
 
-	smbd_lock_socket(req->sconn);
+	smbd_lock_socket(req->xconn);
 	send_file_readX(conn, req, fsp,	startpos, smb_maxcnt);
-	smbd_unlock_socket(req->sconn);
+	smbd_unlock_socket(req->xconn);
 
  out:
 	END_PROFILE(SMBreadX);
-- 
1.8.1.2

